/* This file was genereated on Mon Feb 16 18:08:57 +0100 2009
   source: http://oss.sgi.com/cgi-bin/cvsweb.cgi/~checkout~/projects/ogl-sample/main/doc/registry/specs/enumglu.spec
*/

#ifndef GLU_AUTO_LOAD_MATRIX
#define GLU_AUTO_LOAD_MATRIX 100200
#endif
#ifndef GLU_BEGIN
#define GLU_BEGIN 100100
#endif
#ifndef GLU_CCW
#define GLU_CCW 100121
#endif
#ifndef GLU_CULLING
#define GLU_CULLING 100201
#endif
#ifndef GLU_CW
#define GLU_CW 100120
#endif
#ifndef GLU_DISPLAY_MODE
#define GLU_DISPLAY_MODE 100204
#endif
#ifndef GLU_DOMAIN_DISTANCE
#define GLU_DOMAIN_DISTANCE 100217
#endif
#ifndef GLU_EDGE_FLAG
#define GLU_EDGE_FLAG 100104
#endif
#ifndef GLU_END
#define GLU_END 100102
#endif
#ifndef GLU_ERROR
#define GLU_ERROR 100103
#endif
#ifndef GLU_EXTENSIONS
#define GLU_EXTENSIONS 100801
#endif
#ifndef GLU_EXTERIOR
#define GLU_EXTERIOR 100123
#endif
#ifndef GLU_EXT_nurbs_tessellator
#define GLU_EXT_nurbs_tessellator 1
#endif
#ifndef GLU_EXT_object_space_tess
#define GLU_EXT_object_space_tess 1
#endif
#ifndef GLU_FILL
#define GLU_FILL 100012
#endif
#ifndef GLU_FLAT
#define GLU_FLAT 100001
#endif
#ifndef GLU_INSIDE
#define GLU_INSIDE 100021
#endif
#ifndef GLU_INTERIOR
#define GLU_INTERIOR 100122
#endif
#ifndef GLU_INVALID_ENUM
#define GLU_INVALID_ENUM 100900
#endif
#ifndef GLU_INVALID_OPERATION
#define GLU_INVALID_OPERATION 100904
#endif
#ifndef GLU_INVALID_VALUE
#define GLU_INVALID_VALUE 100901
#endif
#ifndef GLU_LAGRANGIAN_SGI
#define GLU_LAGRANGIAN_SGI 100300
#endif
#ifndef GLU_LINE
#define GLU_LINE 100011
#endif
#ifndef GLU_MAP1_TRIM_2
#define GLU_MAP1_TRIM_2 100210
#endif
#ifndef GLU_MAP1_TRIM_3
#define GLU_MAP1_TRIM_3 100211
#endif
#ifndef GLU_MITCHELL_NETRAVALI_SGI
#define GLU_MITCHELL_NETRAVALI_SGI 100301
#endif
#ifndef GLU_NONE
#define GLU_NONE 100002
#endif
#ifndef GLU_NURBS_BEGIN
#define GLU_NURBS_BEGIN 100164
#endif
#ifndef GLU_NURBS_BEGIN_DATA
#define GLU_NURBS_BEGIN_DATA 100170
#endif
#ifndef GLU_NURBS_BEGIN_DATA_EXT
#define GLU_NURBS_BEGIN_DATA_EXT 100170
#endif
#ifndef GLU_NURBS_BEGIN_EXT
#define GLU_NURBS_BEGIN_EXT 100164
#endif
#ifndef GLU_NURBS_COLOR
#define GLU_NURBS_COLOR 100167
#endif
#ifndef GLU_NURBS_COLOR_DATA
#define GLU_NURBS_COLOR_DATA 100173
#endif
#ifndef GLU_NURBS_COLOR_DATA_EXT
#define GLU_NURBS_COLOR_DATA_EXT 100173
#endif
#ifndef GLU_NURBS_COLOR_EXT
#define GLU_NURBS_COLOR_EXT 100167
#endif
#ifndef GLU_NURBS_END
#define GLU_NURBS_END 100169
#endif
#ifndef GLU_NURBS_END_DATA
#define GLU_NURBS_END_DATA 100175
#endif
#ifndef GLU_NURBS_END_DATA_EXT
#define GLU_NURBS_END_DATA_EXT 100175
#endif
#ifndef GLU_NURBS_END_EXT
#define GLU_NURBS_END_EXT 100169
#endif
#ifndef GLU_NURBS_ERROR
#define GLU_NURBS_ERROR 100103
#endif
#ifndef GLU_NURBS_ERROR1
#define GLU_NURBS_ERROR1 100251
#endif
#ifndef GLU_NURBS_ERROR10
#define GLU_NURBS_ERROR10 100260
#endif
#ifndef GLU_NURBS_ERROR11
#define GLU_NURBS_ERROR11 100261
#endif
#ifndef GLU_NURBS_ERROR12
#define GLU_NURBS_ERROR12 100262
#endif
#ifndef GLU_NURBS_ERROR13
#define GLU_NURBS_ERROR13 100263
#endif
#ifndef GLU_NURBS_ERROR14
#define GLU_NURBS_ERROR14 100264
#endif
#ifndef GLU_NURBS_ERROR15
#define GLU_NURBS_ERROR15 100265
#endif
#ifndef GLU_NURBS_ERROR16
#define GLU_NURBS_ERROR16 100266
#endif
#ifndef GLU_NURBS_ERROR17
#define GLU_NURBS_ERROR17 100267
#endif
#ifndef GLU_NURBS_ERROR18
#define GLU_NURBS_ERROR18 100268
#endif
#ifndef GLU_NURBS_ERROR19
#define GLU_NURBS_ERROR19 100269
#endif
#ifndef GLU_NURBS_ERROR2
#define GLU_NURBS_ERROR2 100252
#endif
#ifndef GLU_NURBS_ERROR20
#define GLU_NURBS_ERROR20 100270
#endif
#ifndef GLU_NURBS_ERROR21
#define GLU_NURBS_ERROR21 100271
#endif
#ifndef GLU_NURBS_ERROR22
#define GLU_NURBS_ERROR22 100272
#endif
#ifndef GLU_NURBS_ERROR23
#define GLU_NURBS_ERROR23 100273
#endif
#ifndef GLU_NURBS_ERROR24
#define GLU_NURBS_ERROR24 100274
#endif
#ifndef GLU_NURBS_ERROR25
#define GLU_NURBS_ERROR25 100275
#endif
#ifndef GLU_NURBS_ERROR26
#define GLU_NURBS_ERROR26 100276
#endif
#ifndef GLU_NURBS_ERROR27
#define GLU_NURBS_ERROR27 100277
#endif
#ifndef GLU_NURBS_ERROR28
#define GLU_NURBS_ERROR28 100278
#endif
#ifndef GLU_NURBS_ERROR29
#define GLU_NURBS_ERROR29 100279
#endif
#ifndef GLU_NURBS_ERROR3
#define GLU_NURBS_ERROR3 100253
#endif
#ifndef GLU_NURBS_ERROR30
#define GLU_NURBS_ERROR30 100280
#endif
#ifndef GLU_NURBS_ERROR31
#define GLU_NURBS_ERROR31 100281
#endif
#ifndef GLU_NURBS_ERROR32
#define GLU_NURBS_ERROR32 100282
#endif
#ifndef GLU_NURBS_ERROR33
#define GLU_NURBS_ERROR33 100283
#endif
#ifndef GLU_NURBS_ERROR34
#define GLU_NURBS_ERROR34 100284
#endif
#ifndef GLU_NURBS_ERROR35
#define GLU_NURBS_ERROR35 100285
#endif
#ifndef GLU_NURBS_ERROR36
#define GLU_NURBS_ERROR36 100286
#endif
#ifndef GLU_NURBS_ERROR37
#define GLU_NURBS_ERROR37 100287
#endif
#ifndef GLU_NURBS_ERROR4
#define GLU_NURBS_ERROR4 100254
#endif
#ifndef GLU_NURBS_ERROR5
#define GLU_NURBS_ERROR5 100255
#endif
#ifndef GLU_NURBS_ERROR6
#define GLU_NURBS_ERROR6 100256
#endif
#ifndef GLU_NURBS_ERROR7
#define GLU_NURBS_ERROR7 100257
#endif
#ifndef GLU_NURBS_ERROR8
#define GLU_NURBS_ERROR8 100258
#endif
#ifndef GLU_NURBS_ERROR9
#define GLU_NURBS_ERROR9 100259
#endif
#ifndef GLU_NURBS_MODE
#define GLU_NURBS_MODE 100160
#endif
#ifndef GLU_NURBS_MODE_EXT
#define GLU_NURBS_MODE_EXT 100160
#endif
#ifndef GLU_NURBS_NORMAL
#define GLU_NURBS_NORMAL 100166
#endif
#ifndef GLU_NURBS_NORMAL_DATA
#define GLU_NURBS_NORMAL_DATA 100172
#endif
#ifndef GLU_NURBS_NORMAL_DATA_EXT
#define GLU_NURBS_NORMAL_DATA_EXT 100172
#endif
#ifndef GLU_NURBS_NORMAL_EXT
#define GLU_NURBS_NORMAL_EXT 100166
#endif
#ifndef GLU_NURBS_RENDERER
#define GLU_NURBS_RENDERER 100162
#endif
#ifndef GLU_NURBS_RENDERER_EXT
#define GLU_NURBS_RENDERER_EXT 100162
#endif
#ifndef GLU_NURBS_TESSELLATOR
#define GLU_NURBS_TESSELLATOR 100161
#endif
#ifndef GLU_NURBS_TESSELLATOR_EXT
#define GLU_NURBS_TESSELLATOR_EXT 100161
#endif
#ifndef GLU_NURBS_TEXTURE_COORD
#define GLU_NURBS_TEXTURE_COORD 100168
#endif
#ifndef GLU_NURBS_TEXTURE_COORD_DATA
#define GLU_NURBS_TEXTURE_COORD_DATA 100174
#endif
#ifndef GLU_NURBS_TEXTURE_COORD_DATA_EXT
#define GLU_NURBS_TEXTURE_COORD_DATA_EXT 100174
#endif
#ifndef GLU_NURBS_TEXTURE_COORD_EXT
#define GLU_NURBS_TEXTURE_COORD_EXT 100168
#endif
#ifndef GLU_NURBS_VERTEX
#define GLU_NURBS_VERTEX 100165
#endif
#ifndef GLU_NURBS_VERTEX_DATA
#define GLU_NURBS_VERTEX_DATA 100171
#endif
#ifndef GLU_NURBS_VERTEX_DATA_EXT
#define GLU_NURBS_VERTEX_DATA_EXT 100171
#endif
#ifndef GLU_NURBS_VERTEX_EXT
#define GLU_NURBS_VERTEX_EXT 100165
#endif
#ifndef GLU_OBJECT_PARAMETRIC_ERROR
#define GLU_OBJECT_PARAMETRIC_ERROR 100208
#endif
#ifndef GLU_OBJECT_PARAMETRIC_ERROR_EXT
#define GLU_OBJECT_PARAMETRIC_ERROR_EXT 100208
#endif
#ifndef GLU_OBJECT_PATH_LENGTH
#define GLU_OBJECT_PATH_LENGTH 100209
#endif
#ifndef GLU_OBJECT_PATH_LENGTH_EXT
#define GLU_OBJECT_PATH_LENGTH_EXT 100209
#endif
#ifndef GLU_OUTLINE_PATCH
#define GLU_OUTLINE_PATCH 100241
#endif
#ifndef GLU_OUTLINE_POLYGON
#define GLU_OUTLINE_POLYGON 100240
#endif
#ifndef GLU_OUTSIDE
#define GLU_OUTSIDE 100020
#endif
#ifndef GLU_OUT_OF_MEMORY
#define GLU_OUT_OF_MEMORY 100902
#endif
#ifndef GLU_PARAMETRIC_ERROR
#define GLU_PARAMETRIC_ERROR 100216
#endif
#ifndef GLU_PARAMETRIC_TOLERANCE
#define GLU_PARAMETRIC_TOLERANCE 100202
#endif
#ifndef GLU_PATH_LENGTH
#define GLU_PATH_LENGTH 100215
#endif
#ifndef GLU_POINT
#define GLU_POINT 100010
#endif
#ifndef GLU_SAMPLING_METHOD
#define GLU_SAMPLING_METHOD 100205
#endif
#ifndef GLU_SAMPLING_TOLERANCE
#define GLU_SAMPLING_TOLERANCE 100203
#endif
#ifndef GLU_SILHOUETTE
#define GLU_SILHOUETTE 100013
#endif
#ifndef GLU_SMOOTH
#define GLU_SMOOTH 100000
#endif
#ifndef GLU_TESS_BEGIN
#define GLU_TESS_BEGIN 100100
#endif
#ifndef GLU_TESS_BEGIN_DATA
#define GLU_TESS_BEGIN_DATA 100106
#endif
#ifndef GLU_TESS_BOUNDARY_ONLY
#define GLU_TESS_BOUNDARY_ONLY 100141
#endif
#ifndef GLU_TESS_COMBINE
#define GLU_TESS_COMBINE 100105
#endif
#ifndef GLU_TESS_COMBINE_DATA
#define GLU_TESS_COMBINE_DATA 100111
#endif
#ifndef GLU_TESS_COORD_TOO_LARGE
#define GLU_TESS_COORD_TOO_LARGE 100155
#endif
#ifndef GLU_TESS_EDGE_FLAG
#define GLU_TESS_EDGE_FLAG 100104
#endif
#ifndef GLU_TESS_EDGE_FLAG_DATA
#define GLU_TESS_EDGE_FLAG_DATA 100110
#endif
#ifndef GLU_TESS_END
#define GLU_TESS_END 100102
#endif
#ifndef GLU_TESS_END_DATA
#define GLU_TESS_END_DATA 100108
#endif
#ifndef GLU_TESS_ERROR
#define GLU_TESS_ERROR 100103
#endif
#ifndef GLU_TESS_ERROR1
#define GLU_TESS_ERROR1 100151
#endif
#ifndef GLU_TESS_ERROR2
#define GLU_TESS_ERROR2 100152
#endif
#ifndef GLU_TESS_ERROR3
#define GLU_TESS_ERROR3 100153
#endif
#ifndef GLU_TESS_ERROR4
#define GLU_TESS_ERROR4 100154
#endif
#ifndef GLU_TESS_ERROR5
#define GLU_TESS_ERROR5 100155
#endif
#ifndef GLU_TESS_ERROR6
#define GLU_TESS_ERROR6 100156
#endif
#ifndef GLU_TESS_ERROR7
#define GLU_TESS_ERROR7 100157
#endif
#ifndef GLU_TESS_ERROR8
#define GLU_TESS_ERROR8 100158
#endif
#ifndef GLU_TESS_ERROR_DATA
#define GLU_TESS_ERROR_DATA 100109
#endif
#ifndef GLU_TESS_MISSING_BEGIN_CONTOUR
#define GLU_TESS_MISSING_BEGIN_CONTOUR 100152
#endif
#ifndef GLU_TESS_MISSING_BEGIN_POLYGON
#define GLU_TESS_MISSING_BEGIN_POLYGON 100151
#endif
#ifndef GLU_TESS_MISSING_END_CONTOUR
#define GLU_TESS_MISSING_END_CONTOUR 100154
#endif
#ifndef GLU_TESS_MISSING_END_POLYGON
#define GLU_TESS_MISSING_END_POLYGON 100153
#endif
#ifndef GLU_TESS_NEED_COMBINE_CALLBACK
#define GLU_TESS_NEED_COMBINE_CALLBACK 100156
#endif
#ifndef GLU_TESS_TOLERANCE
#define GLU_TESS_TOLERANCE 100142
#endif
#ifndef GLU_TESS_VERTEX
#define GLU_TESS_VERTEX 100101
#endif
#ifndef GLU_TESS_VERTEX_DATA
#define GLU_TESS_VERTEX_DATA 100107
#endif
#ifndef GLU_TESS_WINDING_ABS_GEQ_TWO
#define GLU_TESS_WINDING_ABS_GEQ_TWO 100134
#endif
#ifndef GLU_TESS_WINDING_NEGATIVE
#define GLU_TESS_WINDING_NEGATIVE 100133
#endif
#ifndef GLU_TESS_WINDING_NONZERO
#define GLU_TESS_WINDING_NONZERO 100131
#endif
#ifndef GLU_TESS_WINDING_ODD
#define GLU_TESS_WINDING_ODD 100130
#endif
#ifndef GLU_TESS_WINDING_POSITIVE
#define GLU_TESS_WINDING_POSITIVE 100132
#endif
#ifndef GLU_TESS_WINDING_RULE
#define GLU_TESS_WINDING_RULE 100140
#endif
#ifndef GLU_UNKNOWN
#define GLU_UNKNOWN 100124
#endif
#ifndef GLU_U_STEP
#define GLU_U_STEP 100206
#endif
#ifndef GLU_VERSION
#define GLU_VERSION 100800
#endif
#ifndef GLU_VERSION_1_1
#define GLU_VERSION_1_1 1
#endif
#ifndef GLU_VERSION_1_2
#define GLU_VERSION_1_2 1
#endif
#ifndef GLU_VERSION_1_3
#define GLU_VERSION_1_3 1
#endif
#ifndef GLU_VERTEX
#define GLU_VERTEX 100101
#endif
#ifndef GLU_V_STEP
#define GLU_V_STEP 100207
#endif
