/* This file was genereated on Mon Feb 16 18:08:56 +0100 2009
   source: http://www.opengl.org/registry/api/enum.spec
   source: http://www.opengl.org/registry/api/enumext.spec
*/

#include "../common/common.h"
void gl_init_enums(VALUE module)
{
	rb_define_const(module, "GL_TRUE", Qtrue);
	rb_define_const(module, "GL_FALSE", Qfalse);

	rb_define_const(module, "GL_1PASS_EXT", INT2NUM(GL_1PASS_EXT));
	rb_define_const(module, "GL_1PASS_SGIS", INT2NUM(GL_1PASS_SGIS));
	rb_define_const(module, "GL_2D", INT2NUM(GL_2D));
	rb_define_const(module, "GL_2PASS_0_EXT", INT2NUM(GL_2PASS_0_EXT));
	rb_define_const(module, "GL_2PASS_0_SGIS", INT2NUM(GL_2PASS_0_SGIS));
	rb_define_const(module, "GL_2PASS_1_EXT", INT2NUM(GL_2PASS_1_EXT));
	rb_define_const(module, "GL_2PASS_1_SGIS", INT2NUM(GL_2PASS_1_SGIS));
	rb_define_const(module, "GL_2X_BIT_ATI", INT2NUM(GL_2X_BIT_ATI));
	rb_define_const(module, "GL_2_BYTES", INT2NUM(GL_2_BYTES));
	rb_define_const(module, "GL_3D", INT2NUM(GL_3D));
	rb_define_const(module, "GL_3D_COLOR", INT2NUM(GL_3D_COLOR));
	rb_define_const(module, "GL_3D_COLOR_TEXTURE", INT2NUM(GL_3D_COLOR_TEXTURE));
	rb_define_const(module, "GL_3_BYTES", INT2NUM(GL_3_BYTES));
	rb_define_const(module, "GL_422_AVERAGE_EXT", INT2NUM(GL_422_AVERAGE_EXT));
	rb_define_const(module, "GL_422_EXT", INT2NUM(GL_422_EXT));
	rb_define_const(module, "GL_422_REV_AVERAGE_EXT", INT2NUM(GL_422_REV_AVERAGE_EXT));
	rb_define_const(module, "GL_422_REV_EXT", INT2NUM(GL_422_REV_EXT));
	rb_define_const(module, "GL_4D_COLOR_TEXTURE", INT2NUM(GL_4D_COLOR_TEXTURE));
	rb_define_const(module, "GL_4PASS_0_EXT", INT2NUM(GL_4PASS_0_EXT));
	rb_define_const(module, "GL_4PASS_0_SGIS", INT2NUM(GL_4PASS_0_SGIS));
	rb_define_const(module, "GL_4PASS_1_EXT", INT2NUM(GL_4PASS_1_EXT));
	rb_define_const(module, "GL_4PASS_1_SGIS", INT2NUM(GL_4PASS_1_SGIS));
	rb_define_const(module, "GL_4PASS_2_EXT", INT2NUM(GL_4PASS_2_EXT));
	rb_define_const(module, "GL_4PASS_2_SGIS", INT2NUM(GL_4PASS_2_SGIS));
	rb_define_const(module, "GL_4PASS_3_EXT", INT2NUM(GL_4PASS_3_EXT));
	rb_define_const(module, "GL_4PASS_3_SGIS", INT2NUM(GL_4PASS_3_SGIS));
	rb_define_const(module, "GL_4X_BIT_ATI", INT2NUM(GL_4X_BIT_ATI));
	rb_define_const(module, "GL_4_BYTES", INT2NUM(GL_4_BYTES));
	rb_define_const(module, "GL_8X_BIT_ATI", INT2NUM(GL_8X_BIT_ATI));
	rb_define_const(module, "GL_ABGR_EXT", INT2NUM(GL_ABGR_EXT));
	rb_define_const(module, "GL_ACCUM", INT2NUM(GL_ACCUM));
	rb_define_const(module, "GL_ACCUM_ALPHA_BITS", INT2NUM(GL_ACCUM_ALPHA_BITS));
	rb_define_const(module, "GL_ACCUM_BLUE_BITS", INT2NUM(GL_ACCUM_BLUE_BITS));
	rb_define_const(module, "GL_ACCUM_BUFFER_BIT", INT2NUM(GL_ACCUM_BUFFER_BIT));
	rb_define_const(module, "GL_ACCUM_CLEAR_VALUE", INT2NUM(GL_ACCUM_CLEAR_VALUE));
	rb_define_const(module, "GL_ACCUM_GREEN_BITS", INT2NUM(GL_ACCUM_GREEN_BITS));
	rb_define_const(module, "GL_ACCUM_RED_BITS", INT2NUM(GL_ACCUM_RED_BITS));
	rb_define_const(module, "GL_ACTIVE_ATTRIBUTES", INT2NUM(GL_ACTIVE_ATTRIBUTES));
	rb_define_const(module, "GL_ACTIVE_ATTRIBUTE_MAX_LENGTH", INT2NUM(GL_ACTIVE_ATTRIBUTE_MAX_LENGTH));
	rb_define_const(module, "GL_ACTIVE_STENCIL_FACE_EXT", INT2NUM(GL_ACTIVE_STENCIL_FACE_EXT));
	rb_define_const(module, "GL_ACTIVE_TEXTURE", INT2NUM(GL_ACTIVE_TEXTURE));
	rb_define_const(module, "GL_ACTIVE_TEXTURE_ARB", INT2NUM(GL_ACTIVE_TEXTURE_ARB));
	rb_define_const(module, "GL_ACTIVE_UNIFORMS", INT2NUM(GL_ACTIVE_UNIFORMS));
	rb_define_const(module, "GL_ACTIVE_UNIFORM_MAX_LENGTH", INT2NUM(GL_ACTIVE_UNIFORM_MAX_LENGTH));
	rb_define_const(module, "GL_ACTIVE_VARYINGS_NV", INT2NUM(GL_ACTIVE_VARYINGS_NV));
	rb_define_const(module, "GL_ACTIVE_VARYING_MAX_LENGTH_NV", INT2NUM(GL_ACTIVE_VARYING_MAX_LENGTH_NV));
	rb_define_const(module, "GL_ACTIVE_VERTEX_UNITS_ARB", INT2NUM(GL_ACTIVE_VERTEX_UNITS_ARB));
	rb_define_const(module, "GL_ADD", INT2NUM(GL_ADD));
	rb_define_const(module, "GL_ADD_ATI", INT2NUM(GL_ADD_ATI));
	rb_define_const(module, "GL_ADD_SIGNED", INT2NUM(GL_ADD_SIGNED));
	rb_define_const(module, "GL_ADD_SIGNED_ARB", INT2NUM(GL_ADD_SIGNED_ARB));
	rb_define_const(module, "GL_ADD_SIGNED_EXT", INT2NUM(GL_ADD_SIGNED_EXT));
	rb_define_const(module, "GL_ALIASED_LINE_WIDTH_RANGE", INT2NUM(GL_ALIASED_LINE_WIDTH_RANGE));
	rb_define_const(module, "GL_ALIASED_POINT_SIZE_RANGE", INT2NUM(GL_ALIASED_POINT_SIZE_RANGE));
	rb_define_const(module, "GL_ALLOW_DRAW_FRG_HINT_PGI", INT2NUM(GL_ALLOW_DRAW_FRG_HINT_PGI));
	rb_define_const(module, "GL_ALLOW_DRAW_MEM_HINT_PGI", INT2NUM(GL_ALLOW_DRAW_MEM_HINT_PGI));
	rb_define_const(module, "GL_ALLOW_DRAW_OBJ_HINT_PGI", INT2NUM(GL_ALLOW_DRAW_OBJ_HINT_PGI));
	rb_define_const(module, "GL_ALLOW_DRAW_WIN_HINT_PGI", INT2NUM(GL_ALLOW_DRAW_WIN_HINT_PGI));
	rb_define_const(module, "GL_ALL_ATTRIB_BITS", INT2NUM(GL_ALL_ATTRIB_BITS));
	rb_define_const(module, "GL_ALL_COMPLETED_NV", INT2NUM(GL_ALL_COMPLETED_NV));
	rb_define_const(module, "GL_ALPHA", INT2NUM(GL_ALPHA));
	rb_define_const(module, "GL_ALPHA12", INT2NUM(GL_ALPHA12));
	rb_define_const(module, "GL_ALPHA12_EXT", INT2NUM(GL_ALPHA12_EXT));
	rb_define_const(module, "GL_ALPHA16", INT2NUM(GL_ALPHA16));
	rb_define_const(module, "GL_ALPHA16F_ARB", INT2NUM(GL_ALPHA16F_ARB));
	rb_define_const(module, "GL_ALPHA16I_EXT", INT2NUM(GL_ALPHA16I_EXT));
	rb_define_const(module, "GL_ALPHA16UI_EXT", INT2NUM(GL_ALPHA16UI_EXT));
	rb_define_const(module, "GL_ALPHA16_EXT", INT2NUM(GL_ALPHA16_EXT));
	rb_define_const(module, "GL_ALPHA32F_ARB", INT2NUM(GL_ALPHA32F_ARB));
	rb_define_const(module, "GL_ALPHA32I_EXT", INT2NUM(GL_ALPHA32I_EXT));
	rb_define_const(module, "GL_ALPHA32UI_EXT", INT2NUM(GL_ALPHA32UI_EXT));
	rb_define_const(module, "GL_ALPHA4", INT2NUM(GL_ALPHA4));
	rb_define_const(module, "GL_ALPHA4_EXT", INT2NUM(GL_ALPHA4_EXT));
	rb_define_const(module, "GL_ALPHA8", INT2NUM(GL_ALPHA8));
	rb_define_const(module, "GL_ALPHA8I_EXT", INT2NUM(GL_ALPHA8I_EXT));
	rb_define_const(module, "GL_ALPHA8UI_EXT", INT2NUM(GL_ALPHA8UI_EXT));
	rb_define_const(module, "GL_ALPHA8_EXT", INT2NUM(GL_ALPHA8_EXT));
	rb_define_const(module, "GL_ALPHA_BIAS", INT2NUM(GL_ALPHA_BIAS));
	rb_define_const(module, "GL_ALPHA_BITS", INT2NUM(GL_ALPHA_BITS));
	rb_define_const(module, "GL_ALPHA_FLOAT16_ATI", INT2NUM(GL_ALPHA_FLOAT16_ATI));
	rb_define_const(module, "GL_ALPHA_FLOAT32_ATI", INT2NUM(GL_ALPHA_FLOAT32_ATI));
	rb_define_const(module, "GL_ALPHA_INTEGER", INT2NUM(GL_ALPHA_INTEGER));
	rb_define_const(module, "GL_ALPHA_INTEGER_EXT", INT2NUM(GL_ALPHA_INTEGER_EXT));
	rb_define_const(module, "GL_ALPHA_MAX_CLAMP_INGR", INT2NUM(GL_ALPHA_MAX_CLAMP_INGR));
	rb_define_const(module, "GL_ALPHA_MAX_SGIX", INT2NUM(GL_ALPHA_MAX_SGIX));
	rb_define_const(module, "GL_ALPHA_MIN_CLAMP_INGR", INT2NUM(GL_ALPHA_MIN_CLAMP_INGR));
	rb_define_const(module, "GL_ALPHA_MIN_SGIX", INT2NUM(GL_ALPHA_MIN_SGIX));
	rb_define_const(module, "GL_ALPHA_SCALE", INT2NUM(GL_ALPHA_SCALE));
	rb_define_const(module, "GL_ALPHA_TEST", INT2NUM(GL_ALPHA_TEST));
	rb_define_const(module, "GL_ALPHA_TEST_FUNC", INT2NUM(GL_ALPHA_TEST_FUNC));
	rb_define_const(module, "GL_ALPHA_TEST_REF", INT2NUM(GL_ALPHA_TEST_REF));
	rb_define_const(module, "GL_ALWAYS", INT2NUM(GL_ALWAYS));
	rb_define_const(module, "GL_ALWAYS_FAST_HINT_PGI", INT2NUM(GL_ALWAYS_FAST_HINT_PGI));
	rb_define_const(module, "GL_ALWAYS_SOFT_HINT_PGI", INT2NUM(GL_ALWAYS_SOFT_HINT_PGI));
	rb_define_const(module, "GL_AMBIENT", INT2NUM(GL_AMBIENT));
	rb_define_const(module, "GL_AMBIENT_AND_DIFFUSE", INT2NUM(GL_AMBIENT_AND_DIFFUSE));
	rb_define_const(module, "GL_AND", INT2NUM(GL_AND));
	rb_define_const(module, "GL_AND_INVERTED", INT2NUM(GL_AND_INVERTED));
	rb_define_const(module, "GL_AND_REVERSE", INT2NUM(GL_AND_REVERSE));
	rb_define_const(module, "GL_ARB_imaging", INT2NUM(GL_ARB_imaging));
	rb_define_const(module, "GL_ARRAY_BUFFER", INT2NUM(GL_ARRAY_BUFFER));
	rb_define_const(module, "GL_ARRAY_BUFFER_ARB", INT2NUM(GL_ARRAY_BUFFER_ARB));
	rb_define_const(module, "GL_ARRAY_BUFFER_BINDING", INT2NUM(GL_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_ARRAY_ELEMENT_LOCK_COUNT_EXT", INT2NUM(GL_ARRAY_ELEMENT_LOCK_COUNT_EXT));
	rb_define_const(module, "GL_ARRAY_ELEMENT_LOCK_FIRST_EXT", INT2NUM(GL_ARRAY_ELEMENT_LOCK_FIRST_EXT));
	rb_define_const(module, "GL_ARRAY_OBJECT_BUFFER_ATI", INT2NUM(GL_ARRAY_OBJECT_BUFFER_ATI));
	rb_define_const(module, "GL_ARRAY_OBJECT_OFFSET_ATI", INT2NUM(GL_ARRAY_OBJECT_OFFSET_ATI));
	rb_define_const(module, "GL_ASYNC_DRAW_PIXELS_SGIX", INT2NUM(GL_ASYNC_DRAW_PIXELS_SGIX));
	rb_define_const(module, "GL_ASYNC_HISTOGRAM_SGIX", INT2NUM(GL_ASYNC_HISTOGRAM_SGIX));
	rb_define_const(module, "GL_ASYNC_MARKER_SGIX", INT2NUM(GL_ASYNC_MARKER_SGIX));
	rb_define_const(module, "GL_ASYNC_READ_PIXELS_SGIX", INT2NUM(GL_ASYNC_READ_PIXELS_SGIX));
	rb_define_const(module, "GL_ASYNC_TEX_IMAGE_SGIX", INT2NUM(GL_ASYNC_TEX_IMAGE_SGIX));
	rb_define_const(module, "GL_ATTACHED_SHADERS", INT2NUM(GL_ATTACHED_SHADERS));
	rb_define_const(module, "GL_ATTENUATION_EXT", INT2NUM(GL_ATTENUATION_EXT));
	rb_define_const(module, "GL_ATTRIB_ARRAY_POINTER_NV", INT2NUM(GL_ATTRIB_ARRAY_POINTER_NV));
	rb_define_const(module, "GL_ATTRIB_ARRAY_SIZE_NV", INT2NUM(GL_ATTRIB_ARRAY_SIZE_NV));
	rb_define_const(module, "GL_ATTRIB_ARRAY_STRIDE_NV", INT2NUM(GL_ATTRIB_ARRAY_STRIDE_NV));
	rb_define_const(module, "GL_ATTRIB_ARRAY_TYPE_NV", INT2NUM(GL_ATTRIB_ARRAY_TYPE_NV));
	rb_define_const(module, "GL_ATTRIB_STACK_DEPTH", INT2NUM(GL_ATTRIB_STACK_DEPTH));
	rb_define_const(module, "GL_AUTO_NORMAL", INT2NUM(GL_AUTO_NORMAL));
	rb_define_const(module, "GL_AUX0", INT2NUM(GL_AUX0));
	rb_define_const(module, "GL_AUX1", INT2NUM(GL_AUX1));
	rb_define_const(module, "GL_AUX2", INT2NUM(GL_AUX2));
	rb_define_const(module, "GL_AUX3", INT2NUM(GL_AUX3));
	rb_define_const(module, "GL_AUX_BUFFERS", INT2NUM(GL_AUX_BUFFERS));
	rb_define_const(module, "GL_AVERAGE_EXT", INT2NUM(GL_AVERAGE_EXT));
	rb_define_const(module, "GL_AVERAGE_HP", INT2NUM(GL_AVERAGE_HP));
	rb_define_const(module, "GL_BACK", INT2NUM(GL_BACK));
	rb_define_const(module, "GL_BACK_LEFT", INT2NUM(GL_BACK_LEFT));
	rb_define_const(module, "GL_BACK_NORMALS_HINT_PGI", INT2NUM(GL_BACK_NORMALS_HINT_PGI));
	rb_define_const(module, "GL_BACK_PRIMARY_COLOR_NV", INT2NUM(GL_BACK_PRIMARY_COLOR_NV));
	rb_define_const(module, "GL_BACK_RIGHT", INT2NUM(GL_BACK_RIGHT));
	rb_define_const(module, "GL_BACK_SECONDARY_COLOR_NV", INT2NUM(GL_BACK_SECONDARY_COLOR_NV));
	rb_define_const(module, "GL_BGR", INT2NUM(GL_BGR));
	rb_define_const(module, "GL_BGRA", INT2NUM(GL_BGRA));
	rb_define_const(module, "GL_BGRA_EXT", INT2NUM(GL_BGRA_EXT));
	rb_define_const(module, "GL_BGRA_INTEGER", INT2NUM(GL_BGRA_INTEGER));
	rb_define_const(module, "GL_BGRA_INTEGER_EXT", INT2NUM(GL_BGRA_INTEGER_EXT));
	rb_define_const(module, "GL_BGR_EXT", INT2NUM(GL_BGR_EXT));
	rb_define_const(module, "GL_BGR_INTEGER", INT2NUM(GL_BGR_INTEGER));
	rb_define_const(module, "GL_BGR_INTEGER_EXT", INT2NUM(GL_BGR_INTEGER_EXT));
	rb_define_const(module, "GL_BIAS_BIT_ATI", INT2NUM(GL_BIAS_BIT_ATI));
	rb_define_const(module, "GL_BIAS_BY_NEGATIVE_ONE_HALF_NV", INT2NUM(GL_BIAS_BY_NEGATIVE_ONE_HALF_NV));
	rb_define_const(module, "GL_BINORMAL_ARRAY_EXT", INT2NUM(GL_BINORMAL_ARRAY_EXT));
	rb_define_const(module, "GL_BINORMAL_ARRAY_POINTER_EXT", INT2NUM(GL_BINORMAL_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_BINORMAL_ARRAY_STRIDE_EXT", INT2NUM(GL_BINORMAL_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_BINORMAL_ARRAY_TYPE_EXT", INT2NUM(GL_BINORMAL_ARRAY_TYPE_EXT));
	rb_define_const(module, "GL_BITMAP", INT2NUM(GL_BITMAP));
	rb_define_const(module, "GL_BITMAP_TOKEN", INT2NUM(GL_BITMAP_TOKEN));
	rb_define_const(module, "GL_BLEND", INT2NUM(GL_BLEND));
	rb_define_const(module, "GL_BLEND_COLOR", INT2NUM(GL_BLEND_COLOR));
	rb_define_const(module, "GL_BLEND_COLOR_EXT", INT2NUM(GL_BLEND_COLOR_EXT));
	rb_define_const(module, "GL_BLEND_DST", INT2NUM(GL_BLEND_DST));
	rb_define_const(module, "GL_BLEND_DST_ALPHA", INT2NUM(GL_BLEND_DST_ALPHA));
	rb_define_const(module, "GL_BLEND_DST_ALPHA_EXT", INT2NUM(GL_BLEND_DST_ALPHA_EXT));
	rb_define_const(module, "GL_BLEND_DST_RGB", INT2NUM(GL_BLEND_DST_RGB));
	rb_define_const(module, "GL_BLEND_DST_RGB_EXT", INT2NUM(GL_BLEND_DST_RGB_EXT));
	rb_define_const(module, "GL_BLEND_EQUATION", INT2NUM(GL_BLEND_EQUATION));
	rb_define_const(module, "GL_BLEND_EQUATION_ALPHA", INT2NUM(GL_BLEND_EQUATION_ALPHA));
	rb_define_const(module, "GL_BLEND_EQUATION_ALPHA_EXT", INT2NUM(GL_BLEND_EQUATION_ALPHA_EXT));
	rb_define_const(module, "GL_BLEND_EQUATION_EXT", INT2NUM(GL_BLEND_EQUATION_EXT));
	rb_define_const(module, "GL_BLEND_EQUATION_RGB", INT2NUM(GL_BLEND_EQUATION_RGB));
	rb_define_const(module, "GL_BLEND_EQUATION_RGB_EXT", INT2NUM(GL_BLEND_EQUATION_RGB_EXT));
	rb_define_const(module, "GL_BLEND_SRC", INT2NUM(GL_BLEND_SRC));
	rb_define_const(module, "GL_BLEND_SRC_ALPHA", INT2NUM(GL_BLEND_SRC_ALPHA));
	rb_define_const(module, "GL_BLEND_SRC_ALPHA_EXT", INT2NUM(GL_BLEND_SRC_ALPHA_EXT));
	rb_define_const(module, "GL_BLEND_SRC_RGB", INT2NUM(GL_BLEND_SRC_RGB));
	rb_define_const(module, "GL_BLEND_SRC_RGB_EXT", INT2NUM(GL_BLEND_SRC_RGB_EXT));
	rb_define_const(module, "GL_BLUE", INT2NUM(GL_BLUE));
	rb_define_const(module, "GL_BLUE_BIAS", INT2NUM(GL_BLUE_BIAS));
	rb_define_const(module, "GL_BLUE_BITS", INT2NUM(GL_BLUE_BITS));
	rb_define_const(module, "GL_BLUE_BIT_ATI", INT2NUM(GL_BLUE_BIT_ATI));
	rb_define_const(module, "GL_BLUE_INTEGER", INT2NUM(GL_BLUE_INTEGER));
	rb_define_const(module, "GL_BLUE_INTEGER_EXT", INT2NUM(GL_BLUE_INTEGER_EXT));
	rb_define_const(module, "GL_BLUE_MAX_CLAMP_INGR", INT2NUM(GL_BLUE_MAX_CLAMP_INGR));
	rb_define_const(module, "GL_BLUE_MIN_CLAMP_INGR", INT2NUM(GL_BLUE_MIN_CLAMP_INGR));
	rb_define_const(module, "GL_BLUE_SCALE", INT2NUM(GL_BLUE_SCALE));
	rb_define_const(module, "GL_BOOL", INT2NUM(GL_BOOL));
	rb_define_const(module, "GL_BOOL_ARB", INT2NUM(GL_BOOL_ARB));
	rb_define_const(module, "GL_BOOL_VEC2", INT2NUM(GL_BOOL_VEC2));
	rb_define_const(module, "GL_BOOL_VEC2_ARB", INT2NUM(GL_BOOL_VEC2_ARB));
	rb_define_const(module, "GL_BOOL_VEC3", INT2NUM(GL_BOOL_VEC3));
	rb_define_const(module, "GL_BOOL_VEC3_ARB", INT2NUM(GL_BOOL_VEC3_ARB));
	rb_define_const(module, "GL_BOOL_VEC4", INT2NUM(GL_BOOL_VEC4));
	rb_define_const(module, "GL_BOOL_VEC4_ARB", INT2NUM(GL_BOOL_VEC4_ARB));
	rb_define_const(module, "GL_BUFFER_ACCESS", INT2NUM(GL_BUFFER_ACCESS));
	rb_define_const(module, "GL_BUFFER_ACCESS_ARB", INT2NUM(GL_BUFFER_ACCESS_ARB));
	rb_define_const(module, "GL_BUFFER_FLUSHING_UNMAP_APPLE", INT2NUM(GL_BUFFER_FLUSHING_UNMAP_APPLE));
	rb_define_const(module, "GL_BUFFER_MAPPED", INT2NUM(GL_BUFFER_MAPPED));
	rb_define_const(module, "GL_BUFFER_MAPPED_ARB", INT2NUM(GL_BUFFER_MAPPED_ARB));
	rb_define_const(module, "GL_BUFFER_MAP_POINTER", INT2NUM(GL_BUFFER_MAP_POINTER));
	rb_define_const(module, "GL_BUFFER_MAP_POINTER_ARB", INT2NUM(GL_BUFFER_MAP_POINTER_ARB));
	rb_define_const(module, "GL_BUFFER_SERIALIZED_MODIFY_APPLE", INT2NUM(GL_BUFFER_SERIALIZED_MODIFY_APPLE));
	rb_define_const(module, "GL_BUFFER_SIZE", INT2NUM(GL_BUFFER_SIZE));
	rb_define_const(module, "GL_BUFFER_SIZE_ARB", INT2NUM(GL_BUFFER_SIZE_ARB));
	rb_define_const(module, "GL_BUFFER_USAGE", INT2NUM(GL_BUFFER_USAGE));
	rb_define_const(module, "GL_BUFFER_USAGE_ARB", INT2NUM(GL_BUFFER_USAGE_ARB));
	rb_define_const(module, "GL_BUMP_ENVMAP_ATI", INT2NUM(GL_BUMP_ENVMAP_ATI));
	rb_define_const(module, "GL_BUMP_NUM_TEX_UNITS_ATI", INT2NUM(GL_BUMP_NUM_TEX_UNITS_ATI));
	rb_define_const(module, "GL_BUMP_ROT_MATRIX_ATI", INT2NUM(GL_BUMP_ROT_MATRIX_ATI));
	rb_define_const(module, "GL_BUMP_ROT_MATRIX_SIZE_ATI", INT2NUM(GL_BUMP_ROT_MATRIX_SIZE_ATI));
	rb_define_const(module, "GL_BUMP_TARGET_ATI", INT2NUM(GL_BUMP_TARGET_ATI));
	rb_define_const(module, "GL_BUMP_TEX_UNITS_ATI", INT2NUM(GL_BUMP_TEX_UNITS_ATI));
	rb_define_const(module, "GL_BYTE", INT2NUM(GL_BYTE));
	rb_define_const(module, "GL_C3F_V3F", INT2NUM(GL_C3F_V3F));
	rb_define_const(module, "GL_C4F_N3F_V3F", INT2NUM(GL_C4F_N3F_V3F));
	rb_define_const(module, "GL_C4UB_V2F", INT2NUM(GL_C4UB_V2F));
	rb_define_const(module, "GL_C4UB_V3F", INT2NUM(GL_C4UB_V3F));
	rb_define_const(module, "GL_CALLIGRAPHIC_FRAGMENT_SGIX", INT2NUM(GL_CALLIGRAPHIC_FRAGMENT_SGIX));
	rb_define_const(module, "GL_CCW", INT2NUM(GL_CCW));
	rb_define_const(module, "GL_CLAMP", INT2NUM(GL_CLAMP));
	rb_define_const(module, "GL_CLAMP_FRAGMENT_COLOR", INT2NUM(GL_CLAMP_FRAGMENT_COLOR));
	rb_define_const(module, "GL_CLAMP_FRAGMENT_COLOR_ARB", INT2NUM(GL_CLAMP_FRAGMENT_COLOR_ARB));
	rb_define_const(module, "GL_CLAMP_READ_COLOR", INT2NUM(GL_CLAMP_READ_COLOR));
	rb_define_const(module, "GL_CLAMP_READ_COLOR_ARB", INT2NUM(GL_CLAMP_READ_COLOR_ARB));
	rb_define_const(module, "GL_CLAMP_TO_BORDER", INT2NUM(GL_CLAMP_TO_BORDER));
	rb_define_const(module, "GL_CLAMP_TO_BORDER_ARB", INT2NUM(GL_CLAMP_TO_BORDER_ARB));
	rb_define_const(module, "GL_CLAMP_TO_BORDER_SGIS", INT2NUM(GL_CLAMP_TO_BORDER_SGIS));
	rb_define_const(module, "GL_CLAMP_TO_EDGE", INT2NUM(GL_CLAMP_TO_EDGE));
	rb_define_const(module, "GL_CLAMP_TO_EDGE_SGIS", INT2NUM(GL_CLAMP_TO_EDGE_SGIS));
	rb_define_const(module, "GL_CLAMP_VERTEX_COLOR", INT2NUM(GL_CLAMP_VERTEX_COLOR));
	rb_define_const(module, "GL_CLAMP_VERTEX_COLOR_ARB", INT2NUM(GL_CLAMP_VERTEX_COLOR_ARB));
	rb_define_const(module, "GL_CLEAR", INT2NUM(GL_CLEAR));
	rb_define_const(module, "GL_CLIENT_ACTIVE_TEXTURE", INT2NUM(GL_CLIENT_ACTIVE_TEXTURE));
	rb_define_const(module, "GL_CLIENT_ACTIVE_TEXTURE_ARB", INT2NUM(GL_CLIENT_ACTIVE_TEXTURE_ARB));
	rb_define_const(module, "GL_CLIENT_ALL_ATTRIB_BITS", INT2NUM(GL_CLIENT_ALL_ATTRIB_BITS));
	rb_define_const(module, "GL_CLIENT_ATTRIB_STACK_DEPTH", INT2NUM(GL_CLIENT_ATTRIB_STACK_DEPTH));
	rb_define_const(module, "GL_CLIENT_PIXEL_STORE_BIT", INT2NUM(GL_CLIENT_PIXEL_STORE_BIT));
	rb_define_const(module, "GL_CLIENT_VERTEX_ARRAY_BIT", INT2NUM(GL_CLIENT_VERTEX_ARRAY_BIT));
	rb_define_const(module, "GL_CLIP_DISTANCE0", INT2NUM(GL_CLIP_DISTANCE0));
	rb_define_const(module, "GL_CLIP_DISTANCE1", INT2NUM(GL_CLIP_DISTANCE1));
	rb_define_const(module, "GL_CLIP_DISTANCE2", INT2NUM(GL_CLIP_DISTANCE2));
	rb_define_const(module, "GL_CLIP_DISTANCE3", INT2NUM(GL_CLIP_DISTANCE3));
	rb_define_const(module, "GL_CLIP_DISTANCE4", INT2NUM(GL_CLIP_DISTANCE4));
	rb_define_const(module, "GL_CLIP_DISTANCE5", INT2NUM(GL_CLIP_DISTANCE5));
	rb_define_const(module, "GL_CLIP_DISTANCE_NV", INT2NUM(GL_CLIP_DISTANCE_NV));
	rb_define_const(module, "GL_CLIP_FAR_HINT_PGI", INT2NUM(GL_CLIP_FAR_HINT_PGI));
	rb_define_const(module, "GL_CLIP_NEAR_HINT_PGI", INT2NUM(GL_CLIP_NEAR_HINT_PGI));
	rb_define_const(module, "GL_CLIP_PLANE0", INT2NUM(GL_CLIP_PLANE0));
	rb_define_const(module, "GL_CLIP_PLANE1", INT2NUM(GL_CLIP_PLANE1));
	rb_define_const(module, "GL_CLIP_PLANE2", INT2NUM(GL_CLIP_PLANE2));
	rb_define_const(module, "GL_CLIP_PLANE3", INT2NUM(GL_CLIP_PLANE3));
	rb_define_const(module, "GL_CLIP_PLANE4", INT2NUM(GL_CLIP_PLANE4));
	rb_define_const(module, "GL_CLIP_PLANE5", INT2NUM(GL_CLIP_PLANE5));
	rb_define_const(module, "GL_CLIP_VOLUME_CLIPPING_HINT_EXT", INT2NUM(GL_CLIP_VOLUME_CLIPPING_HINT_EXT));
	rb_define_const(module, "GL_CMYKA_EXT", INT2NUM(GL_CMYKA_EXT));
	rb_define_const(module, "GL_CMYK_EXT", INT2NUM(GL_CMYK_EXT));
	rb_define_const(module, "GL_CND0_ATI", INT2NUM(GL_CND0_ATI));
	rb_define_const(module, "GL_CND_ATI", INT2NUM(GL_CND_ATI));
	rb_define_const(module, "GL_COEFF", INT2NUM(GL_COEFF));
	rb_define_const(module, "GL_COLOR", INT2NUM(GL_COLOR));
	rb_define_const(module, "GL_COLOR3_BIT_PGI", INT2NUM(GL_COLOR3_BIT_PGI));
	rb_define_const(module, "GL_COLOR4_BIT_PGI", INT2NUM(GL_COLOR4_BIT_PGI));
	rb_define_const(module, "GL_COLOR_ALPHA_PAIRING_ATI", INT2NUM(GL_COLOR_ALPHA_PAIRING_ATI));
	rb_define_const(module, "GL_COLOR_ARRAY", INT2NUM(GL_COLOR_ARRAY));
	rb_define_const(module, "GL_COLOR_ARRAY_BUFFER_BINDING", INT2NUM(GL_COLOR_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_COLOR_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_COLOR_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_COLOR_ARRAY_COUNT_EXT", INT2NUM(GL_COLOR_ARRAY_COUNT_EXT));
	rb_define_const(module, "GL_COLOR_ARRAY_EXT", INT2NUM(GL_COLOR_ARRAY_EXT));
	rb_define_const(module, "GL_COLOR_ARRAY_LIST_IBM", INT2NUM(GL_COLOR_ARRAY_LIST_IBM));
	rb_define_const(module, "GL_COLOR_ARRAY_LIST_STRIDE_IBM", INT2NUM(GL_COLOR_ARRAY_LIST_STRIDE_IBM));
	rb_define_const(module, "GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL", INT2NUM(GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL));
	rb_define_const(module, "GL_COLOR_ARRAY_POINTER", INT2NUM(GL_COLOR_ARRAY_POINTER));
	rb_define_const(module, "GL_COLOR_ARRAY_POINTER_EXT", INT2NUM(GL_COLOR_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_COLOR_ARRAY_SIZE", INT2NUM(GL_COLOR_ARRAY_SIZE));
	rb_define_const(module, "GL_COLOR_ARRAY_SIZE_EXT", INT2NUM(GL_COLOR_ARRAY_SIZE_EXT));
	rb_define_const(module, "GL_COLOR_ARRAY_STRIDE", INT2NUM(GL_COLOR_ARRAY_STRIDE));
	rb_define_const(module, "GL_COLOR_ARRAY_STRIDE_EXT", INT2NUM(GL_COLOR_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_COLOR_ARRAY_TYPE", INT2NUM(GL_COLOR_ARRAY_TYPE));
	rb_define_const(module, "GL_COLOR_ARRAY_TYPE_EXT", INT2NUM(GL_COLOR_ARRAY_TYPE_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT0", INT2NUM(GL_COLOR_ATTACHMENT0));
	rb_define_const(module, "GL_COLOR_ATTACHMENT0_EXT", INT2NUM(GL_COLOR_ATTACHMENT0_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT1", INT2NUM(GL_COLOR_ATTACHMENT1));
	rb_define_const(module, "GL_COLOR_ATTACHMENT10", INT2NUM(GL_COLOR_ATTACHMENT10));
	rb_define_const(module, "GL_COLOR_ATTACHMENT10_EXT", INT2NUM(GL_COLOR_ATTACHMENT10_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT11", INT2NUM(GL_COLOR_ATTACHMENT11));
	rb_define_const(module, "GL_COLOR_ATTACHMENT11_EXT", INT2NUM(GL_COLOR_ATTACHMENT11_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT12", INT2NUM(GL_COLOR_ATTACHMENT12));
	rb_define_const(module, "GL_COLOR_ATTACHMENT12_EXT", INT2NUM(GL_COLOR_ATTACHMENT12_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT13", INT2NUM(GL_COLOR_ATTACHMENT13));
	rb_define_const(module, "GL_COLOR_ATTACHMENT13_EXT", INT2NUM(GL_COLOR_ATTACHMENT13_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT14", INT2NUM(GL_COLOR_ATTACHMENT14));
	rb_define_const(module, "GL_COLOR_ATTACHMENT14_EXT", INT2NUM(GL_COLOR_ATTACHMENT14_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT15", INT2NUM(GL_COLOR_ATTACHMENT15));
	rb_define_const(module, "GL_COLOR_ATTACHMENT15_EXT", INT2NUM(GL_COLOR_ATTACHMENT15_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT1_EXT", INT2NUM(GL_COLOR_ATTACHMENT1_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT2", INT2NUM(GL_COLOR_ATTACHMENT2));
	rb_define_const(module, "GL_COLOR_ATTACHMENT2_EXT", INT2NUM(GL_COLOR_ATTACHMENT2_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT3", INT2NUM(GL_COLOR_ATTACHMENT3));
	rb_define_const(module, "GL_COLOR_ATTACHMENT3_EXT", INT2NUM(GL_COLOR_ATTACHMENT3_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT4", INT2NUM(GL_COLOR_ATTACHMENT4));
	rb_define_const(module, "GL_COLOR_ATTACHMENT4_EXT", INT2NUM(GL_COLOR_ATTACHMENT4_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT5", INT2NUM(GL_COLOR_ATTACHMENT5));
	rb_define_const(module, "GL_COLOR_ATTACHMENT5_EXT", INT2NUM(GL_COLOR_ATTACHMENT5_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT6", INT2NUM(GL_COLOR_ATTACHMENT6));
	rb_define_const(module, "GL_COLOR_ATTACHMENT6_EXT", INT2NUM(GL_COLOR_ATTACHMENT6_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT7", INT2NUM(GL_COLOR_ATTACHMENT7));
	rb_define_const(module, "GL_COLOR_ATTACHMENT7_EXT", INT2NUM(GL_COLOR_ATTACHMENT7_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT8", INT2NUM(GL_COLOR_ATTACHMENT8));
	rb_define_const(module, "GL_COLOR_ATTACHMENT8_EXT", INT2NUM(GL_COLOR_ATTACHMENT8_EXT));
	rb_define_const(module, "GL_COLOR_ATTACHMENT9", INT2NUM(GL_COLOR_ATTACHMENT9));
	rb_define_const(module, "GL_COLOR_ATTACHMENT9_EXT", INT2NUM(GL_COLOR_ATTACHMENT9_EXT));
	rb_define_const(module, "GL_COLOR_BUFFER_BIT", INT2NUM(GL_COLOR_BUFFER_BIT));
	rb_define_const(module, "GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI", INT2NUM(GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI));
	rb_define_const(module, "GL_COLOR_CLEAR_VALUE", INT2NUM(GL_COLOR_CLEAR_VALUE));
	rb_define_const(module, "GL_COLOR_INDEX", INT2NUM(GL_COLOR_INDEX));
	rb_define_const(module, "GL_COLOR_INDEX12_EXT", INT2NUM(GL_COLOR_INDEX12_EXT));
	rb_define_const(module, "GL_COLOR_INDEX16_EXT", INT2NUM(GL_COLOR_INDEX16_EXT));
	rb_define_const(module, "GL_COLOR_INDEX1_EXT", INT2NUM(GL_COLOR_INDEX1_EXT));
	rb_define_const(module, "GL_COLOR_INDEX2_EXT", INT2NUM(GL_COLOR_INDEX2_EXT));
	rb_define_const(module, "GL_COLOR_INDEX4_EXT", INT2NUM(GL_COLOR_INDEX4_EXT));
	rb_define_const(module, "GL_COLOR_INDEX8_EXT", INT2NUM(GL_COLOR_INDEX8_EXT));
	rb_define_const(module, "GL_COLOR_INDEXES", INT2NUM(GL_COLOR_INDEXES));
	rb_define_const(module, "GL_COLOR_LOGIC_OP", INT2NUM(GL_COLOR_LOGIC_OP));
	rb_define_const(module, "GL_COLOR_MATERIAL", INT2NUM(GL_COLOR_MATERIAL));
	rb_define_const(module, "GL_COLOR_MATERIAL_FACE", INT2NUM(GL_COLOR_MATERIAL_FACE));
	rb_define_const(module, "GL_COLOR_MATERIAL_PARAMETER", INT2NUM(GL_COLOR_MATERIAL_PARAMETER));
	rb_define_const(module, "GL_COLOR_MATRIX", INT2NUM(GL_COLOR_MATRIX));
	rb_define_const(module, "GL_COLOR_MATRIX_SGI", INT2NUM(GL_COLOR_MATRIX_SGI));
	rb_define_const(module, "GL_COLOR_MATRIX_STACK_DEPTH", INT2NUM(GL_COLOR_MATRIX_STACK_DEPTH));
	rb_define_const(module, "GL_COLOR_MATRIX_STACK_DEPTH_SGI", INT2NUM(GL_COLOR_MATRIX_STACK_DEPTH_SGI));
	rb_define_const(module, "GL_COLOR_SUM", INT2NUM(GL_COLOR_SUM));
	rb_define_const(module, "GL_COLOR_SUM_ARB", INT2NUM(GL_COLOR_SUM_ARB));
	rb_define_const(module, "GL_COLOR_SUM_CLAMP_NV", INT2NUM(GL_COLOR_SUM_CLAMP_NV));
	rb_define_const(module, "GL_COLOR_SUM_EXT", INT2NUM(GL_COLOR_SUM_EXT));
	rb_define_const(module, "GL_COLOR_TABLE", INT2NUM(GL_COLOR_TABLE));
	rb_define_const(module, "GL_COLOR_TABLE_ALPHA_SIZE", INT2NUM(GL_COLOR_TABLE_ALPHA_SIZE));
	rb_define_const(module, "GL_COLOR_TABLE_ALPHA_SIZE_SGI", INT2NUM(GL_COLOR_TABLE_ALPHA_SIZE_SGI));
	rb_define_const(module, "GL_COLOR_TABLE_BIAS", INT2NUM(GL_COLOR_TABLE_BIAS));
	rb_define_const(module, "GL_COLOR_TABLE_BIAS_SGI", INT2NUM(GL_COLOR_TABLE_BIAS_SGI));
	rb_define_const(module, "GL_COLOR_TABLE_BLUE_SIZE", INT2NUM(GL_COLOR_TABLE_BLUE_SIZE));
	rb_define_const(module, "GL_COLOR_TABLE_BLUE_SIZE_SGI", INT2NUM(GL_COLOR_TABLE_BLUE_SIZE_SGI));
	rb_define_const(module, "GL_COLOR_TABLE_FORMAT", INT2NUM(GL_COLOR_TABLE_FORMAT));
	rb_define_const(module, "GL_COLOR_TABLE_FORMAT_SGI", INT2NUM(GL_COLOR_TABLE_FORMAT_SGI));
	rb_define_const(module, "GL_COLOR_TABLE_GREEN_SIZE", INT2NUM(GL_COLOR_TABLE_GREEN_SIZE));
	rb_define_const(module, "GL_COLOR_TABLE_GREEN_SIZE_SGI", INT2NUM(GL_COLOR_TABLE_GREEN_SIZE_SGI));
	rb_define_const(module, "GL_COLOR_TABLE_INTENSITY_SIZE", INT2NUM(GL_COLOR_TABLE_INTENSITY_SIZE));
	rb_define_const(module, "GL_COLOR_TABLE_INTENSITY_SIZE_SGI", INT2NUM(GL_COLOR_TABLE_INTENSITY_SIZE_SGI));
	rb_define_const(module, "GL_COLOR_TABLE_LUMINANCE_SIZE", INT2NUM(GL_COLOR_TABLE_LUMINANCE_SIZE));
	rb_define_const(module, "GL_COLOR_TABLE_LUMINANCE_SIZE_SGI", INT2NUM(GL_COLOR_TABLE_LUMINANCE_SIZE_SGI));
	rb_define_const(module, "GL_COLOR_TABLE_RED_SIZE", INT2NUM(GL_COLOR_TABLE_RED_SIZE));
	rb_define_const(module, "GL_COLOR_TABLE_RED_SIZE_SGI", INT2NUM(GL_COLOR_TABLE_RED_SIZE_SGI));
	rb_define_const(module, "GL_COLOR_TABLE_SCALE", INT2NUM(GL_COLOR_TABLE_SCALE));
	rb_define_const(module, "GL_COLOR_TABLE_SCALE_SGI", INT2NUM(GL_COLOR_TABLE_SCALE_SGI));
	rb_define_const(module, "GL_COLOR_TABLE_SGI", INT2NUM(GL_COLOR_TABLE_SGI));
	rb_define_const(module, "GL_COLOR_TABLE_WIDTH", INT2NUM(GL_COLOR_TABLE_WIDTH));
	rb_define_const(module, "GL_COLOR_TABLE_WIDTH_SGI", INT2NUM(GL_COLOR_TABLE_WIDTH_SGI));
	rb_define_const(module, "GL_COLOR_WRITEMASK", INT2NUM(GL_COLOR_WRITEMASK));
	rb_define_const(module, "GL_COMBINE", INT2NUM(GL_COMBINE));
	rb_define_const(module, "GL_COMBINE4_NV", INT2NUM(GL_COMBINE4_NV));
	rb_define_const(module, "GL_COMBINER0_NV", INT2NUM(GL_COMBINER0_NV));
	rb_define_const(module, "GL_COMBINER1_NV", INT2NUM(GL_COMBINER1_NV));
	rb_define_const(module, "GL_COMBINER2_NV", INT2NUM(GL_COMBINER2_NV));
	rb_define_const(module, "GL_COMBINER3_NV", INT2NUM(GL_COMBINER3_NV));
	rb_define_const(module, "GL_COMBINER4_NV", INT2NUM(GL_COMBINER4_NV));
	rb_define_const(module, "GL_COMBINER5_NV", INT2NUM(GL_COMBINER5_NV));
	rb_define_const(module, "GL_COMBINER6_NV", INT2NUM(GL_COMBINER6_NV));
	rb_define_const(module, "GL_COMBINER7_NV", INT2NUM(GL_COMBINER7_NV));
	rb_define_const(module, "GL_COMBINER_AB_DOT_PRODUCT_NV", INT2NUM(GL_COMBINER_AB_DOT_PRODUCT_NV));
	rb_define_const(module, "GL_COMBINER_AB_OUTPUT_NV", INT2NUM(GL_COMBINER_AB_OUTPUT_NV));
	rb_define_const(module, "GL_COMBINER_BIAS_NV", INT2NUM(GL_COMBINER_BIAS_NV));
	rb_define_const(module, "GL_COMBINER_CD_DOT_PRODUCT_NV", INT2NUM(GL_COMBINER_CD_DOT_PRODUCT_NV));
	rb_define_const(module, "GL_COMBINER_CD_OUTPUT_NV", INT2NUM(GL_COMBINER_CD_OUTPUT_NV));
	rb_define_const(module, "GL_COMBINER_COMPONENT_USAGE_NV", INT2NUM(GL_COMBINER_COMPONENT_USAGE_NV));
	rb_define_const(module, "GL_COMBINER_INPUT_NV", INT2NUM(GL_COMBINER_INPUT_NV));
	rb_define_const(module, "GL_COMBINER_MAPPING_NV", INT2NUM(GL_COMBINER_MAPPING_NV));
	rb_define_const(module, "GL_COMBINER_MUX_SUM_NV", INT2NUM(GL_COMBINER_MUX_SUM_NV));
	rb_define_const(module, "GL_COMBINER_SCALE_NV", INT2NUM(GL_COMBINER_SCALE_NV));
	rb_define_const(module, "GL_COMBINER_SUM_OUTPUT_NV", INT2NUM(GL_COMBINER_SUM_OUTPUT_NV));
	rb_define_const(module, "GL_COMBINE_ALPHA", INT2NUM(GL_COMBINE_ALPHA));
	rb_define_const(module, "GL_COMBINE_ALPHA_ARB", INT2NUM(GL_COMBINE_ALPHA_ARB));
	rb_define_const(module, "GL_COMBINE_ALPHA_EXT", INT2NUM(GL_COMBINE_ALPHA_EXT));
	rb_define_const(module, "GL_COMBINE_ARB", INT2NUM(GL_COMBINE_ARB));
	rb_define_const(module, "GL_COMBINE_EXT", INT2NUM(GL_COMBINE_EXT));
	rb_define_const(module, "GL_COMBINE_RGB", INT2NUM(GL_COMBINE_RGB));
	rb_define_const(module, "GL_COMBINE_RGB_ARB", INT2NUM(GL_COMBINE_RGB_ARB));
	rb_define_const(module, "GL_COMBINE_RGB_EXT", INT2NUM(GL_COMBINE_RGB_EXT));
	rb_define_const(module, "GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT", INT2NUM(GL_COMPARE_REF_DEPTH_TO_TEXTURE_EXT));
	rb_define_const(module, "GL_COMPARE_REF_TO_TEXTURE", INT2NUM(GL_COMPARE_REF_TO_TEXTURE));
	rb_define_const(module, "GL_COMPARE_R_TO_TEXTURE", INT2NUM(GL_COMPARE_R_TO_TEXTURE));
	rb_define_const(module, "GL_COMPARE_R_TO_TEXTURE_ARB", INT2NUM(GL_COMPARE_R_TO_TEXTURE_ARB));
	rb_define_const(module, "GL_COMPILE", INT2NUM(GL_COMPILE));
	rb_define_const(module, "GL_COMPILE_AND_EXECUTE", INT2NUM(GL_COMPILE_AND_EXECUTE));
	rb_define_const(module, "GL_COMPILE_STATUS", INT2NUM(GL_COMPILE_STATUS));
	rb_define_const(module, "GL_COMPRESSED_ALPHA", INT2NUM(GL_COMPRESSED_ALPHA));
	rb_define_const(module, "GL_COMPRESSED_ALPHA_ARB", INT2NUM(GL_COMPRESSED_ALPHA_ARB));
	rb_define_const(module, "GL_COMPRESSED_INTENSITY", INT2NUM(GL_COMPRESSED_INTENSITY));
	rb_define_const(module, "GL_COMPRESSED_INTENSITY_ARB", INT2NUM(GL_COMPRESSED_INTENSITY_ARB));
	rb_define_const(module, "GL_COMPRESSED_LUMINANCE", INT2NUM(GL_COMPRESSED_LUMINANCE));
	rb_define_const(module, "GL_COMPRESSED_LUMINANCE_ALPHA", INT2NUM(GL_COMPRESSED_LUMINANCE_ALPHA));
	rb_define_const(module, "GL_COMPRESSED_LUMINANCE_ALPHA_ARB", INT2NUM(GL_COMPRESSED_LUMINANCE_ALPHA_ARB));
	rb_define_const(module, "GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT", INT2NUM(GL_COMPRESSED_LUMINANCE_ALPHA_LATC2_EXT));
	rb_define_const(module, "GL_COMPRESSED_LUMINANCE_ARB", INT2NUM(GL_COMPRESSED_LUMINANCE_ARB));
	rb_define_const(module, "GL_COMPRESSED_LUMINANCE_LATC1_EXT", INT2NUM(GL_COMPRESSED_LUMINANCE_LATC1_EXT));
	rb_define_const(module, "GL_COMPRESSED_RED", INT2NUM(GL_COMPRESSED_RED));
	rb_define_const(module, "GL_COMPRESSED_RED_GREEN_RGTC2_EXT", INT2NUM(GL_COMPRESSED_RED_GREEN_RGTC2_EXT));
	rb_define_const(module, "GL_COMPRESSED_RED_RGTC1", INT2NUM(GL_COMPRESSED_RED_RGTC1));
	rb_define_const(module, "GL_COMPRESSED_RED_RGTC1_EXT", INT2NUM(GL_COMPRESSED_RED_RGTC1_EXT));
	rb_define_const(module, "GL_COMPRESSED_RG", INT2NUM(GL_COMPRESSED_RG));
	rb_define_const(module, "GL_COMPRESSED_RGB", INT2NUM(GL_COMPRESSED_RGB));
	rb_define_const(module, "GL_COMPRESSED_RGBA", INT2NUM(GL_COMPRESSED_RGBA));
	rb_define_const(module, "GL_COMPRESSED_RGBA_ARB", INT2NUM(GL_COMPRESSED_RGBA_ARB));
	rb_define_const(module, "GL_COMPRESSED_RGBA_FXT1_3DFX", INT2NUM(GL_COMPRESSED_RGBA_FXT1_3DFX));
	rb_define_const(module, "GL_COMPRESSED_RGBA_S3TC_DXT1_EXT", INT2NUM(GL_COMPRESSED_RGBA_S3TC_DXT1_EXT));
	rb_define_const(module, "GL_COMPRESSED_RGBA_S3TC_DXT3_EXT", INT2NUM(GL_COMPRESSED_RGBA_S3TC_DXT3_EXT));
	rb_define_const(module, "GL_COMPRESSED_RGBA_S3TC_DXT5_EXT", INT2NUM(GL_COMPRESSED_RGBA_S3TC_DXT5_EXT));
	rb_define_const(module, "GL_COMPRESSED_RGB_ARB", INT2NUM(GL_COMPRESSED_RGB_ARB));
	rb_define_const(module, "GL_COMPRESSED_RGB_FXT1_3DFX", INT2NUM(GL_COMPRESSED_RGB_FXT1_3DFX));
	rb_define_const(module, "GL_COMPRESSED_RGB_S3TC_DXT1_EXT", INT2NUM(GL_COMPRESSED_RGB_S3TC_DXT1_EXT));
	rb_define_const(module, "GL_COMPRESSED_RG_RGTC2", INT2NUM(GL_COMPRESSED_RG_RGTC2));
	rb_define_const(module, "GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT", INT2NUM(GL_COMPRESSED_SIGNED_LUMINANCE_ALPHA_LATC2_EXT));
	rb_define_const(module, "GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT", INT2NUM(GL_COMPRESSED_SIGNED_LUMINANCE_LATC1_EXT));
	rb_define_const(module, "GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT", INT2NUM(GL_COMPRESSED_SIGNED_RED_GREEN_RGTC2_EXT));
	rb_define_const(module, "GL_COMPRESSED_SIGNED_RED_RGTC1", INT2NUM(GL_COMPRESSED_SIGNED_RED_RGTC1));
	rb_define_const(module, "GL_COMPRESSED_SIGNED_RED_RGTC1_EXT", INT2NUM(GL_COMPRESSED_SIGNED_RED_RGTC1_EXT));
	rb_define_const(module, "GL_COMPRESSED_SIGNED_RG_RGTC2", INT2NUM(GL_COMPRESSED_SIGNED_RG_RGTC2));
	rb_define_const(module, "GL_COMPRESSED_SLUMINANCE", INT2NUM(GL_COMPRESSED_SLUMINANCE));
	rb_define_const(module, "GL_COMPRESSED_SLUMINANCE_ALPHA", INT2NUM(GL_COMPRESSED_SLUMINANCE_ALPHA));
	rb_define_const(module, "GL_COMPRESSED_SLUMINANCE_ALPHA_EXT", INT2NUM(GL_COMPRESSED_SLUMINANCE_ALPHA_EXT));
	rb_define_const(module, "GL_COMPRESSED_SLUMINANCE_EXT", INT2NUM(GL_COMPRESSED_SLUMINANCE_EXT));
	rb_define_const(module, "GL_COMPRESSED_SRGB", INT2NUM(GL_COMPRESSED_SRGB));
	rb_define_const(module, "GL_COMPRESSED_SRGB_ALPHA", INT2NUM(GL_COMPRESSED_SRGB_ALPHA));
	rb_define_const(module, "GL_COMPRESSED_SRGB_ALPHA_EXT", INT2NUM(GL_COMPRESSED_SRGB_ALPHA_EXT));
	rb_define_const(module, "GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT", INT2NUM(GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT));
	rb_define_const(module, "GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT", INT2NUM(GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT));
	rb_define_const(module, "GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT", INT2NUM(GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT));
	rb_define_const(module, "GL_COMPRESSED_SRGB_EXT", INT2NUM(GL_COMPRESSED_SRGB_EXT));
	rb_define_const(module, "GL_COMPRESSED_SRGB_S3TC_DXT1_EXT", INT2NUM(GL_COMPRESSED_SRGB_S3TC_DXT1_EXT));
	rb_define_const(module, "GL_COMPRESSED_TEXTURE_FORMATS", INT2NUM(GL_COMPRESSED_TEXTURE_FORMATS));
	rb_define_const(module, "GL_COMPRESSED_TEXTURE_FORMATS_ARB", INT2NUM(GL_COMPRESSED_TEXTURE_FORMATS_ARB));
	rb_define_const(module, "GL_COMP_BIT_ATI", INT2NUM(GL_COMP_BIT_ATI));
	rb_define_const(module, "GL_CONSERVE_MEMORY_HINT_PGI", INT2NUM(GL_CONSERVE_MEMORY_HINT_PGI));
	rb_define_const(module, "GL_CONSTANT", INT2NUM(GL_CONSTANT));
	rb_define_const(module, "GL_CONSTANT_ALPHA", INT2NUM(GL_CONSTANT_ALPHA));
	rb_define_const(module, "GL_CONSTANT_ALPHA_EXT", INT2NUM(GL_CONSTANT_ALPHA_EXT));
	rb_define_const(module, "GL_CONSTANT_ARB", INT2NUM(GL_CONSTANT_ARB));
	rb_define_const(module, "GL_CONSTANT_ATTENUATION", INT2NUM(GL_CONSTANT_ATTENUATION));
	rb_define_const(module, "GL_CONSTANT_BORDER", INT2NUM(GL_CONSTANT_BORDER));
	rb_define_const(module, "GL_CONSTANT_BORDER_HP", INT2NUM(GL_CONSTANT_BORDER_HP));
	rb_define_const(module, "GL_CONSTANT_COLOR", INT2NUM(GL_CONSTANT_COLOR));
	rb_define_const(module, "GL_CONSTANT_COLOR0_NV", INT2NUM(GL_CONSTANT_COLOR0_NV));
	rb_define_const(module, "GL_CONSTANT_COLOR1_NV", INT2NUM(GL_CONSTANT_COLOR1_NV));
	rb_define_const(module, "GL_CONSTANT_COLOR_EXT", INT2NUM(GL_CONSTANT_COLOR_EXT));
	rb_define_const(module, "GL_CONSTANT_EXT", INT2NUM(GL_CONSTANT_EXT));
	rb_define_const(module, "GL_CONST_EYE_NV", INT2NUM(GL_CONST_EYE_NV));
	rb_define_const(module, "GL_CONTEXT_FLAGS", INT2NUM(GL_CONTEXT_FLAGS));
	rb_define_const(module, "GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT", INT2NUM(GL_CONTEXT_FLAG_FORWARD_COMPATIBLE_BIT));
	rb_define_const(module, "GL_CONVOLUTION_1D", INT2NUM(GL_CONVOLUTION_1D));
	rb_define_const(module, "GL_CONVOLUTION_1D_EXT", INT2NUM(GL_CONVOLUTION_1D_EXT));
	rb_define_const(module, "GL_CONVOLUTION_2D", INT2NUM(GL_CONVOLUTION_2D));
	rb_define_const(module, "GL_CONVOLUTION_2D_EXT", INT2NUM(GL_CONVOLUTION_2D_EXT));
	rb_define_const(module, "GL_CONVOLUTION_BORDER_COLOR", INT2NUM(GL_CONVOLUTION_BORDER_COLOR));
	rb_define_const(module, "GL_CONVOLUTION_BORDER_COLOR_HP", INT2NUM(GL_CONVOLUTION_BORDER_COLOR_HP));
	rb_define_const(module, "GL_CONVOLUTION_BORDER_MODE", INT2NUM(GL_CONVOLUTION_BORDER_MODE));
	rb_define_const(module, "GL_CONVOLUTION_BORDER_MODE_EXT", INT2NUM(GL_CONVOLUTION_BORDER_MODE_EXT));
	rb_define_const(module, "GL_CONVOLUTION_FILTER_BIAS", INT2NUM(GL_CONVOLUTION_FILTER_BIAS));
	rb_define_const(module, "GL_CONVOLUTION_FILTER_BIAS_EXT", INT2NUM(GL_CONVOLUTION_FILTER_BIAS_EXT));
	rb_define_const(module, "GL_CONVOLUTION_FILTER_SCALE", INT2NUM(GL_CONVOLUTION_FILTER_SCALE));
	rb_define_const(module, "GL_CONVOLUTION_FILTER_SCALE_EXT", INT2NUM(GL_CONVOLUTION_FILTER_SCALE_EXT));
	rb_define_const(module, "GL_CONVOLUTION_FORMAT", INT2NUM(GL_CONVOLUTION_FORMAT));
	rb_define_const(module, "GL_CONVOLUTION_FORMAT_EXT", INT2NUM(GL_CONVOLUTION_FORMAT_EXT));
	rb_define_const(module, "GL_CONVOLUTION_HEIGHT", INT2NUM(GL_CONVOLUTION_HEIGHT));
	rb_define_const(module, "GL_CONVOLUTION_HEIGHT_EXT", INT2NUM(GL_CONVOLUTION_HEIGHT_EXT));
	rb_define_const(module, "GL_CONVOLUTION_HINT_SGIX", INT2NUM(GL_CONVOLUTION_HINT_SGIX));
	rb_define_const(module, "GL_CONVOLUTION_WIDTH", INT2NUM(GL_CONVOLUTION_WIDTH));
	rb_define_const(module, "GL_CONVOLUTION_WIDTH_EXT", INT2NUM(GL_CONVOLUTION_WIDTH_EXT));
	rb_define_const(module, "GL_CON_0_ATI", INT2NUM(GL_CON_0_ATI));
	rb_define_const(module, "GL_CON_10_ATI", INT2NUM(GL_CON_10_ATI));
	rb_define_const(module, "GL_CON_11_ATI", INT2NUM(GL_CON_11_ATI));
	rb_define_const(module, "GL_CON_12_ATI", INT2NUM(GL_CON_12_ATI));
	rb_define_const(module, "GL_CON_13_ATI", INT2NUM(GL_CON_13_ATI));
	rb_define_const(module, "GL_CON_14_ATI", INT2NUM(GL_CON_14_ATI));
	rb_define_const(module, "GL_CON_15_ATI", INT2NUM(GL_CON_15_ATI));
	rb_define_const(module, "GL_CON_16_ATI", INT2NUM(GL_CON_16_ATI));
	rb_define_const(module, "GL_CON_17_ATI", INT2NUM(GL_CON_17_ATI));
	rb_define_const(module, "GL_CON_18_ATI", INT2NUM(GL_CON_18_ATI));
	rb_define_const(module, "GL_CON_19_ATI", INT2NUM(GL_CON_19_ATI));
	rb_define_const(module, "GL_CON_1_ATI", INT2NUM(GL_CON_1_ATI));
	rb_define_const(module, "GL_CON_20_ATI", INT2NUM(GL_CON_20_ATI));
	rb_define_const(module, "GL_CON_21_ATI", INT2NUM(GL_CON_21_ATI));
	rb_define_const(module, "GL_CON_22_ATI", INT2NUM(GL_CON_22_ATI));
	rb_define_const(module, "GL_CON_23_ATI", INT2NUM(GL_CON_23_ATI));
	rb_define_const(module, "GL_CON_24_ATI", INT2NUM(GL_CON_24_ATI));
	rb_define_const(module, "GL_CON_25_ATI", INT2NUM(GL_CON_25_ATI));
	rb_define_const(module, "GL_CON_26_ATI", INT2NUM(GL_CON_26_ATI));
	rb_define_const(module, "GL_CON_27_ATI", INT2NUM(GL_CON_27_ATI));
	rb_define_const(module, "GL_CON_28_ATI", INT2NUM(GL_CON_28_ATI));
	rb_define_const(module, "GL_CON_29_ATI", INT2NUM(GL_CON_29_ATI));
	rb_define_const(module, "GL_CON_2_ATI", INT2NUM(GL_CON_2_ATI));
	rb_define_const(module, "GL_CON_30_ATI", INT2NUM(GL_CON_30_ATI));
	rb_define_const(module, "GL_CON_31_ATI", INT2NUM(GL_CON_31_ATI));
	rb_define_const(module, "GL_CON_3_ATI", INT2NUM(GL_CON_3_ATI));
	rb_define_const(module, "GL_CON_4_ATI", INT2NUM(GL_CON_4_ATI));
	rb_define_const(module, "GL_CON_5_ATI", INT2NUM(GL_CON_5_ATI));
	rb_define_const(module, "GL_CON_6_ATI", INT2NUM(GL_CON_6_ATI));
	rb_define_const(module, "GL_CON_7_ATI", INT2NUM(GL_CON_7_ATI));
	rb_define_const(module, "GL_CON_8_ATI", INT2NUM(GL_CON_8_ATI));
	rb_define_const(module, "GL_CON_9_ATI", INT2NUM(GL_CON_9_ATI));
	rb_define_const(module, "GL_COORD_REPLACE", INT2NUM(GL_COORD_REPLACE));
	rb_define_const(module, "GL_COORD_REPLACE_ARB", INT2NUM(GL_COORD_REPLACE_ARB));
	rb_define_const(module, "GL_COORD_REPLACE_NV", INT2NUM(GL_COORD_REPLACE_NV));
	rb_define_const(module, "GL_COPY", INT2NUM(GL_COPY));
	rb_define_const(module, "GL_COPY_INVERTED", INT2NUM(GL_COPY_INVERTED));
	rb_define_const(module, "GL_COPY_PIXEL_TOKEN", INT2NUM(GL_COPY_PIXEL_TOKEN));
	rb_define_const(module, "GL_CUBIC_EXT", INT2NUM(GL_CUBIC_EXT));
	rb_define_const(module, "GL_CUBIC_HP", INT2NUM(GL_CUBIC_HP));
	rb_define_const(module, "GL_CULL_FACE", INT2NUM(GL_CULL_FACE));
	rb_define_const(module, "GL_CULL_FACE_MODE", INT2NUM(GL_CULL_FACE_MODE));
	rb_define_const(module, "GL_CULL_FRAGMENT_NV", INT2NUM(GL_CULL_FRAGMENT_NV));
	rb_define_const(module, "GL_CULL_MODES_NV", INT2NUM(GL_CULL_MODES_NV));
	rb_define_const(module, "GL_CULL_VERTEX_EXT", INT2NUM(GL_CULL_VERTEX_EXT));
	rb_define_const(module, "GL_CULL_VERTEX_EYE_POSITION_EXT", INT2NUM(GL_CULL_VERTEX_EYE_POSITION_EXT));
	rb_define_const(module, "GL_CULL_VERTEX_IBM", INT2NUM(GL_CULL_VERTEX_IBM));
	rb_define_const(module, "GL_CULL_VERTEX_OBJECT_POSITION_EXT", INT2NUM(GL_CULL_VERTEX_OBJECT_POSITION_EXT));
	rb_define_const(module, "GL_CURRENT_ATTRIB_NV", INT2NUM(GL_CURRENT_ATTRIB_NV));
	rb_define_const(module, "GL_CURRENT_BINORMAL_EXT", INT2NUM(GL_CURRENT_BINORMAL_EXT));
	rb_define_const(module, "GL_CURRENT_BIT", INT2NUM(GL_CURRENT_BIT));
	rb_define_const(module, "GL_CURRENT_COLOR", INT2NUM(GL_CURRENT_COLOR));
	rb_define_const(module, "GL_CURRENT_FOG_COORD", INT2NUM(GL_CURRENT_FOG_COORD));
	rb_define_const(module, "GL_CURRENT_FOG_COORDINATE", INT2NUM(GL_CURRENT_FOG_COORDINATE));
	rb_define_const(module, "GL_CURRENT_FOG_COORDINATE_EXT", INT2NUM(GL_CURRENT_FOG_COORDINATE_EXT));
	rb_define_const(module, "GL_CURRENT_INDEX", INT2NUM(GL_CURRENT_INDEX));
	rb_define_const(module, "GL_CURRENT_MATRIX_ARB", INT2NUM(GL_CURRENT_MATRIX_ARB));
	rb_define_const(module, "GL_CURRENT_MATRIX_INDEX_ARB", INT2NUM(GL_CURRENT_MATRIX_INDEX_ARB));
	rb_define_const(module, "GL_CURRENT_MATRIX_NV", INT2NUM(GL_CURRENT_MATRIX_NV));
	rb_define_const(module, "GL_CURRENT_MATRIX_STACK_DEPTH_ARB", INT2NUM(GL_CURRENT_MATRIX_STACK_DEPTH_ARB));
	rb_define_const(module, "GL_CURRENT_MATRIX_STACK_DEPTH_NV", INT2NUM(GL_CURRENT_MATRIX_STACK_DEPTH_NV));
	rb_define_const(module, "GL_CURRENT_NORMAL", INT2NUM(GL_CURRENT_NORMAL));
	rb_define_const(module, "GL_CURRENT_OCCLUSION_QUERY_ID_NV", INT2NUM(GL_CURRENT_OCCLUSION_QUERY_ID_NV));
	rb_define_const(module, "GL_CURRENT_PALETTE_MATRIX_ARB", INT2NUM(GL_CURRENT_PALETTE_MATRIX_ARB));
	rb_define_const(module, "GL_CURRENT_PROGRAM", INT2NUM(GL_CURRENT_PROGRAM));
	rb_define_const(module, "GL_CURRENT_QUERY", INT2NUM(GL_CURRENT_QUERY));
	rb_define_const(module, "GL_CURRENT_QUERY_ARB", INT2NUM(GL_CURRENT_QUERY_ARB));
	rb_define_const(module, "GL_CURRENT_RASTER_COLOR", INT2NUM(GL_CURRENT_RASTER_COLOR));
	rb_define_const(module, "GL_CURRENT_RASTER_DISTANCE", INT2NUM(GL_CURRENT_RASTER_DISTANCE));
	rb_define_const(module, "GL_CURRENT_RASTER_INDEX", INT2NUM(GL_CURRENT_RASTER_INDEX));
	rb_define_const(module, "GL_CURRENT_RASTER_NORMAL_SGIX", INT2NUM(GL_CURRENT_RASTER_NORMAL_SGIX));
	rb_define_const(module, "GL_CURRENT_RASTER_POSITION", INT2NUM(GL_CURRENT_RASTER_POSITION));
	rb_define_const(module, "GL_CURRENT_RASTER_POSITION_VALID", INT2NUM(GL_CURRENT_RASTER_POSITION_VALID));
	rb_define_const(module, "GL_CURRENT_RASTER_SECONDARY_COLOR", INT2NUM(GL_CURRENT_RASTER_SECONDARY_COLOR));
	rb_define_const(module, "GL_CURRENT_RASTER_TEXTURE_COORDS", INT2NUM(GL_CURRENT_RASTER_TEXTURE_COORDS));
	rb_define_const(module, "GL_CURRENT_SECONDARY_COLOR", INT2NUM(GL_CURRENT_SECONDARY_COLOR));
	rb_define_const(module, "GL_CURRENT_SECONDARY_COLOR_EXT", INT2NUM(GL_CURRENT_SECONDARY_COLOR_EXT));
	rb_define_const(module, "GL_CURRENT_TANGENT_EXT", INT2NUM(GL_CURRENT_TANGENT_EXT));
	rb_define_const(module, "GL_CURRENT_TEXTURE_COORDS", INT2NUM(GL_CURRENT_TEXTURE_COORDS));
	rb_define_const(module, "GL_CURRENT_TIME_NV", INT2NUM(GL_CURRENT_TIME_NV));
	rb_define_const(module, "GL_CURRENT_VERTEX_ATTRIB", INT2NUM(GL_CURRENT_VERTEX_ATTRIB));
	rb_define_const(module, "GL_CURRENT_VERTEX_ATTRIB_ARB", INT2NUM(GL_CURRENT_VERTEX_ATTRIB_ARB));
	rb_define_const(module, "GL_CURRENT_VERTEX_EXT", INT2NUM(GL_CURRENT_VERTEX_EXT));
	rb_define_const(module, "GL_CURRENT_VERTEX_WEIGHT_EXT", INT2NUM(GL_CURRENT_VERTEX_WEIGHT_EXT));
	rb_define_const(module, "GL_CURRENT_WEIGHT_ARB", INT2NUM(GL_CURRENT_WEIGHT_ARB));
	rb_define_const(module, "GL_CW", INT2NUM(GL_CW));
	rb_define_const(module, "GL_DECAL", INT2NUM(GL_DECAL));
	rb_define_const(module, "GL_DECR", INT2NUM(GL_DECR));
	rb_define_const(module, "GL_DECR_WRAP", INT2NUM(GL_DECR_WRAP));
	rb_define_const(module, "GL_DECR_WRAP_EXT", INT2NUM(GL_DECR_WRAP_EXT));
	rb_define_const(module, "GL_DEFORMATIONS_MASK_SGIX", INT2NUM(GL_DEFORMATIONS_MASK_SGIX));
	rb_define_const(module, "GL_DELETE_STATUS", INT2NUM(GL_DELETE_STATUS));
	rb_define_const(module, "GL_DEPENDENT_AR_TEXTURE_2D_NV", INT2NUM(GL_DEPENDENT_AR_TEXTURE_2D_NV));
	rb_define_const(module, "GL_DEPENDENT_GB_TEXTURE_2D_NV", INT2NUM(GL_DEPENDENT_GB_TEXTURE_2D_NV));
	rb_define_const(module, "GL_DEPENDENT_HILO_TEXTURE_2D_NV", INT2NUM(GL_DEPENDENT_HILO_TEXTURE_2D_NV));
	rb_define_const(module, "GL_DEPENDENT_RGB_TEXTURE_3D_NV", INT2NUM(GL_DEPENDENT_RGB_TEXTURE_3D_NV));
	rb_define_const(module, "GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV", INT2NUM(GL_DEPENDENT_RGB_TEXTURE_CUBE_MAP_NV));
	rb_define_const(module, "GL_DEPTH", INT2NUM(GL_DEPTH));
	rb_define_const(module, "GL_DEPTH24_STENCIL8", INT2NUM(GL_DEPTH24_STENCIL8));
	rb_define_const(module, "GL_DEPTH24_STENCIL8_EXT", INT2NUM(GL_DEPTH24_STENCIL8_EXT));
	rb_define_const(module, "GL_DEPTH32F_STENCIL8", INT2NUM(GL_DEPTH32F_STENCIL8));
	rb_define_const(module, "GL_DEPTH32F_STENCIL8_NV", INT2NUM(GL_DEPTH32F_STENCIL8_NV));
	rb_define_const(module, "GL_DEPTH_ATTACHMENT", INT2NUM(GL_DEPTH_ATTACHMENT));
	rb_define_const(module, "GL_DEPTH_ATTACHMENT_EXT", INT2NUM(GL_DEPTH_ATTACHMENT_EXT));
	rb_define_const(module, "GL_DEPTH_BIAS", INT2NUM(GL_DEPTH_BIAS));
	rb_define_const(module, "GL_DEPTH_BITS", INT2NUM(GL_DEPTH_BITS));
	rb_define_const(module, "GL_DEPTH_BOUNDS_EXT", INT2NUM(GL_DEPTH_BOUNDS_EXT));
	rb_define_const(module, "GL_DEPTH_BOUNDS_TEST_EXT", INT2NUM(GL_DEPTH_BOUNDS_TEST_EXT));
	rb_define_const(module, "GL_DEPTH_BUFFER", INT2NUM(GL_DEPTH_BUFFER));
	rb_define_const(module, "GL_DEPTH_BUFFER_BIT", INT2NUM(GL_DEPTH_BUFFER_BIT));
	rb_define_const(module, "GL_DEPTH_BUFFER_FLOAT_MODE_NV", INT2NUM(GL_DEPTH_BUFFER_FLOAT_MODE_NV));
	rb_define_const(module, "GL_DEPTH_CLAMP_NV", INT2NUM(GL_DEPTH_CLAMP_NV));
	rb_define_const(module, "GL_DEPTH_CLEAR_VALUE", INT2NUM(GL_DEPTH_CLEAR_VALUE));
	rb_define_const(module, "GL_DEPTH_COMPONENT", INT2NUM(GL_DEPTH_COMPONENT));
	rb_define_const(module, "GL_DEPTH_COMPONENT16", INT2NUM(GL_DEPTH_COMPONENT16));
	rb_define_const(module, "GL_DEPTH_COMPONENT16_ARB", INT2NUM(GL_DEPTH_COMPONENT16_ARB));
	rb_define_const(module, "GL_DEPTH_COMPONENT16_SGIX", INT2NUM(GL_DEPTH_COMPONENT16_SGIX));
	rb_define_const(module, "GL_DEPTH_COMPONENT24", INT2NUM(GL_DEPTH_COMPONENT24));
	rb_define_const(module, "GL_DEPTH_COMPONENT24_ARB", INT2NUM(GL_DEPTH_COMPONENT24_ARB));
	rb_define_const(module, "GL_DEPTH_COMPONENT24_SGIX", INT2NUM(GL_DEPTH_COMPONENT24_SGIX));
	rb_define_const(module, "GL_DEPTH_COMPONENT32", INT2NUM(GL_DEPTH_COMPONENT32));
	rb_define_const(module, "GL_DEPTH_COMPONENT32F", INT2NUM(GL_DEPTH_COMPONENT32F));
	rb_define_const(module, "GL_DEPTH_COMPONENT32F_NV", INT2NUM(GL_DEPTH_COMPONENT32F_NV));
	rb_define_const(module, "GL_DEPTH_COMPONENT32_ARB", INT2NUM(GL_DEPTH_COMPONENT32_ARB));
	rb_define_const(module, "GL_DEPTH_COMPONENT32_SGIX", INT2NUM(GL_DEPTH_COMPONENT32_SGIX));
	rb_define_const(module, "GL_DEPTH_FUNC", INT2NUM(GL_DEPTH_FUNC));
	rb_define_const(module, "GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX", INT2NUM(GL_DEPTH_PASS_INSTRUMENT_COUNTERS_SGIX));
	rb_define_const(module, "GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX", INT2NUM(GL_DEPTH_PASS_INSTRUMENT_MAX_SGIX));
	rb_define_const(module, "GL_DEPTH_PASS_INSTRUMENT_SGIX", INT2NUM(GL_DEPTH_PASS_INSTRUMENT_SGIX));
	rb_define_const(module, "GL_DEPTH_RANGE", INT2NUM(GL_DEPTH_RANGE));
	rb_define_const(module, "GL_DEPTH_SCALE", INT2NUM(GL_DEPTH_SCALE));
	rb_define_const(module, "GL_DEPTH_STENCIL", INT2NUM(GL_DEPTH_STENCIL));
	rb_define_const(module, "GL_DEPTH_STENCIL_ATTACHMENT", INT2NUM(GL_DEPTH_STENCIL_ATTACHMENT));
	rb_define_const(module, "GL_DEPTH_STENCIL_EXT", INT2NUM(GL_DEPTH_STENCIL_EXT));
	rb_define_const(module, "GL_DEPTH_STENCIL_NV", INT2NUM(GL_DEPTH_STENCIL_NV));
	rb_define_const(module, "GL_DEPTH_STENCIL_TO_BGRA_NV", INT2NUM(GL_DEPTH_STENCIL_TO_BGRA_NV));
	rb_define_const(module, "GL_DEPTH_STENCIL_TO_RGBA_NV", INT2NUM(GL_DEPTH_STENCIL_TO_RGBA_NV));
	rb_define_const(module, "GL_DEPTH_TEST", INT2NUM(GL_DEPTH_TEST));
	rb_define_const(module, "GL_DEPTH_TEXTURE_MODE", INT2NUM(GL_DEPTH_TEXTURE_MODE));
	rb_define_const(module, "GL_DEPTH_TEXTURE_MODE_ARB", INT2NUM(GL_DEPTH_TEXTURE_MODE_ARB));
	rb_define_const(module, "GL_DEPTH_WRITEMASK", INT2NUM(GL_DEPTH_WRITEMASK));
	rb_define_const(module, "GL_DETAIL_TEXTURE_2D_BINDING_SGIS", INT2NUM(GL_DETAIL_TEXTURE_2D_BINDING_SGIS));
	rb_define_const(module, "GL_DETAIL_TEXTURE_2D_SGIS", INT2NUM(GL_DETAIL_TEXTURE_2D_SGIS));
	rb_define_const(module, "GL_DETAIL_TEXTURE_FUNC_POINTS_SGIS", INT2NUM(GL_DETAIL_TEXTURE_FUNC_POINTS_SGIS));
	rb_define_const(module, "GL_DETAIL_TEXTURE_LEVEL_SGIS", INT2NUM(GL_DETAIL_TEXTURE_LEVEL_SGIS));
	rb_define_const(module, "GL_DETAIL_TEXTURE_MODE_SGIS", INT2NUM(GL_DETAIL_TEXTURE_MODE_SGIS));
	rb_define_const(module, "GL_DIFFUSE", INT2NUM(GL_DIFFUSE));
	rb_define_const(module, "GL_DISCARD_ATI", INT2NUM(GL_DISCARD_ATI));
	rb_define_const(module, "GL_DISCARD_NV", INT2NUM(GL_DISCARD_NV));
	rb_define_const(module, "GL_DISTANCE_ATTENUATION_EXT", INT2NUM(GL_DISTANCE_ATTENUATION_EXT));
	rb_define_const(module, "GL_DISTANCE_ATTENUATION_SGIS", INT2NUM(GL_DISTANCE_ATTENUATION_SGIS));
	rb_define_const(module, "GL_DITHER", INT2NUM(GL_DITHER));
	rb_define_const(module, "GL_DOMAIN", INT2NUM(GL_DOMAIN));
	rb_define_const(module, "GL_DONT_CARE", INT2NUM(GL_DONT_CARE));
	rb_define_const(module, "GL_DOT2_ADD_ATI", INT2NUM(GL_DOT2_ADD_ATI));
	rb_define_const(module, "GL_DOT3_ATI", INT2NUM(GL_DOT3_ATI));
	rb_define_const(module, "GL_DOT3_RGB", INT2NUM(GL_DOT3_RGB));
	rb_define_const(module, "GL_DOT3_RGBA", INT2NUM(GL_DOT3_RGBA));
	rb_define_const(module, "GL_DOT3_RGBA_ARB", INT2NUM(GL_DOT3_RGBA_ARB));
	rb_define_const(module, "GL_DOT3_RGBA_EXT", INT2NUM(GL_DOT3_RGBA_EXT));
	rb_define_const(module, "GL_DOT3_RGB_ARB", INT2NUM(GL_DOT3_RGB_ARB));
	rb_define_const(module, "GL_DOT3_RGB_EXT", INT2NUM(GL_DOT3_RGB_EXT));
	rb_define_const(module, "GL_DOT4_ATI", INT2NUM(GL_DOT4_ATI));
	rb_define_const(module, "GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV", INT2NUM(GL_DOT_PRODUCT_AFFINE_DEPTH_REPLACE_NV));
	rb_define_const(module, "GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV", INT2NUM(GL_DOT_PRODUCT_CONST_EYE_REFLECT_CUBE_MAP_NV));
	rb_define_const(module, "GL_DOT_PRODUCT_DEPTH_REPLACE_NV", INT2NUM(GL_DOT_PRODUCT_DEPTH_REPLACE_NV));
	rb_define_const(module, "GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV", INT2NUM(GL_DOT_PRODUCT_DIFFUSE_CUBE_MAP_NV));
	rb_define_const(module, "GL_DOT_PRODUCT_NV", INT2NUM(GL_DOT_PRODUCT_NV));
	rb_define_const(module, "GL_DOT_PRODUCT_PASS_THROUGH_NV", INT2NUM(GL_DOT_PRODUCT_PASS_THROUGH_NV));
	rb_define_const(module, "GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV", INT2NUM(GL_DOT_PRODUCT_REFLECT_CUBE_MAP_NV));
	rb_define_const(module, "GL_DOT_PRODUCT_TEXTURE_1D_NV", INT2NUM(GL_DOT_PRODUCT_TEXTURE_1D_NV));
	rb_define_const(module, "GL_DOT_PRODUCT_TEXTURE_2D_NV", INT2NUM(GL_DOT_PRODUCT_TEXTURE_2D_NV));
	rb_define_const(module, "GL_DOT_PRODUCT_TEXTURE_3D_NV", INT2NUM(GL_DOT_PRODUCT_TEXTURE_3D_NV));
	rb_define_const(module, "GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV", INT2NUM(GL_DOT_PRODUCT_TEXTURE_CUBE_MAP_NV));
	rb_define_const(module, "GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV", INT2NUM(GL_DOT_PRODUCT_TEXTURE_RECTANGLE_NV));
	rb_define_const(module, "GL_DOUBLE", INT2NUM(GL_DOUBLE));
	rb_define_const(module, "GL_DOUBLEBUFFER", INT2NUM(GL_DOUBLEBUFFER));
	rb_define_const(module, "GL_DOUBLE_EXT", INT2NUM(GL_DOUBLE_EXT));
	rb_define_const(module, "GL_DRAW_BUFFER", INT2NUM(GL_DRAW_BUFFER));
	rb_define_const(module, "GL_DRAW_BUFFER0", INT2NUM(GL_DRAW_BUFFER0));
	rb_define_const(module, "GL_DRAW_BUFFER0_ARB", INT2NUM(GL_DRAW_BUFFER0_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER0_ATI", INT2NUM(GL_DRAW_BUFFER0_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER1", INT2NUM(GL_DRAW_BUFFER1));
	rb_define_const(module, "GL_DRAW_BUFFER10", INT2NUM(GL_DRAW_BUFFER10));
	rb_define_const(module, "GL_DRAW_BUFFER10_ARB", INT2NUM(GL_DRAW_BUFFER10_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER10_ATI", INT2NUM(GL_DRAW_BUFFER10_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER11", INT2NUM(GL_DRAW_BUFFER11));
	rb_define_const(module, "GL_DRAW_BUFFER11_ARB", INT2NUM(GL_DRAW_BUFFER11_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER11_ATI", INT2NUM(GL_DRAW_BUFFER11_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER12", INT2NUM(GL_DRAW_BUFFER12));
	rb_define_const(module, "GL_DRAW_BUFFER12_ARB", INT2NUM(GL_DRAW_BUFFER12_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER12_ATI", INT2NUM(GL_DRAW_BUFFER12_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER13", INT2NUM(GL_DRAW_BUFFER13));
	rb_define_const(module, "GL_DRAW_BUFFER13_ARB", INT2NUM(GL_DRAW_BUFFER13_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER13_ATI", INT2NUM(GL_DRAW_BUFFER13_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER14", INT2NUM(GL_DRAW_BUFFER14));
	rb_define_const(module, "GL_DRAW_BUFFER14_ARB", INT2NUM(GL_DRAW_BUFFER14_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER14_ATI", INT2NUM(GL_DRAW_BUFFER14_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER15", INT2NUM(GL_DRAW_BUFFER15));
	rb_define_const(module, "GL_DRAW_BUFFER15_ARB", INT2NUM(GL_DRAW_BUFFER15_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER15_ATI", INT2NUM(GL_DRAW_BUFFER15_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER1_ARB", INT2NUM(GL_DRAW_BUFFER1_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER1_ATI", INT2NUM(GL_DRAW_BUFFER1_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER2", INT2NUM(GL_DRAW_BUFFER2));
	rb_define_const(module, "GL_DRAW_BUFFER2_ARB", INT2NUM(GL_DRAW_BUFFER2_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER2_ATI", INT2NUM(GL_DRAW_BUFFER2_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER3", INT2NUM(GL_DRAW_BUFFER3));
	rb_define_const(module, "GL_DRAW_BUFFER3_ARB", INT2NUM(GL_DRAW_BUFFER3_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER3_ATI", INT2NUM(GL_DRAW_BUFFER3_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER4", INT2NUM(GL_DRAW_BUFFER4));
	rb_define_const(module, "GL_DRAW_BUFFER4_ARB", INT2NUM(GL_DRAW_BUFFER4_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER4_ATI", INT2NUM(GL_DRAW_BUFFER4_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER5", INT2NUM(GL_DRAW_BUFFER5));
	rb_define_const(module, "GL_DRAW_BUFFER5_ARB", INT2NUM(GL_DRAW_BUFFER5_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER5_ATI", INT2NUM(GL_DRAW_BUFFER5_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER6", INT2NUM(GL_DRAW_BUFFER6));
	rb_define_const(module, "GL_DRAW_BUFFER6_ARB", INT2NUM(GL_DRAW_BUFFER6_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER6_ATI", INT2NUM(GL_DRAW_BUFFER6_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER7", INT2NUM(GL_DRAW_BUFFER7));
	rb_define_const(module, "GL_DRAW_BUFFER7_ARB", INT2NUM(GL_DRAW_BUFFER7_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER7_ATI", INT2NUM(GL_DRAW_BUFFER7_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER8", INT2NUM(GL_DRAW_BUFFER8));
	rb_define_const(module, "GL_DRAW_BUFFER8_ARB", INT2NUM(GL_DRAW_BUFFER8_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER8_ATI", INT2NUM(GL_DRAW_BUFFER8_ATI));
	rb_define_const(module, "GL_DRAW_BUFFER9", INT2NUM(GL_DRAW_BUFFER9));
	rb_define_const(module, "GL_DRAW_BUFFER9_ARB", INT2NUM(GL_DRAW_BUFFER9_ARB));
	rb_define_const(module, "GL_DRAW_BUFFER9_ATI", INT2NUM(GL_DRAW_BUFFER9_ATI));
	rb_define_const(module, "GL_DRAW_FRAMEBUFFER", INT2NUM(GL_DRAW_FRAMEBUFFER));
	rb_define_const(module, "GL_DRAW_FRAMEBUFFER_BINDING", INT2NUM(GL_DRAW_FRAMEBUFFER_BINDING));
	rb_define_const(module, "GL_DRAW_FRAMEBUFFER_BINDING_EXT", INT2NUM(GL_DRAW_FRAMEBUFFER_BINDING_EXT));
	rb_define_const(module, "GL_DRAW_FRAMEBUFFER_EXT", INT2NUM(GL_DRAW_FRAMEBUFFER_EXT));
	rb_define_const(module, "GL_DRAW_PIXELS_APPLE", INT2NUM(GL_DRAW_PIXELS_APPLE));
	rb_define_const(module, "GL_DRAW_PIXEL_TOKEN", INT2NUM(GL_DRAW_PIXEL_TOKEN));
	rb_define_const(module, "GL_DSDT8_MAG8_INTENSITY8_NV", INT2NUM(GL_DSDT8_MAG8_INTENSITY8_NV));
	rb_define_const(module, "GL_DSDT8_MAG8_NV", INT2NUM(GL_DSDT8_MAG8_NV));
	rb_define_const(module, "GL_DSDT8_NV", INT2NUM(GL_DSDT8_NV));
	rb_define_const(module, "GL_DSDT_MAG_INTENSITY_NV", INT2NUM(GL_DSDT_MAG_INTENSITY_NV));
	rb_define_const(module, "GL_DSDT_MAG_NV", INT2NUM(GL_DSDT_MAG_NV));
	rb_define_const(module, "GL_DSDT_MAG_VIB_NV", INT2NUM(GL_DSDT_MAG_VIB_NV));
	rb_define_const(module, "GL_DSDT_NV", INT2NUM(GL_DSDT_NV));
	rb_define_const(module, "GL_DST_ALPHA", INT2NUM(GL_DST_ALPHA));
	rb_define_const(module, "GL_DST_COLOR", INT2NUM(GL_DST_COLOR));
	rb_define_const(module, "GL_DS_BIAS_NV", INT2NUM(GL_DS_BIAS_NV));
	rb_define_const(module, "GL_DS_SCALE_NV", INT2NUM(GL_DS_SCALE_NV));
	rb_define_const(module, "GL_DT_BIAS_NV", INT2NUM(GL_DT_BIAS_NV));
	rb_define_const(module, "GL_DT_SCALE_NV", INT2NUM(GL_DT_SCALE_NV));
	rb_define_const(module, "GL_DU8DV8_ATI", INT2NUM(GL_DU8DV8_ATI));
	rb_define_const(module, "GL_DUAL_ALPHA12_SGIS", INT2NUM(GL_DUAL_ALPHA12_SGIS));
	rb_define_const(module, "GL_DUAL_ALPHA16_SGIS", INT2NUM(GL_DUAL_ALPHA16_SGIS));
	rb_define_const(module, "GL_DUAL_ALPHA4_SGIS", INT2NUM(GL_DUAL_ALPHA4_SGIS));
	rb_define_const(module, "GL_DUAL_ALPHA8_SGIS", INT2NUM(GL_DUAL_ALPHA8_SGIS));
	rb_define_const(module, "GL_DUAL_INTENSITY12_SGIS", INT2NUM(GL_DUAL_INTENSITY12_SGIS));
	rb_define_const(module, "GL_DUAL_INTENSITY16_SGIS", INT2NUM(GL_DUAL_INTENSITY16_SGIS));
	rb_define_const(module, "GL_DUAL_INTENSITY4_SGIS", INT2NUM(GL_DUAL_INTENSITY4_SGIS));
	rb_define_const(module, "GL_DUAL_INTENSITY8_SGIS", INT2NUM(GL_DUAL_INTENSITY8_SGIS));
	rb_define_const(module, "GL_DUAL_LUMINANCE12_SGIS", INT2NUM(GL_DUAL_LUMINANCE12_SGIS));
	rb_define_const(module, "GL_DUAL_LUMINANCE16_SGIS", INT2NUM(GL_DUAL_LUMINANCE16_SGIS));
	rb_define_const(module, "GL_DUAL_LUMINANCE4_SGIS", INT2NUM(GL_DUAL_LUMINANCE4_SGIS));
	rb_define_const(module, "GL_DUAL_LUMINANCE8_SGIS", INT2NUM(GL_DUAL_LUMINANCE8_SGIS));
	rb_define_const(module, "GL_DUAL_LUMINANCE_ALPHA4_SGIS", INT2NUM(GL_DUAL_LUMINANCE_ALPHA4_SGIS));
	rb_define_const(module, "GL_DUAL_LUMINANCE_ALPHA8_SGIS", INT2NUM(GL_DUAL_LUMINANCE_ALPHA8_SGIS));
	rb_define_const(module, "GL_DUAL_TEXTURE_SELECT_SGIS", INT2NUM(GL_DUAL_TEXTURE_SELECT_SGIS));
	rb_define_const(module, "GL_DUDV_ATI", INT2NUM(GL_DUDV_ATI));
	rb_define_const(module, "GL_DYNAMIC_ATI", INT2NUM(GL_DYNAMIC_ATI));
	rb_define_const(module, "GL_DYNAMIC_COPY", INT2NUM(GL_DYNAMIC_COPY));
	rb_define_const(module, "GL_DYNAMIC_COPY_ARB", INT2NUM(GL_DYNAMIC_COPY_ARB));
	rb_define_const(module, "GL_DYNAMIC_DRAW", INT2NUM(GL_DYNAMIC_DRAW));
	rb_define_const(module, "GL_DYNAMIC_DRAW_ARB", INT2NUM(GL_DYNAMIC_DRAW_ARB));
	rb_define_const(module, "GL_DYNAMIC_READ", INT2NUM(GL_DYNAMIC_READ));
	rb_define_const(module, "GL_DYNAMIC_READ_ARB", INT2NUM(GL_DYNAMIC_READ_ARB));
	rb_define_const(module, "GL_EDGEFLAG_BIT_PGI", INT2NUM(GL_EDGEFLAG_BIT_PGI));
	rb_define_const(module, "GL_EDGE_FLAG", INT2NUM(GL_EDGE_FLAG));
	rb_define_const(module, "GL_EDGE_FLAG_ARRAY", INT2NUM(GL_EDGE_FLAG_ARRAY));
	rb_define_const(module, "GL_EDGE_FLAG_ARRAY_BUFFER_BINDING", INT2NUM(GL_EDGE_FLAG_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_EDGE_FLAG_ARRAY_COUNT_EXT", INT2NUM(GL_EDGE_FLAG_ARRAY_COUNT_EXT));
	rb_define_const(module, "GL_EDGE_FLAG_ARRAY_EXT", INT2NUM(GL_EDGE_FLAG_ARRAY_EXT));
	rb_define_const(module, "GL_EDGE_FLAG_ARRAY_LIST_IBM", INT2NUM(GL_EDGE_FLAG_ARRAY_LIST_IBM));
	rb_define_const(module, "GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM", INT2NUM(GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM));
	rb_define_const(module, "GL_EDGE_FLAG_ARRAY_POINTER", INT2NUM(GL_EDGE_FLAG_ARRAY_POINTER));
	rb_define_const(module, "GL_EDGE_FLAG_ARRAY_POINTER_EXT", INT2NUM(GL_EDGE_FLAG_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_EDGE_FLAG_ARRAY_STRIDE", INT2NUM(GL_EDGE_FLAG_ARRAY_STRIDE));
	rb_define_const(module, "GL_EDGE_FLAG_ARRAY_STRIDE_EXT", INT2NUM(GL_EDGE_FLAG_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_EIGHTH_BIT_ATI", INT2NUM(GL_EIGHTH_BIT_ATI));
	rb_define_const(module, "GL_ELEMENT_ARRAY_APPLE", INT2NUM(GL_ELEMENT_ARRAY_APPLE));
	rb_define_const(module, "GL_ELEMENT_ARRAY_ATI", INT2NUM(GL_ELEMENT_ARRAY_ATI));
	rb_define_const(module, "GL_ELEMENT_ARRAY_BUFFER", INT2NUM(GL_ELEMENT_ARRAY_BUFFER));
	rb_define_const(module, "GL_ELEMENT_ARRAY_BUFFER_ARB", INT2NUM(GL_ELEMENT_ARRAY_BUFFER_ARB));
	rb_define_const(module, "GL_ELEMENT_ARRAY_BUFFER_BINDING", INT2NUM(GL_ELEMENT_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_ELEMENT_ARRAY_POINTER_APPLE", INT2NUM(GL_ELEMENT_ARRAY_POINTER_APPLE));
	rb_define_const(module, "GL_ELEMENT_ARRAY_POINTER_ATI", INT2NUM(GL_ELEMENT_ARRAY_POINTER_ATI));
	rb_define_const(module, "GL_ELEMENT_ARRAY_TYPE_APPLE", INT2NUM(GL_ELEMENT_ARRAY_TYPE_APPLE));
	rb_define_const(module, "GL_ELEMENT_ARRAY_TYPE_ATI", INT2NUM(GL_ELEMENT_ARRAY_TYPE_ATI));
	rb_define_const(module, "GL_EMBOSS_CONSTANT_NV", INT2NUM(GL_EMBOSS_CONSTANT_NV));
	rb_define_const(module, "GL_EMBOSS_LIGHT_NV", INT2NUM(GL_EMBOSS_LIGHT_NV));
	rb_define_const(module, "GL_EMBOSS_MAP_NV", INT2NUM(GL_EMBOSS_MAP_NV));
	rb_define_const(module, "GL_EMISSION", INT2NUM(GL_EMISSION));
	rb_define_const(module, "GL_ENABLE_BIT", INT2NUM(GL_ENABLE_BIT));
	rb_define_const(module, "GL_EQUAL", INT2NUM(GL_EQUAL));
	rb_define_const(module, "GL_EQUIV", INT2NUM(GL_EQUIV));
	rb_define_const(module, "GL_EVAL_2D_NV", INT2NUM(GL_EVAL_2D_NV));
	rb_define_const(module, "GL_EVAL_BIT", INT2NUM(GL_EVAL_BIT));
	rb_define_const(module, "GL_EVAL_FRACTIONAL_TESSELLATION_NV", INT2NUM(GL_EVAL_FRACTIONAL_TESSELLATION_NV));
	rb_define_const(module, "GL_EVAL_TRIANGULAR_2D_NV", INT2NUM(GL_EVAL_TRIANGULAR_2D_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB0_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB0_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB10_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB10_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB11_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB11_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB12_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB12_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB13_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB13_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB14_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB14_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB15_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB15_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB1_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB1_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB2_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB2_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB3_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB3_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB4_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB4_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB5_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB5_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB6_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB6_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB7_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB7_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB8_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB8_NV));
	rb_define_const(module, "GL_EVAL_VERTEX_ATTRIB9_NV", INT2NUM(GL_EVAL_VERTEX_ATTRIB9_NV));
	rb_define_const(module, "GL_EXP", INT2NUM(GL_EXP));
	rb_define_const(module, "GL_EXP2", INT2NUM(GL_EXP2));
	rb_define_const(module, "GL_EXPAND_NEGATE_NV", INT2NUM(GL_EXPAND_NEGATE_NV));
	rb_define_const(module, "GL_EXPAND_NORMAL_NV", INT2NUM(GL_EXPAND_NORMAL_NV));
	rb_define_const(module, "GL_EXTENSIONS", INT2NUM(GL_EXTENSIONS));
	rb_define_const(module, "GL_EXT_abgr", INT2NUM(GL_EXT_abgr));
	rb_define_const(module, "GL_EXT_blend_color", INT2NUM(GL_EXT_blend_color));
	rb_define_const(module, "GL_EXT_blend_logic_op", INT2NUM(GL_EXT_blend_logic_op));
	rb_define_const(module, "GL_EXT_blend_minmax", INT2NUM(GL_EXT_blend_minmax));
	rb_define_const(module, "GL_EXT_blend_subtract", INT2NUM(GL_EXT_blend_subtract));
	rb_define_const(module, "GL_EXT_cmyka", INT2NUM(GL_EXT_cmyka));
	rb_define_const(module, "GL_EXT_convolution", INT2NUM(GL_EXT_convolution));
	rb_define_const(module, "GL_EXT_copy_texture", INT2NUM(GL_EXT_copy_texture));
	rb_define_const(module, "GL_EXT_histogram", INT2NUM(GL_EXT_histogram));
	rb_define_const(module, "GL_EXT_packed_pixels", INT2NUM(GL_EXT_packed_pixels));
	rb_define_const(module, "GL_EXT_point_parameters", INT2NUM(GL_EXT_point_parameters));
	rb_define_const(module, "GL_EXT_polygon_offset", INT2NUM(GL_EXT_polygon_offset));
	rb_define_const(module, "GL_EXT_rescale_normal", INT2NUM(GL_EXT_rescale_normal));
	rb_define_const(module, "GL_EXT_shared_texture_palette", INT2NUM(GL_EXT_shared_texture_palette));
	rb_define_const(module, "GL_EXT_subtexture", INT2NUM(GL_EXT_subtexture));
	rb_define_const(module, "GL_EXT_texture", INT2NUM(GL_EXT_texture));
	rb_define_const(module, "GL_EXT_texture3D", INT2NUM(GL_EXT_texture3D));
	rb_define_const(module, "GL_EXT_texture_object", INT2NUM(GL_EXT_texture_object));
	rb_define_const(module, "GL_EXT_vertex_array", INT2NUM(GL_EXT_vertex_array));
	rb_define_const(module, "GL_EYE_DISTANCE_TO_LINE_SGIS", INT2NUM(GL_EYE_DISTANCE_TO_LINE_SGIS));
	rb_define_const(module, "GL_EYE_DISTANCE_TO_POINT_SGIS", INT2NUM(GL_EYE_DISTANCE_TO_POINT_SGIS));
	rb_define_const(module, "GL_EYE_LINEAR", INT2NUM(GL_EYE_LINEAR));
	rb_define_const(module, "GL_EYE_LINE_SGIS", INT2NUM(GL_EYE_LINE_SGIS));
	rb_define_const(module, "GL_EYE_PLANE", INT2NUM(GL_EYE_PLANE));
	rb_define_const(module, "GL_EYE_PLANE_ABSOLUTE_NV", INT2NUM(GL_EYE_PLANE_ABSOLUTE_NV));
	rb_define_const(module, "GL_EYE_POINT_SGIS", INT2NUM(GL_EYE_POINT_SGIS));
	rb_define_const(module, "GL_EYE_RADIAL_NV", INT2NUM(GL_EYE_RADIAL_NV));
	rb_define_const(module, "GL_E_TIMES_F_NV", INT2NUM(GL_E_TIMES_F_NV));
	rb_define_const(module, "GL_FASTEST", INT2NUM(GL_FASTEST));
	rb_define_const(module, "GL_FEEDBACK", INT2NUM(GL_FEEDBACK));
	rb_define_const(module, "GL_FEEDBACK_BUFFER_POINTER", INT2NUM(GL_FEEDBACK_BUFFER_POINTER));
	rb_define_const(module, "GL_FEEDBACK_BUFFER_SIZE", INT2NUM(GL_FEEDBACK_BUFFER_SIZE));
	rb_define_const(module, "GL_FEEDBACK_BUFFER_TYPE", INT2NUM(GL_FEEDBACK_BUFFER_TYPE));
	rb_define_const(module, "GL_FENCE_APPLE", INT2NUM(GL_FENCE_APPLE));
	rb_define_const(module, "GL_FENCE_CONDITION_NV", INT2NUM(GL_FENCE_CONDITION_NV));
	rb_define_const(module, "GL_FENCE_STATUS_NV", INT2NUM(GL_FENCE_STATUS_NV));
	rb_define_const(module, "GL_FIELDS_NV", INT2NUM(GL_FIELDS_NV));
	rb_define_const(module, "GL_FILL", INT2NUM(GL_FILL));
	rb_define_const(module, "GL_FILTER4_SGIS", INT2NUM(GL_FILTER4_SGIS));
	rb_define_const(module, "GL_FIXED_ONLY", INT2NUM(GL_FIXED_ONLY));
	rb_define_const(module, "GL_FIXED_ONLY_ARB", INT2NUM(GL_FIXED_ONLY_ARB));
	rb_define_const(module, "GL_FLAT", INT2NUM(GL_FLAT));
	rb_define_const(module, "GL_FLOAT", INT2NUM(GL_FLOAT));
	rb_define_const(module, "GL_FLOAT_32_UNSIGNED_INT_24_8_REV", INT2NUM(GL_FLOAT_32_UNSIGNED_INT_24_8_REV));
	rb_define_const(module, "GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV", INT2NUM(GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV));
	rb_define_const(module, "GL_FLOAT_CLEAR_COLOR_VALUE_NV", INT2NUM(GL_FLOAT_CLEAR_COLOR_VALUE_NV));
	rb_define_const(module, "GL_FLOAT_MAT2", INT2NUM(GL_FLOAT_MAT2));
	rb_define_const(module, "GL_FLOAT_MAT2_ARB", INT2NUM(GL_FLOAT_MAT2_ARB));
	rb_define_const(module, "GL_FLOAT_MAT2x3", INT2NUM(GL_FLOAT_MAT2x3));
	rb_define_const(module, "GL_FLOAT_MAT2x4", INT2NUM(GL_FLOAT_MAT2x4));
	rb_define_const(module, "GL_FLOAT_MAT3", INT2NUM(GL_FLOAT_MAT3));
	rb_define_const(module, "GL_FLOAT_MAT3_ARB", INT2NUM(GL_FLOAT_MAT3_ARB));
	rb_define_const(module, "GL_FLOAT_MAT3x2", INT2NUM(GL_FLOAT_MAT3x2));
	rb_define_const(module, "GL_FLOAT_MAT3x4", INT2NUM(GL_FLOAT_MAT3x4));
	rb_define_const(module, "GL_FLOAT_MAT4", INT2NUM(GL_FLOAT_MAT4));
	rb_define_const(module, "GL_FLOAT_MAT4_ARB", INT2NUM(GL_FLOAT_MAT4_ARB));
	rb_define_const(module, "GL_FLOAT_MAT4x2", INT2NUM(GL_FLOAT_MAT4x2));
	rb_define_const(module, "GL_FLOAT_MAT4x3", INT2NUM(GL_FLOAT_MAT4x3));
	rb_define_const(module, "GL_FLOAT_R16_NV", INT2NUM(GL_FLOAT_R16_NV));
	rb_define_const(module, "GL_FLOAT_R32_NV", INT2NUM(GL_FLOAT_R32_NV));
	rb_define_const(module, "GL_FLOAT_RG16_NV", INT2NUM(GL_FLOAT_RG16_NV));
	rb_define_const(module, "GL_FLOAT_RG32_NV", INT2NUM(GL_FLOAT_RG32_NV));
	rb_define_const(module, "GL_FLOAT_RGB16_NV", INT2NUM(GL_FLOAT_RGB16_NV));
	rb_define_const(module, "GL_FLOAT_RGB32_NV", INT2NUM(GL_FLOAT_RGB32_NV));
	rb_define_const(module, "GL_FLOAT_RGBA16_NV", INT2NUM(GL_FLOAT_RGBA16_NV));
	rb_define_const(module, "GL_FLOAT_RGBA32_NV", INT2NUM(GL_FLOAT_RGBA32_NV));
	rb_define_const(module, "GL_FLOAT_RGBA_MODE_NV", INT2NUM(GL_FLOAT_RGBA_MODE_NV));
	rb_define_const(module, "GL_FLOAT_RGBA_NV", INT2NUM(GL_FLOAT_RGBA_NV));
	rb_define_const(module, "GL_FLOAT_RGB_NV", INT2NUM(GL_FLOAT_RGB_NV));
	rb_define_const(module, "GL_FLOAT_RG_NV", INT2NUM(GL_FLOAT_RG_NV));
	rb_define_const(module, "GL_FLOAT_R_NV", INT2NUM(GL_FLOAT_R_NV));
	rb_define_const(module, "GL_FLOAT_VEC2", INT2NUM(GL_FLOAT_VEC2));
	rb_define_const(module, "GL_FLOAT_VEC2_ARB", INT2NUM(GL_FLOAT_VEC2_ARB));
	rb_define_const(module, "GL_FLOAT_VEC3", INT2NUM(GL_FLOAT_VEC3));
	rb_define_const(module, "GL_FLOAT_VEC3_ARB", INT2NUM(GL_FLOAT_VEC3_ARB));
	rb_define_const(module, "GL_FLOAT_VEC4", INT2NUM(GL_FLOAT_VEC4));
	rb_define_const(module, "GL_FLOAT_VEC4_ARB", INT2NUM(GL_FLOAT_VEC4_ARB));
	rb_define_const(module, "GL_FOG", INT2NUM(GL_FOG));
	rb_define_const(module, "GL_FOG_BIT", INT2NUM(GL_FOG_BIT));
	rb_define_const(module, "GL_FOG_COLOR", INT2NUM(GL_FOG_COLOR));
	rb_define_const(module, "GL_FOG_COORD", INT2NUM(GL_FOG_COORD));
	rb_define_const(module, "GL_FOG_COORDINATE", INT2NUM(GL_FOG_COORDINATE));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY", INT2NUM(GL_FOG_COORDINATE_ARRAY));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING", INT2NUM(GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY_EXT", INT2NUM(GL_FOG_COORDINATE_ARRAY_EXT));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY_LIST_IBM", INT2NUM(GL_FOG_COORDINATE_ARRAY_LIST_IBM));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM", INT2NUM(GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY_POINTER", INT2NUM(GL_FOG_COORDINATE_ARRAY_POINTER));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY_POINTER_EXT", INT2NUM(GL_FOG_COORDINATE_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY_STRIDE", INT2NUM(GL_FOG_COORDINATE_ARRAY_STRIDE));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY_STRIDE_EXT", INT2NUM(GL_FOG_COORDINATE_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY_TYPE", INT2NUM(GL_FOG_COORDINATE_ARRAY_TYPE));
	rb_define_const(module, "GL_FOG_COORDINATE_ARRAY_TYPE_EXT", INT2NUM(GL_FOG_COORDINATE_ARRAY_TYPE_EXT));
	rb_define_const(module, "GL_FOG_COORDINATE_EXT", INT2NUM(GL_FOG_COORDINATE_EXT));
	rb_define_const(module, "GL_FOG_COORDINATE_SOURCE", INT2NUM(GL_FOG_COORDINATE_SOURCE));
	rb_define_const(module, "GL_FOG_COORDINATE_SOURCE_EXT", INT2NUM(GL_FOG_COORDINATE_SOURCE_EXT));
	rb_define_const(module, "GL_FOG_COORD_ARRAY", INT2NUM(GL_FOG_COORD_ARRAY));
	rb_define_const(module, "GL_FOG_COORD_ARRAY_BUFFER_BINDING", INT2NUM(GL_FOG_COORD_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_FOG_COORD_ARRAY_POINTER", INT2NUM(GL_FOG_COORD_ARRAY_POINTER));
	rb_define_const(module, "GL_FOG_COORD_ARRAY_STRIDE", INT2NUM(GL_FOG_COORD_ARRAY_STRIDE));
	rb_define_const(module, "GL_FOG_COORD_ARRAY_TYPE", INT2NUM(GL_FOG_COORD_ARRAY_TYPE));
	rb_define_const(module, "GL_FOG_COORD_SRC", INT2NUM(GL_FOG_COORD_SRC));
	rb_define_const(module, "GL_FOG_DENSITY", INT2NUM(GL_FOG_DENSITY));
	rb_define_const(module, "GL_FOG_DISTANCE_MODE_NV", INT2NUM(GL_FOG_DISTANCE_MODE_NV));
	rb_define_const(module, "GL_FOG_END", INT2NUM(GL_FOG_END));
	rb_define_const(module, "GL_FOG_FACTOR_TO_ALPHA_SGIX", INT2NUM(GL_FOG_FACTOR_TO_ALPHA_SGIX));
	rb_define_const(module, "GL_FOG_FUNC_POINTS_SGIS", INT2NUM(GL_FOG_FUNC_POINTS_SGIS));
	rb_define_const(module, "GL_FOG_FUNC_SGIS", INT2NUM(GL_FOG_FUNC_SGIS));
	rb_define_const(module, "GL_FOG_HINT", INT2NUM(GL_FOG_HINT));
	rb_define_const(module, "GL_FOG_INDEX", INT2NUM(GL_FOG_INDEX));
	rb_define_const(module, "GL_FOG_MODE", INT2NUM(GL_FOG_MODE));
	rb_define_const(module, "GL_FOG_OFFSET_SGIX", INT2NUM(GL_FOG_OFFSET_SGIX));
	rb_define_const(module, "GL_FOG_OFFSET_VALUE_SGIX", INT2NUM(GL_FOG_OFFSET_VALUE_SGIX));
	rb_define_const(module, "GL_FOG_SCALE_SGIX", INT2NUM(GL_FOG_SCALE_SGIX));
	rb_define_const(module, "GL_FOG_SCALE_VALUE_SGIX", INT2NUM(GL_FOG_SCALE_VALUE_SGIX));
	rb_define_const(module, "GL_FOG_SPECULAR_TEXTURE_WIN", INT2NUM(GL_FOG_SPECULAR_TEXTURE_WIN));
	rb_define_const(module, "GL_FOG_START", INT2NUM(GL_FOG_START));
	rb_define_const(module, "GL_FORCE_BLUE_TO_ONE_NV", INT2NUM(GL_FORCE_BLUE_TO_ONE_NV));
	rb_define_const(module, "GL_FORMAT_SUBSAMPLE_244_244_OML", INT2NUM(GL_FORMAT_SUBSAMPLE_244_244_OML));
	rb_define_const(module, "GL_FORMAT_SUBSAMPLE_24_24_OML", INT2NUM(GL_FORMAT_SUBSAMPLE_24_24_OML));
	rb_define_const(module, "GL_FRAGMENT_COLOR_EXT", INT2NUM(GL_FRAGMENT_COLOR_EXT));
	rb_define_const(module, "GL_FRAGMENT_COLOR_MATERIAL_FACE_SGIX", INT2NUM(GL_FRAGMENT_COLOR_MATERIAL_FACE_SGIX));
	rb_define_const(module, "GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_SGIX", INT2NUM(GL_FRAGMENT_COLOR_MATERIAL_PARAMETER_SGIX));
	rb_define_const(module, "GL_FRAGMENT_COLOR_MATERIAL_SGIX", INT2NUM(GL_FRAGMENT_COLOR_MATERIAL_SGIX));
	rb_define_const(module, "GL_FRAGMENT_DEPTH", INT2NUM(GL_FRAGMENT_DEPTH));
	rb_define_const(module, "GL_FRAGMENT_DEPTH_EXT", INT2NUM(GL_FRAGMENT_DEPTH_EXT));
	rb_define_const(module, "GL_FRAGMENT_LIGHT0_SGIX", INT2NUM(GL_FRAGMENT_LIGHT0_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHT1_SGIX", INT2NUM(GL_FRAGMENT_LIGHT1_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHT2_SGIX", INT2NUM(GL_FRAGMENT_LIGHT2_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHT3_SGIX", INT2NUM(GL_FRAGMENT_LIGHT3_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHT4_SGIX", INT2NUM(GL_FRAGMENT_LIGHT4_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHT5_SGIX", INT2NUM(GL_FRAGMENT_LIGHT5_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHT6_SGIX", INT2NUM(GL_FRAGMENT_LIGHT6_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHT7_SGIX", INT2NUM(GL_FRAGMENT_LIGHT7_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHTING_SGIX", INT2NUM(GL_FRAGMENT_LIGHTING_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHT_MODEL_AMBIENT_SGIX", INT2NUM(GL_FRAGMENT_LIGHT_MODEL_AMBIENT_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_SGIX", INT2NUM(GL_FRAGMENT_LIGHT_MODEL_LOCAL_VIEWER_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_SGIX", INT2NUM(GL_FRAGMENT_LIGHT_MODEL_NORMAL_INTERPOLATION_SGIX));
	rb_define_const(module, "GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_SGIX", INT2NUM(GL_FRAGMENT_LIGHT_MODEL_TWO_SIDE_SGIX));
	rb_define_const(module, "GL_FRAGMENT_MATERIAL_EXT", INT2NUM(GL_FRAGMENT_MATERIAL_EXT));
	rb_define_const(module, "GL_FRAGMENT_NORMAL_EXT", INT2NUM(GL_FRAGMENT_NORMAL_EXT));
	rb_define_const(module, "GL_FRAGMENT_PROGRAM_ARB", INT2NUM(GL_FRAGMENT_PROGRAM_ARB));
	rb_define_const(module, "GL_FRAGMENT_PROGRAM_BINDING_NV", INT2NUM(GL_FRAGMENT_PROGRAM_BINDING_NV));
	rb_define_const(module, "GL_FRAGMENT_PROGRAM_NV", INT2NUM(GL_FRAGMENT_PROGRAM_NV));
	rb_define_const(module, "GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV", INT2NUM(GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV));
	rb_define_const(module, "GL_FRAGMENT_SHADER", INT2NUM(GL_FRAGMENT_SHADER));
	rb_define_const(module, "GL_FRAGMENT_SHADER_ARB", INT2NUM(GL_FRAGMENT_SHADER_ARB));
	rb_define_const(module, "GL_FRAGMENT_SHADER_ATI", INT2NUM(GL_FRAGMENT_SHADER_ATI));
	rb_define_const(module, "GL_FRAGMENT_SHADER_DERIVATIVE_HINT", INT2NUM(GL_FRAGMENT_SHADER_DERIVATIVE_HINT));
	rb_define_const(module, "GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB", INT2NUM(GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB));
	rb_define_const(module, "GL_FRAMEBUFFER", INT2NUM(GL_FRAMEBUFFER));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_LAYERED_ARB));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL));
	rb_define_const(module, "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT", INT2NUM(GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_BINDING", INT2NUM(GL_FRAMEBUFFER_BINDING));
	rb_define_const(module, "GL_FRAMEBUFFER_BINDING_EXT", INT2NUM(GL_FRAMEBUFFER_BINDING_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_COMPLETE", INT2NUM(GL_FRAMEBUFFER_COMPLETE));
	rb_define_const(module, "GL_FRAMEBUFFER_COMPLETE_EXT", INT2NUM(GL_FRAMEBUFFER_COMPLETE_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_DEFAULT", INT2NUM(GL_FRAMEBUFFER_DEFAULT));
	rb_define_const(module, "GL_FRAMEBUFFER_EXT", INT2NUM(GL_FRAMEBUFFER_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_ARB));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_ARB));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER));
	rb_define_const(module, "GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT", INT2NUM(GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_SRGB", INT2NUM(GL_FRAMEBUFFER_SRGB));
	rb_define_const(module, "GL_FRAMEBUFFER_SRGB_CAPABLE_EXT", INT2NUM(GL_FRAMEBUFFER_SRGB_CAPABLE_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_SRGB_EXT", INT2NUM(GL_FRAMEBUFFER_SRGB_EXT));
	rb_define_const(module, "GL_FRAMEBUFFER_UNDEFINED", INT2NUM(GL_FRAMEBUFFER_UNDEFINED));
	rb_define_const(module, "GL_FRAMEBUFFER_UNSUPPORTED", INT2NUM(GL_FRAMEBUFFER_UNSUPPORTED));
	rb_define_const(module, "GL_FRAMEBUFFER_UNSUPPORTED_EXT", INT2NUM(GL_FRAMEBUFFER_UNSUPPORTED_EXT));
	rb_define_const(module, "GL_FRAMEZOOM_FACTOR_SGIX", INT2NUM(GL_FRAMEZOOM_FACTOR_SGIX));
	rb_define_const(module, "GL_FRAMEZOOM_SGIX", INT2NUM(GL_FRAMEZOOM_SGIX));
	rb_define_const(module, "GL_FRAME_NV", INT2NUM(GL_FRAME_NV));
	rb_define_const(module, "GL_FRONT", INT2NUM(GL_FRONT));
	rb_define_const(module, "GL_FRONT_AND_BACK", INT2NUM(GL_FRONT_AND_BACK));
	rb_define_const(module, "GL_FRONT_FACE", INT2NUM(GL_FRONT_FACE));
	rb_define_const(module, "GL_FRONT_LEFT", INT2NUM(GL_FRONT_LEFT));
	rb_define_const(module, "GL_FRONT_RIGHT", INT2NUM(GL_FRONT_RIGHT));
	rb_define_const(module, "GL_FULL_RANGE_EXT", INT2NUM(GL_FULL_RANGE_EXT));
	rb_define_const(module, "GL_FULL_STIPPLE_HINT_PGI", INT2NUM(GL_FULL_STIPPLE_HINT_PGI));
	rb_define_const(module, "GL_FUNC_ADD", INT2NUM(GL_FUNC_ADD));
	rb_define_const(module, "GL_FUNC_ADD_EXT", INT2NUM(GL_FUNC_ADD_EXT));
	rb_define_const(module, "GL_FUNC_REVERSE_SUBTRACT", INT2NUM(GL_FUNC_REVERSE_SUBTRACT));
	rb_define_const(module, "GL_FUNC_REVERSE_SUBTRACT_EXT", INT2NUM(GL_FUNC_REVERSE_SUBTRACT_EXT));
	rb_define_const(module, "GL_FUNC_SUBTRACT", INT2NUM(GL_FUNC_SUBTRACT));
	rb_define_const(module, "GL_FUNC_SUBTRACT_EXT", INT2NUM(GL_FUNC_SUBTRACT_EXT));
	rb_define_const(module, "GL_GENERATE_MIPMAP", INT2NUM(GL_GENERATE_MIPMAP));
	rb_define_const(module, "GL_GENERATE_MIPMAP_HINT", INT2NUM(GL_GENERATE_MIPMAP_HINT));
	rb_define_const(module, "GL_GENERATE_MIPMAP_HINT_SGIS", INT2NUM(GL_GENERATE_MIPMAP_HINT_SGIS));
	rb_define_const(module, "GL_GENERATE_MIPMAP_SGIS", INT2NUM(GL_GENERATE_MIPMAP_SGIS));
	rb_define_const(module, "GL_GENERIC_ATTRIB_NV", INT2NUM(GL_GENERIC_ATTRIB_NV));
	rb_define_const(module, "GL_GEOMETRY_DEFORMATION_BIT_SGIX", INT2NUM(GL_GEOMETRY_DEFORMATION_BIT_SGIX));
	rb_define_const(module, "GL_GEOMETRY_DEFORMATION_SGIX", INT2NUM(GL_GEOMETRY_DEFORMATION_SGIX));
	rb_define_const(module, "GL_GEOMETRY_INPUT_TYPE_ARB", INT2NUM(GL_GEOMETRY_INPUT_TYPE_ARB));
	rb_define_const(module, "GL_GEOMETRY_INPUT_TYPE_EXT", INT2NUM(GL_GEOMETRY_INPUT_TYPE_EXT));
	rb_define_const(module, "GL_GEOMETRY_OUTPUT_TYPE_ARB", INT2NUM(GL_GEOMETRY_OUTPUT_TYPE_ARB));
	rb_define_const(module, "GL_GEOMETRY_OUTPUT_TYPE_EXT", INT2NUM(GL_GEOMETRY_OUTPUT_TYPE_EXT));
	rb_define_const(module, "GL_GEOMETRY_PROGRAM_NV", INT2NUM(GL_GEOMETRY_PROGRAM_NV));
	rb_define_const(module, "GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV", INT2NUM(GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV));
	rb_define_const(module, "GL_GEOMETRY_SHADER_ARB", INT2NUM(GL_GEOMETRY_SHADER_ARB));
	rb_define_const(module, "GL_GEOMETRY_SHADER_EXT", INT2NUM(GL_GEOMETRY_SHADER_EXT));
	rb_define_const(module, "GL_GEOMETRY_VERTICES_OUT_ARB", INT2NUM(GL_GEOMETRY_VERTICES_OUT_ARB));
	rb_define_const(module, "GL_GEOMETRY_VERTICES_OUT_EXT", INT2NUM(GL_GEOMETRY_VERTICES_OUT_EXT));
	rb_define_const(module, "GL_GEQUAL", INT2NUM(GL_GEQUAL));
	rb_define_const(module, "GL_GLOBAL_ALPHA_FACTOR_SUN", INT2NUM(GL_GLOBAL_ALPHA_FACTOR_SUN));
	rb_define_const(module, "GL_GLOBAL_ALPHA_SUN", INT2NUM(GL_GLOBAL_ALPHA_SUN));
	rb_define_const(module, "GL_GREATER", INT2NUM(GL_GREATER));
	rb_define_const(module, "GL_GREEN", INT2NUM(GL_GREEN));
	rb_define_const(module, "GL_GREEN_BIAS", INT2NUM(GL_GREEN_BIAS));
	rb_define_const(module, "GL_GREEN_BITS", INT2NUM(GL_GREEN_BITS));
	rb_define_const(module, "GL_GREEN_BIT_ATI", INT2NUM(GL_GREEN_BIT_ATI));
	rb_define_const(module, "GL_GREEN_INTEGER", INT2NUM(GL_GREEN_INTEGER));
	rb_define_const(module, "GL_GREEN_INTEGER_EXT", INT2NUM(GL_GREEN_INTEGER_EXT));
	rb_define_const(module, "GL_GREEN_MAX_CLAMP_INGR", INT2NUM(GL_GREEN_MAX_CLAMP_INGR));
	rb_define_const(module, "GL_GREEN_MIN_CLAMP_INGR", INT2NUM(GL_GREEN_MIN_CLAMP_INGR));
	rb_define_const(module, "GL_GREEN_SCALE", INT2NUM(GL_GREEN_SCALE));
	rb_define_const(module, "GL_HALF_BIAS_NEGATE_NV", INT2NUM(GL_HALF_BIAS_NEGATE_NV));
	rb_define_const(module, "GL_HALF_BIAS_NORMAL_NV", INT2NUM(GL_HALF_BIAS_NORMAL_NV));
	rb_define_const(module, "GL_HALF_BIT_ATI", INT2NUM(GL_HALF_BIT_ATI));
	rb_define_const(module, "GL_HALF_FLOAT", INT2NUM(GL_HALF_FLOAT));
	rb_define_const(module, "GL_HALF_FLOAT_ARB", INT2NUM(GL_HALF_FLOAT_ARB));
	rb_define_const(module, "GL_HALF_FLOAT_NV", INT2NUM(GL_HALF_FLOAT_NV));
	rb_define_const(module, "GL_HILO16_NV", INT2NUM(GL_HILO16_NV));
	rb_define_const(module, "GL_HILO8_NV", INT2NUM(GL_HILO8_NV));
	rb_define_const(module, "GL_HILO_NV", INT2NUM(GL_HILO_NV));
	rb_define_const(module, "GL_HINT_BIT", INT2NUM(GL_HINT_BIT));
	rb_define_const(module, "GL_HISTOGRAM", INT2NUM(GL_HISTOGRAM));
	rb_define_const(module, "GL_HISTOGRAM_ALPHA_SIZE", INT2NUM(GL_HISTOGRAM_ALPHA_SIZE));
	rb_define_const(module, "GL_HISTOGRAM_ALPHA_SIZE_EXT", INT2NUM(GL_HISTOGRAM_ALPHA_SIZE_EXT));
	rb_define_const(module, "GL_HISTOGRAM_BLUE_SIZE", INT2NUM(GL_HISTOGRAM_BLUE_SIZE));
	rb_define_const(module, "GL_HISTOGRAM_BLUE_SIZE_EXT", INT2NUM(GL_HISTOGRAM_BLUE_SIZE_EXT));
	rb_define_const(module, "GL_HISTOGRAM_EXT", INT2NUM(GL_HISTOGRAM_EXT));
	rb_define_const(module, "GL_HISTOGRAM_FORMAT", INT2NUM(GL_HISTOGRAM_FORMAT));
	rb_define_const(module, "GL_HISTOGRAM_FORMAT_EXT", INT2NUM(GL_HISTOGRAM_FORMAT_EXT));
	rb_define_const(module, "GL_HISTOGRAM_GREEN_SIZE", INT2NUM(GL_HISTOGRAM_GREEN_SIZE));
	rb_define_const(module, "GL_HISTOGRAM_GREEN_SIZE_EXT", INT2NUM(GL_HISTOGRAM_GREEN_SIZE_EXT));
	rb_define_const(module, "GL_HISTOGRAM_LUMINANCE_SIZE", INT2NUM(GL_HISTOGRAM_LUMINANCE_SIZE));
	rb_define_const(module, "GL_HISTOGRAM_LUMINANCE_SIZE_EXT", INT2NUM(GL_HISTOGRAM_LUMINANCE_SIZE_EXT));
	rb_define_const(module, "GL_HISTOGRAM_RED_SIZE", INT2NUM(GL_HISTOGRAM_RED_SIZE));
	rb_define_const(module, "GL_HISTOGRAM_RED_SIZE_EXT", INT2NUM(GL_HISTOGRAM_RED_SIZE_EXT));
	rb_define_const(module, "GL_HISTOGRAM_SINK", INT2NUM(GL_HISTOGRAM_SINK));
	rb_define_const(module, "GL_HISTOGRAM_SINK_EXT", INT2NUM(GL_HISTOGRAM_SINK_EXT));
	rb_define_const(module, "GL_HISTOGRAM_WIDTH", INT2NUM(GL_HISTOGRAM_WIDTH));
	rb_define_const(module, "GL_HISTOGRAM_WIDTH_EXT", INT2NUM(GL_HISTOGRAM_WIDTH_EXT));
	rb_define_const(module, "GL_HI_BIAS_NV", INT2NUM(GL_HI_BIAS_NV));
	rb_define_const(module, "GL_HI_SCALE_NV", INT2NUM(GL_HI_SCALE_NV));
	rb_define_const(module, "GL_IDENTITY_NV", INT2NUM(GL_IDENTITY_NV));
	rb_define_const(module, "GL_IGNORE_BORDER_HP", INT2NUM(GL_IGNORE_BORDER_HP));
	rb_define_const(module, "GL_IMAGE_CUBIC_WEIGHT_HP", INT2NUM(GL_IMAGE_CUBIC_WEIGHT_HP));
	rb_define_const(module, "GL_IMAGE_MAG_FILTER_HP", INT2NUM(GL_IMAGE_MAG_FILTER_HP));
	rb_define_const(module, "GL_IMAGE_MIN_FILTER_HP", INT2NUM(GL_IMAGE_MIN_FILTER_HP));
	rb_define_const(module, "GL_IMAGE_ROTATE_ANGLE_HP", INT2NUM(GL_IMAGE_ROTATE_ANGLE_HP));
	rb_define_const(module, "GL_IMAGE_ROTATE_ORIGIN_X_HP", INT2NUM(GL_IMAGE_ROTATE_ORIGIN_X_HP));
	rb_define_const(module, "GL_IMAGE_ROTATE_ORIGIN_Y_HP", INT2NUM(GL_IMAGE_ROTATE_ORIGIN_Y_HP));
	rb_define_const(module, "GL_IMAGE_SCALE_X_HP", INT2NUM(GL_IMAGE_SCALE_X_HP));
	rb_define_const(module, "GL_IMAGE_SCALE_Y_HP", INT2NUM(GL_IMAGE_SCALE_Y_HP));
	rb_define_const(module, "GL_IMAGE_TRANSFORM_2D_HP", INT2NUM(GL_IMAGE_TRANSFORM_2D_HP));
	rb_define_const(module, "GL_IMAGE_TRANSLATE_X_HP", INT2NUM(GL_IMAGE_TRANSLATE_X_HP));
	rb_define_const(module, "GL_IMAGE_TRANSLATE_Y_HP", INT2NUM(GL_IMAGE_TRANSLATE_Y_HP));
	rb_define_const(module, "GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES", INT2NUM(GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES));
	rb_define_const(module, "GL_IMPLEMENTATION_COLOR_READ_TYPE_OES", INT2NUM(GL_IMPLEMENTATION_COLOR_READ_TYPE_OES));
	rb_define_const(module, "GL_INCR", INT2NUM(GL_INCR));
	rb_define_const(module, "GL_INCR_WRAP", INT2NUM(GL_INCR_WRAP));
	rb_define_const(module, "GL_INCR_WRAP_EXT", INT2NUM(GL_INCR_WRAP_EXT));
	rb_define_const(module, "GL_INDEX", INT2NUM(GL_INDEX));
	rb_define_const(module, "GL_INDEX_ARRAY", INT2NUM(GL_INDEX_ARRAY));
	rb_define_const(module, "GL_INDEX_ARRAY_BUFFER_BINDING", INT2NUM(GL_INDEX_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_INDEX_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_INDEX_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_INDEX_ARRAY_COUNT_EXT", INT2NUM(GL_INDEX_ARRAY_COUNT_EXT));
	rb_define_const(module, "GL_INDEX_ARRAY_EXT", INT2NUM(GL_INDEX_ARRAY_EXT));
	rb_define_const(module, "GL_INDEX_ARRAY_LIST_IBM", INT2NUM(GL_INDEX_ARRAY_LIST_IBM));
	rb_define_const(module, "GL_INDEX_ARRAY_LIST_STRIDE_IBM", INT2NUM(GL_INDEX_ARRAY_LIST_STRIDE_IBM));
	rb_define_const(module, "GL_INDEX_ARRAY_POINTER", INT2NUM(GL_INDEX_ARRAY_POINTER));
	rb_define_const(module, "GL_INDEX_ARRAY_POINTER_EXT", INT2NUM(GL_INDEX_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_INDEX_ARRAY_STRIDE", INT2NUM(GL_INDEX_ARRAY_STRIDE));
	rb_define_const(module, "GL_INDEX_ARRAY_STRIDE_EXT", INT2NUM(GL_INDEX_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_INDEX_ARRAY_TYPE", INT2NUM(GL_INDEX_ARRAY_TYPE));
	rb_define_const(module, "GL_INDEX_ARRAY_TYPE_EXT", INT2NUM(GL_INDEX_ARRAY_TYPE_EXT));
	rb_define_const(module, "GL_INDEX_BITS", INT2NUM(GL_INDEX_BITS));
	rb_define_const(module, "GL_INDEX_BIT_PGI", INT2NUM(GL_INDEX_BIT_PGI));
	rb_define_const(module, "GL_INDEX_CLEAR_VALUE", INT2NUM(GL_INDEX_CLEAR_VALUE));
	rb_define_const(module, "GL_INDEX_LOGIC_OP", INT2NUM(GL_INDEX_LOGIC_OP));
	rb_define_const(module, "GL_INDEX_MATERIAL_EXT", INT2NUM(GL_INDEX_MATERIAL_EXT));
	rb_define_const(module, "GL_INDEX_MATERIAL_FACE_EXT", INT2NUM(GL_INDEX_MATERIAL_FACE_EXT));
	rb_define_const(module, "GL_INDEX_MATERIAL_PARAMETER_EXT", INT2NUM(GL_INDEX_MATERIAL_PARAMETER_EXT));
	rb_define_const(module, "GL_INDEX_MODE", INT2NUM(GL_INDEX_MODE));
	rb_define_const(module, "GL_INDEX_OFFSET", INT2NUM(GL_INDEX_OFFSET));
	rb_define_const(module, "GL_INDEX_SHIFT", INT2NUM(GL_INDEX_SHIFT));
	rb_define_const(module, "GL_INDEX_TEST_EXT", INT2NUM(GL_INDEX_TEST_EXT));
	rb_define_const(module, "GL_INDEX_TEST_FUNC_EXT", INT2NUM(GL_INDEX_TEST_FUNC_EXT));
	rb_define_const(module, "GL_INDEX_TEST_REF_EXT", INT2NUM(GL_INDEX_TEST_REF_EXT));
	rb_define_const(module, "GL_INDEX_WRITEMASK", INT2NUM(GL_INDEX_WRITEMASK));
	rb_define_const(module, "GL_INFO_LOG_LENGTH", INT2NUM(GL_INFO_LOG_LENGTH));
	rb_define_const(module, "GL_INSTRUMENT_BUFFER_POINTER_SGIX", INT2NUM(GL_INSTRUMENT_BUFFER_POINTER_SGIX));
	rb_define_const(module, "GL_INSTRUMENT_MEASUREMENTS_SGIX", INT2NUM(GL_INSTRUMENT_MEASUREMENTS_SGIX));
	rb_define_const(module, "GL_INT", INT2NUM(GL_INT));
	rb_define_const(module, "GL_INTENSITY", INT2NUM(GL_INTENSITY));
	rb_define_const(module, "GL_INTENSITY12", INT2NUM(GL_INTENSITY12));
	rb_define_const(module, "GL_INTENSITY12_EXT", INT2NUM(GL_INTENSITY12_EXT));
	rb_define_const(module, "GL_INTENSITY16", INT2NUM(GL_INTENSITY16));
	rb_define_const(module, "GL_INTENSITY16F_ARB", INT2NUM(GL_INTENSITY16F_ARB));
	rb_define_const(module, "GL_INTENSITY16I_EXT", INT2NUM(GL_INTENSITY16I_EXT));
	rb_define_const(module, "GL_INTENSITY16UI_EXT", INT2NUM(GL_INTENSITY16UI_EXT));
	rb_define_const(module, "GL_INTENSITY16_EXT", INT2NUM(GL_INTENSITY16_EXT));
	rb_define_const(module, "GL_INTENSITY32F_ARB", INT2NUM(GL_INTENSITY32F_ARB));
	rb_define_const(module, "GL_INTENSITY32I_EXT", INT2NUM(GL_INTENSITY32I_EXT));
	rb_define_const(module, "GL_INTENSITY32UI_EXT", INT2NUM(GL_INTENSITY32UI_EXT));
	rb_define_const(module, "GL_INTENSITY4", INT2NUM(GL_INTENSITY4));
	rb_define_const(module, "GL_INTENSITY4_EXT", INT2NUM(GL_INTENSITY4_EXT));
	rb_define_const(module, "GL_INTENSITY8", INT2NUM(GL_INTENSITY8));
	rb_define_const(module, "GL_INTENSITY8I_EXT", INT2NUM(GL_INTENSITY8I_EXT));
	rb_define_const(module, "GL_INTENSITY8UI_EXT", INT2NUM(GL_INTENSITY8UI_EXT));
	rb_define_const(module, "GL_INTENSITY8_EXT", INT2NUM(GL_INTENSITY8_EXT));
	rb_define_const(module, "GL_INTENSITY_EXT", INT2NUM(GL_INTENSITY_EXT));
	rb_define_const(module, "GL_INTENSITY_FLOAT16_ATI", INT2NUM(GL_INTENSITY_FLOAT16_ATI));
	rb_define_const(module, "GL_INTENSITY_FLOAT32_ATI", INT2NUM(GL_INTENSITY_FLOAT32_ATI));
	rb_define_const(module, "GL_INTERLACE_OML", INT2NUM(GL_INTERLACE_OML));
	rb_define_const(module, "GL_INTERLACE_READ_INGR", INT2NUM(GL_INTERLACE_READ_INGR));
	rb_define_const(module, "GL_INTERLACE_READ_OML", INT2NUM(GL_INTERLACE_READ_OML));
	rb_define_const(module, "GL_INTERLACE_SGIX", INT2NUM(GL_INTERLACE_SGIX));
	rb_define_const(module, "GL_INTERLEAVED_ATTRIBS", INT2NUM(GL_INTERLEAVED_ATTRIBS));
	rb_define_const(module, "GL_INTERLEAVED_ATTRIBS_EXT", INT2NUM(GL_INTERLEAVED_ATTRIBS_EXT));
	rb_define_const(module, "GL_INTERLEAVED_ATTRIBS_NV", INT2NUM(GL_INTERLEAVED_ATTRIBS_NV));
	rb_define_const(module, "GL_INTERPOLATE", INT2NUM(GL_INTERPOLATE));
	rb_define_const(module, "GL_INTERPOLATE_ARB", INT2NUM(GL_INTERPOLATE_ARB));
	rb_define_const(module, "GL_INTERPOLATE_EXT", INT2NUM(GL_INTERPOLATE_EXT));
	rb_define_const(module, "GL_INT_SAMPLER_1D", INT2NUM(GL_INT_SAMPLER_1D));
	rb_define_const(module, "GL_INT_SAMPLER_1D_ARRAY", INT2NUM(GL_INT_SAMPLER_1D_ARRAY));
	rb_define_const(module, "GL_INT_SAMPLER_1D_ARRAY_EXT", INT2NUM(GL_INT_SAMPLER_1D_ARRAY_EXT));
	rb_define_const(module, "GL_INT_SAMPLER_1D_EXT", INT2NUM(GL_INT_SAMPLER_1D_EXT));
	rb_define_const(module, "GL_INT_SAMPLER_2D", INT2NUM(GL_INT_SAMPLER_2D));
	rb_define_const(module, "GL_INT_SAMPLER_2D_ARRAY", INT2NUM(GL_INT_SAMPLER_2D_ARRAY));
	rb_define_const(module, "GL_INT_SAMPLER_2D_ARRAY_EXT", INT2NUM(GL_INT_SAMPLER_2D_ARRAY_EXT));
	rb_define_const(module, "GL_INT_SAMPLER_2D_EXT", INT2NUM(GL_INT_SAMPLER_2D_EXT));
	rb_define_const(module, "GL_INT_SAMPLER_2D_RECT_EXT", INT2NUM(GL_INT_SAMPLER_2D_RECT_EXT));
	rb_define_const(module, "GL_INT_SAMPLER_3D", INT2NUM(GL_INT_SAMPLER_3D));
	rb_define_const(module, "GL_INT_SAMPLER_3D_EXT", INT2NUM(GL_INT_SAMPLER_3D_EXT));
	rb_define_const(module, "GL_INT_SAMPLER_BUFFER_EXT", INT2NUM(GL_INT_SAMPLER_BUFFER_EXT));
	rb_define_const(module, "GL_INT_SAMPLER_CUBE", INT2NUM(GL_INT_SAMPLER_CUBE));
	rb_define_const(module, "GL_INT_SAMPLER_CUBE_EXT", INT2NUM(GL_INT_SAMPLER_CUBE_EXT));
	rb_define_const(module, "GL_INT_SAMPLER_RENDERBUFFER_NV", INT2NUM(GL_INT_SAMPLER_RENDERBUFFER_NV));
	rb_define_const(module, "GL_INT_VEC2", INT2NUM(GL_INT_VEC2));
	rb_define_const(module, "GL_INT_VEC2_ARB", INT2NUM(GL_INT_VEC2_ARB));
	rb_define_const(module, "GL_INT_VEC3", INT2NUM(GL_INT_VEC3));
	rb_define_const(module, "GL_INT_VEC3_ARB", INT2NUM(GL_INT_VEC3_ARB));
	rb_define_const(module, "GL_INT_VEC4", INT2NUM(GL_INT_VEC4));
	rb_define_const(module, "GL_INT_VEC4_ARB", INT2NUM(GL_INT_VEC4_ARB));
	rb_define_const(module, "GL_INVALID_ENUM", INT2NUM(GL_INVALID_ENUM));
	rb_define_const(module, "GL_INVALID_FRAMEBUFFER_OPERATION", INT2NUM(GL_INVALID_FRAMEBUFFER_OPERATION));
	rb_define_const(module, "GL_INVALID_FRAMEBUFFER_OPERATION_EXT", INT2NUM(GL_INVALID_FRAMEBUFFER_OPERATION_EXT));
	rb_define_const(module, "GL_INVALID_OPERATION", INT2NUM(GL_INVALID_OPERATION));
	rb_define_const(module, "GL_INVALID_VALUE", INT2NUM(GL_INVALID_VALUE));
	rb_define_const(module, "GL_INVARIANT_DATATYPE_EXT", INT2NUM(GL_INVARIANT_DATATYPE_EXT));
	rb_define_const(module, "GL_INVARIANT_EXT", INT2NUM(GL_INVARIANT_EXT));
	rb_define_const(module, "GL_INVARIANT_VALUE_EXT", INT2NUM(GL_INVARIANT_VALUE_EXT));
	rb_define_const(module, "GL_INVERSE_NV", INT2NUM(GL_INVERSE_NV));
	rb_define_const(module, "GL_INVERSE_TRANSPOSE_NV", INT2NUM(GL_INVERSE_TRANSPOSE_NV));
	rb_define_const(module, "GL_INVERT", INT2NUM(GL_INVERT));
	rb_define_const(module, "GL_INVERTED_SCREEN_W_REND", INT2NUM(GL_INVERTED_SCREEN_W_REND));
	rb_define_const(module, "GL_IR_INSTRUMENT1_SGIX", INT2NUM(GL_IR_INSTRUMENT1_SGIX));
	rb_define_const(module, "GL_IUI_N3F_V2F_EXT", INT2NUM(GL_IUI_N3F_V2F_EXT));
	rb_define_const(module, "GL_IUI_N3F_V3F_EXT", INT2NUM(GL_IUI_N3F_V3F_EXT));
	rb_define_const(module, "GL_IUI_V2F_EXT", INT2NUM(GL_IUI_V2F_EXT));
	rb_define_const(module, "GL_IUI_V3F_EXT", INT2NUM(GL_IUI_V3F_EXT));
	rb_define_const(module, "GL_KEEP", INT2NUM(GL_KEEP));
	rb_define_const(module, "GL_LEFT", INT2NUM(GL_LEFT));
	rb_define_const(module, "GL_LEQUAL", INT2NUM(GL_LEQUAL));
	rb_define_const(module, "GL_LERP_ATI", INT2NUM(GL_LERP_ATI));
	rb_define_const(module, "GL_LESS", INT2NUM(GL_LESS));
	rb_define_const(module, "GL_LIGHT0", INT2NUM(GL_LIGHT0));
	rb_define_const(module, "GL_LIGHT1", INT2NUM(GL_LIGHT1));
	rb_define_const(module, "GL_LIGHT2", INT2NUM(GL_LIGHT2));
	rb_define_const(module, "GL_LIGHT3", INT2NUM(GL_LIGHT3));
	rb_define_const(module, "GL_LIGHT4", INT2NUM(GL_LIGHT4));
	rb_define_const(module, "GL_LIGHT5", INT2NUM(GL_LIGHT5));
	rb_define_const(module, "GL_LIGHT6", INT2NUM(GL_LIGHT6));
	rb_define_const(module, "GL_LIGHT7", INT2NUM(GL_LIGHT7));
	rb_define_const(module, "GL_LIGHTING", INT2NUM(GL_LIGHTING));
	rb_define_const(module, "GL_LIGHTING_BIT", INT2NUM(GL_LIGHTING_BIT));
	rb_define_const(module, "GL_LIGHT_ENV_MODE_SGIX", INT2NUM(GL_LIGHT_ENV_MODE_SGIX));
	rb_define_const(module, "GL_LIGHT_MODEL_AMBIENT", INT2NUM(GL_LIGHT_MODEL_AMBIENT));
	rb_define_const(module, "GL_LIGHT_MODEL_COLOR_CONTROL", INT2NUM(GL_LIGHT_MODEL_COLOR_CONTROL));
	rb_define_const(module, "GL_LIGHT_MODEL_COLOR_CONTROL_EXT", INT2NUM(GL_LIGHT_MODEL_COLOR_CONTROL_EXT));
	rb_define_const(module, "GL_LIGHT_MODEL_LOCAL_VIEWER", INT2NUM(GL_LIGHT_MODEL_LOCAL_VIEWER));
	rb_define_const(module, "GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE", INT2NUM(GL_LIGHT_MODEL_SPECULAR_VECTOR_APPLE));
	rb_define_const(module, "GL_LIGHT_MODEL_TWO_SIDE", INT2NUM(GL_LIGHT_MODEL_TWO_SIDE));
	rb_define_const(module, "GL_LINE", INT2NUM(GL_LINE));
	rb_define_const(module, "GL_LINEAR", INT2NUM(GL_LINEAR));
	rb_define_const(module, "GL_LINEAR_ATTENUATION", INT2NUM(GL_LINEAR_ATTENUATION));
	rb_define_const(module, "GL_LINEAR_CLIPMAP_LINEAR_SGIX", INT2NUM(GL_LINEAR_CLIPMAP_LINEAR_SGIX));
	rb_define_const(module, "GL_LINEAR_CLIPMAP_NEAREST_SGIX", INT2NUM(GL_LINEAR_CLIPMAP_NEAREST_SGIX));
	rb_define_const(module, "GL_LINEAR_DETAIL_ALPHA_SGIS", INT2NUM(GL_LINEAR_DETAIL_ALPHA_SGIS));
	rb_define_const(module, "GL_LINEAR_DETAIL_COLOR_SGIS", INT2NUM(GL_LINEAR_DETAIL_COLOR_SGIS));
	rb_define_const(module, "GL_LINEAR_DETAIL_SGIS", INT2NUM(GL_LINEAR_DETAIL_SGIS));
	rb_define_const(module, "GL_LINEAR_MIPMAP_LINEAR", INT2NUM(GL_LINEAR_MIPMAP_LINEAR));
	rb_define_const(module, "GL_LINEAR_MIPMAP_NEAREST", INT2NUM(GL_LINEAR_MIPMAP_NEAREST));
	rb_define_const(module, "GL_LINEAR_SHARPEN_ALPHA_SGIS", INT2NUM(GL_LINEAR_SHARPEN_ALPHA_SGIS));
	rb_define_const(module, "GL_LINEAR_SHARPEN_COLOR_SGIS", INT2NUM(GL_LINEAR_SHARPEN_COLOR_SGIS));
	rb_define_const(module, "GL_LINEAR_SHARPEN_SGIS", INT2NUM(GL_LINEAR_SHARPEN_SGIS));
	rb_define_const(module, "GL_LINES", INT2NUM(GL_LINES));
	rb_define_const(module, "GL_LINES_ADJACENCY_ARB", INT2NUM(GL_LINES_ADJACENCY_ARB));
	rb_define_const(module, "GL_LINES_ADJACENCY_EXT", INT2NUM(GL_LINES_ADJACENCY_EXT));
	rb_define_const(module, "GL_LINE_BIT", INT2NUM(GL_LINE_BIT));
	rb_define_const(module, "GL_LINE_LOOP", INT2NUM(GL_LINE_LOOP));
	rb_define_const(module, "GL_LINE_RESET_TOKEN", INT2NUM(GL_LINE_RESET_TOKEN));
	rb_define_const(module, "GL_LINE_SMOOTH", INT2NUM(GL_LINE_SMOOTH));
	rb_define_const(module, "GL_LINE_SMOOTH_HINT", INT2NUM(GL_LINE_SMOOTH_HINT));
	rb_define_const(module, "GL_LINE_STIPPLE", INT2NUM(GL_LINE_STIPPLE));
	rb_define_const(module, "GL_LINE_STIPPLE_PATTERN", INT2NUM(GL_LINE_STIPPLE_PATTERN));
	rb_define_const(module, "GL_LINE_STIPPLE_REPEAT", INT2NUM(GL_LINE_STIPPLE_REPEAT));
	rb_define_const(module, "GL_LINE_STRIP", INT2NUM(GL_LINE_STRIP));
	rb_define_const(module, "GL_LINE_STRIP_ADJACENCY_ARB", INT2NUM(GL_LINE_STRIP_ADJACENCY_ARB));
	rb_define_const(module, "GL_LINE_STRIP_ADJACENCY_EXT", INT2NUM(GL_LINE_STRIP_ADJACENCY_EXT));
	rb_define_const(module, "GL_LINE_TOKEN", INT2NUM(GL_LINE_TOKEN));
	rb_define_const(module, "GL_LINE_WIDTH", INT2NUM(GL_LINE_WIDTH));
	rb_define_const(module, "GL_LINE_WIDTH_GRANULARITY", INT2NUM(GL_LINE_WIDTH_GRANULARITY));
	rb_define_const(module, "GL_LINE_WIDTH_RANGE", INT2NUM(GL_LINE_WIDTH_RANGE));
	rb_define_const(module, "GL_LINK_STATUS", INT2NUM(GL_LINK_STATUS));
	rb_define_const(module, "GL_LIST_BASE", INT2NUM(GL_LIST_BASE));
	rb_define_const(module, "GL_LIST_BIT", INT2NUM(GL_LIST_BIT));
	rb_define_const(module, "GL_LIST_INDEX", INT2NUM(GL_LIST_INDEX));
	rb_define_const(module, "GL_LIST_MODE", INT2NUM(GL_LIST_MODE));
	rb_define_const(module, "GL_LIST_PRIORITY_SGIX", INT2NUM(GL_LIST_PRIORITY_SGIX));
	rb_define_const(module, "GL_LOAD", INT2NUM(GL_LOAD));
	rb_define_const(module, "GL_LOCAL_CONSTANT_DATATYPE_EXT", INT2NUM(GL_LOCAL_CONSTANT_DATATYPE_EXT));
	rb_define_const(module, "GL_LOCAL_CONSTANT_EXT", INT2NUM(GL_LOCAL_CONSTANT_EXT));
	rb_define_const(module, "GL_LOCAL_CONSTANT_VALUE_EXT", INT2NUM(GL_LOCAL_CONSTANT_VALUE_EXT));
	rb_define_const(module, "GL_LOCAL_EXT", INT2NUM(GL_LOCAL_EXT));
	rb_define_const(module, "GL_LOGIC_OP", INT2NUM(GL_LOGIC_OP));
	rb_define_const(module, "GL_LOGIC_OP_MODE", INT2NUM(GL_LOGIC_OP_MODE));
	rb_define_const(module, "GL_LOWER_LEFT", INT2NUM(GL_LOWER_LEFT));
	rb_define_const(module, "GL_LO_BIAS_NV", INT2NUM(GL_LO_BIAS_NV));
	rb_define_const(module, "GL_LO_SCALE_NV", INT2NUM(GL_LO_SCALE_NV));
	rb_define_const(module, "GL_LUMINANCE", INT2NUM(GL_LUMINANCE));
	rb_define_const(module, "GL_LUMINANCE12", INT2NUM(GL_LUMINANCE12));
	rb_define_const(module, "GL_LUMINANCE12_ALPHA12", INT2NUM(GL_LUMINANCE12_ALPHA12));
	rb_define_const(module, "GL_LUMINANCE12_ALPHA12_EXT", INT2NUM(GL_LUMINANCE12_ALPHA12_EXT));
	rb_define_const(module, "GL_LUMINANCE12_ALPHA4", INT2NUM(GL_LUMINANCE12_ALPHA4));
	rb_define_const(module, "GL_LUMINANCE12_ALPHA4_EXT", INT2NUM(GL_LUMINANCE12_ALPHA4_EXT));
	rb_define_const(module, "GL_LUMINANCE12_EXT", INT2NUM(GL_LUMINANCE12_EXT));
	rb_define_const(module, "GL_LUMINANCE16", INT2NUM(GL_LUMINANCE16));
	rb_define_const(module, "GL_LUMINANCE16F_ARB", INT2NUM(GL_LUMINANCE16F_ARB));
	rb_define_const(module, "GL_LUMINANCE16I_EXT", INT2NUM(GL_LUMINANCE16I_EXT));
	rb_define_const(module, "GL_LUMINANCE16UI_EXT", INT2NUM(GL_LUMINANCE16UI_EXT));
	rb_define_const(module, "GL_LUMINANCE16_ALPHA16", INT2NUM(GL_LUMINANCE16_ALPHA16));
	rb_define_const(module, "GL_LUMINANCE16_ALPHA16_EXT", INT2NUM(GL_LUMINANCE16_ALPHA16_EXT));
	rb_define_const(module, "GL_LUMINANCE16_EXT", INT2NUM(GL_LUMINANCE16_EXT));
	rb_define_const(module, "GL_LUMINANCE32F_ARB", INT2NUM(GL_LUMINANCE32F_ARB));
	rb_define_const(module, "GL_LUMINANCE32I_EXT", INT2NUM(GL_LUMINANCE32I_EXT));
	rb_define_const(module, "GL_LUMINANCE32UI_EXT", INT2NUM(GL_LUMINANCE32UI_EXT));
	rb_define_const(module, "GL_LUMINANCE4", INT2NUM(GL_LUMINANCE4));
	rb_define_const(module, "GL_LUMINANCE4_ALPHA4", INT2NUM(GL_LUMINANCE4_ALPHA4));
	rb_define_const(module, "GL_LUMINANCE4_ALPHA4_EXT", INT2NUM(GL_LUMINANCE4_ALPHA4_EXT));
	rb_define_const(module, "GL_LUMINANCE4_EXT", INT2NUM(GL_LUMINANCE4_EXT));
	rb_define_const(module, "GL_LUMINANCE6_ALPHA2", INT2NUM(GL_LUMINANCE6_ALPHA2));
	rb_define_const(module, "GL_LUMINANCE6_ALPHA2_EXT", INT2NUM(GL_LUMINANCE6_ALPHA2_EXT));
	rb_define_const(module, "GL_LUMINANCE8", INT2NUM(GL_LUMINANCE8));
	rb_define_const(module, "GL_LUMINANCE8I_EXT", INT2NUM(GL_LUMINANCE8I_EXT));
	rb_define_const(module, "GL_LUMINANCE8UI_EXT", INT2NUM(GL_LUMINANCE8UI_EXT));
	rb_define_const(module, "GL_LUMINANCE8_ALPHA8", INT2NUM(GL_LUMINANCE8_ALPHA8));
	rb_define_const(module, "GL_LUMINANCE8_ALPHA8_EXT", INT2NUM(GL_LUMINANCE8_ALPHA8_EXT));
	rb_define_const(module, "GL_LUMINANCE8_EXT", INT2NUM(GL_LUMINANCE8_EXT));
	rb_define_const(module, "GL_LUMINANCE_ALPHA", INT2NUM(GL_LUMINANCE_ALPHA));
	rb_define_const(module, "GL_LUMINANCE_ALPHA16F_ARB", INT2NUM(GL_LUMINANCE_ALPHA16F_ARB));
	rb_define_const(module, "GL_LUMINANCE_ALPHA16I_EXT", INT2NUM(GL_LUMINANCE_ALPHA16I_EXT));
	rb_define_const(module, "GL_LUMINANCE_ALPHA16UI_EXT", INT2NUM(GL_LUMINANCE_ALPHA16UI_EXT));
	rb_define_const(module, "GL_LUMINANCE_ALPHA32F_ARB", INT2NUM(GL_LUMINANCE_ALPHA32F_ARB));
	rb_define_const(module, "GL_LUMINANCE_ALPHA32I_EXT", INT2NUM(GL_LUMINANCE_ALPHA32I_EXT));
	rb_define_const(module, "GL_LUMINANCE_ALPHA32UI_EXT", INT2NUM(GL_LUMINANCE_ALPHA32UI_EXT));
	rb_define_const(module, "GL_LUMINANCE_ALPHA8I_EXT", INT2NUM(GL_LUMINANCE_ALPHA8I_EXT));
	rb_define_const(module, "GL_LUMINANCE_ALPHA8UI_EXT", INT2NUM(GL_LUMINANCE_ALPHA8UI_EXT));
	rb_define_const(module, "GL_LUMINANCE_ALPHA_FLOAT16_ATI", INT2NUM(GL_LUMINANCE_ALPHA_FLOAT16_ATI));
	rb_define_const(module, "GL_LUMINANCE_ALPHA_FLOAT32_ATI", INT2NUM(GL_LUMINANCE_ALPHA_FLOAT32_ATI));
	rb_define_const(module, "GL_LUMINANCE_ALPHA_INTEGER_EXT", INT2NUM(GL_LUMINANCE_ALPHA_INTEGER_EXT));
	rb_define_const(module, "GL_LUMINANCE_FLOAT16_ATI", INT2NUM(GL_LUMINANCE_FLOAT16_ATI));
	rb_define_const(module, "GL_LUMINANCE_FLOAT32_ATI", INT2NUM(GL_LUMINANCE_FLOAT32_ATI));
	rb_define_const(module, "GL_LUMINANCE_INTEGER_EXT", INT2NUM(GL_LUMINANCE_INTEGER_EXT));
	rb_define_const(module, "GL_MAD_ATI", INT2NUM(GL_MAD_ATI));
	rb_define_const(module, "GL_MAGNITUDE_BIAS_NV", INT2NUM(GL_MAGNITUDE_BIAS_NV));
	rb_define_const(module, "GL_MAGNITUDE_SCALE_NV", INT2NUM(GL_MAGNITUDE_SCALE_NV));
	rb_define_const(module, "GL_MAJOR_VERSION", INT2NUM(GL_MAJOR_VERSION));
	rb_define_const(module, "GL_MAP1_BINORMAL_EXT", INT2NUM(GL_MAP1_BINORMAL_EXT));
	rb_define_const(module, "GL_MAP1_COLOR_4", INT2NUM(GL_MAP1_COLOR_4));
	rb_define_const(module, "GL_MAP1_GRID_DOMAIN", INT2NUM(GL_MAP1_GRID_DOMAIN));
	rb_define_const(module, "GL_MAP1_GRID_SEGMENTS", INT2NUM(GL_MAP1_GRID_SEGMENTS));
	rb_define_const(module, "GL_MAP1_INDEX", INT2NUM(GL_MAP1_INDEX));
	rb_define_const(module, "GL_MAP1_NORMAL", INT2NUM(GL_MAP1_NORMAL));
	rb_define_const(module, "GL_MAP1_TANGENT_EXT", INT2NUM(GL_MAP1_TANGENT_EXT));
	rb_define_const(module, "GL_MAP1_TEXTURE_COORD_1", INT2NUM(GL_MAP1_TEXTURE_COORD_1));
	rb_define_const(module, "GL_MAP1_TEXTURE_COORD_2", INT2NUM(GL_MAP1_TEXTURE_COORD_2));
	rb_define_const(module, "GL_MAP1_TEXTURE_COORD_3", INT2NUM(GL_MAP1_TEXTURE_COORD_3));
	rb_define_const(module, "GL_MAP1_TEXTURE_COORD_4", INT2NUM(GL_MAP1_TEXTURE_COORD_4));
	rb_define_const(module, "GL_MAP1_VERTEX_3", INT2NUM(GL_MAP1_VERTEX_3));
	rb_define_const(module, "GL_MAP1_VERTEX_4", INT2NUM(GL_MAP1_VERTEX_4));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB0_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB0_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB10_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB10_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB11_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB11_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB12_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB12_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB13_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB13_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB14_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB14_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB15_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB15_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB1_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB1_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB2_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB2_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB3_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB3_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB4_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB4_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB5_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB5_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB6_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB6_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB7_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB7_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB8_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB8_4_NV));
	rb_define_const(module, "GL_MAP1_VERTEX_ATTRIB9_4_NV", INT2NUM(GL_MAP1_VERTEX_ATTRIB9_4_NV));
	rb_define_const(module, "GL_MAP2_BINORMAL_EXT", INT2NUM(GL_MAP2_BINORMAL_EXT));
	rb_define_const(module, "GL_MAP2_COLOR_4", INT2NUM(GL_MAP2_COLOR_4));
	rb_define_const(module, "GL_MAP2_GRID_DOMAIN", INT2NUM(GL_MAP2_GRID_DOMAIN));
	rb_define_const(module, "GL_MAP2_GRID_SEGMENTS", INT2NUM(GL_MAP2_GRID_SEGMENTS));
	rb_define_const(module, "GL_MAP2_INDEX", INT2NUM(GL_MAP2_INDEX));
	rb_define_const(module, "GL_MAP2_NORMAL", INT2NUM(GL_MAP2_NORMAL));
	rb_define_const(module, "GL_MAP2_TANGENT_EXT", INT2NUM(GL_MAP2_TANGENT_EXT));
	rb_define_const(module, "GL_MAP2_TEXTURE_COORD_1", INT2NUM(GL_MAP2_TEXTURE_COORD_1));
	rb_define_const(module, "GL_MAP2_TEXTURE_COORD_2", INT2NUM(GL_MAP2_TEXTURE_COORD_2));
	rb_define_const(module, "GL_MAP2_TEXTURE_COORD_3", INT2NUM(GL_MAP2_TEXTURE_COORD_3));
	rb_define_const(module, "GL_MAP2_TEXTURE_COORD_4", INT2NUM(GL_MAP2_TEXTURE_COORD_4));
	rb_define_const(module, "GL_MAP2_VERTEX_3", INT2NUM(GL_MAP2_VERTEX_3));
	rb_define_const(module, "GL_MAP2_VERTEX_4", INT2NUM(GL_MAP2_VERTEX_4));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB0_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB0_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB10_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB10_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB11_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB11_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB12_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB12_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB13_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB13_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB14_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB14_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB15_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB15_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB1_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB1_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB2_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB2_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB3_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB3_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB4_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB4_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB5_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB5_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB6_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB6_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB7_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB7_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB8_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB8_4_NV));
	rb_define_const(module, "GL_MAP2_VERTEX_ATTRIB9_4_NV", INT2NUM(GL_MAP2_VERTEX_ATTRIB9_4_NV));
	rb_define_const(module, "GL_MAP_ATTRIB_U_ORDER_NV", INT2NUM(GL_MAP_ATTRIB_U_ORDER_NV));
	rb_define_const(module, "GL_MAP_ATTRIB_V_ORDER_NV", INT2NUM(GL_MAP_ATTRIB_V_ORDER_NV));
	rb_define_const(module, "GL_MAP_COLOR", INT2NUM(GL_MAP_COLOR));
	rb_define_const(module, "GL_MAP_FLUSH_EXPLICIT_BIT", INT2NUM(GL_MAP_FLUSH_EXPLICIT_BIT));
	rb_define_const(module, "GL_MAP_INVALIDATE_BUFFER_BIT", INT2NUM(GL_MAP_INVALIDATE_BUFFER_BIT));
	rb_define_const(module, "GL_MAP_INVALIDATE_RANGE_BIT", INT2NUM(GL_MAP_INVALIDATE_RANGE_BIT));
	rb_define_const(module, "GL_MAP_READ_BIT", INT2NUM(GL_MAP_READ_BIT));
	rb_define_const(module, "GL_MAP_STENCIL", INT2NUM(GL_MAP_STENCIL));
	rb_define_const(module, "GL_MAP_TESSELLATION_NV", INT2NUM(GL_MAP_TESSELLATION_NV));
	rb_define_const(module, "GL_MAP_UNSYNCHRONIZED_BIT", INT2NUM(GL_MAP_UNSYNCHRONIZED_BIT));
	rb_define_const(module, "GL_MAP_WRITE_BIT", INT2NUM(GL_MAP_WRITE_BIT));
	rb_define_const(module, "GL_MATERIAL_SIDE_HINT_PGI", INT2NUM(GL_MATERIAL_SIDE_HINT_PGI));
	rb_define_const(module, "GL_MATRIX0_ARB", INT2NUM(GL_MATRIX0_ARB));
	rb_define_const(module, "GL_MATRIX0_NV", INT2NUM(GL_MATRIX0_NV));
	rb_define_const(module, "GL_MATRIX10_ARB", INT2NUM(GL_MATRIX10_ARB));
	rb_define_const(module, "GL_MATRIX11_ARB", INT2NUM(GL_MATRIX11_ARB));
	rb_define_const(module, "GL_MATRIX12_ARB", INT2NUM(GL_MATRIX12_ARB));
	rb_define_const(module, "GL_MATRIX13_ARB", INT2NUM(GL_MATRIX13_ARB));
	rb_define_const(module, "GL_MATRIX14_ARB", INT2NUM(GL_MATRIX14_ARB));
	rb_define_const(module, "GL_MATRIX15_ARB", INT2NUM(GL_MATRIX15_ARB));
	rb_define_const(module, "GL_MATRIX16_ARB", INT2NUM(GL_MATRIX16_ARB));
	rb_define_const(module, "GL_MATRIX17_ARB", INT2NUM(GL_MATRIX17_ARB));
	rb_define_const(module, "GL_MATRIX18_ARB", INT2NUM(GL_MATRIX18_ARB));
	rb_define_const(module, "GL_MATRIX19_ARB", INT2NUM(GL_MATRIX19_ARB));
	rb_define_const(module, "GL_MATRIX1_ARB", INT2NUM(GL_MATRIX1_ARB));
	rb_define_const(module, "GL_MATRIX1_NV", INT2NUM(GL_MATRIX1_NV));
	rb_define_const(module, "GL_MATRIX20_ARB", INT2NUM(GL_MATRIX20_ARB));
	rb_define_const(module, "GL_MATRIX21_ARB", INT2NUM(GL_MATRIX21_ARB));
	rb_define_const(module, "GL_MATRIX22_ARB", INT2NUM(GL_MATRIX22_ARB));
	rb_define_const(module, "GL_MATRIX23_ARB", INT2NUM(GL_MATRIX23_ARB));
	rb_define_const(module, "GL_MATRIX24_ARB", INT2NUM(GL_MATRIX24_ARB));
	rb_define_const(module, "GL_MATRIX25_ARB", INT2NUM(GL_MATRIX25_ARB));
	rb_define_const(module, "GL_MATRIX26_ARB", INT2NUM(GL_MATRIX26_ARB));
	rb_define_const(module, "GL_MATRIX27_ARB", INT2NUM(GL_MATRIX27_ARB));
	rb_define_const(module, "GL_MATRIX28_ARB", INT2NUM(GL_MATRIX28_ARB));
	rb_define_const(module, "GL_MATRIX29_ARB", INT2NUM(GL_MATRIX29_ARB));
	rb_define_const(module, "GL_MATRIX2_ARB", INT2NUM(GL_MATRIX2_ARB));
	rb_define_const(module, "GL_MATRIX2_NV", INT2NUM(GL_MATRIX2_NV));
	rb_define_const(module, "GL_MATRIX30_ARB", INT2NUM(GL_MATRIX30_ARB));
	rb_define_const(module, "GL_MATRIX31_ARB", INT2NUM(GL_MATRIX31_ARB));
	rb_define_const(module, "GL_MATRIX3_ARB", INT2NUM(GL_MATRIX3_ARB));
	rb_define_const(module, "GL_MATRIX3_NV", INT2NUM(GL_MATRIX3_NV));
	rb_define_const(module, "GL_MATRIX4_ARB", INT2NUM(GL_MATRIX4_ARB));
	rb_define_const(module, "GL_MATRIX4_NV", INT2NUM(GL_MATRIX4_NV));
	rb_define_const(module, "GL_MATRIX5_ARB", INT2NUM(GL_MATRIX5_ARB));
	rb_define_const(module, "GL_MATRIX5_NV", INT2NUM(GL_MATRIX5_NV));
	rb_define_const(module, "GL_MATRIX6_ARB", INT2NUM(GL_MATRIX6_ARB));
	rb_define_const(module, "GL_MATRIX6_NV", INT2NUM(GL_MATRIX6_NV));
	rb_define_const(module, "GL_MATRIX7_ARB", INT2NUM(GL_MATRIX7_ARB));
	rb_define_const(module, "GL_MATRIX7_NV", INT2NUM(GL_MATRIX7_NV));
	rb_define_const(module, "GL_MATRIX8_ARB", INT2NUM(GL_MATRIX8_ARB));
	rb_define_const(module, "GL_MATRIX9_ARB", INT2NUM(GL_MATRIX9_ARB));
	rb_define_const(module, "GL_MATRIX_EXT", INT2NUM(GL_MATRIX_EXT));
	rb_define_const(module, "GL_MATRIX_INDEX_ARRAY_ARB", INT2NUM(GL_MATRIX_INDEX_ARRAY_ARB));
	rb_define_const(module, "GL_MATRIX_INDEX_ARRAY_POINTER_ARB", INT2NUM(GL_MATRIX_INDEX_ARRAY_POINTER_ARB));
	rb_define_const(module, "GL_MATRIX_INDEX_ARRAY_SIZE_ARB", INT2NUM(GL_MATRIX_INDEX_ARRAY_SIZE_ARB));
	rb_define_const(module, "GL_MATRIX_INDEX_ARRAY_STRIDE_ARB", INT2NUM(GL_MATRIX_INDEX_ARRAY_STRIDE_ARB));
	rb_define_const(module, "GL_MATRIX_INDEX_ARRAY_TYPE_ARB", INT2NUM(GL_MATRIX_INDEX_ARRAY_TYPE_ARB));
	rb_define_const(module, "GL_MATRIX_MODE", INT2NUM(GL_MATRIX_MODE));
	rb_define_const(module, "GL_MATRIX_PALETTE_ARB", INT2NUM(GL_MATRIX_PALETTE_ARB));
	rb_define_const(module, "GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI", INT2NUM(GL_MAT_AMBIENT_AND_DIFFUSE_BIT_PGI));
	rb_define_const(module, "GL_MAT_AMBIENT_BIT_PGI", INT2NUM(GL_MAT_AMBIENT_BIT_PGI));
	rb_define_const(module, "GL_MAT_COLOR_INDEXES_BIT_PGI", INT2NUM(GL_MAT_COLOR_INDEXES_BIT_PGI));
	rb_define_const(module, "GL_MAT_DIFFUSE_BIT_PGI", INT2NUM(GL_MAT_DIFFUSE_BIT_PGI));
	rb_define_const(module, "GL_MAT_EMISSION_BIT_PGI", INT2NUM(GL_MAT_EMISSION_BIT_PGI));
	rb_define_const(module, "GL_MAT_SHININESS_BIT_PGI", INT2NUM(GL_MAT_SHININESS_BIT_PGI));
	rb_define_const(module, "GL_MAT_SPECULAR_BIT_PGI", INT2NUM(GL_MAT_SPECULAR_BIT_PGI));
	rb_define_const(module, "GL_MAX", INT2NUM(GL_MAX));
	rb_define_const(module, "GL_MAX_3D_TEXTURE_SIZE", INT2NUM(GL_MAX_3D_TEXTURE_SIZE));
	rb_define_const(module, "GL_MAX_3D_TEXTURE_SIZE_EXT", INT2NUM(GL_MAX_3D_TEXTURE_SIZE_EXT));
	rb_define_const(module, "GL_MAX_4D_TEXTURE_SIZE_SGIS", INT2NUM(GL_MAX_4D_TEXTURE_SIZE_SGIS));
	rb_define_const(module, "GL_MAX_ACTIVE_LIGHTS_SGIX", INT2NUM(GL_MAX_ACTIVE_LIGHTS_SGIX));
	rb_define_const(module, "GL_MAX_ARRAY_TEXTURE_LAYERS", INT2NUM(GL_MAX_ARRAY_TEXTURE_LAYERS));
	rb_define_const(module, "GL_MAX_ARRAY_TEXTURE_LAYERS_EXT", INT2NUM(GL_MAX_ARRAY_TEXTURE_LAYERS_EXT));
	rb_define_const(module, "GL_MAX_ASYNC_DRAW_PIXELS_SGIX", INT2NUM(GL_MAX_ASYNC_DRAW_PIXELS_SGIX));
	rb_define_const(module, "GL_MAX_ASYNC_HISTOGRAM_SGIX", INT2NUM(GL_MAX_ASYNC_HISTOGRAM_SGIX));
	rb_define_const(module, "GL_MAX_ASYNC_READ_PIXELS_SGIX", INT2NUM(GL_MAX_ASYNC_READ_PIXELS_SGIX));
	rb_define_const(module, "GL_MAX_ASYNC_TEX_IMAGE_SGIX", INT2NUM(GL_MAX_ASYNC_TEX_IMAGE_SGIX));
	rb_define_const(module, "GL_MAX_ATTRIB_STACK_DEPTH", INT2NUM(GL_MAX_ATTRIB_STACK_DEPTH));
	rb_define_const(module, "GL_MAX_BINDABLE_UNIFORM_SIZE_EXT", INT2NUM(GL_MAX_BINDABLE_UNIFORM_SIZE_EXT));
	rb_define_const(module, "GL_MAX_CLIENT_ATTRIB_STACK_DEPTH", INT2NUM(GL_MAX_CLIENT_ATTRIB_STACK_DEPTH));
	rb_define_const(module, "GL_MAX_CLIPMAP_DEPTH_SGIX", INT2NUM(GL_MAX_CLIPMAP_DEPTH_SGIX));
	rb_define_const(module, "GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX", INT2NUM(GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX));
	rb_define_const(module, "GL_MAX_CLIP_DISTANCES", INT2NUM(GL_MAX_CLIP_DISTANCES));
	rb_define_const(module, "GL_MAX_CLIP_PLANES", INT2NUM(GL_MAX_CLIP_PLANES));
	rb_define_const(module, "GL_MAX_COLOR_ATTACHMENTS", INT2NUM(GL_MAX_COLOR_ATTACHMENTS));
	rb_define_const(module, "GL_MAX_COLOR_ATTACHMENTS_EXT", INT2NUM(GL_MAX_COLOR_ATTACHMENTS_EXT));
	rb_define_const(module, "GL_MAX_COLOR_MATRIX_STACK_DEPTH", INT2NUM(GL_MAX_COLOR_MATRIX_STACK_DEPTH));
	rb_define_const(module, "GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI", INT2NUM(GL_MAX_COLOR_MATRIX_STACK_DEPTH_SGI));
	rb_define_const(module, "GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS", INT2NUM(GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS));
	rb_define_const(module, "GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB", INT2NUM(GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB));
	rb_define_const(module, "GL_MAX_CONVOLUTION_HEIGHT", INT2NUM(GL_MAX_CONVOLUTION_HEIGHT));
	rb_define_const(module, "GL_MAX_CONVOLUTION_HEIGHT_EXT", INT2NUM(GL_MAX_CONVOLUTION_HEIGHT_EXT));
	rb_define_const(module, "GL_MAX_CONVOLUTION_WIDTH", INT2NUM(GL_MAX_CONVOLUTION_WIDTH));
	rb_define_const(module, "GL_MAX_CONVOLUTION_WIDTH_EXT", INT2NUM(GL_MAX_CONVOLUTION_WIDTH_EXT));
	rb_define_const(module, "GL_MAX_CUBE_MAP_TEXTURE_SIZE", INT2NUM(GL_MAX_CUBE_MAP_TEXTURE_SIZE));
	rb_define_const(module, "GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB", INT2NUM(GL_MAX_CUBE_MAP_TEXTURE_SIZE_ARB));
	rb_define_const(module, "GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT", INT2NUM(GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT));
	rb_define_const(module, "GL_MAX_DEFORMATION_ORDER_SGIX", INT2NUM(GL_MAX_DEFORMATION_ORDER_SGIX));
	rb_define_const(module, "GL_MAX_DRAW_BUFFERS", INT2NUM(GL_MAX_DRAW_BUFFERS));
	rb_define_const(module, "GL_MAX_DRAW_BUFFERS_ARB", INT2NUM(GL_MAX_DRAW_BUFFERS_ARB));
	rb_define_const(module, "GL_MAX_DRAW_BUFFERS_ATI", INT2NUM(GL_MAX_DRAW_BUFFERS_ATI));
	rb_define_const(module, "GL_MAX_ELEMENTS_INDICES", INT2NUM(GL_MAX_ELEMENTS_INDICES));
	rb_define_const(module, "GL_MAX_ELEMENTS_INDICES_EXT", INT2NUM(GL_MAX_ELEMENTS_INDICES_EXT));
	rb_define_const(module, "GL_MAX_ELEMENTS_VERTICES", INT2NUM(GL_MAX_ELEMENTS_VERTICES));
	rb_define_const(module, "GL_MAX_ELEMENTS_VERTICES_EXT", INT2NUM(GL_MAX_ELEMENTS_VERTICES_EXT));
	rb_define_const(module, "GL_MAX_EVAL_ORDER", INT2NUM(GL_MAX_EVAL_ORDER));
	rb_define_const(module, "GL_MAX_EXT", INT2NUM(GL_MAX_EXT));
	rb_define_const(module, "GL_MAX_FOG_FUNC_POINTS_SGIS", INT2NUM(GL_MAX_FOG_FUNC_POINTS_SGIS));
	rb_define_const(module, "GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT", INT2NUM(GL_MAX_FRAGMENT_BINDABLE_UNIFORMS_EXT));
	rb_define_const(module, "GL_MAX_FRAGMENT_LIGHTS_SGIX", INT2NUM(GL_MAX_FRAGMENT_LIGHTS_SGIX));
	rb_define_const(module, "GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV", INT2NUM(GL_MAX_FRAGMENT_PROGRAM_LOCAL_PARAMETERS_NV));
	rb_define_const(module, "GL_MAX_FRAGMENT_UNIFORM_COMPONENTS", INT2NUM(GL_MAX_FRAGMENT_UNIFORM_COMPONENTS));
	rb_define_const(module, "GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB", INT2NUM(GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB));
	rb_define_const(module, "GL_MAX_FRAMEZOOM_FACTOR_SGIX", INT2NUM(GL_MAX_FRAMEZOOM_FACTOR_SGIX));
	rb_define_const(module, "GL_MAX_GENERAL_COMBINERS_NV", INT2NUM(GL_MAX_GENERAL_COMBINERS_NV));
	rb_define_const(module, "GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT", INT2NUM(GL_MAX_GEOMETRY_BINDABLE_UNIFORMS_EXT));
	rb_define_const(module, "GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB", INT2NUM(GL_MAX_GEOMETRY_OUTPUT_VERTICES_ARB));
	rb_define_const(module, "GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT", INT2NUM(GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT));
	rb_define_const(module, "GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB", INT2NUM(GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_ARB));
	rb_define_const(module, "GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT", INT2NUM(GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT));
	rb_define_const(module, "GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB", INT2NUM(GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_ARB));
	rb_define_const(module, "GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT", INT2NUM(GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS_EXT));
	rb_define_const(module, "GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB", INT2NUM(GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_ARB));
	rb_define_const(module, "GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT", INT2NUM(GL_MAX_GEOMETRY_UNIFORM_COMPONENTS_EXT));
	rb_define_const(module, "GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB", INT2NUM(GL_MAX_GEOMETRY_VARYING_COMPONENTS_ARB));
	rb_define_const(module, "GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT", INT2NUM(GL_MAX_GEOMETRY_VARYING_COMPONENTS_EXT));
	rb_define_const(module, "GL_MAX_LIGHTS", INT2NUM(GL_MAX_LIGHTS));
	rb_define_const(module, "GL_MAX_LIST_NESTING", INT2NUM(GL_MAX_LIST_NESTING));
	rb_define_const(module, "GL_MAX_MAP_TESSELLATION_NV", INT2NUM(GL_MAX_MAP_TESSELLATION_NV));
	rb_define_const(module, "GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB", INT2NUM(GL_MAX_MATRIX_PALETTE_STACK_DEPTH_ARB));
	rb_define_const(module, "GL_MAX_MODELVIEW_STACK_DEPTH", INT2NUM(GL_MAX_MODELVIEW_STACK_DEPTH));
	rb_define_const(module, "GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV", INT2NUM(GL_MAX_MULTISAMPLE_COVERAGE_MODES_NV));
	rb_define_const(module, "GL_MAX_NAME_STACK_DEPTH", INT2NUM(GL_MAX_NAME_STACK_DEPTH));
	rb_define_const(module, "GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT", INT2NUM(GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT));
	rb_define_const(module, "GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT", INT2NUM(GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT));
	rb_define_const(module, "GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT", INT2NUM(GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT));
	rb_define_const(module, "GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT", INT2NUM(GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT));
	rb_define_const(module, "GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT", INT2NUM(GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT));
	rb_define_const(module, "GL_MAX_PALETTE_MATRICES_ARB", INT2NUM(GL_MAX_PALETTE_MATRICES_ARB));
	rb_define_const(module, "GL_MAX_PIXEL_MAP_TABLE", INT2NUM(GL_MAX_PIXEL_MAP_TABLE));
	rb_define_const(module, "GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT", INT2NUM(GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT));
	rb_define_const(module, "GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI", INT2NUM(GL_MAX_PN_TRIANGLES_TESSELATION_LEVEL_ATI));
	rb_define_const(module, "GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB", INT2NUM(GL_MAX_PROGRAM_ADDRESS_REGISTERS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB", INT2NUM(GL_MAX_PROGRAM_ALU_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_ATTRIBS_ARB", INT2NUM(GL_MAX_PROGRAM_ATTRIBS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV", INT2NUM(GL_MAX_PROGRAM_ATTRIB_COMPONENTS_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_CALL_DEPTH_NV", INT2NUM(GL_MAX_PROGRAM_CALL_DEPTH_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_ENV_PARAMETERS_ARB", INT2NUM(GL_MAX_PROGRAM_ENV_PARAMETERS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV", INT2NUM(GL_MAX_PROGRAM_EXEC_INSTRUCTIONS_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV", INT2NUM(GL_MAX_PROGRAM_GENERIC_ATTRIBS_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_GENERIC_RESULTS_NV", INT2NUM(GL_MAX_PROGRAM_GENERIC_RESULTS_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_IF_DEPTH_NV", INT2NUM(GL_MAX_PROGRAM_IF_DEPTH_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_INSTRUCTIONS_ARB", INT2NUM(GL_MAX_PROGRAM_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB", INT2NUM(GL_MAX_PROGRAM_LOCAL_PARAMETERS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_LOOP_COUNT_NV", INT2NUM(GL_MAX_PROGRAM_LOOP_COUNT_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_LOOP_DEPTH_NV", INT2NUM(GL_MAX_PROGRAM_LOOP_DEPTH_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_MATRICES_ARB", INT2NUM(GL_MAX_PROGRAM_MATRICES_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB", INT2NUM(GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB", INT2NUM(GL_MAX_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB", INT2NUM(GL_MAX_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB", INT2NUM(GL_MAX_PROGRAM_NATIVE_ATTRIBS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB", INT2NUM(GL_MAX_PROGRAM_NATIVE_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB", INT2NUM(GL_MAX_PROGRAM_NATIVE_PARAMETERS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB", INT2NUM(GL_MAX_PROGRAM_NATIVE_TEMPORARIES_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB", INT2NUM(GL_MAX_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB", INT2NUM(GL_MAX_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_OUTPUT_VERTICES_NV", INT2NUM(GL_MAX_PROGRAM_OUTPUT_VERTICES_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_PARAMETERS_ARB", INT2NUM(GL_MAX_PROGRAM_PARAMETERS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV", INT2NUM(GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV", INT2NUM(GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_RESULT_COMPONENTS_NV", INT2NUM(GL_MAX_PROGRAM_RESULT_COMPONENTS_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_TEMPORARIES_ARB", INT2NUM(GL_MAX_PROGRAM_TEMPORARIES_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_TEXEL_OFFSET", INT2NUM(GL_MAX_PROGRAM_TEXEL_OFFSET));
	rb_define_const(module, "GL_MAX_PROGRAM_TEXEL_OFFSET_NV", INT2NUM(GL_MAX_PROGRAM_TEXEL_OFFSET_NV));
	rb_define_const(module, "GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB", INT2NUM(GL_MAX_PROGRAM_TEX_INDIRECTIONS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB", INT2NUM(GL_MAX_PROGRAM_TEX_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV", INT2NUM(GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV));
	rb_define_const(module, "GL_MAX_PROJECTION_STACK_DEPTH", INT2NUM(GL_MAX_PROJECTION_STACK_DEPTH));
	rb_define_const(module, "GL_MAX_RATIONAL_EVAL_ORDER_NV", INT2NUM(GL_MAX_RATIONAL_EVAL_ORDER_NV));
	rb_define_const(module, "GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB", INT2NUM(GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB));
	rb_define_const(module, "GL_MAX_RECTANGLE_TEXTURE_SIZE_NV", INT2NUM(GL_MAX_RECTANGLE_TEXTURE_SIZE_NV));
	rb_define_const(module, "GL_MAX_RENDERBUFFER_SIZE", INT2NUM(GL_MAX_RENDERBUFFER_SIZE));
	rb_define_const(module, "GL_MAX_RENDERBUFFER_SIZE_EXT", INT2NUM(GL_MAX_RENDERBUFFER_SIZE_EXT));
	rb_define_const(module, "GL_MAX_SAMPLES", INT2NUM(GL_MAX_SAMPLES));
	rb_define_const(module, "GL_MAX_SAMPLES_EXT", INT2NUM(GL_MAX_SAMPLES_EXT));
	rb_define_const(module, "GL_MAX_SAMPLE_MASK_WORDS_NV", INT2NUM(GL_MAX_SAMPLE_MASK_WORDS_NV));
	rb_define_const(module, "GL_MAX_SHININESS_NV", INT2NUM(GL_MAX_SHININESS_NV));
	rb_define_const(module, "GL_MAX_SPOT_EXPONENT_NV", INT2NUM(GL_MAX_SPOT_EXPONENT_NV));
	rb_define_const(module, "GL_MAX_TEXTURE_BUFFER_SIZE_ARB", INT2NUM(GL_MAX_TEXTURE_BUFFER_SIZE_ARB));
	rb_define_const(module, "GL_MAX_TEXTURE_BUFFER_SIZE_EXT", INT2NUM(GL_MAX_TEXTURE_BUFFER_SIZE_EXT));
	rb_define_const(module, "GL_MAX_TEXTURE_COORDS", INT2NUM(GL_MAX_TEXTURE_COORDS));
	rb_define_const(module, "GL_MAX_TEXTURE_COORDS_ARB", INT2NUM(GL_MAX_TEXTURE_COORDS_ARB));
	rb_define_const(module, "GL_MAX_TEXTURE_COORDS_NV", INT2NUM(GL_MAX_TEXTURE_COORDS_NV));
	rb_define_const(module, "GL_MAX_TEXTURE_IMAGE_UNITS", INT2NUM(GL_MAX_TEXTURE_IMAGE_UNITS));
	rb_define_const(module, "GL_MAX_TEXTURE_IMAGE_UNITS_ARB", INT2NUM(GL_MAX_TEXTURE_IMAGE_UNITS_ARB));
	rb_define_const(module, "GL_MAX_TEXTURE_IMAGE_UNITS_NV", INT2NUM(GL_MAX_TEXTURE_IMAGE_UNITS_NV));
	rb_define_const(module, "GL_MAX_TEXTURE_LOD_BIAS", INT2NUM(GL_MAX_TEXTURE_LOD_BIAS));
	rb_define_const(module, "GL_MAX_TEXTURE_LOD_BIAS_EXT", INT2NUM(GL_MAX_TEXTURE_LOD_BIAS_EXT));
	rb_define_const(module, "GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT", INT2NUM(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT));
	rb_define_const(module, "GL_MAX_TEXTURE_SIZE", INT2NUM(GL_MAX_TEXTURE_SIZE));
	rb_define_const(module, "GL_MAX_TEXTURE_STACK_DEPTH", INT2NUM(GL_MAX_TEXTURE_STACK_DEPTH));
	rb_define_const(module, "GL_MAX_TEXTURE_UNITS", INT2NUM(GL_MAX_TEXTURE_UNITS));
	rb_define_const(module, "GL_MAX_TEXTURE_UNITS_ARB", INT2NUM(GL_MAX_TEXTURE_UNITS_ARB));
	rb_define_const(module, "GL_MAX_TRACK_MATRICES_NV", INT2NUM(GL_MAX_TRACK_MATRICES_NV));
	rb_define_const(module, "GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV", INT2NUM(GL_MAX_TRACK_MATRIX_STACK_DEPTH_NV));
	rb_define_const(module, "GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV", INT2NUM(GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_ATTRIBS_NV));
	rb_define_const(module, "GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS", INT2NUM(GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS));
	rb_define_const(module, "GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT", INT2NUM(GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT));
	rb_define_const(module, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS", INT2NUM(GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS));
	rb_define_const(module, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT", INT2NUM(GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT));
	rb_define_const(module, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV", INT2NUM(GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_NV));
	rb_define_const(module, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS", INT2NUM(GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS));
	rb_define_const(module, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT", INT2NUM(GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT));
	rb_define_const(module, "GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV", INT2NUM(GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_NV));
	rb_define_const(module, "GL_MAX_VARYING_COMPONENTS", INT2NUM(GL_MAX_VARYING_COMPONENTS));
	rb_define_const(module, "GL_MAX_VARYING_COMPONENTS_EXT", INT2NUM(GL_MAX_VARYING_COMPONENTS_EXT));
	rb_define_const(module, "GL_MAX_VARYING_FLOATS", INT2NUM(GL_MAX_VARYING_FLOATS));
	rb_define_const(module, "GL_MAX_VARYING_FLOATS_ARB", INT2NUM(GL_MAX_VARYING_FLOATS_ARB));
	rb_define_const(module, "GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV", INT2NUM(GL_MAX_VERTEX_ARRAY_RANGE_ELEMENT_NV));
	rb_define_const(module, "GL_MAX_VERTEX_ATTRIBS", INT2NUM(GL_MAX_VERTEX_ATTRIBS));
	rb_define_const(module, "GL_MAX_VERTEX_ATTRIBS_ARB", INT2NUM(GL_MAX_VERTEX_ATTRIBS_ARB));
	rb_define_const(module, "GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT", INT2NUM(GL_MAX_VERTEX_BINDABLE_UNIFORMS_EXT));
	rb_define_const(module, "GL_MAX_VERTEX_HINT_PGI", INT2NUM(GL_MAX_VERTEX_HINT_PGI));
	rb_define_const(module, "GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT", INT2NUM(GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT));
	rb_define_const(module, "GL_MAX_VERTEX_SHADER_INVARIANTS_EXT", INT2NUM(GL_MAX_VERTEX_SHADER_INVARIANTS_EXT));
	rb_define_const(module, "GL_MAX_VERTEX_SHADER_LOCALS_EXT", INT2NUM(GL_MAX_VERTEX_SHADER_LOCALS_EXT));
	rb_define_const(module, "GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT", INT2NUM(GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT));
	rb_define_const(module, "GL_MAX_VERTEX_SHADER_VARIANTS_EXT", INT2NUM(GL_MAX_VERTEX_SHADER_VARIANTS_EXT));
	rb_define_const(module, "GL_MAX_VERTEX_STREAMS_ATI", INT2NUM(GL_MAX_VERTEX_STREAMS_ATI));
	rb_define_const(module, "GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS", INT2NUM(GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS));
	rb_define_const(module, "GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB", INT2NUM(GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB));
	rb_define_const(module, "GL_MAX_VERTEX_UNIFORM_COMPONENTS", INT2NUM(GL_MAX_VERTEX_UNIFORM_COMPONENTS));
	rb_define_const(module, "GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB", INT2NUM(GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB));
	rb_define_const(module, "GL_MAX_VERTEX_UNITS_ARB", INT2NUM(GL_MAX_VERTEX_UNITS_ARB));
	rb_define_const(module, "GL_MAX_VERTEX_VARYING_COMPONENTS_ARB", INT2NUM(GL_MAX_VERTEX_VARYING_COMPONENTS_ARB));
	rb_define_const(module, "GL_MAX_VERTEX_VARYING_COMPONENTS_EXT", INT2NUM(GL_MAX_VERTEX_VARYING_COMPONENTS_EXT));
	rb_define_const(module, "GL_MAX_VIEWPORT_DIMS", INT2NUM(GL_MAX_VIEWPORT_DIMS));
	rb_define_const(module, "GL_MIN", INT2NUM(GL_MIN));
	rb_define_const(module, "GL_MINMAX", INT2NUM(GL_MINMAX));
	rb_define_const(module, "GL_MINMAX_EXT", INT2NUM(GL_MINMAX_EXT));
	rb_define_const(module, "GL_MINMAX_FORMAT", INT2NUM(GL_MINMAX_FORMAT));
	rb_define_const(module, "GL_MINMAX_FORMAT_EXT", INT2NUM(GL_MINMAX_FORMAT_EXT));
	rb_define_const(module, "GL_MINMAX_SINK", INT2NUM(GL_MINMAX_SINK));
	rb_define_const(module, "GL_MINMAX_SINK_EXT", INT2NUM(GL_MINMAX_SINK_EXT));
	rb_define_const(module, "GL_MINOR_VERSION", INT2NUM(GL_MINOR_VERSION));
	rb_define_const(module, "GL_MIN_EXT", INT2NUM(GL_MIN_EXT));
	rb_define_const(module, "GL_MIN_PROGRAM_TEXEL_OFFSET", INT2NUM(GL_MIN_PROGRAM_TEXEL_OFFSET));
	rb_define_const(module, "GL_MIN_PROGRAM_TEXEL_OFFSET_NV", INT2NUM(GL_MIN_PROGRAM_TEXEL_OFFSET_NV));
	rb_define_const(module, "GL_MIRRORED_REPEAT", INT2NUM(GL_MIRRORED_REPEAT));
	rb_define_const(module, "GL_MIRRORED_REPEAT_ARB", INT2NUM(GL_MIRRORED_REPEAT_ARB));
	rb_define_const(module, "GL_MIRRORED_REPEAT_IBM", INT2NUM(GL_MIRRORED_REPEAT_IBM));
	rb_define_const(module, "GL_MIRROR_CLAMP_ATI", INT2NUM(GL_MIRROR_CLAMP_ATI));
	rb_define_const(module, "GL_MIRROR_CLAMP_EXT", INT2NUM(GL_MIRROR_CLAMP_EXT));
	rb_define_const(module, "GL_MIRROR_CLAMP_TO_BORDER_EXT", INT2NUM(GL_MIRROR_CLAMP_TO_BORDER_EXT));
	rb_define_const(module, "GL_MIRROR_CLAMP_TO_EDGE_ATI", INT2NUM(GL_MIRROR_CLAMP_TO_EDGE_ATI));
	rb_define_const(module, "GL_MIRROR_CLAMP_TO_EDGE_EXT", INT2NUM(GL_MIRROR_CLAMP_TO_EDGE_EXT));
	rb_define_const(module, "GL_MODELVIEW", INT2NUM(GL_MODELVIEW));
	rb_define_const(module, "GL_MODELVIEW0_ARB", INT2NUM(GL_MODELVIEW0_ARB));
	rb_define_const(module, "GL_MODELVIEW0_EXT", INT2NUM(GL_MODELVIEW0_EXT));
	rb_define_const(module, "GL_MODELVIEW0_MATRIX_EXT", INT2NUM(GL_MODELVIEW0_MATRIX_EXT));
	rb_define_const(module, "GL_MODELVIEW0_STACK_DEPTH_EXT", INT2NUM(GL_MODELVIEW0_STACK_DEPTH_EXT));
	rb_define_const(module, "GL_MODELVIEW10_ARB", INT2NUM(GL_MODELVIEW10_ARB));
	rb_define_const(module, "GL_MODELVIEW11_ARB", INT2NUM(GL_MODELVIEW11_ARB));
	rb_define_const(module, "GL_MODELVIEW12_ARB", INT2NUM(GL_MODELVIEW12_ARB));
	rb_define_const(module, "GL_MODELVIEW13_ARB", INT2NUM(GL_MODELVIEW13_ARB));
	rb_define_const(module, "GL_MODELVIEW14_ARB", INT2NUM(GL_MODELVIEW14_ARB));
	rb_define_const(module, "GL_MODELVIEW15_ARB", INT2NUM(GL_MODELVIEW15_ARB));
	rb_define_const(module, "GL_MODELVIEW16_ARB", INT2NUM(GL_MODELVIEW16_ARB));
	rb_define_const(module, "GL_MODELVIEW17_ARB", INT2NUM(GL_MODELVIEW17_ARB));
	rb_define_const(module, "GL_MODELVIEW18_ARB", INT2NUM(GL_MODELVIEW18_ARB));
	rb_define_const(module, "GL_MODELVIEW19_ARB", INT2NUM(GL_MODELVIEW19_ARB));
	rb_define_const(module, "GL_MODELVIEW1_ARB", INT2NUM(GL_MODELVIEW1_ARB));
	rb_define_const(module, "GL_MODELVIEW1_EXT", INT2NUM(GL_MODELVIEW1_EXT));
	rb_define_const(module, "GL_MODELVIEW1_MATRIX_EXT", INT2NUM(GL_MODELVIEW1_MATRIX_EXT));
	rb_define_const(module, "GL_MODELVIEW1_STACK_DEPTH_EXT", INT2NUM(GL_MODELVIEW1_STACK_DEPTH_EXT));
	rb_define_const(module, "GL_MODELVIEW20_ARB", INT2NUM(GL_MODELVIEW20_ARB));
	rb_define_const(module, "GL_MODELVIEW21_ARB", INT2NUM(GL_MODELVIEW21_ARB));
	rb_define_const(module, "GL_MODELVIEW22_ARB", INT2NUM(GL_MODELVIEW22_ARB));
	rb_define_const(module, "GL_MODELVIEW23_ARB", INT2NUM(GL_MODELVIEW23_ARB));
	rb_define_const(module, "GL_MODELVIEW24_ARB", INT2NUM(GL_MODELVIEW24_ARB));
	rb_define_const(module, "GL_MODELVIEW25_ARB", INT2NUM(GL_MODELVIEW25_ARB));
	rb_define_const(module, "GL_MODELVIEW26_ARB", INT2NUM(GL_MODELVIEW26_ARB));
	rb_define_const(module, "GL_MODELVIEW27_ARB", INT2NUM(GL_MODELVIEW27_ARB));
	rb_define_const(module, "GL_MODELVIEW28_ARB", INT2NUM(GL_MODELVIEW28_ARB));
	rb_define_const(module, "GL_MODELVIEW29_ARB", INT2NUM(GL_MODELVIEW29_ARB));
	rb_define_const(module, "GL_MODELVIEW2_ARB", INT2NUM(GL_MODELVIEW2_ARB));
	rb_define_const(module, "GL_MODELVIEW30_ARB", INT2NUM(GL_MODELVIEW30_ARB));
	rb_define_const(module, "GL_MODELVIEW31_ARB", INT2NUM(GL_MODELVIEW31_ARB));
	rb_define_const(module, "GL_MODELVIEW3_ARB", INT2NUM(GL_MODELVIEW3_ARB));
	rb_define_const(module, "GL_MODELVIEW4_ARB", INT2NUM(GL_MODELVIEW4_ARB));
	rb_define_const(module, "GL_MODELVIEW5_ARB", INT2NUM(GL_MODELVIEW5_ARB));
	rb_define_const(module, "GL_MODELVIEW6_ARB", INT2NUM(GL_MODELVIEW6_ARB));
	rb_define_const(module, "GL_MODELVIEW7_ARB", INT2NUM(GL_MODELVIEW7_ARB));
	rb_define_const(module, "GL_MODELVIEW8_ARB", INT2NUM(GL_MODELVIEW8_ARB));
	rb_define_const(module, "GL_MODELVIEW9_ARB", INT2NUM(GL_MODELVIEW9_ARB));
	rb_define_const(module, "GL_MODELVIEW_MATRIX", INT2NUM(GL_MODELVIEW_MATRIX));
	rb_define_const(module, "GL_MODELVIEW_PROJECTION_NV", INT2NUM(GL_MODELVIEW_PROJECTION_NV));
	rb_define_const(module, "GL_MODELVIEW_STACK_DEPTH", INT2NUM(GL_MODELVIEW_STACK_DEPTH));
	rb_define_const(module, "GL_MODULATE", INT2NUM(GL_MODULATE));
	rb_define_const(module, "GL_MODULATE_ADD_ATI", INT2NUM(GL_MODULATE_ADD_ATI));
	rb_define_const(module, "GL_MODULATE_SIGNED_ADD_ATI", INT2NUM(GL_MODULATE_SIGNED_ADD_ATI));
	rb_define_const(module, "GL_MODULATE_SUBTRACT_ATI", INT2NUM(GL_MODULATE_SUBTRACT_ATI));
	rb_define_const(module, "GL_MOV_ATI", INT2NUM(GL_MOV_ATI));
	rb_define_const(module, "GL_MULT", INT2NUM(GL_MULT));
	rb_define_const(module, "GL_MULTISAMPLE", INT2NUM(GL_MULTISAMPLE));
	rb_define_const(module, "GL_MULTISAMPLE_3DFX", INT2NUM(GL_MULTISAMPLE_3DFX));
	rb_define_const(module, "GL_MULTISAMPLE_ARB", INT2NUM(GL_MULTISAMPLE_ARB));
	rb_define_const(module, "GL_MULTISAMPLE_BIT", INT2NUM(GL_MULTISAMPLE_BIT));
	rb_define_const(module, "GL_MULTISAMPLE_BIT_3DFX", INT2NUM(GL_MULTISAMPLE_BIT_3DFX));
	rb_define_const(module, "GL_MULTISAMPLE_BIT_ARB", INT2NUM(GL_MULTISAMPLE_BIT_ARB));
	rb_define_const(module, "GL_MULTISAMPLE_BIT_EXT", INT2NUM(GL_MULTISAMPLE_BIT_EXT));
	rb_define_const(module, "GL_MULTISAMPLE_COVERAGE_MODES_NV", INT2NUM(GL_MULTISAMPLE_COVERAGE_MODES_NV));
	rb_define_const(module, "GL_MULTISAMPLE_EXT", INT2NUM(GL_MULTISAMPLE_EXT));
	rb_define_const(module, "GL_MULTISAMPLE_FILTER_HINT_NV", INT2NUM(GL_MULTISAMPLE_FILTER_HINT_NV));
	rb_define_const(module, "GL_MULTISAMPLE_SGIS", INT2NUM(GL_MULTISAMPLE_SGIS));
	rb_define_const(module, "GL_MUL_ATI", INT2NUM(GL_MUL_ATI));
	rb_define_const(module, "GL_MVP_MATRIX_EXT", INT2NUM(GL_MVP_MATRIX_EXT));
	rb_define_const(module, "GL_N3F_V3F", INT2NUM(GL_N3F_V3F));
	rb_define_const(module, "GL_NAME_STACK_DEPTH", INT2NUM(GL_NAME_STACK_DEPTH));
	rb_define_const(module, "GL_NAND", INT2NUM(GL_NAND));
	rb_define_const(module, "GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI", INT2NUM(GL_NATIVE_GRAPHICS_BEGIN_HINT_PGI));
	rb_define_const(module, "GL_NATIVE_GRAPHICS_END_HINT_PGI", INT2NUM(GL_NATIVE_GRAPHICS_END_HINT_PGI));
	rb_define_const(module, "GL_NATIVE_GRAPHICS_HANDLE_PGI", INT2NUM(GL_NATIVE_GRAPHICS_HANDLE_PGI));
	rb_define_const(module, "GL_NEAREST", INT2NUM(GL_NEAREST));
	rb_define_const(module, "GL_NEAREST_CLIPMAP_LINEAR_SGIX", INT2NUM(GL_NEAREST_CLIPMAP_LINEAR_SGIX));
	rb_define_const(module, "GL_NEAREST_CLIPMAP_NEAREST_SGIX", INT2NUM(GL_NEAREST_CLIPMAP_NEAREST_SGIX));
	rb_define_const(module, "GL_NEAREST_MIPMAP_LINEAR", INT2NUM(GL_NEAREST_MIPMAP_LINEAR));
	rb_define_const(module, "GL_NEAREST_MIPMAP_NEAREST", INT2NUM(GL_NEAREST_MIPMAP_NEAREST));
	rb_define_const(module, "GL_NEGATE_BIT_ATI", INT2NUM(GL_NEGATE_BIT_ATI));
	rb_define_const(module, "GL_NEGATIVE_ONE_EXT", INT2NUM(GL_NEGATIVE_ONE_EXT));
	rb_define_const(module, "GL_NEGATIVE_W_EXT", INT2NUM(GL_NEGATIVE_W_EXT));
	rb_define_const(module, "GL_NEGATIVE_X_EXT", INT2NUM(GL_NEGATIVE_X_EXT));
	rb_define_const(module, "GL_NEGATIVE_Y_EXT", INT2NUM(GL_NEGATIVE_Y_EXT));
	rb_define_const(module, "GL_NEGATIVE_Z_EXT", INT2NUM(GL_NEGATIVE_Z_EXT));
	rb_define_const(module, "GL_NEVER", INT2NUM(GL_NEVER));
	rb_define_const(module, "GL_NICEST", INT2NUM(GL_NICEST));
	rb_define_const(module, "GL_NONE", INT2NUM(GL_NONE));
	rb_define_const(module, "GL_NOOP", INT2NUM(GL_NOOP));
	rb_define_const(module, "GL_NOR", INT2NUM(GL_NOR));
	rb_define_const(module, "GL_NORMALIZE", INT2NUM(GL_NORMALIZE));
	rb_define_const(module, "GL_NORMALIZED_RANGE_EXT", INT2NUM(GL_NORMALIZED_RANGE_EXT));
	rb_define_const(module, "GL_NORMAL_ARRAY", INT2NUM(GL_NORMAL_ARRAY));
	rb_define_const(module, "GL_NORMAL_ARRAY_BUFFER_BINDING", INT2NUM(GL_NORMAL_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_NORMAL_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_NORMAL_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_NORMAL_ARRAY_COUNT_EXT", INT2NUM(GL_NORMAL_ARRAY_COUNT_EXT));
	rb_define_const(module, "GL_NORMAL_ARRAY_EXT", INT2NUM(GL_NORMAL_ARRAY_EXT));
	rb_define_const(module, "GL_NORMAL_ARRAY_LIST_IBM", INT2NUM(GL_NORMAL_ARRAY_LIST_IBM));
	rb_define_const(module, "GL_NORMAL_ARRAY_LIST_STRIDE_IBM", INT2NUM(GL_NORMAL_ARRAY_LIST_STRIDE_IBM));
	rb_define_const(module, "GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL", INT2NUM(GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL));
	rb_define_const(module, "GL_NORMAL_ARRAY_POINTER", INT2NUM(GL_NORMAL_ARRAY_POINTER));
	rb_define_const(module, "GL_NORMAL_ARRAY_POINTER_EXT", INT2NUM(GL_NORMAL_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_NORMAL_ARRAY_STRIDE", INT2NUM(GL_NORMAL_ARRAY_STRIDE));
	rb_define_const(module, "GL_NORMAL_ARRAY_STRIDE_EXT", INT2NUM(GL_NORMAL_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_NORMAL_ARRAY_TYPE", INT2NUM(GL_NORMAL_ARRAY_TYPE));
	rb_define_const(module, "GL_NORMAL_ARRAY_TYPE_EXT", INT2NUM(GL_NORMAL_ARRAY_TYPE_EXT));
	rb_define_const(module, "GL_NORMAL_BIT_PGI", INT2NUM(GL_NORMAL_BIT_PGI));
	rb_define_const(module, "GL_NORMAL_MAP", INT2NUM(GL_NORMAL_MAP));
	rb_define_const(module, "GL_NORMAL_MAP_ARB", INT2NUM(GL_NORMAL_MAP_ARB));
	rb_define_const(module, "GL_NORMAL_MAP_EXT", INT2NUM(GL_NORMAL_MAP_EXT));
	rb_define_const(module, "GL_NORMAL_MAP_NV", INT2NUM(GL_NORMAL_MAP_NV));
	rb_define_const(module, "GL_NOTEQUAL", INT2NUM(GL_NOTEQUAL));
	rb_define_const(module, "GL_NO_ERROR", INT2NUM(GL_NO_ERROR));
	rb_define_const(module, "GL_NUM_COMPRESSED_TEXTURE_FORMATS", INT2NUM(GL_NUM_COMPRESSED_TEXTURE_FORMATS));
	rb_define_const(module, "GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB", INT2NUM(GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB));
	rb_define_const(module, "GL_NUM_EXTENSIONS", INT2NUM(GL_NUM_EXTENSIONS));
	rb_define_const(module, "GL_NUM_FILL_STREAMS_NV", INT2NUM(GL_NUM_FILL_STREAMS_NV));
	rb_define_const(module, "GL_NUM_FRAGMENT_CONSTANTS_ATI", INT2NUM(GL_NUM_FRAGMENT_CONSTANTS_ATI));
	rb_define_const(module, "GL_NUM_FRAGMENT_REGISTERS_ATI", INT2NUM(GL_NUM_FRAGMENT_REGISTERS_ATI));
	rb_define_const(module, "GL_NUM_GENERAL_COMBINERS_NV", INT2NUM(GL_NUM_GENERAL_COMBINERS_NV));
	rb_define_const(module, "GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI", INT2NUM(GL_NUM_INPUT_INTERPOLATOR_COMPONENTS_ATI));
	rb_define_const(module, "GL_NUM_INSTRUCTIONS_PER_PASS_ATI", INT2NUM(GL_NUM_INSTRUCTIONS_PER_PASS_ATI));
	rb_define_const(module, "GL_NUM_INSTRUCTIONS_TOTAL_ATI", INT2NUM(GL_NUM_INSTRUCTIONS_TOTAL_ATI));
	rb_define_const(module, "GL_NUM_LOOPBACK_COMPONENTS_ATI", INT2NUM(GL_NUM_LOOPBACK_COMPONENTS_ATI));
	rb_define_const(module, "GL_NUM_PASSES_ATI", INT2NUM(GL_NUM_PASSES_ATI));
	rb_define_const(module, "GL_OBJECT_ACTIVE_ATTRIBUTES_ARB", INT2NUM(GL_OBJECT_ACTIVE_ATTRIBUTES_ARB));
	rb_define_const(module, "GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB", INT2NUM(GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB));
	rb_define_const(module, "GL_OBJECT_ACTIVE_UNIFORMS_ARB", INT2NUM(GL_OBJECT_ACTIVE_UNIFORMS_ARB));
	rb_define_const(module, "GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB", INT2NUM(GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB));
	rb_define_const(module, "GL_OBJECT_ATTACHED_OBJECTS_ARB", INT2NUM(GL_OBJECT_ATTACHED_OBJECTS_ARB));
	rb_define_const(module, "GL_OBJECT_BUFFER_SIZE_ATI", INT2NUM(GL_OBJECT_BUFFER_SIZE_ATI));
	rb_define_const(module, "GL_OBJECT_BUFFER_USAGE_ATI", INT2NUM(GL_OBJECT_BUFFER_USAGE_ATI));
	rb_define_const(module, "GL_OBJECT_COMPILE_STATUS_ARB", INT2NUM(GL_OBJECT_COMPILE_STATUS_ARB));
	rb_define_const(module, "GL_OBJECT_DELETE_STATUS_ARB", INT2NUM(GL_OBJECT_DELETE_STATUS_ARB));
	rb_define_const(module, "GL_OBJECT_DISTANCE_TO_LINE_SGIS", INT2NUM(GL_OBJECT_DISTANCE_TO_LINE_SGIS));
	rb_define_const(module, "GL_OBJECT_DISTANCE_TO_POINT_SGIS", INT2NUM(GL_OBJECT_DISTANCE_TO_POINT_SGIS));
	rb_define_const(module, "GL_OBJECT_INFO_LOG_LENGTH_ARB", INT2NUM(GL_OBJECT_INFO_LOG_LENGTH_ARB));
	rb_define_const(module, "GL_OBJECT_LINEAR", INT2NUM(GL_OBJECT_LINEAR));
	rb_define_const(module, "GL_OBJECT_LINE_SGIS", INT2NUM(GL_OBJECT_LINE_SGIS));
	rb_define_const(module, "GL_OBJECT_LINK_STATUS_ARB", INT2NUM(GL_OBJECT_LINK_STATUS_ARB));
	rb_define_const(module, "GL_OBJECT_PLANE", INT2NUM(GL_OBJECT_PLANE));
	rb_define_const(module, "GL_OBJECT_POINT_SGIS", INT2NUM(GL_OBJECT_POINT_SGIS));
	rb_define_const(module, "GL_OBJECT_SHADER_SOURCE_LENGTH_ARB", INT2NUM(GL_OBJECT_SHADER_SOURCE_LENGTH_ARB));
	rb_define_const(module, "GL_OBJECT_SUBTYPE_ARB", INT2NUM(GL_OBJECT_SUBTYPE_ARB));
	rb_define_const(module, "GL_OBJECT_TYPE_ARB", INT2NUM(GL_OBJECT_TYPE_ARB));
	rb_define_const(module, "GL_OBJECT_VALIDATE_STATUS_ARB", INT2NUM(GL_OBJECT_VALIDATE_STATUS_ARB));
	rb_define_const(module, "GL_OCCLUSION_TEST_HP", INT2NUM(GL_OCCLUSION_TEST_HP));
	rb_define_const(module, "GL_OCCLUSION_TEST_RESULT_HP", INT2NUM(GL_OCCLUSION_TEST_RESULT_HP));
	rb_define_const(module, "GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV", INT2NUM(GL_OFFSET_HILO_PROJECTIVE_TEXTURE_2D_NV));
	rb_define_const(module, "GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV", INT2NUM(GL_OFFSET_HILO_PROJECTIVE_TEXTURE_RECTANGLE_NV));
	rb_define_const(module, "GL_OFFSET_HILO_TEXTURE_2D_NV", INT2NUM(GL_OFFSET_HILO_TEXTURE_2D_NV));
	rb_define_const(module, "GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV", INT2NUM(GL_OFFSET_HILO_TEXTURE_RECTANGLE_NV));
	rb_define_const(module, "GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV", INT2NUM(GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV));
	rb_define_const(module, "GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV", INT2NUM(GL_OFFSET_PROJECTIVE_TEXTURE_2D_SCALE_NV));
	rb_define_const(module, "GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV", INT2NUM(GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_NV));
	rb_define_const(module, "GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV", INT2NUM(GL_OFFSET_PROJECTIVE_TEXTURE_RECTANGLE_SCALE_NV));
	rb_define_const(module, "GL_OFFSET_TEXTURE_2D_BIAS_NV", INT2NUM(GL_OFFSET_TEXTURE_2D_BIAS_NV));
	rb_define_const(module, "GL_OFFSET_TEXTURE_2D_MATRIX_NV", INT2NUM(GL_OFFSET_TEXTURE_2D_MATRIX_NV));
	rb_define_const(module, "GL_OFFSET_TEXTURE_2D_NV", INT2NUM(GL_OFFSET_TEXTURE_2D_NV));
	rb_define_const(module, "GL_OFFSET_TEXTURE_2D_SCALE_NV", INT2NUM(GL_OFFSET_TEXTURE_2D_SCALE_NV));
	rb_define_const(module, "GL_OFFSET_TEXTURE_BIAS_NV", INT2NUM(GL_OFFSET_TEXTURE_BIAS_NV));
	rb_define_const(module, "GL_OFFSET_TEXTURE_MATRIX_NV", INT2NUM(GL_OFFSET_TEXTURE_MATRIX_NV));
	rb_define_const(module, "GL_OFFSET_TEXTURE_RECTANGLE_NV", INT2NUM(GL_OFFSET_TEXTURE_RECTANGLE_NV));
	rb_define_const(module, "GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV", INT2NUM(GL_OFFSET_TEXTURE_RECTANGLE_SCALE_NV));
	rb_define_const(module, "GL_OFFSET_TEXTURE_SCALE_NV", INT2NUM(GL_OFFSET_TEXTURE_SCALE_NV));
	rb_define_const(module, "GL_ONE", INT2NUM(GL_ONE));
	rb_define_const(module, "GL_ONE_EXT", INT2NUM(GL_ONE_EXT));
	rb_define_const(module, "GL_ONE_MINUS_CONSTANT_ALPHA", INT2NUM(GL_ONE_MINUS_CONSTANT_ALPHA));
	rb_define_const(module, "GL_ONE_MINUS_CONSTANT_ALPHA_EXT", INT2NUM(GL_ONE_MINUS_CONSTANT_ALPHA_EXT));
	rb_define_const(module, "GL_ONE_MINUS_CONSTANT_COLOR", INT2NUM(GL_ONE_MINUS_CONSTANT_COLOR));
	rb_define_const(module, "GL_ONE_MINUS_CONSTANT_COLOR_EXT", INT2NUM(GL_ONE_MINUS_CONSTANT_COLOR_EXT));
	rb_define_const(module, "GL_ONE_MINUS_DST_ALPHA", INT2NUM(GL_ONE_MINUS_DST_ALPHA));
	rb_define_const(module, "GL_ONE_MINUS_DST_COLOR", INT2NUM(GL_ONE_MINUS_DST_COLOR));
	rb_define_const(module, "GL_ONE_MINUS_SRC_ALPHA", INT2NUM(GL_ONE_MINUS_SRC_ALPHA));
	rb_define_const(module, "GL_ONE_MINUS_SRC_COLOR", INT2NUM(GL_ONE_MINUS_SRC_COLOR));
	rb_define_const(module, "GL_OPERAND0_ALPHA", INT2NUM(GL_OPERAND0_ALPHA));
	rb_define_const(module, "GL_OPERAND0_ALPHA_ARB", INT2NUM(GL_OPERAND0_ALPHA_ARB));
	rb_define_const(module, "GL_OPERAND0_ALPHA_EXT", INT2NUM(GL_OPERAND0_ALPHA_EXT));
	rb_define_const(module, "GL_OPERAND0_RGB", INT2NUM(GL_OPERAND0_RGB));
	rb_define_const(module, "GL_OPERAND0_RGB_ARB", INT2NUM(GL_OPERAND0_RGB_ARB));
	rb_define_const(module, "GL_OPERAND0_RGB_EXT", INT2NUM(GL_OPERAND0_RGB_EXT));
	rb_define_const(module, "GL_OPERAND1_ALPHA", INT2NUM(GL_OPERAND1_ALPHA));
	rb_define_const(module, "GL_OPERAND1_ALPHA_ARB", INT2NUM(GL_OPERAND1_ALPHA_ARB));
	rb_define_const(module, "GL_OPERAND1_ALPHA_EXT", INT2NUM(GL_OPERAND1_ALPHA_EXT));
	rb_define_const(module, "GL_OPERAND1_RGB", INT2NUM(GL_OPERAND1_RGB));
	rb_define_const(module, "GL_OPERAND1_RGB_ARB", INT2NUM(GL_OPERAND1_RGB_ARB));
	rb_define_const(module, "GL_OPERAND1_RGB_EXT", INT2NUM(GL_OPERAND1_RGB_EXT));
	rb_define_const(module, "GL_OPERAND2_ALPHA", INT2NUM(GL_OPERAND2_ALPHA));
	rb_define_const(module, "GL_OPERAND2_ALPHA_ARB", INT2NUM(GL_OPERAND2_ALPHA_ARB));
	rb_define_const(module, "GL_OPERAND2_ALPHA_EXT", INT2NUM(GL_OPERAND2_ALPHA_EXT));
	rb_define_const(module, "GL_OPERAND2_RGB", INT2NUM(GL_OPERAND2_RGB));
	rb_define_const(module, "GL_OPERAND2_RGB_ARB", INT2NUM(GL_OPERAND2_RGB_ARB));
	rb_define_const(module, "GL_OPERAND2_RGB_EXT", INT2NUM(GL_OPERAND2_RGB_EXT));
	rb_define_const(module, "GL_OPERAND3_ALPHA_NV", INT2NUM(GL_OPERAND3_ALPHA_NV));
	rb_define_const(module, "GL_OPERAND3_RGB_NV", INT2NUM(GL_OPERAND3_RGB_NV));
	rb_define_const(module, "GL_OP_ADD_EXT", INT2NUM(GL_OP_ADD_EXT));
	rb_define_const(module, "GL_OP_CLAMP_EXT", INT2NUM(GL_OP_CLAMP_EXT));
	rb_define_const(module, "GL_OP_CROSS_PRODUCT_EXT", INT2NUM(GL_OP_CROSS_PRODUCT_EXT));
	rb_define_const(module, "GL_OP_DOT3_EXT", INT2NUM(GL_OP_DOT3_EXT));
	rb_define_const(module, "GL_OP_DOT4_EXT", INT2NUM(GL_OP_DOT4_EXT));
	rb_define_const(module, "GL_OP_EXP_BASE_2_EXT", INT2NUM(GL_OP_EXP_BASE_2_EXT));
	rb_define_const(module, "GL_OP_FLOOR_EXT", INT2NUM(GL_OP_FLOOR_EXT));
	rb_define_const(module, "GL_OP_FRAC_EXT", INT2NUM(GL_OP_FRAC_EXT));
	rb_define_const(module, "GL_OP_INDEX_EXT", INT2NUM(GL_OP_INDEX_EXT));
	rb_define_const(module, "GL_OP_LOG_BASE_2_EXT", INT2NUM(GL_OP_LOG_BASE_2_EXT));
	rb_define_const(module, "GL_OP_MADD_EXT", INT2NUM(GL_OP_MADD_EXT));
	rb_define_const(module, "GL_OP_MAX_EXT", INT2NUM(GL_OP_MAX_EXT));
	rb_define_const(module, "GL_OP_MIN_EXT", INT2NUM(GL_OP_MIN_EXT));
	rb_define_const(module, "GL_OP_MOV_EXT", INT2NUM(GL_OP_MOV_EXT));
	rb_define_const(module, "GL_OP_MULTIPLY_MATRIX_EXT", INT2NUM(GL_OP_MULTIPLY_MATRIX_EXT));
	rb_define_const(module, "GL_OP_MUL_EXT", INT2NUM(GL_OP_MUL_EXT));
	rb_define_const(module, "GL_OP_NEGATE_EXT", INT2NUM(GL_OP_NEGATE_EXT));
	rb_define_const(module, "GL_OP_POWER_EXT", INT2NUM(GL_OP_POWER_EXT));
	rb_define_const(module, "GL_OP_RECIP_EXT", INT2NUM(GL_OP_RECIP_EXT));
	rb_define_const(module, "GL_OP_RECIP_SQRT_EXT", INT2NUM(GL_OP_RECIP_SQRT_EXT));
	rb_define_const(module, "GL_OP_ROUND_EXT", INT2NUM(GL_OP_ROUND_EXT));
	rb_define_const(module, "GL_OP_SET_GE_EXT", INT2NUM(GL_OP_SET_GE_EXT));
	rb_define_const(module, "GL_OP_SET_LT_EXT", INT2NUM(GL_OP_SET_LT_EXT));
	rb_define_const(module, "GL_OP_SUB_EXT", INT2NUM(GL_OP_SUB_EXT));
	rb_define_const(module, "GL_OR", INT2NUM(GL_OR));
	rb_define_const(module, "GL_ORDER", INT2NUM(GL_ORDER));
	rb_define_const(module, "GL_OR_INVERTED", INT2NUM(GL_OR_INVERTED));
	rb_define_const(module, "GL_OR_REVERSE", INT2NUM(GL_OR_REVERSE));
	rb_define_const(module, "GL_OUTPUT_COLOR0_EXT", INT2NUM(GL_OUTPUT_COLOR0_EXT));
	rb_define_const(module, "GL_OUTPUT_COLOR1_EXT", INT2NUM(GL_OUTPUT_COLOR1_EXT));
	rb_define_const(module, "GL_OUTPUT_FOG_EXT", INT2NUM(GL_OUTPUT_FOG_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD0_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD0_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD10_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD10_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD11_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD11_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD12_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD12_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD13_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD13_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD14_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD14_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD15_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD15_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD16_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD16_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD17_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD17_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD18_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD18_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD19_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD19_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD1_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD1_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD20_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD20_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD21_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD21_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD22_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD22_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD23_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD23_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD24_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD24_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD25_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD25_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD26_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD26_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD27_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD27_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD28_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD28_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD29_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD29_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD2_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD2_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD30_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD30_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD31_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD31_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD3_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD3_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD4_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD4_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD5_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD5_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD6_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD6_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD7_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD7_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD8_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD8_EXT));
	rb_define_const(module, "GL_OUTPUT_TEXTURE_COORD9_EXT", INT2NUM(GL_OUTPUT_TEXTURE_COORD9_EXT));
	rb_define_const(module, "GL_OUTPUT_VERTEX_EXT", INT2NUM(GL_OUTPUT_VERTEX_EXT));
	rb_define_const(module, "GL_OUT_OF_MEMORY", INT2NUM(GL_OUT_OF_MEMORY));
	rb_define_const(module, "GL_PACK_ALIGNMENT", INT2NUM(GL_PACK_ALIGNMENT));
	rb_define_const(module, "GL_PACK_CMYK_HINT_EXT", INT2NUM(GL_PACK_CMYK_HINT_EXT));
	rb_define_const(module, "GL_PACK_IMAGE_DEPTH_SGIS", INT2NUM(GL_PACK_IMAGE_DEPTH_SGIS));
	rb_define_const(module, "GL_PACK_IMAGE_HEIGHT", INT2NUM(GL_PACK_IMAGE_HEIGHT));
	rb_define_const(module, "GL_PACK_IMAGE_HEIGHT_EXT", INT2NUM(GL_PACK_IMAGE_HEIGHT_EXT));
	rb_define_const(module, "GL_PACK_INVERT_MESA", INT2NUM(GL_PACK_INVERT_MESA));
	rb_define_const(module, "GL_PACK_LSB_FIRST", INT2NUM(GL_PACK_LSB_FIRST));
	rb_define_const(module, "GL_PACK_RESAMPLE_OML", INT2NUM(GL_PACK_RESAMPLE_OML));
	rb_define_const(module, "GL_PACK_RESAMPLE_SGIX", INT2NUM(GL_PACK_RESAMPLE_SGIX));
	rb_define_const(module, "GL_PACK_ROW_LENGTH", INT2NUM(GL_PACK_ROW_LENGTH));
	rb_define_const(module, "GL_PACK_SKIP_IMAGES", INT2NUM(GL_PACK_SKIP_IMAGES));
	rb_define_const(module, "GL_PACK_SKIP_IMAGES_EXT", INT2NUM(GL_PACK_SKIP_IMAGES_EXT));
	rb_define_const(module, "GL_PACK_SKIP_PIXELS", INT2NUM(GL_PACK_SKIP_PIXELS));
	rb_define_const(module, "GL_PACK_SKIP_ROWS", INT2NUM(GL_PACK_SKIP_ROWS));
	rb_define_const(module, "GL_PACK_SKIP_VOLUMES_SGIS", INT2NUM(GL_PACK_SKIP_VOLUMES_SGIS));
	rb_define_const(module, "GL_PACK_SUBSAMPLE_RATE_SGIX", INT2NUM(GL_PACK_SUBSAMPLE_RATE_SGIX));
	rb_define_const(module, "GL_PACK_SWAP_BYTES", INT2NUM(GL_PACK_SWAP_BYTES));
	rb_define_const(module, "GL_PARALLEL_ARRAYS_INTEL", INT2NUM(GL_PARALLEL_ARRAYS_INTEL));
	rb_define_const(module, "GL_PASS_THROUGH_NV", INT2NUM(GL_PASS_THROUGH_NV));
	rb_define_const(module, "GL_PASS_THROUGH_TOKEN", INT2NUM(GL_PASS_THROUGH_TOKEN));
	rb_define_const(module, "GL_PERSPECTIVE_CORRECTION_HINT", INT2NUM(GL_PERSPECTIVE_CORRECTION_HINT));
	rb_define_const(module, "GL_PERTURB_EXT", INT2NUM(GL_PERTURB_EXT));
	rb_define_const(module, "GL_PER_STAGE_CONSTANTS_NV", INT2NUM(GL_PER_STAGE_CONSTANTS_NV));
	rb_define_const(module, "GL_PHONG_HINT_WIN", INT2NUM(GL_PHONG_HINT_WIN));
	rb_define_const(module, "GL_PHONG_WIN", INT2NUM(GL_PHONG_WIN));
	rb_define_const(module, "GL_PIXEL_COUNTER_BITS_NV", INT2NUM(GL_PIXEL_COUNTER_BITS_NV));
	rb_define_const(module, "GL_PIXEL_COUNT_AVAILABLE_NV", INT2NUM(GL_PIXEL_COUNT_AVAILABLE_NV));
	rb_define_const(module, "GL_PIXEL_COUNT_NV", INT2NUM(GL_PIXEL_COUNT_NV));
	rb_define_const(module, "GL_PIXEL_CUBIC_WEIGHT_EXT", INT2NUM(GL_PIXEL_CUBIC_WEIGHT_EXT));
	rb_define_const(module, "GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS", INT2NUM(GL_PIXEL_FRAGMENT_ALPHA_SOURCE_SGIS));
	rb_define_const(module, "GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS", INT2NUM(GL_PIXEL_FRAGMENT_RGB_SOURCE_SGIS));
	rb_define_const(module, "GL_PIXEL_GROUP_COLOR_SGIS", INT2NUM(GL_PIXEL_GROUP_COLOR_SGIS));
	rb_define_const(module, "GL_PIXEL_MAG_FILTER_EXT", INT2NUM(GL_PIXEL_MAG_FILTER_EXT));
	rb_define_const(module, "GL_PIXEL_MAP_A_TO_A", INT2NUM(GL_PIXEL_MAP_A_TO_A));
	rb_define_const(module, "GL_PIXEL_MAP_A_TO_A_SIZE", INT2NUM(GL_PIXEL_MAP_A_TO_A_SIZE));
	rb_define_const(module, "GL_PIXEL_MAP_B_TO_B", INT2NUM(GL_PIXEL_MAP_B_TO_B));
	rb_define_const(module, "GL_PIXEL_MAP_B_TO_B_SIZE", INT2NUM(GL_PIXEL_MAP_B_TO_B_SIZE));
	rb_define_const(module, "GL_PIXEL_MAP_G_TO_G", INT2NUM(GL_PIXEL_MAP_G_TO_G));
	rb_define_const(module, "GL_PIXEL_MAP_G_TO_G_SIZE", INT2NUM(GL_PIXEL_MAP_G_TO_G_SIZE));
	rb_define_const(module, "GL_PIXEL_MAP_I_TO_A", INT2NUM(GL_PIXEL_MAP_I_TO_A));
	rb_define_const(module, "GL_PIXEL_MAP_I_TO_A_SIZE", INT2NUM(GL_PIXEL_MAP_I_TO_A_SIZE));
	rb_define_const(module, "GL_PIXEL_MAP_I_TO_B", INT2NUM(GL_PIXEL_MAP_I_TO_B));
	rb_define_const(module, "GL_PIXEL_MAP_I_TO_B_SIZE", INT2NUM(GL_PIXEL_MAP_I_TO_B_SIZE));
	rb_define_const(module, "GL_PIXEL_MAP_I_TO_G", INT2NUM(GL_PIXEL_MAP_I_TO_G));
	rb_define_const(module, "GL_PIXEL_MAP_I_TO_G_SIZE", INT2NUM(GL_PIXEL_MAP_I_TO_G_SIZE));
	rb_define_const(module, "GL_PIXEL_MAP_I_TO_I", INT2NUM(GL_PIXEL_MAP_I_TO_I));
	rb_define_const(module, "GL_PIXEL_MAP_I_TO_I_SIZE", INT2NUM(GL_PIXEL_MAP_I_TO_I_SIZE));
	rb_define_const(module, "GL_PIXEL_MAP_I_TO_R", INT2NUM(GL_PIXEL_MAP_I_TO_R));
	rb_define_const(module, "GL_PIXEL_MAP_I_TO_R_SIZE", INT2NUM(GL_PIXEL_MAP_I_TO_R_SIZE));
	rb_define_const(module, "GL_PIXEL_MAP_R_TO_R", INT2NUM(GL_PIXEL_MAP_R_TO_R));
	rb_define_const(module, "GL_PIXEL_MAP_R_TO_R_SIZE", INT2NUM(GL_PIXEL_MAP_R_TO_R_SIZE));
	rb_define_const(module, "GL_PIXEL_MAP_S_TO_S", INT2NUM(GL_PIXEL_MAP_S_TO_S));
	rb_define_const(module, "GL_PIXEL_MAP_S_TO_S_SIZE", INT2NUM(GL_PIXEL_MAP_S_TO_S_SIZE));
	rb_define_const(module, "GL_PIXEL_MIN_FILTER_EXT", INT2NUM(GL_PIXEL_MIN_FILTER_EXT));
	rb_define_const(module, "GL_PIXEL_MODE_BIT", INT2NUM(GL_PIXEL_MODE_BIT));
	rb_define_const(module, "GL_PIXEL_PACK_BUFFER", INT2NUM(GL_PIXEL_PACK_BUFFER));
	rb_define_const(module, "GL_PIXEL_PACK_BUFFER_ARB", INT2NUM(GL_PIXEL_PACK_BUFFER_ARB));
	rb_define_const(module, "GL_PIXEL_PACK_BUFFER_BINDING", INT2NUM(GL_PIXEL_PACK_BUFFER_BINDING));
	rb_define_const(module, "GL_PIXEL_PACK_BUFFER_BINDING_ARB", INT2NUM(GL_PIXEL_PACK_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_PIXEL_PACK_BUFFER_BINDING_EXT", INT2NUM(GL_PIXEL_PACK_BUFFER_BINDING_EXT));
	rb_define_const(module, "GL_PIXEL_PACK_BUFFER_EXT", INT2NUM(GL_PIXEL_PACK_BUFFER_EXT));
	rb_define_const(module, "GL_PIXEL_SUBSAMPLE_2424_SGIX", INT2NUM(GL_PIXEL_SUBSAMPLE_2424_SGIX));
	rb_define_const(module, "GL_PIXEL_SUBSAMPLE_4242_SGIX", INT2NUM(GL_PIXEL_SUBSAMPLE_4242_SGIX));
	rb_define_const(module, "GL_PIXEL_SUBSAMPLE_4444_SGIX", INT2NUM(GL_PIXEL_SUBSAMPLE_4444_SGIX));
	rb_define_const(module, "GL_PIXEL_TEXTURE_SGIS", INT2NUM(GL_PIXEL_TEXTURE_SGIS));
	rb_define_const(module, "GL_PIXEL_TEX_GEN_ALPHA_LS_SGIX", INT2NUM(GL_PIXEL_TEX_GEN_ALPHA_LS_SGIX));
	rb_define_const(module, "GL_PIXEL_TEX_GEN_ALPHA_MS_SGIX", INT2NUM(GL_PIXEL_TEX_GEN_ALPHA_MS_SGIX));
	rb_define_const(module, "GL_PIXEL_TEX_GEN_ALPHA_NO_REPLACE_SGIX", INT2NUM(GL_PIXEL_TEX_GEN_ALPHA_NO_REPLACE_SGIX));
	rb_define_const(module, "GL_PIXEL_TEX_GEN_ALPHA_REPLACE_SGIX", INT2NUM(GL_PIXEL_TEX_GEN_ALPHA_REPLACE_SGIX));
	rb_define_const(module, "GL_PIXEL_TEX_GEN_MODE_SGIX", INT2NUM(GL_PIXEL_TEX_GEN_MODE_SGIX));
	rb_define_const(module, "GL_PIXEL_TEX_GEN_Q_CEILING_SGIX", INT2NUM(GL_PIXEL_TEX_GEN_Q_CEILING_SGIX));
	rb_define_const(module, "GL_PIXEL_TEX_GEN_Q_FLOOR_SGIX", INT2NUM(GL_PIXEL_TEX_GEN_Q_FLOOR_SGIX));
	rb_define_const(module, "GL_PIXEL_TEX_GEN_Q_ROUND_SGIX", INT2NUM(GL_PIXEL_TEX_GEN_Q_ROUND_SGIX));
	rb_define_const(module, "GL_PIXEL_TEX_GEN_SGIX", INT2NUM(GL_PIXEL_TEX_GEN_SGIX));
	rb_define_const(module, "GL_PIXEL_TILE_BEST_ALIGNMENT_SGIX", INT2NUM(GL_PIXEL_TILE_BEST_ALIGNMENT_SGIX));
	rb_define_const(module, "GL_PIXEL_TILE_CACHE_INCREMENT_SGIX", INT2NUM(GL_PIXEL_TILE_CACHE_INCREMENT_SGIX));
	rb_define_const(module, "GL_PIXEL_TILE_CACHE_SIZE_SGIX", INT2NUM(GL_PIXEL_TILE_CACHE_SIZE_SGIX));
	rb_define_const(module, "GL_PIXEL_TILE_GRID_DEPTH_SGIX", INT2NUM(GL_PIXEL_TILE_GRID_DEPTH_SGIX));
	rb_define_const(module, "GL_PIXEL_TILE_GRID_HEIGHT_SGIX", INT2NUM(GL_PIXEL_TILE_GRID_HEIGHT_SGIX));
	rb_define_const(module, "GL_PIXEL_TILE_GRID_WIDTH_SGIX", INT2NUM(GL_PIXEL_TILE_GRID_WIDTH_SGIX));
	rb_define_const(module, "GL_PIXEL_TILE_HEIGHT_SGIX", INT2NUM(GL_PIXEL_TILE_HEIGHT_SGIX));
	rb_define_const(module, "GL_PIXEL_TILE_WIDTH_SGIX", INT2NUM(GL_PIXEL_TILE_WIDTH_SGIX));
	rb_define_const(module, "GL_PIXEL_TRANSFORM_2D_EXT", INT2NUM(GL_PIXEL_TRANSFORM_2D_EXT));
	rb_define_const(module, "GL_PIXEL_TRANSFORM_2D_MATRIX_EXT", INT2NUM(GL_PIXEL_TRANSFORM_2D_MATRIX_EXT));
	rb_define_const(module, "GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT", INT2NUM(GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT));
	rb_define_const(module, "GL_PIXEL_UNPACK_BUFFER", INT2NUM(GL_PIXEL_UNPACK_BUFFER));
	rb_define_const(module, "GL_PIXEL_UNPACK_BUFFER_ARB", INT2NUM(GL_PIXEL_UNPACK_BUFFER_ARB));
	rb_define_const(module, "GL_PIXEL_UNPACK_BUFFER_BINDING", INT2NUM(GL_PIXEL_UNPACK_BUFFER_BINDING));
	rb_define_const(module, "GL_PIXEL_UNPACK_BUFFER_BINDING_ARB", INT2NUM(GL_PIXEL_UNPACK_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_PIXEL_UNPACK_BUFFER_BINDING_EXT", INT2NUM(GL_PIXEL_UNPACK_BUFFER_BINDING_EXT));
	rb_define_const(module, "GL_PIXEL_UNPACK_BUFFER_EXT", INT2NUM(GL_PIXEL_UNPACK_BUFFER_EXT));
	rb_define_const(module, "GL_PN_TRIANGLES_ATI", INT2NUM(GL_PN_TRIANGLES_ATI));
	rb_define_const(module, "GL_PN_TRIANGLES_NORMAL_MODE_ATI", INT2NUM(GL_PN_TRIANGLES_NORMAL_MODE_ATI));
	rb_define_const(module, "GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI", INT2NUM(GL_PN_TRIANGLES_NORMAL_MODE_LINEAR_ATI));
	rb_define_const(module, "GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI", INT2NUM(GL_PN_TRIANGLES_NORMAL_MODE_QUADRATIC_ATI));
	rb_define_const(module, "GL_PN_TRIANGLES_POINT_MODE_ATI", INT2NUM(GL_PN_TRIANGLES_POINT_MODE_ATI));
	rb_define_const(module, "GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI", INT2NUM(GL_PN_TRIANGLES_POINT_MODE_CUBIC_ATI));
	rb_define_const(module, "GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI", INT2NUM(GL_PN_TRIANGLES_POINT_MODE_LINEAR_ATI));
	rb_define_const(module, "GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI", INT2NUM(GL_PN_TRIANGLES_TESSELATION_LEVEL_ATI));
	rb_define_const(module, "GL_POINT", INT2NUM(GL_POINT));
	rb_define_const(module, "GL_POINTS", INT2NUM(GL_POINTS));
	rb_define_const(module, "GL_POINT_BIT", INT2NUM(GL_POINT_BIT));
	rb_define_const(module, "GL_POINT_DISTANCE_ATTENUATION", INT2NUM(GL_POINT_DISTANCE_ATTENUATION));
	rb_define_const(module, "GL_POINT_DISTANCE_ATTENUATION_ARB", INT2NUM(GL_POINT_DISTANCE_ATTENUATION_ARB));
	rb_define_const(module, "GL_POINT_FADE_THRESHOLD_SIZE", INT2NUM(GL_POINT_FADE_THRESHOLD_SIZE));
	rb_define_const(module, "GL_POINT_FADE_THRESHOLD_SIZE_ARB", INT2NUM(GL_POINT_FADE_THRESHOLD_SIZE_ARB));
	rb_define_const(module, "GL_POINT_FADE_THRESHOLD_SIZE_EXT", INT2NUM(GL_POINT_FADE_THRESHOLD_SIZE_EXT));
	rb_define_const(module, "GL_POINT_FADE_THRESHOLD_SIZE_SGIS", INT2NUM(GL_POINT_FADE_THRESHOLD_SIZE_SGIS));
	rb_define_const(module, "GL_POINT_SIZE", INT2NUM(GL_POINT_SIZE));
	rb_define_const(module, "GL_POINT_SIZE_GRANULARITY", INT2NUM(GL_POINT_SIZE_GRANULARITY));
	rb_define_const(module, "GL_POINT_SIZE_MAX", INT2NUM(GL_POINT_SIZE_MAX));
	rb_define_const(module, "GL_POINT_SIZE_MAX_ARB", INT2NUM(GL_POINT_SIZE_MAX_ARB));
	rb_define_const(module, "GL_POINT_SIZE_MAX_EXT", INT2NUM(GL_POINT_SIZE_MAX_EXT));
	rb_define_const(module, "GL_POINT_SIZE_MAX_SGIS", INT2NUM(GL_POINT_SIZE_MAX_SGIS));
	rb_define_const(module, "GL_POINT_SIZE_MIN", INT2NUM(GL_POINT_SIZE_MIN));
	rb_define_const(module, "GL_POINT_SIZE_MIN_ARB", INT2NUM(GL_POINT_SIZE_MIN_ARB));
	rb_define_const(module, "GL_POINT_SIZE_MIN_EXT", INT2NUM(GL_POINT_SIZE_MIN_EXT));
	rb_define_const(module, "GL_POINT_SIZE_MIN_SGIS", INT2NUM(GL_POINT_SIZE_MIN_SGIS));
	rb_define_const(module, "GL_POINT_SIZE_RANGE", INT2NUM(GL_POINT_SIZE_RANGE));
	rb_define_const(module, "GL_POINT_SMOOTH", INT2NUM(GL_POINT_SMOOTH));
	rb_define_const(module, "GL_POINT_SMOOTH_HINT", INT2NUM(GL_POINT_SMOOTH_HINT));
	rb_define_const(module, "GL_POINT_SPRITE", INT2NUM(GL_POINT_SPRITE));
	rb_define_const(module, "GL_POINT_SPRITE_ARB", INT2NUM(GL_POINT_SPRITE_ARB));
	rb_define_const(module, "GL_POINT_SPRITE_COORD_ORIGIN", INT2NUM(GL_POINT_SPRITE_COORD_ORIGIN));
	rb_define_const(module, "GL_POINT_SPRITE_NV", INT2NUM(GL_POINT_SPRITE_NV));
	rb_define_const(module, "GL_POINT_SPRITE_R_MODE_NV", INT2NUM(GL_POINT_SPRITE_R_MODE_NV));
	rb_define_const(module, "GL_POINT_TOKEN", INT2NUM(GL_POINT_TOKEN));
	rb_define_const(module, "GL_POLYGON", INT2NUM(GL_POLYGON));
	rb_define_const(module, "GL_POLYGON_BIT", INT2NUM(GL_POLYGON_BIT));
	rb_define_const(module, "GL_POLYGON_MODE", INT2NUM(GL_POLYGON_MODE));
	rb_define_const(module, "GL_POLYGON_OFFSET_BIAS_EXT", INT2NUM(GL_POLYGON_OFFSET_BIAS_EXT));
	rb_define_const(module, "GL_POLYGON_OFFSET_EXT", INT2NUM(GL_POLYGON_OFFSET_EXT));
	rb_define_const(module, "GL_POLYGON_OFFSET_FACTOR", INT2NUM(GL_POLYGON_OFFSET_FACTOR));
	rb_define_const(module, "GL_POLYGON_OFFSET_FACTOR_EXT", INT2NUM(GL_POLYGON_OFFSET_FACTOR_EXT));
	rb_define_const(module, "GL_POLYGON_OFFSET_FILL", INT2NUM(GL_POLYGON_OFFSET_FILL));
	rb_define_const(module, "GL_POLYGON_OFFSET_LINE", INT2NUM(GL_POLYGON_OFFSET_LINE));
	rb_define_const(module, "GL_POLYGON_OFFSET_POINT", INT2NUM(GL_POLYGON_OFFSET_POINT));
	rb_define_const(module, "GL_POLYGON_OFFSET_UNITS", INT2NUM(GL_POLYGON_OFFSET_UNITS));
	rb_define_const(module, "GL_POLYGON_SMOOTH", INT2NUM(GL_POLYGON_SMOOTH));
	rb_define_const(module, "GL_POLYGON_SMOOTH_HINT", INT2NUM(GL_POLYGON_SMOOTH_HINT));
	rb_define_const(module, "GL_POLYGON_STIPPLE", INT2NUM(GL_POLYGON_STIPPLE));
	rb_define_const(module, "GL_POLYGON_STIPPLE_BIT", INT2NUM(GL_POLYGON_STIPPLE_BIT));
	rb_define_const(module, "GL_POLYGON_TOKEN", INT2NUM(GL_POLYGON_TOKEN));
	rb_define_const(module, "GL_POSITION", INT2NUM(GL_POSITION));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_ALPHA_BIAS", INT2NUM(GL_POST_COLOR_MATRIX_ALPHA_BIAS));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI", INT2NUM(GL_POST_COLOR_MATRIX_ALPHA_BIAS_SGI));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_ALPHA_SCALE", INT2NUM(GL_POST_COLOR_MATRIX_ALPHA_SCALE));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI", INT2NUM(GL_POST_COLOR_MATRIX_ALPHA_SCALE_SGI));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_BLUE_BIAS", INT2NUM(GL_POST_COLOR_MATRIX_BLUE_BIAS));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI", INT2NUM(GL_POST_COLOR_MATRIX_BLUE_BIAS_SGI));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_BLUE_SCALE", INT2NUM(GL_POST_COLOR_MATRIX_BLUE_SCALE));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI", INT2NUM(GL_POST_COLOR_MATRIX_BLUE_SCALE_SGI));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_COLOR_TABLE", INT2NUM(GL_POST_COLOR_MATRIX_COLOR_TABLE));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI", INT2NUM(GL_POST_COLOR_MATRIX_COLOR_TABLE_SGI));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_GREEN_BIAS", INT2NUM(GL_POST_COLOR_MATRIX_GREEN_BIAS));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI", INT2NUM(GL_POST_COLOR_MATRIX_GREEN_BIAS_SGI));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_GREEN_SCALE", INT2NUM(GL_POST_COLOR_MATRIX_GREEN_SCALE));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI", INT2NUM(GL_POST_COLOR_MATRIX_GREEN_SCALE_SGI));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_RED_BIAS", INT2NUM(GL_POST_COLOR_MATRIX_RED_BIAS));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_RED_BIAS_SGI", INT2NUM(GL_POST_COLOR_MATRIX_RED_BIAS_SGI));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_RED_SCALE", INT2NUM(GL_POST_COLOR_MATRIX_RED_SCALE));
	rb_define_const(module, "GL_POST_COLOR_MATRIX_RED_SCALE_SGI", INT2NUM(GL_POST_COLOR_MATRIX_RED_SCALE_SGI));
	rb_define_const(module, "GL_POST_CONVOLUTION_ALPHA_BIAS", INT2NUM(GL_POST_CONVOLUTION_ALPHA_BIAS));
	rb_define_const(module, "GL_POST_CONVOLUTION_ALPHA_BIAS_EXT", INT2NUM(GL_POST_CONVOLUTION_ALPHA_BIAS_EXT));
	rb_define_const(module, "GL_POST_CONVOLUTION_ALPHA_SCALE", INT2NUM(GL_POST_CONVOLUTION_ALPHA_SCALE));
	rb_define_const(module, "GL_POST_CONVOLUTION_ALPHA_SCALE_EXT", INT2NUM(GL_POST_CONVOLUTION_ALPHA_SCALE_EXT));
	rb_define_const(module, "GL_POST_CONVOLUTION_BLUE_BIAS", INT2NUM(GL_POST_CONVOLUTION_BLUE_BIAS));
	rb_define_const(module, "GL_POST_CONVOLUTION_BLUE_BIAS_EXT", INT2NUM(GL_POST_CONVOLUTION_BLUE_BIAS_EXT));
	rb_define_const(module, "GL_POST_CONVOLUTION_BLUE_SCALE", INT2NUM(GL_POST_CONVOLUTION_BLUE_SCALE));
	rb_define_const(module, "GL_POST_CONVOLUTION_BLUE_SCALE_EXT", INT2NUM(GL_POST_CONVOLUTION_BLUE_SCALE_EXT));
	rb_define_const(module, "GL_POST_CONVOLUTION_COLOR_TABLE", INT2NUM(GL_POST_CONVOLUTION_COLOR_TABLE));
	rb_define_const(module, "GL_POST_CONVOLUTION_COLOR_TABLE_SGI", INT2NUM(GL_POST_CONVOLUTION_COLOR_TABLE_SGI));
	rb_define_const(module, "GL_POST_CONVOLUTION_GREEN_BIAS", INT2NUM(GL_POST_CONVOLUTION_GREEN_BIAS));
	rb_define_const(module, "GL_POST_CONVOLUTION_GREEN_BIAS_EXT", INT2NUM(GL_POST_CONVOLUTION_GREEN_BIAS_EXT));
	rb_define_const(module, "GL_POST_CONVOLUTION_GREEN_SCALE", INT2NUM(GL_POST_CONVOLUTION_GREEN_SCALE));
	rb_define_const(module, "GL_POST_CONVOLUTION_GREEN_SCALE_EXT", INT2NUM(GL_POST_CONVOLUTION_GREEN_SCALE_EXT));
	rb_define_const(module, "GL_POST_CONVOLUTION_RED_BIAS", INT2NUM(GL_POST_CONVOLUTION_RED_BIAS));
	rb_define_const(module, "GL_POST_CONVOLUTION_RED_BIAS_EXT", INT2NUM(GL_POST_CONVOLUTION_RED_BIAS_EXT));
	rb_define_const(module, "GL_POST_CONVOLUTION_RED_SCALE", INT2NUM(GL_POST_CONVOLUTION_RED_SCALE));
	rb_define_const(module, "GL_POST_CONVOLUTION_RED_SCALE_EXT", INT2NUM(GL_POST_CONVOLUTION_RED_SCALE_EXT));
	rb_define_const(module, "GL_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP", INT2NUM(GL_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP));
	rb_define_const(module, "GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX", INT2NUM(GL_POST_TEXTURE_FILTER_BIAS_RANGE_SGIX));
	rb_define_const(module, "GL_POST_TEXTURE_FILTER_BIAS_SGIX", INT2NUM(GL_POST_TEXTURE_FILTER_BIAS_SGIX));
	rb_define_const(module, "GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX", INT2NUM(GL_POST_TEXTURE_FILTER_SCALE_RANGE_SGIX));
	rb_define_const(module, "GL_POST_TEXTURE_FILTER_SCALE_SGIX", INT2NUM(GL_POST_TEXTURE_FILTER_SCALE_SGIX));
	rb_define_const(module, "GL_PREFER_DOUBLEBUFFER_HINT_PGI", INT2NUM(GL_PREFER_DOUBLEBUFFER_HINT_PGI));
	rb_define_const(module, "GL_PRESENT_DURATION_NV", INT2NUM(GL_PRESENT_DURATION_NV));
	rb_define_const(module, "GL_PRESENT_TIME_NV", INT2NUM(GL_PRESENT_TIME_NV));
	rb_define_const(module, "GL_PRESERVE_ATI", INT2NUM(GL_PRESERVE_ATI));
	rb_define_const(module, "GL_PREVIOUS", INT2NUM(GL_PREVIOUS));
	rb_define_const(module, "GL_PREVIOUS_ARB", INT2NUM(GL_PREVIOUS_ARB));
	rb_define_const(module, "GL_PREVIOUS_EXT", INT2NUM(GL_PREVIOUS_EXT));
	rb_define_const(module, "GL_PREVIOUS_TEXTURE_INPUT_NV", INT2NUM(GL_PREVIOUS_TEXTURE_INPUT_NV));
	rb_define_const(module, "GL_PRIMARY_COLOR", INT2NUM(GL_PRIMARY_COLOR));
	rb_define_const(module, "GL_PRIMARY_COLOR_ARB", INT2NUM(GL_PRIMARY_COLOR_ARB));
	rb_define_const(module, "GL_PRIMARY_COLOR_EXT", INT2NUM(GL_PRIMARY_COLOR_EXT));
	rb_define_const(module, "GL_PRIMARY_COLOR_NV", INT2NUM(GL_PRIMARY_COLOR_NV));
	rb_define_const(module, "GL_PRIMITIVES_GENERATED", INT2NUM(GL_PRIMITIVES_GENERATED));
	rb_define_const(module, "GL_PRIMITIVES_GENERATED_EXT", INT2NUM(GL_PRIMITIVES_GENERATED_EXT));
	rb_define_const(module, "GL_PRIMITIVES_GENERATED_NV", INT2NUM(GL_PRIMITIVES_GENERATED_NV));
	rb_define_const(module, "GL_PRIMITIVE_ID_NV", INT2NUM(GL_PRIMITIVE_ID_NV));
	rb_define_const(module, "GL_PRIMITIVE_RESTART_INDEX_NV", INT2NUM(GL_PRIMITIVE_RESTART_INDEX_NV));
	rb_define_const(module, "GL_PRIMITIVE_RESTART_NV", INT2NUM(GL_PRIMITIVE_RESTART_NV));
	rb_define_const(module, "GL_PROGRAM_ADDRESS_REGISTERS_ARB", INT2NUM(GL_PROGRAM_ADDRESS_REGISTERS_ARB));
	rb_define_const(module, "GL_PROGRAM_ALU_INSTRUCTIONS_ARB", INT2NUM(GL_PROGRAM_ALU_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_PROGRAM_ATTRIBS_ARB", INT2NUM(GL_PROGRAM_ATTRIBS_ARB));
	rb_define_const(module, "GL_PROGRAM_ATTRIB_COMPONENTS_NV", INT2NUM(GL_PROGRAM_ATTRIB_COMPONENTS_NV));
	rb_define_const(module, "GL_PROGRAM_BINDING_ARB", INT2NUM(GL_PROGRAM_BINDING_ARB));
	rb_define_const(module, "GL_PROGRAM_ERROR_POSITION_ARB", INT2NUM(GL_PROGRAM_ERROR_POSITION_ARB));
	rb_define_const(module, "GL_PROGRAM_ERROR_POSITION_NV", INT2NUM(GL_PROGRAM_ERROR_POSITION_NV));
	rb_define_const(module, "GL_PROGRAM_ERROR_STRING_ARB", INT2NUM(GL_PROGRAM_ERROR_STRING_ARB));
	rb_define_const(module, "GL_PROGRAM_ERROR_STRING_NV", INT2NUM(GL_PROGRAM_ERROR_STRING_NV));
	rb_define_const(module, "GL_PROGRAM_FORMAT_ARB", INT2NUM(GL_PROGRAM_FORMAT_ARB));
	rb_define_const(module, "GL_PROGRAM_FORMAT_ASCII_ARB", INT2NUM(GL_PROGRAM_FORMAT_ASCII_ARB));
	rb_define_const(module, "GL_PROGRAM_INSTRUCTIONS_ARB", INT2NUM(GL_PROGRAM_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_PROGRAM_LENGTH_ARB", INT2NUM(GL_PROGRAM_LENGTH_ARB));
	rb_define_const(module, "GL_PROGRAM_LENGTH_NV", INT2NUM(GL_PROGRAM_LENGTH_NV));
	rb_define_const(module, "GL_PROGRAM_MATRIX_EXT", INT2NUM(GL_PROGRAM_MATRIX_EXT));
	rb_define_const(module, "GL_PROGRAM_MATRIX_STACK_DEPTH_EXT", INT2NUM(GL_PROGRAM_MATRIX_STACK_DEPTH_EXT));
	rb_define_const(module, "GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB", INT2NUM(GL_PROGRAM_NATIVE_ADDRESS_REGISTERS_ARB));
	rb_define_const(module, "GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB", INT2NUM(GL_PROGRAM_NATIVE_ALU_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_PROGRAM_NATIVE_ATTRIBS_ARB", INT2NUM(GL_PROGRAM_NATIVE_ATTRIBS_ARB));
	rb_define_const(module, "GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB", INT2NUM(GL_PROGRAM_NATIVE_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_PROGRAM_NATIVE_PARAMETERS_ARB", INT2NUM(GL_PROGRAM_NATIVE_PARAMETERS_ARB));
	rb_define_const(module, "GL_PROGRAM_NATIVE_TEMPORARIES_ARB", INT2NUM(GL_PROGRAM_NATIVE_TEMPORARIES_ARB));
	rb_define_const(module, "GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB", INT2NUM(GL_PROGRAM_NATIVE_TEX_INDIRECTIONS_ARB));
	rb_define_const(module, "GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB", INT2NUM(GL_PROGRAM_NATIVE_TEX_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_PROGRAM_OBJECT_ARB", INT2NUM(GL_PROGRAM_OBJECT_ARB));
	rb_define_const(module, "GL_PROGRAM_PARAMETERS_ARB", INT2NUM(GL_PROGRAM_PARAMETERS_ARB));
	rb_define_const(module, "GL_PROGRAM_PARAMETER_NV", INT2NUM(GL_PROGRAM_PARAMETER_NV));
	rb_define_const(module, "GL_PROGRAM_POINT_SIZE_ARB", INT2NUM(GL_PROGRAM_POINT_SIZE_ARB));
	rb_define_const(module, "GL_PROGRAM_POINT_SIZE_EXT", INT2NUM(GL_PROGRAM_POINT_SIZE_EXT));
	rb_define_const(module, "GL_PROGRAM_RESIDENT_NV", INT2NUM(GL_PROGRAM_RESIDENT_NV));
	rb_define_const(module, "GL_PROGRAM_RESULT_COMPONENTS_NV", INT2NUM(GL_PROGRAM_RESULT_COMPONENTS_NV));
	rb_define_const(module, "GL_PROGRAM_STRING_ARB", INT2NUM(GL_PROGRAM_STRING_ARB));
	rb_define_const(module, "GL_PROGRAM_STRING_NV", INT2NUM(GL_PROGRAM_STRING_NV));
	rb_define_const(module, "GL_PROGRAM_TARGET_NV", INT2NUM(GL_PROGRAM_TARGET_NV));
	rb_define_const(module, "GL_PROGRAM_TEMPORARIES_ARB", INT2NUM(GL_PROGRAM_TEMPORARIES_ARB));
	rb_define_const(module, "GL_PROGRAM_TEX_INDIRECTIONS_ARB", INT2NUM(GL_PROGRAM_TEX_INDIRECTIONS_ARB));
	rb_define_const(module, "GL_PROGRAM_TEX_INSTRUCTIONS_ARB", INT2NUM(GL_PROGRAM_TEX_INSTRUCTIONS_ARB));
	rb_define_const(module, "GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB", INT2NUM(GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB));
	rb_define_const(module, "GL_PROJECTION", INT2NUM(GL_PROJECTION));
	rb_define_const(module, "GL_PROJECTION_MATRIX", INT2NUM(GL_PROJECTION_MATRIX));
	rb_define_const(module, "GL_PROJECTION_STACK_DEPTH", INT2NUM(GL_PROJECTION_STACK_DEPTH));
	rb_define_const(module, "GL_PROXY_COLOR_TABLE", INT2NUM(GL_PROXY_COLOR_TABLE));
	rb_define_const(module, "GL_PROXY_COLOR_TABLE_SGI", INT2NUM(GL_PROXY_COLOR_TABLE_SGI));
	rb_define_const(module, "GL_PROXY_HISTOGRAM", INT2NUM(GL_PROXY_HISTOGRAM));
	rb_define_const(module, "GL_PROXY_HISTOGRAM_EXT", INT2NUM(GL_PROXY_HISTOGRAM_EXT));
	rb_define_const(module, "GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE", INT2NUM(GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE));
	rb_define_const(module, "GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI", INT2NUM(GL_PROXY_POST_COLOR_MATRIX_COLOR_TABLE_SGI));
	rb_define_const(module, "GL_PROXY_POST_CONVOLUTION_COLOR_TABLE", INT2NUM(GL_PROXY_POST_CONVOLUTION_COLOR_TABLE));
	rb_define_const(module, "GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI", INT2NUM(GL_PROXY_POST_CONVOLUTION_COLOR_TABLE_SGI));
	rb_define_const(module, "GL_PROXY_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP", INT2NUM(GL_PROXY_POST_IMAGE_TRANSFORM_COLOR_TABLE_HP));
	rb_define_const(module, "GL_PROXY_TEXTURE_1D", INT2NUM(GL_PROXY_TEXTURE_1D));
	rb_define_const(module, "GL_PROXY_TEXTURE_1D_ARRAY", INT2NUM(GL_PROXY_TEXTURE_1D_ARRAY));
	rb_define_const(module, "GL_PROXY_TEXTURE_1D_ARRAY_EXT", INT2NUM(GL_PROXY_TEXTURE_1D_ARRAY_EXT));
	rb_define_const(module, "GL_PROXY_TEXTURE_1D_EXT", INT2NUM(GL_PROXY_TEXTURE_1D_EXT));
	rb_define_const(module, "GL_PROXY_TEXTURE_1D_STACK_MESAX", INT2NUM(GL_PROXY_TEXTURE_1D_STACK_MESAX));
	rb_define_const(module, "GL_PROXY_TEXTURE_2D", INT2NUM(GL_PROXY_TEXTURE_2D));
	rb_define_const(module, "GL_PROXY_TEXTURE_2D_ARRAY", INT2NUM(GL_PROXY_TEXTURE_2D_ARRAY));
	rb_define_const(module, "GL_PROXY_TEXTURE_2D_ARRAY_EXT", INT2NUM(GL_PROXY_TEXTURE_2D_ARRAY_EXT));
	rb_define_const(module, "GL_PROXY_TEXTURE_2D_EXT", INT2NUM(GL_PROXY_TEXTURE_2D_EXT));
	rb_define_const(module, "GL_PROXY_TEXTURE_2D_STACK_MESAX", INT2NUM(GL_PROXY_TEXTURE_2D_STACK_MESAX));
	rb_define_const(module, "GL_PROXY_TEXTURE_3D", INT2NUM(GL_PROXY_TEXTURE_3D));
	rb_define_const(module, "GL_PROXY_TEXTURE_3D_EXT", INT2NUM(GL_PROXY_TEXTURE_3D_EXT));
	rb_define_const(module, "GL_PROXY_TEXTURE_4D_SGIS", INT2NUM(GL_PROXY_TEXTURE_4D_SGIS));
	rb_define_const(module, "GL_PROXY_TEXTURE_COLOR_TABLE_SGI", INT2NUM(GL_PROXY_TEXTURE_COLOR_TABLE_SGI));
	rb_define_const(module, "GL_PROXY_TEXTURE_CUBE_MAP", INT2NUM(GL_PROXY_TEXTURE_CUBE_MAP));
	rb_define_const(module, "GL_PROXY_TEXTURE_CUBE_MAP_ARB", INT2NUM(GL_PROXY_TEXTURE_CUBE_MAP_ARB));
	rb_define_const(module, "GL_PROXY_TEXTURE_CUBE_MAP_EXT", INT2NUM(GL_PROXY_TEXTURE_CUBE_MAP_EXT));
	rb_define_const(module, "GL_PROXY_TEXTURE_RECTANGLE_ARB", INT2NUM(GL_PROXY_TEXTURE_RECTANGLE_ARB));
	rb_define_const(module, "GL_PROXY_TEXTURE_RECTANGLE_NV", INT2NUM(GL_PROXY_TEXTURE_RECTANGLE_NV));
	rb_define_const(module, "GL_Q", INT2NUM(GL_Q));
	rb_define_const(module, "GL_QUADRATIC_ATTENUATION", INT2NUM(GL_QUADRATIC_ATTENUATION));
	rb_define_const(module, "GL_QUADS", INT2NUM(GL_QUADS));
	rb_define_const(module, "GL_QUAD_ALPHA4_SGIS", INT2NUM(GL_QUAD_ALPHA4_SGIS));
	rb_define_const(module, "GL_QUAD_ALPHA8_SGIS", INT2NUM(GL_QUAD_ALPHA8_SGIS));
	rb_define_const(module, "GL_QUAD_INTENSITY4_SGIS", INT2NUM(GL_QUAD_INTENSITY4_SGIS));
	rb_define_const(module, "GL_QUAD_INTENSITY8_SGIS", INT2NUM(GL_QUAD_INTENSITY8_SGIS));
	rb_define_const(module, "GL_QUAD_LUMINANCE4_SGIS", INT2NUM(GL_QUAD_LUMINANCE4_SGIS));
	rb_define_const(module, "GL_QUAD_LUMINANCE8_SGIS", INT2NUM(GL_QUAD_LUMINANCE8_SGIS));
	rb_define_const(module, "GL_QUAD_MESH_SUN", INT2NUM(GL_QUAD_MESH_SUN));
	rb_define_const(module, "GL_QUAD_STRIP", INT2NUM(GL_QUAD_STRIP));
	rb_define_const(module, "GL_QUAD_TEXTURE_SELECT_SGIS", INT2NUM(GL_QUAD_TEXTURE_SELECT_SGIS));
	rb_define_const(module, "GL_QUARTER_BIT_ATI", INT2NUM(GL_QUARTER_BIT_ATI));
	rb_define_const(module, "GL_QUERY_BY_REGION_NO_WAIT", INT2NUM(GL_QUERY_BY_REGION_NO_WAIT));
	rb_define_const(module, "GL_QUERY_BY_REGION_NO_WAIT_NV", INT2NUM(GL_QUERY_BY_REGION_NO_WAIT_NV));
	rb_define_const(module, "GL_QUERY_BY_REGION_WAIT", INT2NUM(GL_QUERY_BY_REGION_WAIT));
	rb_define_const(module, "GL_QUERY_BY_REGION_WAIT_NV", INT2NUM(GL_QUERY_BY_REGION_WAIT_NV));
	rb_define_const(module, "GL_QUERY_COUNTER_BITS", INT2NUM(GL_QUERY_COUNTER_BITS));
	rb_define_const(module, "GL_QUERY_COUNTER_BITS_ARB", INT2NUM(GL_QUERY_COUNTER_BITS_ARB));
	rb_define_const(module, "GL_QUERY_NO_WAIT", INT2NUM(GL_QUERY_NO_WAIT));
	rb_define_const(module, "GL_QUERY_NO_WAIT_NV", INT2NUM(GL_QUERY_NO_WAIT_NV));
	rb_define_const(module, "GL_QUERY_RESULT", INT2NUM(GL_QUERY_RESULT));
	rb_define_const(module, "GL_QUERY_RESULT_ARB", INT2NUM(GL_QUERY_RESULT_ARB));
	rb_define_const(module, "GL_QUERY_RESULT_AVAILABLE", INT2NUM(GL_QUERY_RESULT_AVAILABLE));
	rb_define_const(module, "GL_QUERY_RESULT_AVAILABLE_ARB", INT2NUM(GL_QUERY_RESULT_AVAILABLE_ARB));
	rb_define_const(module, "GL_QUERY_WAIT", INT2NUM(GL_QUERY_WAIT));
	rb_define_const(module, "GL_QUERY_WAIT_NV", INT2NUM(GL_QUERY_WAIT_NV));
	rb_define_const(module, "GL_R", INT2NUM(GL_R));
	rb_define_const(module, "GL_R11F_G11F_B10F", INT2NUM(GL_R11F_G11F_B10F));
	rb_define_const(module, "GL_R11F_G11F_B10F_EXT", INT2NUM(GL_R11F_G11F_B10F_EXT));
	rb_define_const(module, "GL_R16", INT2NUM(GL_R16));
	rb_define_const(module, "GL_R16F", INT2NUM(GL_R16F));
	rb_define_const(module, "GL_R16I", INT2NUM(GL_R16I));
	rb_define_const(module, "GL_R16UI", INT2NUM(GL_R16UI));
	rb_define_const(module, "GL_R1UI_C3F_V3F_SUN", INT2NUM(GL_R1UI_C3F_V3F_SUN));
	rb_define_const(module, "GL_R1UI_C4F_N3F_V3F_SUN", INT2NUM(GL_R1UI_C4F_N3F_V3F_SUN));
	rb_define_const(module, "GL_R1UI_C4UB_V3F_SUN", INT2NUM(GL_R1UI_C4UB_V3F_SUN));
	rb_define_const(module, "GL_R1UI_N3F_V3F_SUN", INT2NUM(GL_R1UI_N3F_V3F_SUN));
	rb_define_const(module, "GL_R1UI_T2F_C4F_N3F_V3F_SUN", INT2NUM(GL_R1UI_T2F_C4F_N3F_V3F_SUN));
	rb_define_const(module, "GL_R1UI_T2F_N3F_V3F_SUN", INT2NUM(GL_R1UI_T2F_N3F_V3F_SUN));
	rb_define_const(module, "GL_R1UI_T2F_V3F_SUN", INT2NUM(GL_R1UI_T2F_V3F_SUN));
	rb_define_const(module, "GL_R1UI_V3F_SUN", INT2NUM(GL_R1UI_V3F_SUN));
	rb_define_const(module, "GL_R32F", INT2NUM(GL_R32F));
	rb_define_const(module, "GL_R32I", INT2NUM(GL_R32I));
	rb_define_const(module, "GL_R32UI", INT2NUM(GL_R32UI));
	rb_define_const(module, "GL_R3_G3_B2", INT2NUM(GL_R3_G3_B2));
	rb_define_const(module, "GL_R8", INT2NUM(GL_R8));
	rb_define_const(module, "GL_R8I", INT2NUM(GL_R8I));
	rb_define_const(module, "GL_R8UI", INT2NUM(GL_R8UI));
	rb_define_const(module, "GL_RASTERIZER_DISCARD", INT2NUM(GL_RASTERIZER_DISCARD));
	rb_define_const(module, "GL_RASTERIZER_DISCARD_EXT", INT2NUM(GL_RASTERIZER_DISCARD_EXT));
	rb_define_const(module, "GL_RASTERIZER_DISCARD_NV", INT2NUM(GL_RASTERIZER_DISCARD_NV));
	rb_define_const(module, "GL_RASTER_POSITION_UNCLIPPED_IBM", INT2NUM(GL_RASTER_POSITION_UNCLIPPED_IBM));
	rb_define_const(module, "GL_READ_BUFFER", INT2NUM(GL_READ_BUFFER));
	rb_define_const(module, "GL_READ_FRAMEBUFFER", INT2NUM(GL_READ_FRAMEBUFFER));
	rb_define_const(module, "GL_READ_FRAMEBUFFER_BINDING", INT2NUM(GL_READ_FRAMEBUFFER_BINDING));
	rb_define_const(module, "GL_READ_FRAMEBUFFER_BINDING_EXT", INT2NUM(GL_READ_FRAMEBUFFER_BINDING_EXT));
	rb_define_const(module, "GL_READ_FRAMEBUFFER_EXT", INT2NUM(GL_READ_FRAMEBUFFER_EXT));
	rb_define_const(module, "GL_READ_ONLY", INT2NUM(GL_READ_ONLY));
	rb_define_const(module, "GL_READ_ONLY_ARB", INT2NUM(GL_READ_ONLY_ARB));
	rb_define_const(module, "GL_READ_PIXEL_DATA_RANGE_LENGTH_NV", INT2NUM(GL_READ_PIXEL_DATA_RANGE_LENGTH_NV));
	rb_define_const(module, "GL_READ_PIXEL_DATA_RANGE_NV", INT2NUM(GL_READ_PIXEL_DATA_RANGE_NV));
	rb_define_const(module, "GL_READ_PIXEL_DATA_RANGE_POINTER_NV", INT2NUM(GL_READ_PIXEL_DATA_RANGE_POINTER_NV));
	rb_define_const(module, "GL_READ_WRITE", INT2NUM(GL_READ_WRITE));
	rb_define_const(module, "GL_READ_WRITE_ARB", INT2NUM(GL_READ_WRITE_ARB));
	rb_define_const(module, "GL_RECLAIM_MEMORY_HINT_PGI", INT2NUM(GL_RECLAIM_MEMORY_HINT_PGI));
	rb_define_const(module, "GL_RED", INT2NUM(GL_RED));
	rb_define_const(module, "GL_REDUCE", INT2NUM(GL_REDUCE));
	rb_define_const(module, "GL_REDUCE_EXT", INT2NUM(GL_REDUCE_EXT));
	rb_define_const(module, "GL_RED_BIAS", INT2NUM(GL_RED_BIAS));
	rb_define_const(module, "GL_RED_BITS", INT2NUM(GL_RED_BITS));
	rb_define_const(module, "GL_RED_BIT_ATI", INT2NUM(GL_RED_BIT_ATI));
	rb_define_const(module, "GL_RED_INTEGER", INT2NUM(GL_RED_INTEGER));
	rb_define_const(module, "GL_RED_INTEGER_EXT", INT2NUM(GL_RED_INTEGER_EXT));
	rb_define_const(module, "GL_RED_MAX_CLAMP_INGR", INT2NUM(GL_RED_MAX_CLAMP_INGR));
	rb_define_const(module, "GL_RED_MIN_CLAMP_INGR", INT2NUM(GL_RED_MIN_CLAMP_INGR));
	rb_define_const(module, "GL_RED_SCALE", INT2NUM(GL_RED_SCALE));
	rb_define_const(module, "GL_REFERENCE_PLANE_EQUATION_SGIX", INT2NUM(GL_REFERENCE_PLANE_EQUATION_SGIX));
	rb_define_const(module, "GL_REFERENCE_PLANE_SGIX", INT2NUM(GL_REFERENCE_PLANE_SGIX));
	rb_define_const(module, "GL_REFLECTION_MAP", INT2NUM(GL_REFLECTION_MAP));
	rb_define_const(module, "GL_REFLECTION_MAP_ARB", INT2NUM(GL_REFLECTION_MAP_ARB));
	rb_define_const(module, "GL_REFLECTION_MAP_EXT", INT2NUM(GL_REFLECTION_MAP_EXT));
	rb_define_const(module, "GL_REFLECTION_MAP_NV", INT2NUM(GL_REFLECTION_MAP_NV));
	rb_define_const(module, "GL_REGISTER_COMBINERS_NV", INT2NUM(GL_REGISTER_COMBINERS_NV));
	rb_define_const(module, "GL_REG_0_ATI", INT2NUM(GL_REG_0_ATI));
	rb_define_const(module, "GL_REG_10_ATI", INT2NUM(GL_REG_10_ATI));
	rb_define_const(module, "GL_REG_11_ATI", INT2NUM(GL_REG_11_ATI));
	rb_define_const(module, "GL_REG_12_ATI", INT2NUM(GL_REG_12_ATI));
	rb_define_const(module, "GL_REG_13_ATI", INT2NUM(GL_REG_13_ATI));
	rb_define_const(module, "GL_REG_14_ATI", INT2NUM(GL_REG_14_ATI));
	rb_define_const(module, "GL_REG_15_ATI", INT2NUM(GL_REG_15_ATI));
	rb_define_const(module, "GL_REG_16_ATI", INT2NUM(GL_REG_16_ATI));
	rb_define_const(module, "GL_REG_17_ATI", INT2NUM(GL_REG_17_ATI));
	rb_define_const(module, "GL_REG_18_ATI", INT2NUM(GL_REG_18_ATI));
	rb_define_const(module, "GL_REG_19_ATI", INT2NUM(GL_REG_19_ATI));
	rb_define_const(module, "GL_REG_1_ATI", INT2NUM(GL_REG_1_ATI));
	rb_define_const(module, "GL_REG_20_ATI", INT2NUM(GL_REG_20_ATI));
	rb_define_const(module, "GL_REG_21_ATI", INT2NUM(GL_REG_21_ATI));
	rb_define_const(module, "GL_REG_22_ATI", INT2NUM(GL_REG_22_ATI));
	rb_define_const(module, "GL_REG_23_ATI", INT2NUM(GL_REG_23_ATI));
	rb_define_const(module, "GL_REG_24_ATI", INT2NUM(GL_REG_24_ATI));
	rb_define_const(module, "GL_REG_25_ATI", INT2NUM(GL_REG_25_ATI));
	rb_define_const(module, "GL_REG_26_ATI", INT2NUM(GL_REG_26_ATI));
	rb_define_const(module, "GL_REG_27_ATI", INT2NUM(GL_REG_27_ATI));
	rb_define_const(module, "GL_REG_28_ATI", INT2NUM(GL_REG_28_ATI));
	rb_define_const(module, "GL_REG_29_ATI", INT2NUM(GL_REG_29_ATI));
	rb_define_const(module, "GL_REG_2_ATI", INT2NUM(GL_REG_2_ATI));
	rb_define_const(module, "GL_REG_30_ATI", INT2NUM(GL_REG_30_ATI));
	rb_define_const(module, "GL_REG_31_ATI", INT2NUM(GL_REG_31_ATI));
	rb_define_const(module, "GL_REG_3_ATI", INT2NUM(GL_REG_3_ATI));
	rb_define_const(module, "GL_REG_4_ATI", INT2NUM(GL_REG_4_ATI));
	rb_define_const(module, "GL_REG_5_ATI", INT2NUM(GL_REG_5_ATI));
	rb_define_const(module, "GL_REG_6_ATI", INT2NUM(GL_REG_6_ATI));
	rb_define_const(module, "GL_REG_7_ATI", INT2NUM(GL_REG_7_ATI));
	rb_define_const(module, "GL_REG_8_ATI", INT2NUM(GL_REG_8_ATI));
	rb_define_const(module, "GL_REG_9_ATI", INT2NUM(GL_REG_9_ATI));
	rb_define_const(module, "GL_RENDER", INT2NUM(GL_RENDER));
	rb_define_const(module, "GL_RENDERBUFFER", INT2NUM(GL_RENDERBUFFER));
	rb_define_const(module, "GL_RENDERBUFFER_ALPHA_SIZE", INT2NUM(GL_RENDERBUFFER_ALPHA_SIZE));
	rb_define_const(module, "GL_RENDERBUFFER_ALPHA_SIZE_EXT", INT2NUM(GL_RENDERBUFFER_ALPHA_SIZE_EXT));
	rb_define_const(module, "GL_RENDERBUFFER_BINDING", INT2NUM(GL_RENDERBUFFER_BINDING));
	rb_define_const(module, "GL_RENDERBUFFER_BINDING_EXT", INT2NUM(GL_RENDERBUFFER_BINDING_EXT));
	rb_define_const(module, "GL_RENDERBUFFER_BLUE_SIZE", INT2NUM(GL_RENDERBUFFER_BLUE_SIZE));
	rb_define_const(module, "GL_RENDERBUFFER_BLUE_SIZE_EXT", INT2NUM(GL_RENDERBUFFER_BLUE_SIZE_EXT));
	rb_define_const(module, "GL_RENDERBUFFER_COLOR_SAMPLES_NV", INT2NUM(GL_RENDERBUFFER_COLOR_SAMPLES_NV));
	rb_define_const(module, "GL_RENDERBUFFER_COVERAGE_SAMPLES_NV", INT2NUM(GL_RENDERBUFFER_COVERAGE_SAMPLES_NV));
	rb_define_const(module, "GL_RENDERBUFFER_DEPTH_SIZE", INT2NUM(GL_RENDERBUFFER_DEPTH_SIZE));
	rb_define_const(module, "GL_RENDERBUFFER_DEPTH_SIZE_EXT", INT2NUM(GL_RENDERBUFFER_DEPTH_SIZE_EXT));
	rb_define_const(module, "GL_RENDERBUFFER_EXT", INT2NUM(GL_RENDERBUFFER_EXT));
	rb_define_const(module, "GL_RENDERBUFFER_GREEN_SIZE", INT2NUM(GL_RENDERBUFFER_GREEN_SIZE));
	rb_define_const(module, "GL_RENDERBUFFER_GREEN_SIZE_EXT", INT2NUM(GL_RENDERBUFFER_GREEN_SIZE_EXT));
	rb_define_const(module, "GL_RENDERBUFFER_HEIGHT", INT2NUM(GL_RENDERBUFFER_HEIGHT));
	rb_define_const(module, "GL_RENDERBUFFER_HEIGHT_EXT", INT2NUM(GL_RENDERBUFFER_HEIGHT_EXT));
	rb_define_const(module, "GL_RENDERBUFFER_INTERNAL_FORMAT", INT2NUM(GL_RENDERBUFFER_INTERNAL_FORMAT));
	rb_define_const(module, "GL_RENDERBUFFER_INTERNAL_FORMAT_EXT", INT2NUM(GL_RENDERBUFFER_INTERNAL_FORMAT_EXT));
	rb_define_const(module, "GL_RENDERBUFFER_RED_SIZE", INT2NUM(GL_RENDERBUFFER_RED_SIZE));
	rb_define_const(module, "GL_RENDERBUFFER_RED_SIZE_EXT", INT2NUM(GL_RENDERBUFFER_RED_SIZE_EXT));
	rb_define_const(module, "GL_RENDERBUFFER_SAMPLES", INT2NUM(GL_RENDERBUFFER_SAMPLES));
	rb_define_const(module, "GL_RENDERBUFFER_SAMPLES_EXT", INT2NUM(GL_RENDERBUFFER_SAMPLES_EXT));
	rb_define_const(module, "GL_RENDERBUFFER_STENCIL_SIZE", INT2NUM(GL_RENDERBUFFER_STENCIL_SIZE));
	rb_define_const(module, "GL_RENDERBUFFER_STENCIL_SIZE_EXT", INT2NUM(GL_RENDERBUFFER_STENCIL_SIZE_EXT));
	rb_define_const(module, "GL_RENDERBUFFER_WIDTH", INT2NUM(GL_RENDERBUFFER_WIDTH));
	rb_define_const(module, "GL_RENDERBUFFER_WIDTH_EXT", INT2NUM(GL_RENDERBUFFER_WIDTH_EXT));
	rb_define_const(module, "GL_RENDERER", INT2NUM(GL_RENDERER));
	rb_define_const(module, "GL_RENDER_MODE", INT2NUM(GL_RENDER_MODE));
	rb_define_const(module, "GL_REPEAT", INT2NUM(GL_REPEAT));
	rb_define_const(module, "GL_REPLACE", INT2NUM(GL_REPLACE));
	rb_define_const(module, "GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN", INT2NUM(GL_REPLACEMENT_CODE_ARRAY_POINTER_SUN));
	rb_define_const(module, "GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN", INT2NUM(GL_REPLACEMENT_CODE_ARRAY_STRIDE_SUN));
	rb_define_const(module, "GL_REPLACEMENT_CODE_ARRAY_SUN", INT2NUM(GL_REPLACEMENT_CODE_ARRAY_SUN));
	rb_define_const(module, "GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN", INT2NUM(GL_REPLACEMENT_CODE_ARRAY_TYPE_SUN));
	rb_define_const(module, "GL_REPLACEMENT_CODE_SUN", INT2NUM(GL_REPLACEMENT_CODE_SUN));
	rb_define_const(module, "GL_REPLACE_EXT", INT2NUM(GL_REPLACE_EXT));
	rb_define_const(module, "GL_REPLACE_MIDDLE_SUN", INT2NUM(GL_REPLACE_MIDDLE_SUN));
	rb_define_const(module, "GL_REPLACE_OLDEST_SUN", INT2NUM(GL_REPLACE_OLDEST_SUN));
	rb_define_const(module, "GL_REPLICATE_BORDER", INT2NUM(GL_REPLICATE_BORDER));
	rb_define_const(module, "GL_REPLICATE_BORDER_HP", INT2NUM(GL_REPLICATE_BORDER_HP));
	rb_define_const(module, "GL_RESAMPLE_AVERAGE_OML", INT2NUM(GL_RESAMPLE_AVERAGE_OML));
	rb_define_const(module, "GL_RESAMPLE_DECIMATE_OML", INT2NUM(GL_RESAMPLE_DECIMATE_OML));
	rb_define_const(module, "GL_RESAMPLE_DECIMATE_SGIX", INT2NUM(GL_RESAMPLE_DECIMATE_SGIX));
	rb_define_const(module, "GL_RESAMPLE_REPLICATE_OML", INT2NUM(GL_RESAMPLE_REPLICATE_OML));
	rb_define_const(module, "GL_RESAMPLE_REPLICATE_SGIX", INT2NUM(GL_RESAMPLE_REPLICATE_SGIX));
	rb_define_const(module, "GL_RESAMPLE_ZERO_FILL_OML", INT2NUM(GL_RESAMPLE_ZERO_FILL_OML));
	rb_define_const(module, "GL_RESAMPLE_ZERO_FILL_SGIX", INT2NUM(GL_RESAMPLE_ZERO_FILL_SGIX));
	rb_define_const(module, "GL_RESCALE_NORMAL", INT2NUM(GL_RESCALE_NORMAL));
	rb_define_const(module, "GL_RESCALE_NORMAL_EXT", INT2NUM(GL_RESCALE_NORMAL_EXT));
	rb_define_const(module, "GL_RESTART_SUN", INT2NUM(GL_RESTART_SUN));
	rb_define_const(module, "GL_RETURN", INT2NUM(GL_RETURN));
	rb_define_const(module, "GL_RG", INT2NUM(GL_RG));
	rb_define_const(module, "GL_RG16", INT2NUM(GL_RG16));
	rb_define_const(module, "GL_RG16F", INT2NUM(GL_RG16F));
	rb_define_const(module, "GL_RG16I", INT2NUM(GL_RG16I));
	rb_define_const(module, "GL_RG16UI", INT2NUM(GL_RG16UI));
	rb_define_const(module, "GL_RG32F", INT2NUM(GL_RG32F));
	rb_define_const(module, "GL_RG32I", INT2NUM(GL_RG32I));
	rb_define_const(module, "GL_RG32UI", INT2NUM(GL_RG32UI));
	rb_define_const(module, "GL_RG8", INT2NUM(GL_RG8));
	rb_define_const(module, "GL_RG8I", INT2NUM(GL_RG8I));
	rb_define_const(module, "GL_RG8UI", INT2NUM(GL_RG8UI));
	rb_define_const(module, "GL_RGB", INT2NUM(GL_RGB));
	rb_define_const(module, "GL_RGB10", INT2NUM(GL_RGB10));
	rb_define_const(module, "GL_RGB10_A2", INT2NUM(GL_RGB10_A2));
	rb_define_const(module, "GL_RGB10_A2_EXT", INT2NUM(GL_RGB10_A2_EXT));
	rb_define_const(module, "GL_RGB10_EXT", INT2NUM(GL_RGB10_EXT));
	rb_define_const(module, "GL_RGB12", INT2NUM(GL_RGB12));
	rb_define_const(module, "GL_RGB12_EXT", INT2NUM(GL_RGB12_EXT));
	rb_define_const(module, "GL_RGB16", INT2NUM(GL_RGB16));
	rb_define_const(module, "GL_RGB16F", INT2NUM(GL_RGB16F));
	rb_define_const(module, "GL_RGB16F_ARB", INT2NUM(GL_RGB16F_ARB));
	rb_define_const(module, "GL_RGB16I", INT2NUM(GL_RGB16I));
	rb_define_const(module, "GL_RGB16I_EXT", INT2NUM(GL_RGB16I_EXT));
	rb_define_const(module, "GL_RGB16UI", INT2NUM(GL_RGB16UI));
	rb_define_const(module, "GL_RGB16UI_EXT", INT2NUM(GL_RGB16UI_EXT));
	rb_define_const(module, "GL_RGB16_EXT", INT2NUM(GL_RGB16_EXT));
	rb_define_const(module, "GL_RGB2_EXT", INT2NUM(GL_RGB2_EXT));
	rb_define_const(module, "GL_RGB32F", INT2NUM(GL_RGB32F));
	rb_define_const(module, "GL_RGB32F_ARB", INT2NUM(GL_RGB32F_ARB));
	rb_define_const(module, "GL_RGB32I", INT2NUM(GL_RGB32I));
	rb_define_const(module, "GL_RGB32I_EXT", INT2NUM(GL_RGB32I_EXT));
	rb_define_const(module, "GL_RGB32UI", INT2NUM(GL_RGB32UI));
	rb_define_const(module, "GL_RGB32UI_EXT", INT2NUM(GL_RGB32UI_EXT));
	rb_define_const(module, "GL_RGB4", INT2NUM(GL_RGB4));
	rb_define_const(module, "GL_RGB4_EXT", INT2NUM(GL_RGB4_EXT));
	rb_define_const(module, "GL_RGB4_S3TC", INT2NUM(GL_RGB4_S3TC));
	rb_define_const(module, "GL_RGB5", INT2NUM(GL_RGB5));
	rb_define_const(module, "GL_RGB5_A1", INT2NUM(GL_RGB5_A1));
	rb_define_const(module, "GL_RGB5_A1_EXT", INT2NUM(GL_RGB5_A1_EXT));
	rb_define_const(module, "GL_RGB5_EXT", INT2NUM(GL_RGB5_EXT));
	rb_define_const(module, "GL_RGB8", INT2NUM(GL_RGB8));
	rb_define_const(module, "GL_RGB8I", INT2NUM(GL_RGB8I));
	rb_define_const(module, "GL_RGB8I_EXT", INT2NUM(GL_RGB8I_EXT));
	rb_define_const(module, "GL_RGB8UI", INT2NUM(GL_RGB8UI));
	rb_define_const(module, "GL_RGB8UI_EXT", INT2NUM(GL_RGB8UI_EXT));
	rb_define_const(module, "GL_RGB8_EXT", INT2NUM(GL_RGB8_EXT));
	rb_define_const(module, "GL_RGB9_E5", INT2NUM(GL_RGB9_E5));
	rb_define_const(module, "GL_RGB9_E5_EXT", INT2NUM(GL_RGB9_E5_EXT));
	rb_define_const(module, "GL_RGBA", INT2NUM(GL_RGBA));
	rb_define_const(module, "GL_RGBA12", INT2NUM(GL_RGBA12));
	rb_define_const(module, "GL_RGBA12_EXT", INT2NUM(GL_RGBA12_EXT));
	rb_define_const(module, "GL_RGBA16", INT2NUM(GL_RGBA16));
	rb_define_const(module, "GL_RGBA16F", INT2NUM(GL_RGBA16F));
	rb_define_const(module, "GL_RGBA16F_ARB", INT2NUM(GL_RGBA16F_ARB));
	rb_define_const(module, "GL_RGBA16I", INT2NUM(GL_RGBA16I));
	rb_define_const(module, "GL_RGBA16I_EXT", INT2NUM(GL_RGBA16I_EXT));
	rb_define_const(module, "GL_RGBA16UI", INT2NUM(GL_RGBA16UI));
	rb_define_const(module, "GL_RGBA16UI_EXT", INT2NUM(GL_RGBA16UI_EXT));
	rb_define_const(module, "GL_RGBA16_EXT", INT2NUM(GL_RGBA16_EXT));
	rb_define_const(module, "GL_RGBA2", INT2NUM(GL_RGBA2));
	rb_define_const(module, "GL_RGBA2_EXT", INT2NUM(GL_RGBA2_EXT));
	rb_define_const(module, "GL_RGBA32F", INT2NUM(GL_RGBA32F));
	rb_define_const(module, "GL_RGBA32F_ARB", INT2NUM(GL_RGBA32F_ARB));
	rb_define_const(module, "GL_RGBA32I", INT2NUM(GL_RGBA32I));
	rb_define_const(module, "GL_RGBA32I_EXT", INT2NUM(GL_RGBA32I_EXT));
	rb_define_const(module, "GL_RGBA32UI", INT2NUM(GL_RGBA32UI));
	rb_define_const(module, "GL_RGBA32UI_EXT", INT2NUM(GL_RGBA32UI_EXT));
	rb_define_const(module, "GL_RGBA4", INT2NUM(GL_RGBA4));
	rb_define_const(module, "GL_RGBA4_EXT", INT2NUM(GL_RGBA4_EXT));
	rb_define_const(module, "GL_RGBA4_S3TC", INT2NUM(GL_RGBA4_S3TC));
	rb_define_const(module, "GL_RGBA8", INT2NUM(GL_RGBA8));
	rb_define_const(module, "GL_RGBA8I", INT2NUM(GL_RGBA8I));
	rb_define_const(module, "GL_RGBA8I_EXT", INT2NUM(GL_RGBA8I_EXT));
	rb_define_const(module, "GL_RGBA8UI", INT2NUM(GL_RGBA8UI));
	rb_define_const(module, "GL_RGBA8UI_EXT", INT2NUM(GL_RGBA8UI_EXT));
	rb_define_const(module, "GL_RGBA8_EXT", INT2NUM(GL_RGBA8_EXT));
	rb_define_const(module, "GL_RGBA_FLOAT16_ATI", INT2NUM(GL_RGBA_FLOAT16_ATI));
	rb_define_const(module, "GL_RGBA_FLOAT32_ATI", INT2NUM(GL_RGBA_FLOAT32_ATI));
	rb_define_const(module, "GL_RGBA_FLOAT_MODE_ARB", INT2NUM(GL_RGBA_FLOAT_MODE_ARB));
	rb_define_const(module, "GL_RGBA_INTEGER", INT2NUM(GL_RGBA_INTEGER));
	rb_define_const(module, "GL_RGBA_INTEGER_EXT", INT2NUM(GL_RGBA_INTEGER_EXT));
	rb_define_const(module, "GL_RGBA_INTEGER_MODE_EXT", INT2NUM(GL_RGBA_INTEGER_MODE_EXT));
	rb_define_const(module, "GL_RGBA_MODE", INT2NUM(GL_RGBA_MODE));
	rb_define_const(module, "GL_RGBA_S3TC", INT2NUM(GL_RGBA_S3TC));
	rb_define_const(module, "GL_RGBA_SIGNED_COMPONENTS_EXT", INT2NUM(GL_RGBA_SIGNED_COMPONENTS_EXT));
	rb_define_const(module, "GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV", INT2NUM(GL_RGBA_UNSIGNED_DOT_PRODUCT_MAPPING_NV));
	rb_define_const(module, "GL_RGB_FLOAT16_ATI", INT2NUM(GL_RGB_FLOAT16_ATI));
	rb_define_const(module, "GL_RGB_FLOAT32_ATI", INT2NUM(GL_RGB_FLOAT32_ATI));
	rb_define_const(module, "GL_RGB_INTEGER", INT2NUM(GL_RGB_INTEGER));
	rb_define_const(module, "GL_RGB_INTEGER_EXT", INT2NUM(GL_RGB_INTEGER_EXT));
	rb_define_const(module, "GL_RGB_S3TC", INT2NUM(GL_RGB_S3TC));
	rb_define_const(module, "GL_RGB_SCALE", INT2NUM(GL_RGB_SCALE));
	rb_define_const(module, "GL_RGB_SCALE_ARB", INT2NUM(GL_RGB_SCALE_ARB));
	rb_define_const(module, "GL_RGB_SCALE_EXT", INT2NUM(GL_RGB_SCALE_EXT));
	rb_define_const(module, "GL_RG_INTEGER", INT2NUM(GL_RG_INTEGER));
	rb_define_const(module, "GL_RIGHT", INT2NUM(GL_RIGHT));
	rb_define_const(module, "GL_S", INT2NUM(GL_S));
	rb_define_const(module, "GL_SAMPLER_1D", INT2NUM(GL_SAMPLER_1D));
	rb_define_const(module, "GL_SAMPLER_1D_ARB", INT2NUM(GL_SAMPLER_1D_ARB));
	rb_define_const(module, "GL_SAMPLER_1D_ARRAY", INT2NUM(GL_SAMPLER_1D_ARRAY));
	rb_define_const(module, "GL_SAMPLER_1D_ARRAY_EXT", INT2NUM(GL_SAMPLER_1D_ARRAY_EXT));
	rb_define_const(module, "GL_SAMPLER_1D_ARRAY_SHADOW", INT2NUM(GL_SAMPLER_1D_ARRAY_SHADOW));
	rb_define_const(module, "GL_SAMPLER_1D_ARRAY_SHADOW_EXT", INT2NUM(GL_SAMPLER_1D_ARRAY_SHADOW_EXT));
	rb_define_const(module, "GL_SAMPLER_1D_SHADOW", INT2NUM(GL_SAMPLER_1D_SHADOW));
	rb_define_const(module, "GL_SAMPLER_1D_SHADOW_ARB", INT2NUM(GL_SAMPLER_1D_SHADOW_ARB));
	rb_define_const(module, "GL_SAMPLER_2D", INT2NUM(GL_SAMPLER_2D));
	rb_define_const(module, "GL_SAMPLER_2D_ARB", INT2NUM(GL_SAMPLER_2D_ARB));
	rb_define_const(module, "GL_SAMPLER_2D_ARRAY", INT2NUM(GL_SAMPLER_2D_ARRAY));
	rb_define_const(module, "GL_SAMPLER_2D_ARRAY_EXT", INT2NUM(GL_SAMPLER_2D_ARRAY_EXT));
	rb_define_const(module, "GL_SAMPLER_2D_ARRAY_SHADOW", INT2NUM(GL_SAMPLER_2D_ARRAY_SHADOW));
	rb_define_const(module, "GL_SAMPLER_2D_ARRAY_SHADOW_EXT", INT2NUM(GL_SAMPLER_2D_ARRAY_SHADOW_EXT));
	rb_define_const(module, "GL_SAMPLER_2D_RECT_ARB", INT2NUM(GL_SAMPLER_2D_RECT_ARB));
	rb_define_const(module, "GL_SAMPLER_2D_RECT_SHADOW_ARB", INT2NUM(GL_SAMPLER_2D_RECT_SHADOW_ARB));
	rb_define_const(module, "GL_SAMPLER_2D_SHADOW", INT2NUM(GL_SAMPLER_2D_SHADOW));
	rb_define_const(module, "GL_SAMPLER_2D_SHADOW_ARB", INT2NUM(GL_SAMPLER_2D_SHADOW_ARB));
	rb_define_const(module, "GL_SAMPLER_3D", INT2NUM(GL_SAMPLER_3D));
	rb_define_const(module, "GL_SAMPLER_3D_ARB", INT2NUM(GL_SAMPLER_3D_ARB));
	rb_define_const(module, "GL_SAMPLER_BUFFER_EXT", INT2NUM(GL_SAMPLER_BUFFER_EXT));
	rb_define_const(module, "GL_SAMPLER_CUBE", INT2NUM(GL_SAMPLER_CUBE));
	rb_define_const(module, "GL_SAMPLER_CUBE_ARB", INT2NUM(GL_SAMPLER_CUBE_ARB));
	rb_define_const(module, "GL_SAMPLER_CUBE_SHADOW", INT2NUM(GL_SAMPLER_CUBE_SHADOW));
	rb_define_const(module, "GL_SAMPLER_CUBE_SHADOW_EXT", INT2NUM(GL_SAMPLER_CUBE_SHADOW_EXT));
	rb_define_const(module, "GL_SAMPLER_RENDERBUFFER_NV", INT2NUM(GL_SAMPLER_RENDERBUFFER_NV));
	rb_define_const(module, "GL_SAMPLES", INT2NUM(GL_SAMPLES));
	rb_define_const(module, "GL_SAMPLES_3DFX", INT2NUM(GL_SAMPLES_3DFX));
	rb_define_const(module, "GL_SAMPLES_ARB", INT2NUM(GL_SAMPLES_ARB));
	rb_define_const(module, "GL_SAMPLES_EXT", INT2NUM(GL_SAMPLES_EXT));
	rb_define_const(module, "GL_SAMPLES_PASSED", INT2NUM(GL_SAMPLES_PASSED));
	rb_define_const(module, "GL_SAMPLES_PASSED_ARB", INT2NUM(GL_SAMPLES_PASSED_ARB));
	rb_define_const(module, "GL_SAMPLES_SGIS", INT2NUM(GL_SAMPLES_SGIS));
	rb_define_const(module, "GL_SAMPLE_ALPHA_TO_COVERAGE", INT2NUM(GL_SAMPLE_ALPHA_TO_COVERAGE));
	rb_define_const(module, "GL_SAMPLE_ALPHA_TO_COVERAGE_ARB", INT2NUM(GL_SAMPLE_ALPHA_TO_COVERAGE_ARB));
	rb_define_const(module, "GL_SAMPLE_ALPHA_TO_MASK_EXT", INT2NUM(GL_SAMPLE_ALPHA_TO_MASK_EXT));
	rb_define_const(module, "GL_SAMPLE_ALPHA_TO_MASK_SGIS", INT2NUM(GL_SAMPLE_ALPHA_TO_MASK_SGIS));
	rb_define_const(module, "GL_SAMPLE_ALPHA_TO_ONE", INT2NUM(GL_SAMPLE_ALPHA_TO_ONE));
	rb_define_const(module, "GL_SAMPLE_ALPHA_TO_ONE_ARB", INT2NUM(GL_SAMPLE_ALPHA_TO_ONE_ARB));
	rb_define_const(module, "GL_SAMPLE_ALPHA_TO_ONE_EXT", INT2NUM(GL_SAMPLE_ALPHA_TO_ONE_EXT));
	rb_define_const(module, "GL_SAMPLE_ALPHA_TO_ONE_SGIS", INT2NUM(GL_SAMPLE_ALPHA_TO_ONE_SGIS));
	rb_define_const(module, "GL_SAMPLE_BUFFERS", INT2NUM(GL_SAMPLE_BUFFERS));
	rb_define_const(module, "GL_SAMPLE_BUFFERS_3DFX", INT2NUM(GL_SAMPLE_BUFFERS_3DFX));
	rb_define_const(module, "GL_SAMPLE_BUFFERS_ARB", INT2NUM(GL_SAMPLE_BUFFERS_ARB));
	rb_define_const(module, "GL_SAMPLE_BUFFERS_EXT", INT2NUM(GL_SAMPLE_BUFFERS_EXT));
	rb_define_const(module, "GL_SAMPLE_BUFFERS_SGIS", INT2NUM(GL_SAMPLE_BUFFERS_SGIS));
	rb_define_const(module, "GL_SAMPLE_COVERAGE", INT2NUM(GL_SAMPLE_COVERAGE));
	rb_define_const(module, "GL_SAMPLE_COVERAGE_ARB", INT2NUM(GL_SAMPLE_COVERAGE_ARB));
	rb_define_const(module, "GL_SAMPLE_COVERAGE_INVERT", INT2NUM(GL_SAMPLE_COVERAGE_INVERT));
	rb_define_const(module, "GL_SAMPLE_COVERAGE_INVERT_ARB", INT2NUM(GL_SAMPLE_COVERAGE_INVERT_ARB));
	rb_define_const(module, "GL_SAMPLE_COVERAGE_VALUE", INT2NUM(GL_SAMPLE_COVERAGE_VALUE));
	rb_define_const(module, "GL_SAMPLE_COVERAGE_VALUE_ARB", INT2NUM(GL_SAMPLE_COVERAGE_VALUE_ARB));
	rb_define_const(module, "GL_SAMPLE_MASK_EXT", INT2NUM(GL_SAMPLE_MASK_EXT));
	rb_define_const(module, "GL_SAMPLE_MASK_INVERT_EXT", INT2NUM(GL_SAMPLE_MASK_INVERT_EXT));
	rb_define_const(module, "GL_SAMPLE_MASK_INVERT_SGIS", INT2NUM(GL_SAMPLE_MASK_INVERT_SGIS));
	rb_define_const(module, "GL_SAMPLE_MASK_NV", INT2NUM(GL_SAMPLE_MASK_NV));
	rb_define_const(module, "GL_SAMPLE_MASK_SGIS", INT2NUM(GL_SAMPLE_MASK_SGIS));
	rb_define_const(module, "GL_SAMPLE_MASK_VALUE_EXT", INT2NUM(GL_SAMPLE_MASK_VALUE_EXT));
	rb_define_const(module, "GL_SAMPLE_MASK_VALUE_NV", INT2NUM(GL_SAMPLE_MASK_VALUE_NV));
	rb_define_const(module, "GL_SAMPLE_MASK_VALUE_SGIS", INT2NUM(GL_SAMPLE_MASK_VALUE_SGIS));
	rb_define_const(module, "GL_SAMPLE_PATTERN_EXT", INT2NUM(GL_SAMPLE_PATTERN_EXT));
	rb_define_const(module, "GL_SAMPLE_PATTERN_SGIS", INT2NUM(GL_SAMPLE_PATTERN_SGIS));
	rb_define_const(module, "GL_SAMPLE_POSITION_NV", INT2NUM(GL_SAMPLE_POSITION_NV));
	rb_define_const(module, "GL_SATURATE_BIT_ATI", INT2NUM(GL_SATURATE_BIT_ATI));
	rb_define_const(module, "GL_SCALAR_EXT", INT2NUM(GL_SCALAR_EXT));
	rb_define_const(module, "GL_SCALEBIAS_HINT_SGIX", INT2NUM(GL_SCALEBIAS_HINT_SGIX));
	rb_define_const(module, "GL_SCALE_BY_FOUR_NV", INT2NUM(GL_SCALE_BY_FOUR_NV));
	rb_define_const(module, "GL_SCALE_BY_ONE_HALF_NV", INT2NUM(GL_SCALE_BY_ONE_HALF_NV));
	rb_define_const(module, "GL_SCALE_BY_TWO_NV", INT2NUM(GL_SCALE_BY_TWO_NV));
	rb_define_const(module, "GL_SCISSOR_BIT", INT2NUM(GL_SCISSOR_BIT));
	rb_define_const(module, "GL_SCISSOR_BOX", INT2NUM(GL_SCISSOR_BOX));
	rb_define_const(module, "GL_SCISSOR_TEST", INT2NUM(GL_SCISSOR_TEST));
	rb_define_const(module, "GL_SCREEN_COORDINATES_REND", INT2NUM(GL_SCREEN_COORDINATES_REND));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY", INT2NUM(GL_SECONDARY_COLOR_ARRAY));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING", INT2NUM(GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_EXT", INT2NUM(GL_SECONDARY_COLOR_ARRAY_EXT));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_LIST_IBM", INT2NUM(GL_SECONDARY_COLOR_ARRAY_LIST_IBM));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM", INT2NUM(GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_POINTER", INT2NUM(GL_SECONDARY_COLOR_ARRAY_POINTER));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_POINTER_EXT", INT2NUM(GL_SECONDARY_COLOR_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_SIZE", INT2NUM(GL_SECONDARY_COLOR_ARRAY_SIZE));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_SIZE_EXT", INT2NUM(GL_SECONDARY_COLOR_ARRAY_SIZE_EXT));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_STRIDE", INT2NUM(GL_SECONDARY_COLOR_ARRAY_STRIDE));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT", INT2NUM(GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_TYPE", INT2NUM(GL_SECONDARY_COLOR_ARRAY_TYPE));
	rb_define_const(module, "GL_SECONDARY_COLOR_ARRAY_TYPE_EXT", INT2NUM(GL_SECONDARY_COLOR_ARRAY_TYPE_EXT));
	rb_define_const(module, "GL_SECONDARY_COLOR_NV", INT2NUM(GL_SECONDARY_COLOR_NV));
	rb_define_const(module, "GL_SECONDARY_INTERPOLATOR_ATI", INT2NUM(GL_SECONDARY_INTERPOLATOR_ATI));
	rb_define_const(module, "GL_SELECT", INT2NUM(GL_SELECT));
	rb_define_const(module, "GL_SELECTION_BUFFER_POINTER", INT2NUM(GL_SELECTION_BUFFER_POINTER));
	rb_define_const(module, "GL_SELECTION_BUFFER_SIZE", INT2NUM(GL_SELECTION_BUFFER_SIZE));
	rb_define_const(module, "GL_SEPARABLE_2D", INT2NUM(GL_SEPARABLE_2D));
	rb_define_const(module, "GL_SEPARABLE_2D_EXT", INT2NUM(GL_SEPARABLE_2D_EXT));
	rb_define_const(module, "GL_SEPARATE_ATTRIBS", INT2NUM(GL_SEPARATE_ATTRIBS));
	rb_define_const(module, "GL_SEPARATE_ATTRIBS_EXT", INT2NUM(GL_SEPARATE_ATTRIBS_EXT));
	rb_define_const(module, "GL_SEPARATE_ATTRIBS_NV", INT2NUM(GL_SEPARATE_ATTRIBS_NV));
	rb_define_const(module, "GL_SEPARATE_SPECULAR_COLOR", INT2NUM(GL_SEPARATE_SPECULAR_COLOR));
	rb_define_const(module, "GL_SEPARATE_SPECULAR_COLOR_EXT", INT2NUM(GL_SEPARATE_SPECULAR_COLOR_EXT));
	rb_define_const(module, "GL_SET", INT2NUM(GL_SET));
	rb_define_const(module, "GL_SGIS_detail_texture", INT2NUM(GL_SGIS_detail_texture));
	rb_define_const(module, "GL_SGIS_fog_function", INT2NUM(GL_SGIS_fog_function));
	rb_define_const(module, "GL_SGIS_generate_mipmap", INT2NUM(GL_SGIS_generate_mipmap));
	rb_define_const(module, "GL_SGIS_multisample", INT2NUM(GL_SGIS_multisample));
	rb_define_const(module, "GL_SGIS_pixel_texture", INT2NUM(GL_SGIS_pixel_texture));
	rb_define_const(module, "GL_SGIS_point_line_texgen", INT2NUM(GL_SGIS_point_line_texgen));
	rb_define_const(module, "GL_SGIS_point_parameters", INT2NUM(GL_SGIS_point_parameters));
	rb_define_const(module, "GL_SGIS_sharpen_texture", INT2NUM(GL_SGIS_sharpen_texture));
	rb_define_const(module, "GL_SGIS_texture4D", INT2NUM(GL_SGIS_texture4D));
	rb_define_const(module, "GL_SGIS_texture_border_clamp", INT2NUM(GL_SGIS_texture_border_clamp));
	rb_define_const(module, "GL_SGIS_texture_edge_clamp", INT2NUM(GL_SGIS_texture_edge_clamp));
	rb_define_const(module, "GL_SGIS_texture_filter4", INT2NUM(GL_SGIS_texture_filter4));
	rb_define_const(module, "GL_SGIS_texture_lod", INT2NUM(GL_SGIS_texture_lod));
	rb_define_const(module, "GL_SGIS_texture_select", INT2NUM(GL_SGIS_texture_select));
	rb_define_const(module, "GL_SGIX_async", INT2NUM(GL_SGIX_async));
	rb_define_const(module, "GL_SGIX_async_histogram", INT2NUM(GL_SGIX_async_histogram));
	rb_define_const(module, "GL_SGIX_async_pixel", INT2NUM(GL_SGIX_async_pixel));
	rb_define_const(module, "GL_SGIX_blend_alpha_minmax", INT2NUM(GL_SGIX_blend_alpha_minmax));
	rb_define_const(module, "GL_SGIX_calligraphic_fragment", INT2NUM(GL_SGIX_calligraphic_fragment));
	rb_define_const(module, "GL_SGIX_clipmap", INT2NUM(GL_SGIX_clipmap));
	rb_define_const(module, "GL_SGIX_convolution_accuracy", INT2NUM(GL_SGIX_convolution_accuracy));
	rb_define_const(module, "GL_SGIX_depth_texture", INT2NUM(GL_SGIX_depth_texture));
	rb_define_const(module, "GL_SGIX_flush_raster", INT2NUM(GL_SGIX_flush_raster));
	rb_define_const(module, "GL_SGIX_fog_offset", INT2NUM(GL_SGIX_fog_offset));
	rb_define_const(module, "GL_SGIX_fragment_lighting", INT2NUM(GL_SGIX_fragment_lighting));
	rb_define_const(module, "GL_SGIX_framezoom", INT2NUM(GL_SGIX_framezoom));
	rb_define_const(module, "GL_SGIX_icc_texture", INT2NUM(GL_SGIX_icc_texture));
	rb_define_const(module, "GL_SGIX_impact_pixel_texture", INT2NUM(GL_SGIX_impact_pixel_texture));
	rb_define_const(module, "GL_SGIX_instruments", INT2NUM(GL_SGIX_instruments));
	rb_define_const(module, "GL_SGIX_interlace", INT2NUM(GL_SGIX_interlace));
	rb_define_const(module, "GL_SGIX_ir_instrument1", INT2NUM(GL_SGIX_ir_instrument1));
	rb_define_const(module, "GL_SGIX_list_priority", INT2NUM(GL_SGIX_list_priority));
	rb_define_const(module, "GL_SGIX_pixel_texture", INT2NUM(GL_SGIX_pixel_texture));
	rb_define_const(module, "GL_SGIX_pixel_tiles", INT2NUM(GL_SGIX_pixel_tiles));
	rb_define_const(module, "GL_SGIX_polynomial_ffd", INT2NUM(GL_SGIX_polynomial_ffd));
	rb_define_const(module, "GL_SGIX_reference_plane", INT2NUM(GL_SGIX_reference_plane));
	rb_define_const(module, "GL_SGIX_resample", INT2NUM(GL_SGIX_resample));
	rb_define_const(module, "GL_SGIX_scalebias_hint", INT2NUM(GL_SGIX_scalebias_hint));
	rb_define_const(module, "GL_SGIX_shadow", INT2NUM(GL_SGIX_shadow));
	rb_define_const(module, "GL_SGIX_shadow_ambient", INT2NUM(GL_SGIX_shadow_ambient));
	rb_define_const(module, "GL_SGIX_sprite", INT2NUM(GL_SGIX_sprite));
	rb_define_const(module, "GL_SGIX_subsample", INT2NUM(GL_SGIX_subsample));
	rb_define_const(module, "GL_SGIX_tag_sample_buffer", INT2NUM(GL_SGIX_tag_sample_buffer));
	rb_define_const(module, "GL_SGIX_texture_add_env", INT2NUM(GL_SGIX_texture_add_env));
	rb_define_const(module, "GL_SGIX_texture_coordinate_clamp", INT2NUM(GL_SGIX_texture_coordinate_clamp));
	rb_define_const(module, "GL_SGIX_texture_lod_bias", INT2NUM(GL_SGIX_texture_lod_bias));
	rb_define_const(module, "GL_SGIX_texture_multi_buffer", INT2NUM(GL_SGIX_texture_multi_buffer));
	rb_define_const(module, "GL_SGIX_texture_scale_bias", INT2NUM(GL_SGIX_texture_scale_bias));
	rb_define_const(module, "GL_SGIX_vertex_preclip", INT2NUM(GL_SGIX_vertex_preclip));
	rb_define_const(module, "GL_SGIX_ycrcb", INT2NUM(GL_SGIX_ycrcb));
	rb_define_const(module, "GL_SGI_color_matrix", INT2NUM(GL_SGI_color_matrix));
	rb_define_const(module, "GL_SGI_color_table", INT2NUM(GL_SGI_color_table));
	rb_define_const(module, "GL_SGI_texture_color_table", INT2NUM(GL_SGI_texture_color_table));
	rb_define_const(module, "GL_SHADER_CONSISTENT_NV", INT2NUM(GL_SHADER_CONSISTENT_NV));
	rb_define_const(module, "GL_SHADER_OBJECT_ARB", INT2NUM(GL_SHADER_OBJECT_ARB));
	rb_define_const(module, "GL_SHADER_OPERATION_NV", INT2NUM(GL_SHADER_OPERATION_NV));
	rb_define_const(module, "GL_SHADER_SOURCE_LENGTH", INT2NUM(GL_SHADER_SOURCE_LENGTH));
	rb_define_const(module, "GL_SHADER_TYPE", INT2NUM(GL_SHADER_TYPE));
	rb_define_const(module, "GL_SHADE_MODEL", INT2NUM(GL_SHADE_MODEL));
	rb_define_const(module, "GL_SHADING_LANGUAGE_VERSION", INT2NUM(GL_SHADING_LANGUAGE_VERSION));
	rb_define_const(module, "GL_SHADING_LANGUAGE_VERSION_ARB", INT2NUM(GL_SHADING_LANGUAGE_VERSION_ARB));
	rb_define_const(module, "GL_SHADOW_AMBIENT_SGIX", INT2NUM(GL_SHADOW_AMBIENT_SGIX));
	rb_define_const(module, "GL_SHADOW_ATTENUATION_EXT", INT2NUM(GL_SHADOW_ATTENUATION_EXT));
	rb_define_const(module, "GL_SHARED_TEXTURE_PALETTE_EXT", INT2NUM(GL_SHARED_TEXTURE_PALETTE_EXT));
	rb_define_const(module, "GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS", INT2NUM(GL_SHARPEN_TEXTURE_FUNC_POINTS_SGIS));
	rb_define_const(module, "GL_SHININESS", INT2NUM(GL_SHININESS));
	rb_define_const(module, "GL_SHORT", INT2NUM(GL_SHORT));
	rb_define_const(module, "GL_SIGNED_ALPHA8_NV", INT2NUM(GL_SIGNED_ALPHA8_NV));
	rb_define_const(module, "GL_SIGNED_ALPHA_NV", INT2NUM(GL_SIGNED_ALPHA_NV));
	rb_define_const(module, "GL_SIGNED_HILO16_NV", INT2NUM(GL_SIGNED_HILO16_NV));
	rb_define_const(module, "GL_SIGNED_HILO8_NV", INT2NUM(GL_SIGNED_HILO8_NV));
	rb_define_const(module, "GL_SIGNED_HILO_NV", INT2NUM(GL_SIGNED_HILO_NV));
	rb_define_const(module, "GL_SIGNED_IDENTITY_NV", INT2NUM(GL_SIGNED_IDENTITY_NV));
	rb_define_const(module, "GL_SIGNED_INTENSITY8_NV", INT2NUM(GL_SIGNED_INTENSITY8_NV));
	rb_define_const(module, "GL_SIGNED_INTENSITY_NV", INT2NUM(GL_SIGNED_INTENSITY_NV));
	rb_define_const(module, "GL_SIGNED_LUMINANCE8_ALPHA8_NV", INT2NUM(GL_SIGNED_LUMINANCE8_ALPHA8_NV));
	rb_define_const(module, "GL_SIGNED_LUMINANCE8_NV", INT2NUM(GL_SIGNED_LUMINANCE8_NV));
	rb_define_const(module, "GL_SIGNED_LUMINANCE_ALPHA_NV", INT2NUM(GL_SIGNED_LUMINANCE_ALPHA_NV));
	rb_define_const(module, "GL_SIGNED_LUMINANCE_NV", INT2NUM(GL_SIGNED_LUMINANCE_NV));
	rb_define_const(module, "GL_SIGNED_NEGATE_NV", INT2NUM(GL_SIGNED_NEGATE_NV));
	rb_define_const(module, "GL_SIGNED_RGB8_NV", INT2NUM(GL_SIGNED_RGB8_NV));
	rb_define_const(module, "GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV", INT2NUM(GL_SIGNED_RGB8_UNSIGNED_ALPHA8_NV));
	rb_define_const(module, "GL_SIGNED_RGBA8_NV", INT2NUM(GL_SIGNED_RGBA8_NV));
	rb_define_const(module, "GL_SIGNED_RGBA_NV", INT2NUM(GL_SIGNED_RGBA_NV));
	rb_define_const(module, "GL_SIGNED_RGB_NV", INT2NUM(GL_SIGNED_RGB_NV));
	rb_define_const(module, "GL_SIGNED_RGB_UNSIGNED_ALPHA_NV", INT2NUM(GL_SIGNED_RGB_UNSIGNED_ALPHA_NV));
	rb_define_const(module, "GL_SINGLE_COLOR", INT2NUM(GL_SINGLE_COLOR));
	rb_define_const(module, "GL_SINGLE_COLOR_EXT", INT2NUM(GL_SINGLE_COLOR_EXT));
	rb_define_const(module, "GL_SLICE_ACCUM_SUN", INT2NUM(GL_SLICE_ACCUM_SUN));
	rb_define_const(module, "GL_SLUMINANCE", INT2NUM(GL_SLUMINANCE));
	rb_define_const(module, "GL_SLUMINANCE8", INT2NUM(GL_SLUMINANCE8));
	rb_define_const(module, "GL_SLUMINANCE8_ALPHA8", INT2NUM(GL_SLUMINANCE8_ALPHA8));
	rb_define_const(module, "GL_SLUMINANCE8_ALPHA8_EXT", INT2NUM(GL_SLUMINANCE8_ALPHA8_EXT));
	rb_define_const(module, "GL_SLUMINANCE8_EXT", INT2NUM(GL_SLUMINANCE8_EXT));
	rb_define_const(module, "GL_SLUMINANCE_ALPHA", INT2NUM(GL_SLUMINANCE_ALPHA));
	rb_define_const(module, "GL_SLUMINANCE_ALPHA_EXT", INT2NUM(GL_SLUMINANCE_ALPHA_EXT));
	rb_define_const(module, "GL_SLUMINANCE_EXT", INT2NUM(GL_SLUMINANCE_EXT));
	rb_define_const(module, "GL_SMOOTH", INT2NUM(GL_SMOOTH));
	rb_define_const(module, "GL_SMOOTH_LINE_WIDTH_GRANULARITY", INT2NUM(GL_SMOOTH_LINE_WIDTH_GRANULARITY));
	rb_define_const(module, "GL_SMOOTH_LINE_WIDTH_RANGE", INT2NUM(GL_SMOOTH_LINE_WIDTH_RANGE));
	rb_define_const(module, "GL_SMOOTH_POINT_SIZE_GRANULARITY", INT2NUM(GL_SMOOTH_POINT_SIZE_GRANULARITY));
	rb_define_const(module, "GL_SMOOTH_POINT_SIZE_RANGE", INT2NUM(GL_SMOOTH_POINT_SIZE_RANGE));
	rb_define_const(module, "GL_SOURCE0_ALPHA", INT2NUM(GL_SOURCE0_ALPHA));
	rb_define_const(module, "GL_SOURCE0_ALPHA_ARB", INT2NUM(GL_SOURCE0_ALPHA_ARB));
	rb_define_const(module, "GL_SOURCE0_ALPHA_EXT", INT2NUM(GL_SOURCE0_ALPHA_EXT));
	rb_define_const(module, "GL_SOURCE0_RGB", INT2NUM(GL_SOURCE0_RGB));
	rb_define_const(module, "GL_SOURCE0_RGB_ARB", INT2NUM(GL_SOURCE0_RGB_ARB));
	rb_define_const(module, "GL_SOURCE0_RGB_EXT", INT2NUM(GL_SOURCE0_RGB_EXT));
	rb_define_const(module, "GL_SOURCE1_ALPHA", INT2NUM(GL_SOURCE1_ALPHA));
	rb_define_const(module, "GL_SOURCE1_ALPHA_ARB", INT2NUM(GL_SOURCE1_ALPHA_ARB));
	rb_define_const(module, "GL_SOURCE1_ALPHA_EXT", INT2NUM(GL_SOURCE1_ALPHA_EXT));
	rb_define_const(module, "GL_SOURCE1_RGB", INT2NUM(GL_SOURCE1_RGB));
	rb_define_const(module, "GL_SOURCE1_RGB_ARB", INT2NUM(GL_SOURCE1_RGB_ARB));
	rb_define_const(module, "GL_SOURCE1_RGB_EXT", INT2NUM(GL_SOURCE1_RGB_EXT));
	rb_define_const(module, "GL_SOURCE2_ALPHA", INT2NUM(GL_SOURCE2_ALPHA));
	rb_define_const(module, "GL_SOURCE2_ALPHA_ARB", INT2NUM(GL_SOURCE2_ALPHA_ARB));
	rb_define_const(module, "GL_SOURCE2_ALPHA_EXT", INT2NUM(GL_SOURCE2_ALPHA_EXT));
	rb_define_const(module, "GL_SOURCE2_RGB", INT2NUM(GL_SOURCE2_RGB));
	rb_define_const(module, "GL_SOURCE2_RGB_ARB", INT2NUM(GL_SOURCE2_RGB_ARB));
	rb_define_const(module, "GL_SOURCE2_RGB_EXT", INT2NUM(GL_SOURCE2_RGB_EXT));
	rb_define_const(module, "GL_SOURCE3_ALPHA_NV", INT2NUM(GL_SOURCE3_ALPHA_NV));
	rb_define_const(module, "GL_SOURCE3_RGB_NV", INT2NUM(GL_SOURCE3_RGB_NV));
	rb_define_const(module, "GL_SPARE0_NV", INT2NUM(GL_SPARE0_NV));
	rb_define_const(module, "GL_SPARE0_PLUS_SECONDARY_COLOR_NV", INT2NUM(GL_SPARE0_PLUS_SECONDARY_COLOR_NV));
	rb_define_const(module, "GL_SPARE1_NV", INT2NUM(GL_SPARE1_NV));
	rb_define_const(module, "GL_SPECULAR", INT2NUM(GL_SPECULAR));
	rb_define_const(module, "GL_SPHERE_MAP", INT2NUM(GL_SPHERE_MAP));
	rb_define_const(module, "GL_SPOT_CUTOFF", INT2NUM(GL_SPOT_CUTOFF));
	rb_define_const(module, "GL_SPOT_DIRECTION", INT2NUM(GL_SPOT_DIRECTION));
	rb_define_const(module, "GL_SPOT_EXPONENT", INT2NUM(GL_SPOT_EXPONENT));
	rb_define_const(module, "GL_SPRITE_AXIAL_SGIX", INT2NUM(GL_SPRITE_AXIAL_SGIX));
	rb_define_const(module, "GL_SPRITE_AXIS_SGIX", INT2NUM(GL_SPRITE_AXIS_SGIX));
	rb_define_const(module, "GL_SPRITE_EYE_ALIGNED_SGIX", INT2NUM(GL_SPRITE_EYE_ALIGNED_SGIX));
	rb_define_const(module, "GL_SPRITE_MODE_SGIX", INT2NUM(GL_SPRITE_MODE_SGIX));
	rb_define_const(module, "GL_SPRITE_OBJECT_ALIGNED_SGIX", INT2NUM(GL_SPRITE_OBJECT_ALIGNED_SGIX));
	rb_define_const(module, "GL_SPRITE_SGIX", INT2NUM(GL_SPRITE_SGIX));
	rb_define_const(module, "GL_SPRITE_TRANSLATION_SGIX", INT2NUM(GL_SPRITE_TRANSLATION_SGIX));
	rb_define_const(module, "GL_SRC0_ALPHA", INT2NUM(GL_SRC0_ALPHA));
	rb_define_const(module, "GL_SRC0_RGB", INT2NUM(GL_SRC0_RGB));
	rb_define_const(module, "GL_SRC1_ALPHA", INT2NUM(GL_SRC1_ALPHA));
	rb_define_const(module, "GL_SRC1_RGB", INT2NUM(GL_SRC1_RGB));
	rb_define_const(module, "GL_SRC2_ALPHA", INT2NUM(GL_SRC2_ALPHA));
	rb_define_const(module, "GL_SRC2_RGB", INT2NUM(GL_SRC2_RGB));
	rb_define_const(module, "GL_SRC_ALPHA", INT2NUM(GL_SRC_ALPHA));
	rb_define_const(module, "GL_SRC_ALPHA_SATURATE", INT2NUM(GL_SRC_ALPHA_SATURATE));
	rb_define_const(module, "GL_SRC_COLOR", INT2NUM(GL_SRC_COLOR));
	rb_define_const(module, "GL_SRGB", INT2NUM(GL_SRGB));
	rb_define_const(module, "GL_SRGB8", INT2NUM(GL_SRGB8));
	rb_define_const(module, "GL_SRGB8_ALPHA8", INT2NUM(GL_SRGB8_ALPHA8));
	rb_define_const(module, "GL_SRGB8_ALPHA8_EXT", INT2NUM(GL_SRGB8_ALPHA8_EXT));
	rb_define_const(module, "GL_SRGB8_EXT", INT2NUM(GL_SRGB8_EXT));
	rb_define_const(module, "GL_SRGB_ALPHA", INT2NUM(GL_SRGB_ALPHA));
	rb_define_const(module, "GL_SRGB_ALPHA_EXT", INT2NUM(GL_SRGB_ALPHA_EXT));
	rb_define_const(module, "GL_SRGB_EXT", INT2NUM(GL_SRGB_EXT));
	rb_define_const(module, "GL_STACK_OVERFLOW", INT2NUM(GL_STACK_OVERFLOW));
	rb_define_const(module, "GL_STACK_UNDERFLOW", INT2NUM(GL_STACK_UNDERFLOW));
	rb_define_const(module, "GL_STATIC_ATI", INT2NUM(GL_STATIC_ATI));
	rb_define_const(module, "GL_STATIC_COPY", INT2NUM(GL_STATIC_COPY));
	rb_define_const(module, "GL_STATIC_COPY_ARB", INT2NUM(GL_STATIC_COPY_ARB));
	rb_define_const(module, "GL_STATIC_DRAW", INT2NUM(GL_STATIC_DRAW));
	rb_define_const(module, "GL_STATIC_DRAW_ARB", INT2NUM(GL_STATIC_DRAW_ARB));
	rb_define_const(module, "GL_STATIC_READ", INT2NUM(GL_STATIC_READ));
	rb_define_const(module, "GL_STATIC_READ_ARB", INT2NUM(GL_STATIC_READ_ARB));
	rb_define_const(module, "GL_STENCIL", INT2NUM(GL_STENCIL));
	rb_define_const(module, "GL_STENCIL_ATTACHMENT", INT2NUM(GL_STENCIL_ATTACHMENT));
	rb_define_const(module, "GL_STENCIL_ATTACHMENT_EXT", INT2NUM(GL_STENCIL_ATTACHMENT_EXT));
	rb_define_const(module, "GL_STENCIL_BACK_FAIL", INT2NUM(GL_STENCIL_BACK_FAIL));
	rb_define_const(module, "GL_STENCIL_BACK_FAIL_ATI", INT2NUM(GL_STENCIL_BACK_FAIL_ATI));
	rb_define_const(module, "GL_STENCIL_BACK_FUNC", INT2NUM(GL_STENCIL_BACK_FUNC));
	rb_define_const(module, "GL_STENCIL_BACK_FUNC_ATI", INT2NUM(GL_STENCIL_BACK_FUNC_ATI));
	rb_define_const(module, "GL_STENCIL_BACK_PASS_DEPTH_FAIL", INT2NUM(GL_STENCIL_BACK_PASS_DEPTH_FAIL));
	rb_define_const(module, "GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI", INT2NUM(GL_STENCIL_BACK_PASS_DEPTH_FAIL_ATI));
	rb_define_const(module, "GL_STENCIL_BACK_PASS_DEPTH_PASS", INT2NUM(GL_STENCIL_BACK_PASS_DEPTH_PASS));
	rb_define_const(module, "GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI", INT2NUM(GL_STENCIL_BACK_PASS_DEPTH_PASS_ATI));
	rb_define_const(module, "GL_STENCIL_BACK_REF", INT2NUM(GL_STENCIL_BACK_REF));
	rb_define_const(module, "GL_STENCIL_BACK_VALUE_MASK", INT2NUM(GL_STENCIL_BACK_VALUE_MASK));
	rb_define_const(module, "GL_STENCIL_BACK_WRITEMASK", INT2NUM(GL_STENCIL_BACK_WRITEMASK));
	rb_define_const(module, "GL_STENCIL_BITS", INT2NUM(GL_STENCIL_BITS));
	rb_define_const(module, "GL_STENCIL_BUFFER", INT2NUM(GL_STENCIL_BUFFER));
	rb_define_const(module, "GL_STENCIL_BUFFER_BIT", INT2NUM(GL_STENCIL_BUFFER_BIT));
	rb_define_const(module, "GL_STENCIL_CLEAR_TAG_VALUE_EXT", INT2NUM(GL_STENCIL_CLEAR_TAG_VALUE_EXT));
	rb_define_const(module, "GL_STENCIL_CLEAR_VALUE", INT2NUM(GL_STENCIL_CLEAR_VALUE));
	rb_define_const(module, "GL_STENCIL_FAIL", INT2NUM(GL_STENCIL_FAIL));
	rb_define_const(module, "GL_STENCIL_FUNC", INT2NUM(GL_STENCIL_FUNC));
	rb_define_const(module, "GL_STENCIL_INDEX", INT2NUM(GL_STENCIL_INDEX));
	rb_define_const(module, "GL_STENCIL_INDEX1", INT2NUM(GL_STENCIL_INDEX1));
	rb_define_const(module, "GL_STENCIL_INDEX16", INT2NUM(GL_STENCIL_INDEX16));
	rb_define_const(module, "GL_STENCIL_INDEX16_EXT", INT2NUM(GL_STENCIL_INDEX16_EXT));
	rb_define_const(module, "GL_STENCIL_INDEX1_EXT", INT2NUM(GL_STENCIL_INDEX1_EXT));
	rb_define_const(module, "GL_STENCIL_INDEX4", INT2NUM(GL_STENCIL_INDEX4));
	rb_define_const(module, "GL_STENCIL_INDEX4_EXT", INT2NUM(GL_STENCIL_INDEX4_EXT));
	rb_define_const(module, "GL_STENCIL_INDEX8", INT2NUM(GL_STENCIL_INDEX8));
	rb_define_const(module, "GL_STENCIL_INDEX8_EXT", INT2NUM(GL_STENCIL_INDEX8_EXT));
	rb_define_const(module, "GL_STENCIL_PASS_DEPTH_FAIL", INT2NUM(GL_STENCIL_PASS_DEPTH_FAIL));
	rb_define_const(module, "GL_STENCIL_PASS_DEPTH_PASS", INT2NUM(GL_STENCIL_PASS_DEPTH_PASS));
	rb_define_const(module, "GL_STENCIL_REF", INT2NUM(GL_STENCIL_REF));
	rb_define_const(module, "GL_STENCIL_TAG_BITS_EXT", INT2NUM(GL_STENCIL_TAG_BITS_EXT));
	rb_define_const(module, "GL_STENCIL_TEST", INT2NUM(GL_STENCIL_TEST));
	rb_define_const(module, "GL_STENCIL_TEST_TWO_SIDE_EXT", INT2NUM(GL_STENCIL_TEST_TWO_SIDE_EXT));
	rb_define_const(module, "GL_STENCIL_VALUE_MASK", INT2NUM(GL_STENCIL_VALUE_MASK));
	rb_define_const(module, "GL_STENCIL_WRITEMASK", INT2NUM(GL_STENCIL_WRITEMASK));
	rb_define_const(module, "GL_STEREO", INT2NUM(GL_STEREO));
	rb_define_const(module, "GL_STORAGE_CACHED_APPLE", INT2NUM(GL_STORAGE_CACHED_APPLE));
	rb_define_const(module, "GL_STORAGE_SHARED_APPLE", INT2NUM(GL_STORAGE_SHARED_APPLE));
	rb_define_const(module, "GL_STREAM_COPY", INT2NUM(GL_STREAM_COPY));
	rb_define_const(module, "GL_STREAM_COPY_ARB", INT2NUM(GL_STREAM_COPY_ARB));
	rb_define_const(module, "GL_STREAM_DRAW", INT2NUM(GL_STREAM_DRAW));
	rb_define_const(module, "GL_STREAM_DRAW_ARB", INT2NUM(GL_STREAM_DRAW_ARB));
	rb_define_const(module, "GL_STREAM_READ", INT2NUM(GL_STREAM_READ));
	rb_define_const(module, "GL_STREAM_READ_ARB", INT2NUM(GL_STREAM_READ_ARB));
	rb_define_const(module, "GL_STRICT_DEPTHFUNC_HINT_PGI", INT2NUM(GL_STRICT_DEPTHFUNC_HINT_PGI));
	rb_define_const(module, "GL_STRICT_LIGHTING_HINT_PGI", INT2NUM(GL_STRICT_LIGHTING_HINT_PGI));
	rb_define_const(module, "GL_STRICT_SCISSOR_HINT_PGI", INT2NUM(GL_STRICT_SCISSOR_HINT_PGI));
	rb_define_const(module, "GL_SUBPIXEL_BITS", INT2NUM(GL_SUBPIXEL_BITS));
	rb_define_const(module, "GL_SUBTRACT", INT2NUM(GL_SUBTRACT));
	rb_define_const(module, "GL_SUBTRACT_ARB", INT2NUM(GL_SUBTRACT_ARB));
	rb_define_const(module, "GL_SUB_ATI", INT2NUM(GL_SUB_ATI));
	rb_define_const(module, "GL_SWIZZLE_STQ_ATI", INT2NUM(GL_SWIZZLE_STQ_ATI));
	rb_define_const(module, "GL_SWIZZLE_STQ_DQ_ATI", INT2NUM(GL_SWIZZLE_STQ_DQ_ATI));
	rb_define_const(module, "GL_SWIZZLE_STRQ_ATI", INT2NUM(GL_SWIZZLE_STRQ_ATI));
	rb_define_const(module, "GL_SWIZZLE_STRQ_DQ_ATI", INT2NUM(GL_SWIZZLE_STRQ_DQ_ATI));
	rb_define_const(module, "GL_SWIZZLE_STR_ATI", INT2NUM(GL_SWIZZLE_STR_ATI));
	rb_define_const(module, "GL_SWIZZLE_STR_DR_ATI", INT2NUM(GL_SWIZZLE_STR_DR_ATI));
	rb_define_const(module, "GL_T", INT2NUM(GL_T));
	rb_define_const(module, "GL_T2F_C3F_V3F", INT2NUM(GL_T2F_C3F_V3F));
	rb_define_const(module, "GL_T2F_C4F_N3F_V3F", INT2NUM(GL_T2F_C4F_N3F_V3F));
	rb_define_const(module, "GL_T2F_C4UB_V3F", INT2NUM(GL_T2F_C4UB_V3F));
	rb_define_const(module, "GL_T2F_IUI_N3F_V2F_EXT", INT2NUM(GL_T2F_IUI_N3F_V2F_EXT));
	rb_define_const(module, "GL_T2F_IUI_N3F_V3F_EXT", INT2NUM(GL_T2F_IUI_N3F_V3F_EXT));
	rb_define_const(module, "GL_T2F_IUI_V2F_EXT", INT2NUM(GL_T2F_IUI_V2F_EXT));
	rb_define_const(module, "GL_T2F_IUI_V3F_EXT", INT2NUM(GL_T2F_IUI_V3F_EXT));
	rb_define_const(module, "GL_T2F_N3F_V3F", INT2NUM(GL_T2F_N3F_V3F));
	rb_define_const(module, "GL_T2F_V3F", INT2NUM(GL_T2F_V3F));
	rb_define_const(module, "GL_T4F_C4F_N3F_V4F", INT2NUM(GL_T4F_C4F_N3F_V4F));
	rb_define_const(module, "GL_T4F_V4F", INT2NUM(GL_T4F_V4F));
	rb_define_const(module, "GL_TABLE_TOO_LARGE", INT2NUM(GL_TABLE_TOO_LARGE));
	rb_define_const(module, "GL_TABLE_TOO_LARGE_EXT", INT2NUM(GL_TABLE_TOO_LARGE_EXT));
	rb_define_const(module, "GL_TANGENT_ARRAY_EXT", INT2NUM(GL_TANGENT_ARRAY_EXT));
	rb_define_const(module, "GL_TANGENT_ARRAY_POINTER_EXT", INT2NUM(GL_TANGENT_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_TANGENT_ARRAY_STRIDE_EXT", INT2NUM(GL_TANGENT_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_TANGENT_ARRAY_TYPE_EXT", INT2NUM(GL_TANGENT_ARRAY_TYPE_EXT));
	rb_define_const(module, "GL_TEXCOORD1_BIT_PGI", INT2NUM(GL_TEXCOORD1_BIT_PGI));
	rb_define_const(module, "GL_TEXCOORD2_BIT_PGI", INT2NUM(GL_TEXCOORD2_BIT_PGI));
	rb_define_const(module, "GL_TEXCOORD3_BIT_PGI", INT2NUM(GL_TEXCOORD3_BIT_PGI));
	rb_define_const(module, "GL_TEXCOORD4_BIT_PGI", INT2NUM(GL_TEXCOORD4_BIT_PGI));
	rb_define_const(module, "GL_TEXTURE", INT2NUM(GL_TEXTURE));
	rb_define_const(module, "GL_TEXTURE0", INT2NUM(GL_TEXTURE0));
	rb_define_const(module, "GL_TEXTURE0_ARB", INT2NUM(GL_TEXTURE0_ARB));
	rb_define_const(module, "GL_TEXTURE1", INT2NUM(GL_TEXTURE1));
	rb_define_const(module, "GL_TEXTURE10", INT2NUM(GL_TEXTURE10));
	rb_define_const(module, "GL_TEXTURE10_ARB", INT2NUM(GL_TEXTURE10_ARB));
	rb_define_const(module, "GL_TEXTURE11", INT2NUM(GL_TEXTURE11));
	rb_define_const(module, "GL_TEXTURE11_ARB", INT2NUM(GL_TEXTURE11_ARB));
	rb_define_const(module, "GL_TEXTURE12", INT2NUM(GL_TEXTURE12));
	rb_define_const(module, "GL_TEXTURE12_ARB", INT2NUM(GL_TEXTURE12_ARB));
	rb_define_const(module, "GL_TEXTURE13", INT2NUM(GL_TEXTURE13));
	rb_define_const(module, "GL_TEXTURE13_ARB", INT2NUM(GL_TEXTURE13_ARB));
	rb_define_const(module, "GL_TEXTURE14", INT2NUM(GL_TEXTURE14));
	rb_define_const(module, "GL_TEXTURE14_ARB", INT2NUM(GL_TEXTURE14_ARB));
	rb_define_const(module, "GL_TEXTURE15", INT2NUM(GL_TEXTURE15));
	rb_define_const(module, "GL_TEXTURE15_ARB", INT2NUM(GL_TEXTURE15_ARB));
	rb_define_const(module, "GL_TEXTURE16", INT2NUM(GL_TEXTURE16));
	rb_define_const(module, "GL_TEXTURE16_ARB", INT2NUM(GL_TEXTURE16_ARB));
	rb_define_const(module, "GL_TEXTURE17", INT2NUM(GL_TEXTURE17));
	rb_define_const(module, "GL_TEXTURE17_ARB", INT2NUM(GL_TEXTURE17_ARB));
	rb_define_const(module, "GL_TEXTURE18", INT2NUM(GL_TEXTURE18));
	rb_define_const(module, "GL_TEXTURE18_ARB", INT2NUM(GL_TEXTURE18_ARB));
	rb_define_const(module, "GL_TEXTURE19", INT2NUM(GL_TEXTURE19));
	rb_define_const(module, "GL_TEXTURE19_ARB", INT2NUM(GL_TEXTURE19_ARB));
	rb_define_const(module, "GL_TEXTURE1_ARB", INT2NUM(GL_TEXTURE1_ARB));
	rb_define_const(module, "GL_TEXTURE2", INT2NUM(GL_TEXTURE2));
	rb_define_const(module, "GL_TEXTURE20", INT2NUM(GL_TEXTURE20));
	rb_define_const(module, "GL_TEXTURE20_ARB", INT2NUM(GL_TEXTURE20_ARB));
	rb_define_const(module, "GL_TEXTURE21", INT2NUM(GL_TEXTURE21));
	rb_define_const(module, "GL_TEXTURE21_ARB", INT2NUM(GL_TEXTURE21_ARB));
	rb_define_const(module, "GL_TEXTURE22", INT2NUM(GL_TEXTURE22));
	rb_define_const(module, "GL_TEXTURE22_ARB", INT2NUM(GL_TEXTURE22_ARB));
	rb_define_const(module, "GL_TEXTURE23", INT2NUM(GL_TEXTURE23));
	rb_define_const(module, "GL_TEXTURE23_ARB", INT2NUM(GL_TEXTURE23_ARB));
	rb_define_const(module, "GL_TEXTURE24", INT2NUM(GL_TEXTURE24));
	rb_define_const(module, "GL_TEXTURE24_ARB", INT2NUM(GL_TEXTURE24_ARB));
	rb_define_const(module, "GL_TEXTURE25", INT2NUM(GL_TEXTURE25));
	rb_define_const(module, "GL_TEXTURE25_ARB", INT2NUM(GL_TEXTURE25_ARB));
	rb_define_const(module, "GL_TEXTURE26", INT2NUM(GL_TEXTURE26));
	rb_define_const(module, "GL_TEXTURE26_ARB", INT2NUM(GL_TEXTURE26_ARB));
	rb_define_const(module, "GL_TEXTURE27", INT2NUM(GL_TEXTURE27));
	rb_define_const(module, "GL_TEXTURE27_ARB", INT2NUM(GL_TEXTURE27_ARB));
	rb_define_const(module, "GL_TEXTURE28", INT2NUM(GL_TEXTURE28));
	rb_define_const(module, "GL_TEXTURE28_ARB", INT2NUM(GL_TEXTURE28_ARB));
	rb_define_const(module, "GL_TEXTURE29", INT2NUM(GL_TEXTURE29));
	rb_define_const(module, "GL_TEXTURE29_ARB", INT2NUM(GL_TEXTURE29_ARB));
	rb_define_const(module, "GL_TEXTURE2_ARB", INT2NUM(GL_TEXTURE2_ARB));
	rb_define_const(module, "GL_TEXTURE3", INT2NUM(GL_TEXTURE3));
	rb_define_const(module, "GL_TEXTURE30", INT2NUM(GL_TEXTURE30));
	rb_define_const(module, "GL_TEXTURE30_ARB", INT2NUM(GL_TEXTURE30_ARB));
	rb_define_const(module, "GL_TEXTURE31", INT2NUM(GL_TEXTURE31));
	rb_define_const(module, "GL_TEXTURE31_ARB", INT2NUM(GL_TEXTURE31_ARB));
	rb_define_const(module, "GL_TEXTURE3_ARB", INT2NUM(GL_TEXTURE3_ARB));
	rb_define_const(module, "GL_TEXTURE4", INT2NUM(GL_TEXTURE4));
	rb_define_const(module, "GL_TEXTURE4_ARB", INT2NUM(GL_TEXTURE4_ARB));
	rb_define_const(module, "GL_TEXTURE5", INT2NUM(GL_TEXTURE5));
	rb_define_const(module, "GL_TEXTURE5_ARB", INT2NUM(GL_TEXTURE5_ARB));
	rb_define_const(module, "GL_TEXTURE6", INT2NUM(GL_TEXTURE6));
	rb_define_const(module, "GL_TEXTURE6_ARB", INT2NUM(GL_TEXTURE6_ARB));
	rb_define_const(module, "GL_TEXTURE7", INT2NUM(GL_TEXTURE7));
	rb_define_const(module, "GL_TEXTURE7_ARB", INT2NUM(GL_TEXTURE7_ARB));
	rb_define_const(module, "GL_TEXTURE8", INT2NUM(GL_TEXTURE8));
	rb_define_const(module, "GL_TEXTURE8_ARB", INT2NUM(GL_TEXTURE8_ARB));
	rb_define_const(module, "GL_TEXTURE9", INT2NUM(GL_TEXTURE9));
	rb_define_const(module, "GL_TEXTURE9_ARB", INT2NUM(GL_TEXTURE9_ARB));
	rb_define_const(module, "GL_TEXTURE_1D", INT2NUM(GL_TEXTURE_1D));
	rb_define_const(module, "GL_TEXTURE_1D_ARRAY", INT2NUM(GL_TEXTURE_1D_ARRAY));
	rb_define_const(module, "GL_TEXTURE_1D_ARRAY_EXT", INT2NUM(GL_TEXTURE_1D_ARRAY_EXT));
	rb_define_const(module, "GL_TEXTURE_1D_BINDING_EXT", INT2NUM(GL_TEXTURE_1D_BINDING_EXT));
	rb_define_const(module, "GL_TEXTURE_1D_STACK_BINDING_MESAX", INT2NUM(GL_TEXTURE_1D_STACK_BINDING_MESAX));
	rb_define_const(module, "GL_TEXTURE_1D_STACK_MESAX", INT2NUM(GL_TEXTURE_1D_STACK_MESAX));
	rb_define_const(module, "GL_TEXTURE_2D", INT2NUM(GL_TEXTURE_2D));
	rb_define_const(module, "GL_TEXTURE_2D_ARRAY", INT2NUM(GL_TEXTURE_2D_ARRAY));
	rb_define_const(module, "GL_TEXTURE_2D_ARRAY_EXT", INT2NUM(GL_TEXTURE_2D_ARRAY_EXT));
	rb_define_const(module, "GL_TEXTURE_2D_BINDING_EXT", INT2NUM(GL_TEXTURE_2D_BINDING_EXT));
	rb_define_const(module, "GL_TEXTURE_2D_STACK_BINDING_MESAX", INT2NUM(GL_TEXTURE_2D_STACK_BINDING_MESAX));
	rb_define_const(module, "GL_TEXTURE_2D_STACK_MESAX", INT2NUM(GL_TEXTURE_2D_STACK_MESAX));
	rb_define_const(module, "GL_TEXTURE_3D", INT2NUM(GL_TEXTURE_3D));
	rb_define_const(module, "GL_TEXTURE_3D_BINDING_EXT", INT2NUM(GL_TEXTURE_3D_BINDING_EXT));
	rb_define_const(module, "GL_TEXTURE_3D_EXT", INT2NUM(GL_TEXTURE_3D_EXT));
	rb_define_const(module, "GL_TEXTURE_4DSIZE_SGIS", INT2NUM(GL_TEXTURE_4DSIZE_SGIS));
	rb_define_const(module, "GL_TEXTURE_4D_BINDING_SGIS", INT2NUM(GL_TEXTURE_4D_BINDING_SGIS));
	rb_define_const(module, "GL_TEXTURE_4D_SGIS", INT2NUM(GL_TEXTURE_4D_SGIS));
	rb_define_const(module, "GL_TEXTURE_ALPHA_SIZE", INT2NUM(GL_TEXTURE_ALPHA_SIZE));
	rb_define_const(module, "GL_TEXTURE_ALPHA_SIZE_EXT", INT2NUM(GL_TEXTURE_ALPHA_SIZE_EXT));
	rb_define_const(module, "GL_TEXTURE_ALPHA_TYPE", INT2NUM(GL_TEXTURE_ALPHA_TYPE));
	rb_define_const(module, "GL_TEXTURE_ALPHA_TYPE_ARB", INT2NUM(GL_TEXTURE_ALPHA_TYPE_ARB));
	rb_define_const(module, "GL_TEXTURE_APPLICATION_MODE_EXT", INT2NUM(GL_TEXTURE_APPLICATION_MODE_EXT));
	rb_define_const(module, "GL_TEXTURE_BASE_LEVEL", INT2NUM(GL_TEXTURE_BASE_LEVEL));
	rb_define_const(module, "GL_TEXTURE_BASE_LEVEL_SGIS", INT2NUM(GL_TEXTURE_BASE_LEVEL_SGIS));
	rb_define_const(module, "GL_TEXTURE_BINDING_1D", INT2NUM(GL_TEXTURE_BINDING_1D));
	rb_define_const(module, "GL_TEXTURE_BINDING_1D_ARRAY", INT2NUM(GL_TEXTURE_BINDING_1D_ARRAY));
	rb_define_const(module, "GL_TEXTURE_BINDING_1D_ARRAY_EXT", INT2NUM(GL_TEXTURE_BINDING_1D_ARRAY_EXT));
	rb_define_const(module, "GL_TEXTURE_BINDING_2D", INT2NUM(GL_TEXTURE_BINDING_2D));
	rb_define_const(module, "GL_TEXTURE_BINDING_2D_ARRAY", INT2NUM(GL_TEXTURE_BINDING_2D_ARRAY));
	rb_define_const(module, "GL_TEXTURE_BINDING_2D_ARRAY_EXT", INT2NUM(GL_TEXTURE_BINDING_2D_ARRAY_EXT));
	rb_define_const(module, "GL_TEXTURE_BINDING_3D", INT2NUM(GL_TEXTURE_BINDING_3D));
	rb_define_const(module, "GL_TEXTURE_BINDING_BUFFER_ARB", INT2NUM(GL_TEXTURE_BINDING_BUFFER_ARB));
	rb_define_const(module, "GL_TEXTURE_BINDING_BUFFER_EXT", INT2NUM(GL_TEXTURE_BINDING_BUFFER_EXT));
	rb_define_const(module, "GL_TEXTURE_BINDING_CUBE_MAP", INT2NUM(GL_TEXTURE_BINDING_CUBE_MAP));
	rb_define_const(module, "GL_TEXTURE_BINDING_CUBE_MAP_ARB", INT2NUM(GL_TEXTURE_BINDING_CUBE_MAP_ARB));
	rb_define_const(module, "GL_TEXTURE_BINDING_CUBE_MAP_EXT", INT2NUM(GL_TEXTURE_BINDING_CUBE_MAP_EXT));
	rb_define_const(module, "GL_TEXTURE_BINDING_RECTANGLE_ARB", INT2NUM(GL_TEXTURE_BINDING_RECTANGLE_ARB));
	rb_define_const(module, "GL_TEXTURE_BINDING_RECTANGLE_NV", INT2NUM(GL_TEXTURE_BINDING_RECTANGLE_NV));
	rb_define_const(module, "GL_TEXTURE_BINDING_RENDERBUFFER_NV", INT2NUM(GL_TEXTURE_BINDING_RENDERBUFFER_NV));
	rb_define_const(module, "GL_TEXTURE_BIT", INT2NUM(GL_TEXTURE_BIT));
	rb_define_const(module, "GL_TEXTURE_BLUE_SIZE", INT2NUM(GL_TEXTURE_BLUE_SIZE));
	rb_define_const(module, "GL_TEXTURE_BLUE_SIZE_EXT", INT2NUM(GL_TEXTURE_BLUE_SIZE_EXT));
	rb_define_const(module, "GL_TEXTURE_BLUE_TYPE", INT2NUM(GL_TEXTURE_BLUE_TYPE));
	rb_define_const(module, "GL_TEXTURE_BLUE_TYPE_ARB", INT2NUM(GL_TEXTURE_BLUE_TYPE_ARB));
	rb_define_const(module, "GL_TEXTURE_BORDER", INT2NUM(GL_TEXTURE_BORDER));
	rb_define_const(module, "GL_TEXTURE_BORDER_COLOR", INT2NUM(GL_TEXTURE_BORDER_COLOR));
	rb_define_const(module, "GL_TEXTURE_BORDER_VALUES_NV", INT2NUM(GL_TEXTURE_BORDER_VALUES_NV));
	rb_define_const(module, "GL_TEXTURE_BUFFER_ARB", INT2NUM(GL_TEXTURE_BUFFER_ARB));
	rb_define_const(module, "GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB", INT2NUM(GL_TEXTURE_BUFFER_DATA_STORE_BINDING_ARB));
	rb_define_const(module, "GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT", INT2NUM(GL_TEXTURE_BUFFER_DATA_STORE_BINDING_EXT));
	rb_define_const(module, "GL_TEXTURE_BUFFER_EXT", INT2NUM(GL_TEXTURE_BUFFER_EXT));
	rb_define_const(module, "GL_TEXTURE_BUFFER_FORMAT_ARB", INT2NUM(GL_TEXTURE_BUFFER_FORMAT_ARB));
	rb_define_const(module, "GL_TEXTURE_BUFFER_FORMAT_EXT", INT2NUM(GL_TEXTURE_BUFFER_FORMAT_EXT));
	rb_define_const(module, "GL_TEXTURE_CLIPMAP_CENTER_SGIX", INT2NUM(GL_TEXTURE_CLIPMAP_CENTER_SGIX));
	rb_define_const(module, "GL_TEXTURE_CLIPMAP_DEPTH_SGIX", INT2NUM(GL_TEXTURE_CLIPMAP_DEPTH_SGIX));
	rb_define_const(module, "GL_TEXTURE_CLIPMAP_FRAME_SGIX", INT2NUM(GL_TEXTURE_CLIPMAP_FRAME_SGIX));
	rb_define_const(module, "GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX", INT2NUM(GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX));
	rb_define_const(module, "GL_TEXTURE_CLIPMAP_OFFSET_SGIX", INT2NUM(GL_TEXTURE_CLIPMAP_OFFSET_SGIX));
	rb_define_const(module, "GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX", INT2NUM(GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX));
	rb_define_const(module, "GL_TEXTURE_COLOR_TABLE_SGI", INT2NUM(GL_TEXTURE_COLOR_TABLE_SGI));
	rb_define_const(module, "GL_TEXTURE_COLOR_WRITEMASK_SGIS", INT2NUM(GL_TEXTURE_COLOR_WRITEMASK_SGIS));
	rb_define_const(module, "GL_TEXTURE_COMPARE_FAIL_VALUE_ARB", INT2NUM(GL_TEXTURE_COMPARE_FAIL_VALUE_ARB));
	rb_define_const(module, "GL_TEXTURE_COMPARE_FUNC", INT2NUM(GL_TEXTURE_COMPARE_FUNC));
	rb_define_const(module, "GL_TEXTURE_COMPARE_FUNC_ARB", INT2NUM(GL_TEXTURE_COMPARE_FUNC_ARB));
	rb_define_const(module, "GL_TEXTURE_COMPARE_MODE", INT2NUM(GL_TEXTURE_COMPARE_MODE));
	rb_define_const(module, "GL_TEXTURE_COMPARE_MODE_ARB", INT2NUM(GL_TEXTURE_COMPARE_MODE_ARB));
	rb_define_const(module, "GL_TEXTURE_COMPARE_OPERATOR_SGIX", INT2NUM(GL_TEXTURE_COMPARE_OPERATOR_SGIX));
	rb_define_const(module, "GL_TEXTURE_COMPARE_SGIX", INT2NUM(GL_TEXTURE_COMPARE_SGIX));
	rb_define_const(module, "GL_TEXTURE_COMPONENTS", INT2NUM(GL_TEXTURE_COMPONENTS));
	rb_define_const(module, "GL_TEXTURE_COMPRESSED", INT2NUM(GL_TEXTURE_COMPRESSED));
	rb_define_const(module, "GL_TEXTURE_COMPRESSED_ARB", INT2NUM(GL_TEXTURE_COMPRESSED_ARB));
	rb_define_const(module, "GL_TEXTURE_COMPRESSED_IMAGE_SIZE", INT2NUM(GL_TEXTURE_COMPRESSED_IMAGE_SIZE));
	rb_define_const(module, "GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB", INT2NUM(GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB));
	rb_define_const(module, "GL_TEXTURE_COMPRESSION_HINT", INT2NUM(GL_TEXTURE_COMPRESSION_HINT));
	rb_define_const(module, "GL_TEXTURE_COMPRESSION_HINT_ARB", INT2NUM(GL_TEXTURE_COMPRESSION_HINT_ARB));
	rb_define_const(module, "GL_TEXTURE_CONSTANT_DATA_SUNX", INT2NUM(GL_TEXTURE_CONSTANT_DATA_SUNX));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY", INT2NUM(GL_TEXTURE_COORD_ARRAY));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING", INT2NUM(GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_COUNT_EXT", INT2NUM(GL_TEXTURE_COORD_ARRAY_COUNT_EXT));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_EXT", INT2NUM(GL_TEXTURE_COORD_ARRAY_EXT));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_LIST_IBM", INT2NUM(GL_TEXTURE_COORD_ARRAY_LIST_IBM));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM", INT2NUM(GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL", INT2NUM(GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_POINTER", INT2NUM(GL_TEXTURE_COORD_ARRAY_POINTER));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_POINTER_EXT", INT2NUM(GL_TEXTURE_COORD_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_SIZE", INT2NUM(GL_TEXTURE_COORD_ARRAY_SIZE));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_SIZE_EXT", INT2NUM(GL_TEXTURE_COORD_ARRAY_SIZE_EXT));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_STRIDE", INT2NUM(GL_TEXTURE_COORD_ARRAY_STRIDE));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_STRIDE_EXT", INT2NUM(GL_TEXTURE_COORD_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_TYPE", INT2NUM(GL_TEXTURE_COORD_ARRAY_TYPE));
	rb_define_const(module, "GL_TEXTURE_COORD_ARRAY_TYPE_EXT", INT2NUM(GL_TEXTURE_COORD_ARRAY_TYPE_EXT));
	rb_define_const(module, "GL_TEXTURE_COORD_NV", INT2NUM(GL_TEXTURE_COORD_NV));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP", INT2NUM(GL_TEXTURE_CUBE_MAP));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_ARB", INT2NUM(GL_TEXTURE_CUBE_MAP_ARB));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_EXT", INT2NUM(GL_TEXTURE_CUBE_MAP_EXT));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_NEGATIVE_X", INT2NUM(GL_TEXTURE_CUBE_MAP_NEGATIVE_X));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB", INT2NUM(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT", INT2NUM(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Y", INT2NUM(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB", INT2NUM(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT", INT2NUM(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Z", INT2NUM(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB", INT2NUM(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT", INT2NUM(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_POSITIVE_X", INT2NUM(GL_TEXTURE_CUBE_MAP_POSITIVE_X));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB", INT2NUM(GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT", INT2NUM(GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_POSITIVE_Y", INT2NUM(GL_TEXTURE_CUBE_MAP_POSITIVE_Y));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB", INT2NUM(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT", INT2NUM(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_POSITIVE_Z", INT2NUM(GL_TEXTURE_CUBE_MAP_POSITIVE_Z));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB", INT2NUM(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB));
	rb_define_const(module, "GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT", INT2NUM(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT));
	rb_define_const(module, "GL_TEXTURE_DEFORMATION_BIT_SGIX", INT2NUM(GL_TEXTURE_DEFORMATION_BIT_SGIX));
	rb_define_const(module, "GL_TEXTURE_DEFORMATION_SGIX", INT2NUM(GL_TEXTURE_DEFORMATION_SGIX));
	rb_define_const(module, "GL_TEXTURE_DEPTH", INT2NUM(GL_TEXTURE_DEPTH));
	rb_define_const(module, "GL_TEXTURE_DEPTH_EXT", INT2NUM(GL_TEXTURE_DEPTH_EXT));
	rb_define_const(module, "GL_TEXTURE_DEPTH_SIZE", INT2NUM(GL_TEXTURE_DEPTH_SIZE));
	rb_define_const(module, "GL_TEXTURE_DEPTH_SIZE_ARB", INT2NUM(GL_TEXTURE_DEPTH_SIZE_ARB));
	rb_define_const(module, "GL_TEXTURE_DEPTH_TYPE", INT2NUM(GL_TEXTURE_DEPTH_TYPE));
	rb_define_const(module, "GL_TEXTURE_DEPTH_TYPE_ARB", INT2NUM(GL_TEXTURE_DEPTH_TYPE_ARB));
	rb_define_const(module, "GL_TEXTURE_DS_SIZE_NV", INT2NUM(GL_TEXTURE_DS_SIZE_NV));
	rb_define_const(module, "GL_TEXTURE_DT_SIZE_NV", INT2NUM(GL_TEXTURE_DT_SIZE_NV));
	rb_define_const(module, "GL_TEXTURE_ENV", INT2NUM(GL_TEXTURE_ENV));
	rb_define_const(module, "GL_TEXTURE_ENV_BIAS_SGIX", INT2NUM(GL_TEXTURE_ENV_BIAS_SGIX));
	rb_define_const(module, "GL_TEXTURE_ENV_COLOR", INT2NUM(GL_TEXTURE_ENV_COLOR));
	rb_define_const(module, "GL_TEXTURE_ENV_MODE", INT2NUM(GL_TEXTURE_ENV_MODE));
	rb_define_const(module, "GL_TEXTURE_FILTER4_SIZE_SGIS", INT2NUM(GL_TEXTURE_FILTER4_SIZE_SGIS));
	rb_define_const(module, "GL_TEXTURE_FILTER_CONTROL", INT2NUM(GL_TEXTURE_FILTER_CONTROL));
	rb_define_const(module, "GL_TEXTURE_FILTER_CONTROL_EXT", INT2NUM(GL_TEXTURE_FILTER_CONTROL_EXT));
	rb_define_const(module, "GL_TEXTURE_FLOAT_COMPONENTS_NV", INT2NUM(GL_TEXTURE_FLOAT_COMPONENTS_NV));
	rb_define_const(module, "GL_TEXTURE_GEN_MODE", INT2NUM(GL_TEXTURE_GEN_MODE));
	rb_define_const(module, "GL_TEXTURE_GEN_Q", INT2NUM(GL_TEXTURE_GEN_Q));
	rb_define_const(module, "GL_TEXTURE_GEN_R", INT2NUM(GL_TEXTURE_GEN_R));
	rb_define_const(module, "GL_TEXTURE_GEN_S", INT2NUM(GL_TEXTURE_GEN_S));
	rb_define_const(module, "GL_TEXTURE_GEN_T", INT2NUM(GL_TEXTURE_GEN_T));
	rb_define_const(module, "GL_TEXTURE_GEQUAL_R_SGIX", INT2NUM(GL_TEXTURE_GEQUAL_R_SGIX));
	rb_define_const(module, "GL_TEXTURE_GREEN_SIZE", INT2NUM(GL_TEXTURE_GREEN_SIZE));
	rb_define_const(module, "GL_TEXTURE_GREEN_SIZE_EXT", INT2NUM(GL_TEXTURE_GREEN_SIZE_EXT));
	rb_define_const(module, "GL_TEXTURE_GREEN_TYPE", INT2NUM(GL_TEXTURE_GREEN_TYPE));
	rb_define_const(module, "GL_TEXTURE_GREEN_TYPE_ARB", INT2NUM(GL_TEXTURE_GREEN_TYPE_ARB));
	rb_define_const(module, "GL_TEXTURE_HEIGHT", INT2NUM(GL_TEXTURE_HEIGHT));
	rb_define_const(module, "GL_TEXTURE_HI_SIZE_NV", INT2NUM(GL_TEXTURE_HI_SIZE_NV));
	rb_define_const(module, "GL_TEXTURE_INDEX_SIZE_EXT", INT2NUM(GL_TEXTURE_INDEX_SIZE_EXT));
	rb_define_const(module, "GL_TEXTURE_INTENSITY_SIZE", INT2NUM(GL_TEXTURE_INTENSITY_SIZE));
	rb_define_const(module, "GL_TEXTURE_INTENSITY_SIZE_EXT", INT2NUM(GL_TEXTURE_INTENSITY_SIZE_EXT));
	rb_define_const(module, "GL_TEXTURE_INTENSITY_TYPE", INT2NUM(GL_TEXTURE_INTENSITY_TYPE));
	rb_define_const(module, "GL_TEXTURE_INTENSITY_TYPE_ARB", INT2NUM(GL_TEXTURE_INTENSITY_TYPE_ARB));
	rb_define_const(module, "GL_TEXTURE_INTERNAL_FORMAT", INT2NUM(GL_TEXTURE_INTERNAL_FORMAT));
	rb_define_const(module, "GL_TEXTURE_LEQUAL_R_SGIX", INT2NUM(GL_TEXTURE_LEQUAL_R_SGIX));
	rb_define_const(module, "GL_TEXTURE_LIGHTING_MODE_HP", INT2NUM(GL_TEXTURE_LIGHTING_MODE_HP));
	rb_define_const(module, "GL_TEXTURE_LIGHT_EXT", INT2NUM(GL_TEXTURE_LIGHT_EXT));
	rb_define_const(module, "GL_TEXTURE_LOD_BIAS", INT2NUM(GL_TEXTURE_LOD_BIAS));
	rb_define_const(module, "GL_TEXTURE_LOD_BIAS_EXT", INT2NUM(GL_TEXTURE_LOD_BIAS_EXT));
	rb_define_const(module, "GL_TEXTURE_LOD_BIAS_R_SGIX", INT2NUM(GL_TEXTURE_LOD_BIAS_R_SGIX));
	rb_define_const(module, "GL_TEXTURE_LOD_BIAS_S_SGIX", INT2NUM(GL_TEXTURE_LOD_BIAS_S_SGIX));
	rb_define_const(module, "GL_TEXTURE_LOD_BIAS_T_SGIX", INT2NUM(GL_TEXTURE_LOD_BIAS_T_SGIX));
	rb_define_const(module, "GL_TEXTURE_LO_SIZE_NV", INT2NUM(GL_TEXTURE_LO_SIZE_NV));
	rb_define_const(module, "GL_TEXTURE_LUMINANCE_SIZE", INT2NUM(GL_TEXTURE_LUMINANCE_SIZE));
	rb_define_const(module, "GL_TEXTURE_LUMINANCE_SIZE_EXT", INT2NUM(GL_TEXTURE_LUMINANCE_SIZE_EXT));
	rb_define_const(module, "GL_TEXTURE_LUMINANCE_TYPE", INT2NUM(GL_TEXTURE_LUMINANCE_TYPE));
	rb_define_const(module, "GL_TEXTURE_LUMINANCE_TYPE_ARB", INT2NUM(GL_TEXTURE_LUMINANCE_TYPE_ARB));
	rb_define_const(module, "GL_TEXTURE_MAG_FILTER", INT2NUM(GL_TEXTURE_MAG_FILTER));
	rb_define_const(module, "GL_TEXTURE_MAG_SIZE_NV", INT2NUM(GL_TEXTURE_MAG_SIZE_NV));
	rb_define_const(module, "GL_TEXTURE_MATERIAL_FACE_EXT", INT2NUM(GL_TEXTURE_MATERIAL_FACE_EXT));
	rb_define_const(module, "GL_TEXTURE_MATERIAL_PARAMETER_EXT", INT2NUM(GL_TEXTURE_MATERIAL_PARAMETER_EXT));
	rb_define_const(module, "GL_TEXTURE_MATRIX", INT2NUM(GL_TEXTURE_MATRIX));
	rb_define_const(module, "GL_TEXTURE_MAX_ANISOTROPY_EXT", INT2NUM(GL_TEXTURE_MAX_ANISOTROPY_EXT));
	rb_define_const(module, "GL_TEXTURE_MAX_CLAMP_R_SGIX", INT2NUM(GL_TEXTURE_MAX_CLAMP_R_SGIX));
	rb_define_const(module, "GL_TEXTURE_MAX_CLAMP_S_SGIX", INT2NUM(GL_TEXTURE_MAX_CLAMP_S_SGIX));
	rb_define_const(module, "GL_TEXTURE_MAX_CLAMP_T_SGIX", INT2NUM(GL_TEXTURE_MAX_CLAMP_T_SGIX));
	rb_define_const(module, "GL_TEXTURE_MAX_LEVEL", INT2NUM(GL_TEXTURE_MAX_LEVEL));
	rb_define_const(module, "GL_TEXTURE_MAX_LEVEL_SGIS", INT2NUM(GL_TEXTURE_MAX_LEVEL_SGIS));
	rb_define_const(module, "GL_TEXTURE_MAX_LOD", INT2NUM(GL_TEXTURE_MAX_LOD));
	rb_define_const(module, "GL_TEXTURE_MAX_LOD_SGIS", INT2NUM(GL_TEXTURE_MAX_LOD_SGIS));
	rb_define_const(module, "GL_TEXTURE_MIN_FILTER", INT2NUM(GL_TEXTURE_MIN_FILTER));
	rb_define_const(module, "GL_TEXTURE_MIN_LOD", INT2NUM(GL_TEXTURE_MIN_LOD));
	rb_define_const(module, "GL_TEXTURE_MIN_LOD_SGIS", INT2NUM(GL_TEXTURE_MIN_LOD_SGIS));
	rb_define_const(module, "GL_TEXTURE_MULTI_BUFFER_HINT_SGIX", INT2NUM(GL_TEXTURE_MULTI_BUFFER_HINT_SGIX));
	rb_define_const(module, "GL_TEXTURE_NORMAL_EXT", INT2NUM(GL_TEXTURE_NORMAL_EXT));
	rb_define_const(module, "GL_TEXTURE_POST_SPECULAR_HP", INT2NUM(GL_TEXTURE_POST_SPECULAR_HP));
	rb_define_const(module, "GL_TEXTURE_PRE_SPECULAR_HP", INT2NUM(GL_TEXTURE_PRE_SPECULAR_HP));
	rb_define_const(module, "GL_TEXTURE_PRIORITY", INT2NUM(GL_TEXTURE_PRIORITY));
	rb_define_const(module, "GL_TEXTURE_PRIORITY_EXT", INT2NUM(GL_TEXTURE_PRIORITY_EXT));
	rb_define_const(module, "GL_TEXTURE_RECTANGLE_ARB", INT2NUM(GL_TEXTURE_RECTANGLE_ARB));
	rb_define_const(module, "GL_TEXTURE_RECTANGLE_NV", INT2NUM(GL_TEXTURE_RECTANGLE_NV));
	rb_define_const(module, "GL_TEXTURE_RED_SIZE", INT2NUM(GL_TEXTURE_RED_SIZE));
	rb_define_const(module, "GL_TEXTURE_RED_SIZE_EXT", INT2NUM(GL_TEXTURE_RED_SIZE_EXT));
	rb_define_const(module, "GL_TEXTURE_RED_TYPE", INT2NUM(GL_TEXTURE_RED_TYPE));
	rb_define_const(module, "GL_TEXTURE_RED_TYPE_ARB", INT2NUM(GL_TEXTURE_RED_TYPE_ARB));
	rb_define_const(module, "GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV", INT2NUM(GL_TEXTURE_RENDERBUFFER_DATA_STORE_BINDING_NV));
	rb_define_const(module, "GL_TEXTURE_RENDERBUFFER_NV", INT2NUM(GL_TEXTURE_RENDERBUFFER_NV));
	rb_define_const(module, "GL_TEXTURE_RESIDENT", INT2NUM(GL_TEXTURE_RESIDENT));
	rb_define_const(module, "GL_TEXTURE_RESIDENT_EXT", INT2NUM(GL_TEXTURE_RESIDENT_EXT));
	rb_define_const(module, "GL_TEXTURE_SHADER_NV", INT2NUM(GL_TEXTURE_SHADER_NV));
	rb_define_const(module, "GL_TEXTURE_SHARED_SIZE", INT2NUM(GL_TEXTURE_SHARED_SIZE));
	rb_define_const(module, "GL_TEXTURE_SHARED_SIZE_EXT", INT2NUM(GL_TEXTURE_SHARED_SIZE_EXT));
	rb_define_const(module, "GL_TEXTURE_STACK_DEPTH", INT2NUM(GL_TEXTURE_STACK_DEPTH));
	rb_define_const(module, "GL_TEXTURE_STENCIL_SIZE", INT2NUM(GL_TEXTURE_STENCIL_SIZE));
	rb_define_const(module, "GL_TEXTURE_STENCIL_SIZE_EXT", INT2NUM(GL_TEXTURE_STENCIL_SIZE_EXT));
	rb_define_const(module, "GL_TEXTURE_SWIZZLE_A_EXT", INT2NUM(GL_TEXTURE_SWIZZLE_A_EXT));
	rb_define_const(module, "GL_TEXTURE_SWIZZLE_B_EXT", INT2NUM(GL_TEXTURE_SWIZZLE_B_EXT));
	rb_define_const(module, "GL_TEXTURE_SWIZZLE_G_EXT", INT2NUM(GL_TEXTURE_SWIZZLE_G_EXT));
	rb_define_const(module, "GL_TEXTURE_SWIZZLE_RGBA_EXT", INT2NUM(GL_TEXTURE_SWIZZLE_RGBA_EXT));
	rb_define_const(module, "GL_TEXTURE_SWIZZLE_R_EXT", INT2NUM(GL_TEXTURE_SWIZZLE_R_EXT));
	rb_define_const(module, "GL_TEXTURE_TOO_LARGE_EXT", INT2NUM(GL_TEXTURE_TOO_LARGE_EXT));
	rb_define_const(module, "GL_TEXTURE_UNSIGNED_REMAP_MODE_NV", INT2NUM(GL_TEXTURE_UNSIGNED_REMAP_MODE_NV));
	rb_define_const(module, "GL_TEXTURE_WIDTH", INT2NUM(GL_TEXTURE_WIDTH));
	rb_define_const(module, "GL_TEXTURE_WRAP_Q_SGIS", INT2NUM(GL_TEXTURE_WRAP_Q_SGIS));
	rb_define_const(module, "GL_TEXTURE_WRAP_R", INT2NUM(GL_TEXTURE_WRAP_R));
	rb_define_const(module, "GL_TEXTURE_WRAP_R_EXT", INT2NUM(GL_TEXTURE_WRAP_R_EXT));
	rb_define_const(module, "GL_TEXTURE_WRAP_S", INT2NUM(GL_TEXTURE_WRAP_S));
	rb_define_const(module, "GL_TEXTURE_WRAP_T", INT2NUM(GL_TEXTURE_WRAP_T));
	rb_define_const(module, "GL_TEXT_FRAGMENT_SHADER_ATI", INT2NUM(GL_TEXT_FRAGMENT_SHADER_ATI));
	rb_define_const(module, "GL_TIME_ELAPSED_EXT", INT2NUM(GL_TIME_ELAPSED_EXT));
	rb_define_const(module, "GL_TRACK_MATRIX_NV", INT2NUM(GL_TRACK_MATRIX_NV));
	rb_define_const(module, "GL_TRACK_MATRIX_TRANSFORM_NV", INT2NUM(GL_TRACK_MATRIX_TRANSFORM_NV));
	rb_define_const(module, "GL_TRANSFORM_BIT", INT2NUM(GL_TRANSFORM_BIT));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_ATTRIBS_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_ATTRIBS_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BINDING_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_BINDING_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_BINDING", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_BINDING));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_EXT", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_EXT));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_MODE", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_MODE));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_MODE_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_SIZE", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_SIZE));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_START", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_START));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_BUFFER_START_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_BUFFER_START_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN", INT2NUM(GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT", INT2NUM(GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_RECORD_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_RECORD_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_VARYINGS", INT2NUM(GL_TRANSFORM_FEEDBACK_VARYINGS));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_VARYINGS_EXT", INT2NUM(GL_TRANSFORM_FEEDBACK_VARYINGS_EXT));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_VARYINGS_NV", INT2NUM(GL_TRANSFORM_FEEDBACK_VARYINGS_NV));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH", INT2NUM(GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH));
	rb_define_const(module, "GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT", INT2NUM(GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT));
	rb_define_const(module, "GL_TRANSFORM_HINT_APPLE", INT2NUM(GL_TRANSFORM_HINT_APPLE));
	rb_define_const(module, "GL_TRANSPOSE_COLOR_MATRIX", INT2NUM(GL_TRANSPOSE_COLOR_MATRIX));
	rb_define_const(module, "GL_TRANSPOSE_COLOR_MATRIX_ARB", INT2NUM(GL_TRANSPOSE_COLOR_MATRIX_ARB));
	rb_define_const(module, "GL_TRANSPOSE_CURRENT_MATRIX_ARB", INT2NUM(GL_TRANSPOSE_CURRENT_MATRIX_ARB));
	rb_define_const(module, "GL_TRANSPOSE_MODELVIEW_MATRIX", INT2NUM(GL_TRANSPOSE_MODELVIEW_MATRIX));
	rb_define_const(module, "GL_TRANSPOSE_MODELVIEW_MATRIX_ARB", INT2NUM(GL_TRANSPOSE_MODELVIEW_MATRIX_ARB));
	rb_define_const(module, "GL_TRANSPOSE_NV", INT2NUM(GL_TRANSPOSE_NV));
	rb_define_const(module, "GL_TRANSPOSE_PROGRAM_MATRIX_EXT", INT2NUM(GL_TRANSPOSE_PROGRAM_MATRIX_EXT));
	rb_define_const(module, "GL_TRANSPOSE_PROJECTION_MATRIX", INT2NUM(GL_TRANSPOSE_PROJECTION_MATRIX));
	rb_define_const(module, "GL_TRANSPOSE_PROJECTION_MATRIX_ARB", INT2NUM(GL_TRANSPOSE_PROJECTION_MATRIX_ARB));
	rb_define_const(module, "GL_TRANSPOSE_TEXTURE_MATRIX", INT2NUM(GL_TRANSPOSE_TEXTURE_MATRIX));
	rb_define_const(module, "GL_TRANSPOSE_TEXTURE_MATRIX_ARB", INT2NUM(GL_TRANSPOSE_TEXTURE_MATRIX_ARB));
	rb_define_const(module, "GL_TRIANGLES", INT2NUM(GL_TRIANGLES));
	rb_define_const(module, "GL_TRIANGLES_ADJACENCY_ARB", INT2NUM(GL_TRIANGLES_ADJACENCY_ARB));
	rb_define_const(module, "GL_TRIANGLES_ADJACENCY_EXT", INT2NUM(GL_TRIANGLES_ADJACENCY_EXT));
	rb_define_const(module, "GL_TRIANGLE_FAN", INT2NUM(GL_TRIANGLE_FAN));
	rb_define_const(module, "GL_TRIANGLE_LIST_SUN", INT2NUM(GL_TRIANGLE_LIST_SUN));
	rb_define_const(module, "GL_TRIANGLE_MESH_SUN", INT2NUM(GL_TRIANGLE_MESH_SUN));
	rb_define_const(module, "GL_TRIANGLE_STRIP", INT2NUM(GL_TRIANGLE_STRIP));
	rb_define_const(module, "GL_TRIANGLE_STRIP_ADJACENCY_ARB", INT2NUM(GL_TRIANGLE_STRIP_ADJACENCY_ARB));
	rb_define_const(module, "GL_TRIANGLE_STRIP_ADJACENCY_EXT", INT2NUM(GL_TRIANGLE_STRIP_ADJACENCY_EXT));
	rb_define_const(module, "GL_TYPE_RGBA_FLOAT_ATI", INT2NUM(GL_TYPE_RGBA_FLOAT_ATI));
	rb_define_const(module, "GL_UNIFORM_BUFFER_BINDING_EXT", INT2NUM(GL_UNIFORM_BUFFER_BINDING_EXT));
	rb_define_const(module, "GL_UNIFORM_BUFFER_EXT", INT2NUM(GL_UNIFORM_BUFFER_EXT));
	rb_define_const(module, "GL_UNPACK_ALIGNMENT", INT2NUM(GL_UNPACK_ALIGNMENT));
	rb_define_const(module, "GL_UNPACK_CLIENT_STORAGE_APPLE", INT2NUM(GL_UNPACK_CLIENT_STORAGE_APPLE));
	rb_define_const(module, "GL_UNPACK_CMYK_HINT_EXT", INT2NUM(GL_UNPACK_CMYK_HINT_EXT));
	rb_define_const(module, "GL_UNPACK_CONSTANT_DATA_SUNX", INT2NUM(GL_UNPACK_CONSTANT_DATA_SUNX));
	rb_define_const(module, "GL_UNPACK_IMAGE_DEPTH_SGIS", INT2NUM(GL_UNPACK_IMAGE_DEPTH_SGIS));
	rb_define_const(module, "GL_UNPACK_IMAGE_HEIGHT", INT2NUM(GL_UNPACK_IMAGE_HEIGHT));
	rb_define_const(module, "GL_UNPACK_IMAGE_HEIGHT_EXT", INT2NUM(GL_UNPACK_IMAGE_HEIGHT_EXT));
	rb_define_const(module, "GL_UNPACK_LSB_FIRST", INT2NUM(GL_UNPACK_LSB_FIRST));
	rb_define_const(module, "GL_UNPACK_RESAMPLE_OML", INT2NUM(GL_UNPACK_RESAMPLE_OML));
	rb_define_const(module, "GL_UNPACK_RESAMPLE_SGIX", INT2NUM(GL_UNPACK_RESAMPLE_SGIX));
	rb_define_const(module, "GL_UNPACK_ROW_LENGTH", INT2NUM(GL_UNPACK_ROW_LENGTH));
	rb_define_const(module, "GL_UNPACK_SKIP_IMAGES", INT2NUM(GL_UNPACK_SKIP_IMAGES));
	rb_define_const(module, "GL_UNPACK_SKIP_IMAGES_EXT", INT2NUM(GL_UNPACK_SKIP_IMAGES_EXT));
	rb_define_const(module, "GL_UNPACK_SKIP_PIXELS", INT2NUM(GL_UNPACK_SKIP_PIXELS));
	rb_define_const(module, "GL_UNPACK_SKIP_ROWS", INT2NUM(GL_UNPACK_SKIP_ROWS));
	rb_define_const(module, "GL_UNPACK_SKIP_VOLUMES_SGIS", INT2NUM(GL_UNPACK_SKIP_VOLUMES_SGIS));
	rb_define_const(module, "GL_UNPACK_SUBSAMPLE_RATE_SGIX", INT2NUM(GL_UNPACK_SUBSAMPLE_RATE_SGIX));
	rb_define_const(module, "GL_UNPACK_SWAP_BYTES", INT2NUM(GL_UNPACK_SWAP_BYTES));
	rb_define_const(module, "GL_UNSIGNED_BYTE", INT2NUM(GL_UNSIGNED_BYTE));
	rb_define_const(module, "GL_UNSIGNED_BYTE_2_3_3_REV", INT2NUM(GL_UNSIGNED_BYTE_2_3_3_REV));
	rb_define_const(module, "GL_UNSIGNED_BYTE_2_3_3_REV_EXT", INT2NUM(GL_UNSIGNED_BYTE_2_3_3_REV_EXT));
	rb_define_const(module, "GL_UNSIGNED_BYTE_3_3_2", INT2NUM(GL_UNSIGNED_BYTE_3_3_2));
	rb_define_const(module, "GL_UNSIGNED_BYTE_3_3_2_EXT", INT2NUM(GL_UNSIGNED_BYTE_3_3_2_EXT));
	rb_define_const(module, "GL_UNSIGNED_IDENTITY_NV", INT2NUM(GL_UNSIGNED_IDENTITY_NV));
	rb_define_const(module, "GL_UNSIGNED_INT", INT2NUM(GL_UNSIGNED_INT));
	rb_define_const(module, "GL_UNSIGNED_INT_10F_11F_11F_REV", INT2NUM(GL_UNSIGNED_INT_10F_11F_11F_REV));
	rb_define_const(module, "GL_UNSIGNED_INT_10F_11F_11F_REV_EXT", INT2NUM(GL_UNSIGNED_INT_10F_11F_11F_REV_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_10_10_10_2", INT2NUM(GL_UNSIGNED_INT_10_10_10_2));
	rb_define_const(module, "GL_UNSIGNED_INT_10_10_10_2_EXT", INT2NUM(GL_UNSIGNED_INT_10_10_10_2_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_24_8", INT2NUM(GL_UNSIGNED_INT_24_8));
	rb_define_const(module, "GL_UNSIGNED_INT_24_8_EXT", INT2NUM(GL_UNSIGNED_INT_24_8_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_24_8_NV", INT2NUM(GL_UNSIGNED_INT_24_8_NV));
	rb_define_const(module, "GL_UNSIGNED_INT_2_10_10_10_REV", INT2NUM(GL_UNSIGNED_INT_2_10_10_10_REV));
	rb_define_const(module, "GL_UNSIGNED_INT_2_10_10_10_REV_EXT", INT2NUM(GL_UNSIGNED_INT_2_10_10_10_REV_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_5_9_9_9_REV", INT2NUM(GL_UNSIGNED_INT_5_9_9_9_REV));
	rb_define_const(module, "GL_UNSIGNED_INT_5_9_9_9_REV_EXT", INT2NUM(GL_UNSIGNED_INT_5_9_9_9_REV_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_8_8_8_8", INT2NUM(GL_UNSIGNED_INT_8_8_8_8));
	rb_define_const(module, "GL_UNSIGNED_INT_8_8_8_8_EXT", INT2NUM(GL_UNSIGNED_INT_8_8_8_8_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_8_8_8_8_REV", INT2NUM(GL_UNSIGNED_INT_8_8_8_8_REV));
	rb_define_const(module, "GL_UNSIGNED_INT_8_8_8_8_REV_EXT", INT2NUM(GL_UNSIGNED_INT_8_8_8_8_REV_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_8_8_S8_S8_REV_NV", INT2NUM(GL_UNSIGNED_INT_8_8_S8_S8_REV_NV));
	rb_define_const(module, "GL_UNSIGNED_INT_S8_S8_8_8_NV", INT2NUM(GL_UNSIGNED_INT_S8_S8_8_8_NV));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_1D", INT2NUM(GL_UNSIGNED_INT_SAMPLER_1D));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_1D_ARRAY", INT2NUM(GL_UNSIGNED_INT_SAMPLER_1D_ARRAY));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT", INT2NUM(GL_UNSIGNED_INT_SAMPLER_1D_ARRAY_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_1D_EXT", INT2NUM(GL_UNSIGNED_INT_SAMPLER_1D_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_2D", INT2NUM(GL_UNSIGNED_INT_SAMPLER_2D));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_2D_ARRAY", INT2NUM(GL_UNSIGNED_INT_SAMPLER_2D_ARRAY));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT", INT2NUM(GL_UNSIGNED_INT_SAMPLER_2D_ARRAY_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_2D_EXT", INT2NUM(GL_UNSIGNED_INT_SAMPLER_2D_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT", INT2NUM(GL_UNSIGNED_INT_SAMPLER_2D_RECT_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_3D", INT2NUM(GL_UNSIGNED_INT_SAMPLER_3D));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_3D_EXT", INT2NUM(GL_UNSIGNED_INT_SAMPLER_3D_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT", INT2NUM(GL_UNSIGNED_INT_SAMPLER_BUFFER_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_CUBE", INT2NUM(GL_UNSIGNED_INT_SAMPLER_CUBE));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_CUBE_EXT", INT2NUM(GL_UNSIGNED_INT_SAMPLER_CUBE_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_SAMPLER_RENDERBUFFER_NV", INT2NUM(GL_UNSIGNED_INT_SAMPLER_RENDERBUFFER_NV));
	rb_define_const(module, "GL_UNSIGNED_INT_VEC2", INT2NUM(GL_UNSIGNED_INT_VEC2));
	rb_define_const(module, "GL_UNSIGNED_INT_VEC2_EXT", INT2NUM(GL_UNSIGNED_INT_VEC2_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_VEC3", INT2NUM(GL_UNSIGNED_INT_VEC3));
	rb_define_const(module, "GL_UNSIGNED_INT_VEC3_EXT", INT2NUM(GL_UNSIGNED_INT_VEC3_EXT));
	rb_define_const(module, "GL_UNSIGNED_INT_VEC4", INT2NUM(GL_UNSIGNED_INT_VEC4));
	rb_define_const(module, "GL_UNSIGNED_INT_VEC4_EXT", INT2NUM(GL_UNSIGNED_INT_VEC4_EXT));
	rb_define_const(module, "GL_UNSIGNED_INVERT_NV", INT2NUM(GL_UNSIGNED_INVERT_NV));
	rb_define_const(module, "GL_UNSIGNED_NORMALIZED", INT2NUM(GL_UNSIGNED_NORMALIZED));
	rb_define_const(module, "GL_UNSIGNED_NORMALIZED_ARB", INT2NUM(GL_UNSIGNED_NORMALIZED_ARB));
	rb_define_const(module, "GL_UNSIGNED_SHORT", INT2NUM(GL_UNSIGNED_SHORT));
	rb_define_const(module, "GL_UNSIGNED_SHORT_1_5_5_5_REV", INT2NUM(GL_UNSIGNED_SHORT_1_5_5_5_REV));
	rb_define_const(module, "GL_UNSIGNED_SHORT_1_5_5_5_REV_EXT", INT2NUM(GL_UNSIGNED_SHORT_1_5_5_5_REV_EXT));
	rb_define_const(module, "GL_UNSIGNED_SHORT_4_4_4_4", INT2NUM(GL_UNSIGNED_SHORT_4_4_4_4));
	rb_define_const(module, "GL_UNSIGNED_SHORT_4_4_4_4_EXT", INT2NUM(GL_UNSIGNED_SHORT_4_4_4_4_EXT));
	rb_define_const(module, "GL_UNSIGNED_SHORT_4_4_4_4_REV", INT2NUM(GL_UNSIGNED_SHORT_4_4_4_4_REV));
	rb_define_const(module, "GL_UNSIGNED_SHORT_4_4_4_4_REV_EXT", INT2NUM(GL_UNSIGNED_SHORT_4_4_4_4_REV_EXT));
	rb_define_const(module, "GL_UNSIGNED_SHORT_5_5_5_1", INT2NUM(GL_UNSIGNED_SHORT_5_5_5_1));
	rb_define_const(module, "GL_UNSIGNED_SHORT_5_5_5_1_EXT", INT2NUM(GL_UNSIGNED_SHORT_5_5_5_1_EXT));
	rb_define_const(module, "GL_UNSIGNED_SHORT_5_6_5", INT2NUM(GL_UNSIGNED_SHORT_5_6_5));
	rb_define_const(module, "GL_UNSIGNED_SHORT_5_6_5_EXT", INT2NUM(GL_UNSIGNED_SHORT_5_6_5_EXT));
	rb_define_const(module, "GL_UNSIGNED_SHORT_5_6_5_REV", INT2NUM(GL_UNSIGNED_SHORT_5_6_5_REV));
	rb_define_const(module, "GL_UNSIGNED_SHORT_5_6_5_REV_EXT", INT2NUM(GL_UNSIGNED_SHORT_5_6_5_REV_EXT));
	rb_define_const(module, "GL_UNSIGNED_SHORT_8_8_APPLE", INT2NUM(GL_UNSIGNED_SHORT_8_8_APPLE));
	rb_define_const(module, "GL_UNSIGNED_SHORT_8_8_MESA", INT2NUM(GL_UNSIGNED_SHORT_8_8_MESA));
	rb_define_const(module, "GL_UNSIGNED_SHORT_8_8_REV_APPLE", INT2NUM(GL_UNSIGNED_SHORT_8_8_REV_APPLE));
	rb_define_const(module, "GL_UNSIGNED_SHORT_8_8_REV_MESA", INT2NUM(GL_UNSIGNED_SHORT_8_8_REV_MESA));
	rb_define_const(module, "GL_UPPER_LEFT", INT2NUM(GL_UPPER_LEFT));
	rb_define_const(module, "GL_V2F", INT2NUM(GL_V2F));
	rb_define_const(module, "GL_V3F", INT2NUM(GL_V3F));
	rb_define_const(module, "GL_VALIDATE_STATUS", INT2NUM(GL_VALIDATE_STATUS));
	rb_define_const(module, "GL_VARIABLE_A_NV", INT2NUM(GL_VARIABLE_A_NV));
	rb_define_const(module, "GL_VARIABLE_B_NV", INT2NUM(GL_VARIABLE_B_NV));
	rb_define_const(module, "GL_VARIABLE_C_NV", INT2NUM(GL_VARIABLE_C_NV));
	rb_define_const(module, "GL_VARIABLE_D_NV", INT2NUM(GL_VARIABLE_D_NV));
	rb_define_const(module, "GL_VARIABLE_E_NV", INT2NUM(GL_VARIABLE_E_NV));
	rb_define_const(module, "GL_VARIABLE_F_NV", INT2NUM(GL_VARIABLE_F_NV));
	rb_define_const(module, "GL_VARIABLE_G_NV", INT2NUM(GL_VARIABLE_G_NV));
	rb_define_const(module, "GL_VARIANT_ARRAY_EXT", INT2NUM(GL_VARIANT_ARRAY_EXT));
	rb_define_const(module, "GL_VARIANT_ARRAY_POINTER_EXT", INT2NUM(GL_VARIANT_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_VARIANT_ARRAY_STRIDE_EXT", INT2NUM(GL_VARIANT_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_VARIANT_ARRAY_TYPE_EXT", INT2NUM(GL_VARIANT_ARRAY_TYPE_EXT));
	rb_define_const(module, "GL_VARIANT_DATATYPE_EXT", INT2NUM(GL_VARIANT_DATATYPE_EXT));
	rb_define_const(module, "GL_VARIANT_EXT", INT2NUM(GL_VARIANT_EXT));
	rb_define_const(module, "GL_VARIANT_VALUE_EXT", INT2NUM(GL_VARIANT_VALUE_EXT));
	rb_define_const(module, "GL_VECTOR_EXT", INT2NUM(GL_VECTOR_EXT));
	rb_define_const(module, "GL_VENDOR", INT2NUM(GL_VENDOR));
	rb_define_const(module, "GL_VERSION", INT2NUM(GL_VERSION));
	rb_define_const(module, "GL_VERSION_1_1", INT2NUM(GL_VERSION_1_1));
	rb_define_const(module, "GL_VERSION_1_2", INT2NUM(GL_VERSION_1_2));
	rb_define_const(module, "GL_VERSION_1_3", INT2NUM(GL_VERSION_1_3));
	rb_define_const(module, "GL_VERSION_1_4", INT2NUM(GL_VERSION_1_4));
	rb_define_const(module, "GL_VERSION_1_5", INT2NUM(GL_VERSION_1_5));
	rb_define_const(module, "GL_VERSION_2_0", INT2NUM(GL_VERSION_2_0));
	rb_define_const(module, "GL_VERSION_2_1", INT2NUM(GL_VERSION_2_1));
	rb_define_const(module, "GL_VERSION_3_0", INT2NUM(GL_VERSION_3_0));
	rb_define_const(module, "GL_VERTEX23_BIT_PGI", INT2NUM(GL_VERTEX23_BIT_PGI));
	rb_define_const(module, "GL_VERTEX4_BIT_PGI", INT2NUM(GL_VERTEX4_BIT_PGI));
	rb_define_const(module, "GL_VERTEX_ARRAY", INT2NUM(GL_VERTEX_ARRAY));
	rb_define_const(module, "GL_VERTEX_ARRAY_BINDING", INT2NUM(GL_VERTEX_ARRAY_BINDING));
	rb_define_const(module, "GL_VERTEX_ARRAY_BINDING_APPLE", INT2NUM(GL_VERTEX_ARRAY_BINDING_APPLE));
	rb_define_const(module, "GL_VERTEX_ARRAY_BUFFER_BINDING", INT2NUM(GL_VERTEX_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_VERTEX_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_VERTEX_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_VERTEX_ARRAY_COUNT_EXT", INT2NUM(GL_VERTEX_ARRAY_COUNT_EXT));
	rb_define_const(module, "GL_VERTEX_ARRAY_EXT", INT2NUM(GL_VERTEX_ARRAY_EXT));
	rb_define_const(module, "GL_VERTEX_ARRAY_LIST_IBM", INT2NUM(GL_VERTEX_ARRAY_LIST_IBM));
	rb_define_const(module, "GL_VERTEX_ARRAY_LIST_STRIDE_IBM", INT2NUM(GL_VERTEX_ARRAY_LIST_STRIDE_IBM));
	rb_define_const(module, "GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL", INT2NUM(GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL));
	rb_define_const(module, "GL_VERTEX_ARRAY_POINTER", INT2NUM(GL_VERTEX_ARRAY_POINTER));
	rb_define_const(module, "GL_VERTEX_ARRAY_POINTER_EXT", INT2NUM(GL_VERTEX_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_VERTEX_ARRAY_RANGE_APPLE", INT2NUM(GL_VERTEX_ARRAY_RANGE_APPLE));
	rb_define_const(module, "GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE", INT2NUM(GL_VERTEX_ARRAY_RANGE_LENGTH_APPLE));
	rb_define_const(module, "GL_VERTEX_ARRAY_RANGE_LENGTH_NV", INT2NUM(GL_VERTEX_ARRAY_RANGE_LENGTH_NV));
	rb_define_const(module, "GL_VERTEX_ARRAY_RANGE_NV", INT2NUM(GL_VERTEX_ARRAY_RANGE_NV));
	rb_define_const(module, "GL_VERTEX_ARRAY_RANGE_POINTER_APPLE", INT2NUM(GL_VERTEX_ARRAY_RANGE_POINTER_APPLE));
	rb_define_const(module, "GL_VERTEX_ARRAY_RANGE_POINTER_NV", INT2NUM(GL_VERTEX_ARRAY_RANGE_POINTER_NV));
	rb_define_const(module, "GL_VERTEX_ARRAY_RANGE_VALID_NV", INT2NUM(GL_VERTEX_ARRAY_RANGE_VALID_NV));
	rb_define_const(module, "GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV", INT2NUM(GL_VERTEX_ARRAY_RANGE_WITHOUT_FLUSH_NV));
	rb_define_const(module, "GL_VERTEX_ARRAY_SIZE", INT2NUM(GL_VERTEX_ARRAY_SIZE));
	rb_define_const(module, "GL_VERTEX_ARRAY_SIZE_EXT", INT2NUM(GL_VERTEX_ARRAY_SIZE_EXT));
	rb_define_const(module, "GL_VERTEX_ARRAY_STORAGE_HINT_APPLE", INT2NUM(GL_VERTEX_ARRAY_STORAGE_HINT_APPLE));
	rb_define_const(module, "GL_VERTEX_ARRAY_STRIDE", INT2NUM(GL_VERTEX_ARRAY_STRIDE));
	rb_define_const(module, "GL_VERTEX_ARRAY_STRIDE_EXT", INT2NUM(GL_VERTEX_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_VERTEX_ARRAY_TYPE", INT2NUM(GL_VERTEX_ARRAY_TYPE));
	rb_define_const(module, "GL_VERTEX_ARRAY_TYPE_EXT", INT2NUM(GL_VERTEX_ARRAY_TYPE_EXT));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY0_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY0_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY10_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY10_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY11_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY11_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY12_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY12_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY13_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY13_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY14_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY14_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY15_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY15_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY1_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY1_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY2_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY2_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY3_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY3_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY4_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY4_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY5_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY5_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY6_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY6_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY7_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY7_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY8_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY8_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY9_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY9_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_ENABLED", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_ENABLED));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_INTEGER", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_INTEGER));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_INTEGER_NV", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_INTEGER_NV));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_NORMALIZED", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_NORMALIZED));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_POINTER", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_POINTER));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_SIZE", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_SIZE));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_STRIDE", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_STRIDE));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_TYPE", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_TYPE));
	rb_define_const(module, "GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB", INT2NUM(GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB));
	rb_define_const(module, "GL_VERTEX_BLEND_ARB", INT2NUM(GL_VERTEX_BLEND_ARB));
	rb_define_const(module, "GL_VERTEX_CONSISTENT_HINT_PGI", INT2NUM(GL_VERTEX_CONSISTENT_HINT_PGI));
	rb_define_const(module, "GL_VERTEX_DATA_HINT_PGI", INT2NUM(GL_VERTEX_DATA_HINT_PGI));
	rb_define_const(module, "GL_VERTEX_ID_NV", INT2NUM(GL_VERTEX_ID_NV));
	rb_define_const(module, "GL_VERTEX_PRECLIP_HINT_SGIX", INT2NUM(GL_VERTEX_PRECLIP_HINT_SGIX));
	rb_define_const(module, "GL_VERTEX_PRECLIP_SGIX", INT2NUM(GL_VERTEX_PRECLIP_SGIX));
	rb_define_const(module, "GL_VERTEX_PROGRAM_ARB", INT2NUM(GL_VERTEX_PROGRAM_ARB));
	rb_define_const(module, "GL_VERTEX_PROGRAM_BINDING_NV", INT2NUM(GL_VERTEX_PROGRAM_BINDING_NV));
	rb_define_const(module, "GL_VERTEX_PROGRAM_NV", INT2NUM(GL_VERTEX_PROGRAM_NV));
	rb_define_const(module, "GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV", INT2NUM(GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV));
	rb_define_const(module, "GL_VERTEX_PROGRAM_POINT_SIZE", INT2NUM(GL_VERTEX_PROGRAM_POINT_SIZE));
	rb_define_const(module, "GL_VERTEX_PROGRAM_POINT_SIZE_ARB", INT2NUM(GL_VERTEX_PROGRAM_POINT_SIZE_ARB));
	rb_define_const(module, "GL_VERTEX_PROGRAM_POINT_SIZE_NV", INT2NUM(GL_VERTEX_PROGRAM_POINT_SIZE_NV));
	rb_define_const(module, "GL_VERTEX_PROGRAM_TWO_SIDE", INT2NUM(GL_VERTEX_PROGRAM_TWO_SIDE));
	rb_define_const(module, "GL_VERTEX_PROGRAM_TWO_SIDE_ARB", INT2NUM(GL_VERTEX_PROGRAM_TWO_SIDE_ARB));
	rb_define_const(module, "GL_VERTEX_PROGRAM_TWO_SIDE_NV", INT2NUM(GL_VERTEX_PROGRAM_TWO_SIDE_NV));
	rb_define_const(module, "GL_VERTEX_SHADER", INT2NUM(GL_VERTEX_SHADER));
	rb_define_const(module, "GL_VERTEX_SHADER_ARB", INT2NUM(GL_VERTEX_SHADER_ARB));
	rb_define_const(module, "GL_VERTEX_SHADER_BINDING_EXT", INT2NUM(GL_VERTEX_SHADER_BINDING_EXT));
	rb_define_const(module, "GL_VERTEX_SHADER_EXT", INT2NUM(GL_VERTEX_SHADER_EXT));
	rb_define_const(module, "GL_VERTEX_SHADER_INSTRUCTIONS_EXT", INT2NUM(GL_VERTEX_SHADER_INSTRUCTIONS_EXT));
	rb_define_const(module, "GL_VERTEX_SHADER_INVARIANTS_EXT", INT2NUM(GL_VERTEX_SHADER_INVARIANTS_EXT));
	rb_define_const(module, "GL_VERTEX_SHADER_LOCALS_EXT", INT2NUM(GL_VERTEX_SHADER_LOCALS_EXT));
	rb_define_const(module, "GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT", INT2NUM(GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT));
	rb_define_const(module, "GL_VERTEX_SHADER_OPTIMIZED_EXT", INT2NUM(GL_VERTEX_SHADER_OPTIMIZED_EXT));
	rb_define_const(module, "GL_VERTEX_SHADER_VARIANTS_EXT", INT2NUM(GL_VERTEX_SHADER_VARIANTS_EXT));
	rb_define_const(module, "GL_VERTEX_SOURCE_ATI", INT2NUM(GL_VERTEX_SOURCE_ATI));
	rb_define_const(module, "GL_VERTEX_STATE_PROGRAM_NV", INT2NUM(GL_VERTEX_STATE_PROGRAM_NV));
	rb_define_const(module, "GL_VERTEX_STREAM0_ATI", INT2NUM(GL_VERTEX_STREAM0_ATI));
	rb_define_const(module, "GL_VERTEX_STREAM1_ATI", INT2NUM(GL_VERTEX_STREAM1_ATI));
	rb_define_const(module, "GL_VERTEX_STREAM2_ATI", INT2NUM(GL_VERTEX_STREAM2_ATI));
	rb_define_const(module, "GL_VERTEX_STREAM3_ATI", INT2NUM(GL_VERTEX_STREAM3_ATI));
	rb_define_const(module, "GL_VERTEX_STREAM4_ATI", INT2NUM(GL_VERTEX_STREAM4_ATI));
	rb_define_const(module, "GL_VERTEX_STREAM5_ATI", INT2NUM(GL_VERTEX_STREAM5_ATI));
	rb_define_const(module, "GL_VERTEX_STREAM6_ATI", INT2NUM(GL_VERTEX_STREAM6_ATI));
	rb_define_const(module, "GL_VERTEX_STREAM7_ATI", INT2NUM(GL_VERTEX_STREAM7_ATI));
	rb_define_const(module, "GL_VERTEX_WEIGHTING_EXT", INT2NUM(GL_VERTEX_WEIGHTING_EXT));
	rb_define_const(module, "GL_VERTEX_WEIGHT_ARRAY_EXT", INT2NUM(GL_VERTEX_WEIGHT_ARRAY_EXT));
	rb_define_const(module, "GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT", INT2NUM(GL_VERTEX_WEIGHT_ARRAY_POINTER_EXT));
	rb_define_const(module, "GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT", INT2NUM(GL_VERTEX_WEIGHT_ARRAY_SIZE_EXT));
	rb_define_const(module, "GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT", INT2NUM(GL_VERTEX_WEIGHT_ARRAY_STRIDE_EXT));
	rb_define_const(module, "GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT", INT2NUM(GL_VERTEX_WEIGHT_ARRAY_TYPE_EXT));
	rb_define_const(module, "GL_VIBRANCE_BIAS_NV", INT2NUM(GL_VIBRANCE_BIAS_NV));
	rb_define_const(module, "GL_VIBRANCE_SCALE_NV", INT2NUM(GL_VIBRANCE_SCALE_NV));
	rb_define_const(module, "GL_VIEWPORT", INT2NUM(GL_VIEWPORT));
	rb_define_const(module, "GL_VIEWPORT_BIT", INT2NUM(GL_VIEWPORT_BIT));
	rb_define_const(module, "GL_WEIGHT_ARRAY_ARB", INT2NUM(GL_WEIGHT_ARRAY_ARB));
	rb_define_const(module, "GL_WEIGHT_ARRAY_BUFFER_BINDING", INT2NUM(GL_WEIGHT_ARRAY_BUFFER_BINDING));
	rb_define_const(module, "GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB", INT2NUM(GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB));
	rb_define_const(module, "GL_WEIGHT_ARRAY_POINTER_ARB", INT2NUM(GL_WEIGHT_ARRAY_POINTER_ARB));
	rb_define_const(module, "GL_WEIGHT_ARRAY_SIZE_ARB", INT2NUM(GL_WEIGHT_ARRAY_SIZE_ARB));
	rb_define_const(module, "GL_WEIGHT_ARRAY_STRIDE_ARB", INT2NUM(GL_WEIGHT_ARRAY_STRIDE_ARB));
	rb_define_const(module, "GL_WEIGHT_ARRAY_TYPE_ARB", INT2NUM(GL_WEIGHT_ARRAY_TYPE_ARB));
	rb_define_const(module, "GL_WEIGHT_SUM_UNITY_ARB", INT2NUM(GL_WEIGHT_SUM_UNITY_ARB));
	rb_define_const(module, "GL_WIDE_LINE_HINT_PGI", INT2NUM(GL_WIDE_LINE_HINT_PGI));
	rb_define_const(module, "GL_WRAP_BORDER_SUN", INT2NUM(GL_WRAP_BORDER_SUN));
	rb_define_const(module, "GL_WRITE_ONLY", INT2NUM(GL_WRITE_ONLY));
	rb_define_const(module, "GL_WRITE_ONLY_ARB", INT2NUM(GL_WRITE_ONLY_ARB));
	rb_define_const(module, "GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV", INT2NUM(GL_WRITE_PIXEL_DATA_RANGE_LENGTH_NV));
	rb_define_const(module, "GL_WRITE_PIXEL_DATA_RANGE_NV", INT2NUM(GL_WRITE_PIXEL_DATA_RANGE_NV));
	rb_define_const(module, "GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV", INT2NUM(GL_WRITE_PIXEL_DATA_RANGE_POINTER_NV));
	rb_define_const(module, "GL_W_EXT", INT2NUM(GL_W_EXT));
	rb_define_const(module, "GL_XOR", INT2NUM(GL_XOR));
	rb_define_const(module, "GL_X_EXT", INT2NUM(GL_X_EXT));
	rb_define_const(module, "GL_YCBCR_422_APPLE", INT2NUM(GL_YCBCR_422_APPLE));
	rb_define_const(module, "GL_YCBCR_MESA", INT2NUM(GL_YCBCR_MESA));
	rb_define_const(module, "GL_YCRCBA_SGIX", INT2NUM(GL_YCRCBA_SGIX));
	rb_define_const(module, "GL_YCRCB_422_SGIX", INT2NUM(GL_YCRCB_422_SGIX));
	rb_define_const(module, "GL_YCRCB_444_SGIX", INT2NUM(GL_YCRCB_444_SGIX));
	rb_define_const(module, "GL_YCRCB_SGIX", INT2NUM(GL_YCRCB_SGIX));
	rb_define_const(module, "GL_Y_EXT", INT2NUM(GL_Y_EXT));
	rb_define_const(module, "GL_ZERO", INT2NUM(GL_ZERO));
	rb_define_const(module, "GL_ZERO_EXT", INT2NUM(GL_ZERO_EXT));
	rb_define_const(module, "GL_ZOOM_X", INT2NUM(GL_ZOOM_X));
	rb_define_const(module, "GL_ZOOM_Y", INT2NUM(GL_ZOOM_Y));
	rb_define_const(module, "GL_Z_EXT", INT2NUM(GL_Z_EXT));
}
